Package: tna
Title: Transition Network Analysis (TNA)
Version: 1.2.3
Authors@R: c(
    person("Mohammed", "Saqr", role = "aut"),
    person("Santtu", "Tikka", role = "aut"),
    person("Sonsoles", "L\u00f3pez-Pernas",
           email = "sonsoles.lopez@uef.fi", role = c("aut", "cre", "cph"))
  )
Description: Provides tools for performing Transition Network Analysis (TNA) to 
    study relational dynamics, including functions for building and plotting TNA 
    models, calculating centrality measures, and identifying dominant events and 
    patterns. TNA statistical techniques (e.g., bootstrapping and permutation 
    tests) ensure the reliability of observed insights and confirm that 
    identified dynamics are meaningful. See (Saqr et al., 2025) 
    <doi:10.1145/3706468.3706513> for more details on TNA.
License: MIT + file LICENSE
URL: https://github.com/sonsoleslp/tna/, https://sonsoles.me/tna/
BugReports: https://github.com/sonsoleslp/tna/issues/
Depends: R (>= 4.1.0)
Imports: checkmate, cli, cluster, cograph, colorspace, dplyr, ggplot2,
        graphics, igraph, RColorBrewer, rlang, stats, tibble, tidyr,
        tidyselect
Suggests: gt, knitr, pracma, rmarkdown, seqHMM, stringdist, testthat
        (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
LazyData: true
Config/Needs/website: rmarkdown
NeedsCompilation: no
Packaged: 2026-04-26 23:22:42 UTC; slopezpe
Author: Mohammed Saqr [aut],
  Santtu Tikka [aut],
  Sonsoles López-Pernas [aut, cre, cph]
Maintainer: Sonsoles López-Pernas <sonsoles.lopez@uef.fi>
Repository: CRAN
Date/Publication: 2026-04-27 00:00:03 UTC
Built: R 4.6.0; ; 2026-05-01 22:29:40 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.igraph,group_tna)
S3method(as.igraph,matrix)
S3method(as.igraph,tna)
S3method(betweenness_network,tna)
S3method(bootstrap,group_tna)
S3method(bootstrap,tna)
S3method(bootstrap_cliques,tna)
S3method(build_model,data.frame)
S3method(build_model,default)
S3method(build_model,matrix)
S3method(build_model,stslist)
S3method(build_model,tna_data)
S3method(build_model,tsn)
S3method(centralities,group_tna)
S3method(centralities,matrix)
S3method(centralities,tna)
S3method(cliques,group_tna)
S3method(cliques,tna)
S3method(communities,group_tna)
S3method(communities,tna)
S3method(compare,group_tna)
S3method(compare,matrix)
S3method(compare,tna)
S3method(compare_sequences,default)
S3method(compare_sequences,group_tna)
S3method(deprune,group_tna)
S3method(deprune,tna)
S3method(estimate_centrality_stability,group_tna)
S3method(estimate_centrality_stability,tna)
S3method(estimate_cs,group_tna)
S3method(estimate_cs,tna)
S3method(group_model,default)
S3method(group_model,mhmm)
S3method(group_model,tna_clustering)
S3method(group_model,tna_mmm)
S3method(hist,group_tna)
S3method(hist,tna)
S3method(mmm_stats,mhmm)
S3method(mmm_stats,tna_mmm)
S3method(permutation_test,group_tna)
S3method(permutation_test,tna)
S3method(plot,group_tna)
S3method(plot,group_tna_bootstrap)
S3method(plot,group_tna_centralities)
S3method(plot,group_tna_cliques)
S3method(plot,group_tna_communities)
S3method(plot,group_tna_permutation)
S3method(plot,group_tna_stability)
S3method(plot,tna)
S3method(plot,tna_bootstrap)
S3method(plot,tna_centralities)
S3method(plot,tna_cliques)
S3method(plot,tna_communities)
S3method(plot,tna_comparison)
S3method(plot,tna_permutation)
S3method(plot,tna_reliability)
S3method(plot,tna_sequence_comparison)
S3method(plot,tna_stability)
S3method(plot_associations,tna)
S3method(plot_compare,group_tna)
S3method(plot_compare,tna)
S3method(plot_frequencies,group_tna)
S3method(plot_frequencies,tna)
S3method(plot_mosaic,group_tna)
S3method(plot_mosaic,tna)
S3method(plot_mosaic,tna_data)
S3method(plot_sequences,default)
S3method(plot_sequences,group_tna)
S3method(plot_sequences,tna)
S3method(plot_sequences,tna_data)
S3method(print,group_tna)
S3method(print,group_tna_bootstrap)
S3method(print,group_tna_centralities)
S3method(print,group_tna_cliques)
S3method(print,group_tna_communities)
S3method(print,group_tna_permutation)
S3method(print,group_tna_stability)
S3method(print,summary.group_tna)
S3method(print,summary.group_tna_bootstrap)
S3method(print,summary.tna)
S3method(print,summary.tna_bootstrap)
S3method(print,tna)
S3method(print,tna_bootstrap)
S3method(print,tna_centralities)
S3method(print,tna_cliques)
S3method(print,tna_clustering)
S3method(print,tna_communities)
S3method(print,tna_comparison)
S3method(print,tna_data)
S3method(print,tna_permutation)
S3method(print,tna_reliability)
S3method(print,tna_sequence_comparison)
S3method(print,tna_stability)
S3method(prune,group_tna)
S3method(prune,tna)
S3method(pruning_details,group_tna)
S3method(pruning_details,tna)
S3method(reliability,tna)
S3method(reprune,group_tna)
S3method(reprune,tna)
S3method(simulate,group_tna)
S3method(simulate,tna)
S3method(summary,group_tna)
S3method(summary,group_tna_bootstrap)
S3method(summary,tna)
S3method(summary,tna_bootstrap)
export(as.igraph)
export(atna)
export(betweenness_network)
export(bootstrap)
export(bootstrap_cliques)
export(build_model)
export(centralities)
export(cliques)
export(cluster_data)
export(cluster_sequences)
export(communities)
export(compare)
export(compare_sequences)
export(ctna)
export(deprune)
export(estimate_centrality_stability)
export(estimate_cs)
export(ftna)
export(group_atna)
export(group_ctna)
export(group_ftna)
export(group_model)
export(group_tna)
export(import_data)
export(import_onehot)
export(list_random_state_pools)
export(mmm_stats)
export(permutation_test)
export(plot_associations)
export(plot_compare)
export(plot_frequencies)
export(plot_model)
export(plot_mosaic)
export(plot_sequences)
export(prepare_data)
export(prune)
export(pruning_details)
export(random_group_tna)
export(random_tna)
export(random_tna_mmm)
export(reliability)
export(rename_groups)
export(reprune)
export(simulate)
export(sna)
export(tna)
export(tsn)
importFrom(igraph,as.igraph)
importFrom(stats,simulate)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# `tna`: An R package for Transition Network Analysis <img src="man/figures/logo.png" align="right" height="139"/></a>

<!-- badges: start -->

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![R-CMD-check](https://github.com/sonsoleslp/tna/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/sonsoleslp/tna/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/sonsoleslp/tna/graph/badge.svg)](https://app.codecov.io/gh/sonsoleslp/tna)
[![tna CRAN
badge](https://www.r-pkg.org/badges/version/tna)](https://cran.r-project.org/package=tna)
[![License:
MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)

<!-- badges: end -->

`tna` is an R package for the analysis of relational dynamics through
Transition Network Analysis (TNA). TNA provides tools for building TNA
models, plotting transition networks, calculating centrality measures,
and identifying dominant events and patterns. TNA statistical techniques
(e.g., bootstrapping and permutation tests) ensure the reliability of
observed insights and confirm that identified dynamics are meaningful.
See [(Saqr et al., 2025)](https://doi.org/10.1145/3706468.3706513) for
more details on TNA.

![](man/figures/method.png)

## Resources

### Companion tutorials

We have also released comprehensive new tutorials for the main TNA
features:

| Tutorial | Link |
|----|----|
| An Updated Comprehensive Tutorial on Transition Network Analysis (TNA) | <https://sonsoles.me/posts/tna-tutorial/> |
| TNA Data Preparation: A Comprehensive Guide to `prepare_data()` | <https://sonsoles.me/posts/tna-data/> |
| TNA Group Analysis: Analysis and Comparison of Groups | <https://sonsoles.me/posts/tna-group/> |
| TNA Clustering: Discovering and Analysis of Clusters | <https://sonsoles.me/posts/tna-clustering/> |
| TNA Model Comparison:TNA Model Comparison: A Comprehensive Guide to Network Comparison | <https://sonsoles.me/posts/tna-compare/> |
| Full reference guide on `tna` functions | <https://sonsoles.me/tna/tna.html> |
| Sequence Patterns, Outcomes, and Indices with `codyna` | <https://sonsoles.me/posts/codyna-seq-tutorial/> |

### Vignettes

Check out the `tna` R package vignettes:

| Vignette | Link |
|----|----|
| Getting started with tna | <https://sonsoles.me/tna/articles/tna.html> |
| A showcase of the main tna functions | <https://sonsoles.me/tna/articles/complete_tutorial.html> |
| How to prepare data for tna | <https://sonsoles.me/tna/articles/prepare_data.html> |
| Frequency-based TNA | <https://sonsoles.me/tna/articles/ftna.html> |
| Attention TNA | <https://sonsoles.me/tna/articles/atna.html> |
| Finding cliques and communities | <https://sonsoles.me/tna/articles/communities_and_cliques.html> |
| Using grouped sequence data | <https://sonsoles.me/tna/articles/grouped_sequences.html> |

### Book chapters

Do not forget to check out our tutorials in the “Advanced learning
analytics methods” book:

| Title | Pages | Tutorial |
|----|----|----|
| Saqr, M., Lopez-Pernas, S., & Tikka, S. *Mapping Relational Dynamics with Transition Network Analysis: A Primer and Tutorial* | <https://doi.org/10.1007/978-3-031-95365-1_15> | [Online tutorial](https://lamethods.org/book2/chapters/ch15-tna/ch15-tna.html) |
| Saqr, M., Lopez-Pernas, S., & Tikka, S. *Capturing the Breadth and Dynamics of the Temporal Processes with Frequency Transition Network Analysis: A Primer and Tutorial* | <https://doi.org/10.1007/978-3-031-95365-1_16> | [Online tutorial](https://lamethods.org/book2/chapters/ch16-ftna/ch16-ftna.html) |
| Lopez-Pernas, S., Tikka, S., & Saqr, M. *Mining Patterns and Clusters with Transition Network Analysis: A Heterogeneity Approach* | <https://doi.org/10.1007/978-3-031-95365-1_17> | [Online tutorial](https://lamethods.org/book2/chapters/ch17-tna-clusters/ch17-tna-clusters.html) |

### Other tools

In addition to the `tna` R package, you can also try our [Shiny
app](https://sonsoleslp.shinyapps.io/tna-app/) and [Jamovi
plugin](https://github.com/SONsoleslp/JTNA).

## Installation

You can install the most recent stable version of `tna` from
[CRAN](https://cran.r-project.org/package=tna) or the development
version from [GitHub](https://github.com/) by running one of the
following:

``` r
install.packages("tna")

# install.packages("devtools")
# devtools::install_github("sonsoleslp/tna")
```

## Example

Load the package

``` r
library("tna")
```

Example data

``` r
data("group_regulation", package = "tna")
```

Build a Markov model

``` r
tna_model <- tna(group_regulation)
```

``` r
summary(tna_model)
```

<div id="wqjybadbrp" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
  &#10;  <table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false" style="-webkit-font-smoothing: antialiased; -moz-osx-font-smoothing: grayscale; font-family: system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; display: table; border-collapse: collapse; line-height: normal; margin-left: auto; margin-right: auto; color: #333333; font-size: 16px; font-weight: normal; font-style: normal; background-color: #FFFFFF; width: auto; border-top-style: solid; border-top-width: 2px; border-top-color: #A8A8A8; border-right-style: none; border-right-width: 2px; border-right-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #A8A8A8; border-left-style: none; border-left-width: 2px; border-left-color: #D3D3D3;" bgcolor="#FFFFFF">
  <thead style="border-style: none;">
    <tr class="gt_col_headings" style="border-style: none; border-top-style: solid; border-top-width: 2px; border-top-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3;">
      <th class="gt_col_heading gt_columns_bottom_border gt_left" rowspan="1" colspan="1" scope="col" id="metric" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: left;" bgcolor="#FFFFFF" valign="bottom" align="left">metric</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="value" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">value</th>
    </tr>
  </thead>
  <tbody class="gt_table_body" style="border-style: none; border-top-style: solid; border-top-width: 2px; border-top-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #D3D3D3;">
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Node Count</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">9.00</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Edge Count</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">78.00</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Network Density</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">1.00</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Mean Distance</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.05</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Mean Out-Strength</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">1.00</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">SD Out-Strength</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.81</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Mean In-Strength</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">1.00</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">SD In-Strength</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.00</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Mean Out-Degree</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">8.67</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">SD Out-Degree</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.71</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Centralization (Out-Degree)</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.02</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Centralization (In-Degree)</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.02</td></tr>
    <tr style="border-style: none;"><td headers="metric" class="gt_row gt_left" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: left;" valign="middle" align="left">Reciprocity</td>
<td headers="value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.99</td></tr>
  </tbody>
  &#10;</table>
</div>

Plot the transition network

``` r
plot(tna_model) 
```

<img src="man/figures/README-tnaplot-1.svg" alt="" width="60%" style='text-align: center; display: block; margin: auto;' style="display: block; margin: auto;" />
Calculate the centrality measures

``` r
cent <- centralities(tna_model)
```

<div id="ondqhamusk" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
  &#10;  <table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false" style="-webkit-font-smoothing: antialiased; -moz-osx-font-smoothing: grayscale; font-family: system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; display: table; border-collapse: collapse; line-height: normal; margin-left: auto; margin-right: auto; color: #333333; font-size: 8px; font-weight: normal; font-style: normal; background-color: #FFFFFF; width: auto; border-top-style: solid; border-top-width: 2px; border-top-color: #A8A8A8; border-right-style: none; border-right-width: 2px; border-right-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #A8A8A8; border-left-style: none; border-left-width: 2px; border-left-color: #D3D3D3;" bgcolor="#FFFFFF">
  <thead style="border-style: none;">
    <tr class="gt_col_headings" style="border-style: none; border-top-style: solid; border-top-width: 2px; border-top-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3;">
      <th class="gt_col_heading gt_columns_bottom_border gt_center" rowspan="1" colspan="1" scope="col" id="state" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: center;" bgcolor="#FFFFFF" valign="bottom" align="center">state</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="OutStrength" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">OutStrength</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="InStrength" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">InStrength</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="ClosenessIn" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">ClosenessIn</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="ClosenessOut" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">ClosenessOut</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="Closeness" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">Closeness</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="Betweenness" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">Betweenness</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="BetweennessRSP" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">BetweennessRSP</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="Diffusion" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">Diffusion</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="Clustering" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">Clustering</th>
    </tr>
  </thead>
  <tbody class="gt_table_body" style="border-style: none; border-top-style: solid; border-top-width: 2px; border-top-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #D3D3D3;">
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">adapt</td>
<td headers="OutStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">1.000</td>
<td headers="InStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.345</td>
<td headers="ClosenessIn" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.008</td>
<td headers="ClosenessOut" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.015</td>
<td headers="Closeness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.025</td>
<td headers="Betweenness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">1.000</td>
<td headers="BetweennessRSP" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">1.000</td>
<td headers="Diffusion" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">5.586</td>
<td headers="Clustering" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.337</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">cohesion</td>
<td headers="OutStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.973</td>
<td headers="InStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.812</td>
<td headers="ClosenessIn" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.014</td>
<td headers="ClosenessOut" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.012</td>
<td headers="Closeness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.027</td>
<td headers="Betweenness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.000</td>
<td headers="BetweennessRSP" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">19.000</td>
<td headers="Diffusion" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">5.209</td>
<td headers="Clustering" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.300</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">consensus</td>
<td headers="OutStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.918</td>
<td headers="InStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">2.667</td>
<td headers="ClosenessIn" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.035</td>
<td headers="ClosenessOut" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.013</td>
<td headers="Closeness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.038</td>
<td headers="Betweenness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">30.000</td>
<td headers="BetweennessRSP" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">103.000</td>
<td headers="Diffusion" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">4.660</td>
<td headers="Clustering" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.161</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">coregulate</td>
<td headers="OutStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.977</td>
<td headers="InStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.567</td>
<td headers="ClosenessIn" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.016</td>
<td headers="ClosenessOut" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.015</td>
<td headers="Closeness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.021</td>
<td headers="Betweenness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.000</td>
<td headers="BetweennessRSP" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">27.000</td>
<td headers="Diffusion" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">5.148</td>
<td headers="Clustering" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.306</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">discuss</td>
<td headers="OutStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.805</td>
<td headers="InStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">1.188</td>
<td headers="ClosenessIn" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.020</td>
<td headers="ClosenessOut" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.013</td>
<td headers="Closeness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.027</td>
<td headers="Betweenness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">16.000</td>
<td headers="BetweennessRSP" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">53.000</td>
<td headers="Diffusion" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">4.628</td>
<td headers="Clustering" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.240</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">emotion</td>
<td headers="OutStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.923</td>
<td headers="InStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.894</td>
<td headers="ClosenessIn" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.014</td>
<td headers="ClosenessOut" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.012</td>
<td headers="Closeness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.023</td>
<td headers="Betweenness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">5.000</td>
<td headers="BetweennessRSP" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">36.000</td>
<td headers="Diffusion" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">5.070</td>
<td headers="Clustering" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.290</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">monitor</td>
<td headers="OutStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.982</td>
<td headers="InStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.346</td>
<td headers="ClosenessIn" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.008</td>
<td headers="ClosenessOut" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.014</td>
<td headers="Closeness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.019</td>
<td headers="Betweenness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.000</td>
<td headers="BetweennessRSP" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">11.000</td>
<td headers="Diffusion" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">5.157</td>
<td headers="Clustering" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.289</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">plan</td>
<td headers="OutStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.626</td>
<td headers="InStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">1.194</td>
<td headers="ClosenessIn" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.027</td>
<td headers="ClosenessOut" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.012</td>
<td headers="Closeness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.027</td>
<td headers="Betweenness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">9.000</td>
<td headers="BetweennessRSP" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">61.000</td>
<td headers="Diffusion" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">3.488</td>
<td headers="Clustering" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.287</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">synthesis</td>
<td headers="OutStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">1.000</td>
<td headers="InStrength" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.192</td>
<td headers="ClosenessIn" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.010</td>
<td headers="ClosenessOut" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.016</td>
<td headers="Closeness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.024</td>
<td headers="Betweenness" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">7.000</td>
<td headers="BetweennessRSP" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">3.000</td>
<td headers="Diffusion" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">5.583</td>
<td headers="Clustering" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.359</td></tr>
  </tbody>
  &#10;</table>
</div>

Plot the centrality measures

``` r
plot(cent, ncol = 3)
```

<img src="man/figures/README-centralitiesplot-1.svg" alt="" width="90%" style='text-align: center; display: block; margin: auto;' style="display: block; margin: auto;" />

Estimate centrality stability

``` r
estimate_centrality_stability(tna_model)
#> Centrality Stability Coefficients
#> 
#>  InStrength OutStrength Betweenness 
#>         0.9         0.9         0.9
```

Identify and plot communities

``` r
coms <- communities(tna_model)
plot(coms)
```

<img src="man/figures/README-unnamed-chunk-12-1.svg" alt="" width="60%" style='text-align: center; display: block; margin: auto;' style="display: block; margin: auto;" />

Find and plot cliques

``` r
cqs <- cliques(tna_model, threshold = 0.12)
plot(cqs)
```

<img src="man/figures/README-unnamed-chunk-14-1.svg" alt="" width="100%" style='text-align: center; display: block; margin: auto;' style="display: block; margin: auto;" />

Compare high achievers (first 1000) with low achievers (last 1000)

``` r
tna_model_start_high <- tna(group_regulation[1:1000, ])
tna_model_start_low <- tna(group_regulation[1001:2000, ])
comparison <- permutation_test(
  tna_model_start_high, 
  tna_model_start_low,
  measures = c("InStrength")
)
```

Simple comparison vs. permutation test comparison

``` r
plot_compare(tna_model_start_high, tna_model_start_low)
plot(comparison)
```

<img src="man/figures/README-unnamed-chunk-16-1.svg" alt="" width="100%" style='text-align: center; display: block; margin: auto;' style="display: block; margin: auto;" />

Compare centralities

``` r
print(comparison$centralities$stats)
```

<div id="bpwgsnxlgh" style="padding-left:0px;padding-right:0px;padding-top:10px;padding-bottom:10px;overflow-x:auto;overflow-y:auto;width:auto;height:auto;">
  &#10;  <table class="gt_table" data-quarto-disable-processing="false" data-quarto-bootstrap="false" style="-webkit-font-smoothing: antialiased; -moz-osx-font-smoothing: grayscale; font-family: system-ui, 'Segoe UI', Roboto, Helvetica, Arial, sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'; display: table; border-collapse: collapse; line-height: normal; margin-left: auto; margin-right: auto; color: #333333; font-size: 16px; font-weight: normal; font-style: normal; background-color: #FFFFFF; width: auto; border-top-style: solid; border-top-width: 2px; border-top-color: #A8A8A8; border-right-style: none; border-right-width: 2px; border-right-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #A8A8A8; border-left-style: none; border-left-width: 2px; border-left-color: #D3D3D3;" bgcolor="#FFFFFF">
  <thead style="border-style: none;">
    <tr class="gt_col_headings" style="border-style: none; border-top-style: solid; border-top-width: 2px; border-top-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3;">
      <th class="gt_col_heading gt_columns_bottom_border gt_center" rowspan="1" colspan="1" scope="col" id="state" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: center;" bgcolor="#FFFFFF" valign="bottom" align="center">state</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_center" rowspan="1" colspan="1" scope="col" id="centrality" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: center;" bgcolor="#FFFFFF" valign="bottom" align="center">centrality</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="diff_true" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">diff_true</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="effect_size" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">effect_size</th>
      <th class="gt_col_heading gt_columns_bottom_border gt_right" rowspan="1" colspan="1" scope="col" id="p_value" style="border-style: none; color: #333333; background-color: #FFFFFF; font-size: 100%; font-weight: normal; text-transform: inherit; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: bottom; padding-top: 5px; padding-bottom: 6px; padding-left: 5px; padding-right: 5px; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" bgcolor="#FFFFFF" valign="bottom" align="right">p_value</th>
    </tr>
  </thead>
  <tbody class="gt_table_body" style="border-style: none; border-top-style: solid; border-top-width: 2px; border-top-color: #D3D3D3; border-bottom-style: solid; border-bottom-width: 2px; border-bottom-color: #D3D3D3;">
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">adapt</td>
<td headers="centrality" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">InStrength</td>
<td headers="diff_true" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-0.23693341</td>
<td headers="effect_size" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-6.5741846</td>
<td headers="p_value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.000999001</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">cohesion</td>
<td headers="centrality" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">InStrength</td>
<td headers="diff_true" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.01634987</td>
<td headers="effect_size" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.3369021</td>
<td headers="p_value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.743256743</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">consensus</td>
<td headers="centrality" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">InStrength</td>
<td headers="diff_true" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.53680793</td>
<td headers="effect_size" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">7.7240021</td>
<td headers="p_value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.000999001</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">coregulate</td>
<td headers="centrality" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">InStrength</td>
<td headers="diff_true" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-0.25275371</td>
<td headers="effect_size" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-7.3308543</td>
<td headers="p_value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.000999001</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">discuss</td>
<td headers="centrality" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">InStrength</td>
<td headers="diff_true" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-0.09009651</td>
<td headers="effect_size" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-1.9372656</td>
<td headers="p_value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.046953047</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">emotion</td>
<td headers="centrality" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">InStrength</td>
<td headers="diff_true" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.19288376</td>
<td headers="effect_size" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">4.0496273</td>
<td headers="p_value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.000999001</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">monitor</td>
<td headers="centrality" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">InStrength</td>
<td headers="diff_true" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-0.09192991</td>
<td headers="effect_size" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-3.5069394</td>
<td headers="p_value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.000999001</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">plan</td>
<td headers="centrality" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">InStrength</td>
<td headers="diff_true" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.12225988</td>
<td headers="effect_size" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">2.7153863</td>
<td headers="p_value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.005994006</td></tr>
    <tr style="border-style: none;"><td headers="state" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">synthesis</td>
<td headers="centrality" class="gt_row gt_center" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: center;" valign="middle" align="center">InStrength</td>
<td headers="diff_true" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-0.04909607</td>
<td headers="effect_size" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">-3.0577480</td>
<td headers="p_value" class="gt_row gt_right" style="border-style: none; padding-top: 8px; padding-bottom: 8px; padding-left: 5px; padding-right: 5px; margin: 10px; border-top-style: solid; border-top-width: 1px; border-top-color: #D3D3D3; border-left-style: none; border-left-width: 1px; border-left-color: #D3D3D3; border-right-style: none; border-right-width: 1px; border-right-color: #D3D3D3; vertical-align: middle; overflow-x: hidden; text-align: right; font-variant-numeric: tabular-nums;" valign="middle" align="right">0.002997003</td></tr>
  </tbody>
  &#10;</table>
</div>

# Papers using TNA

- Saqr, M., Lopez-Pernas, S., Tormanen, T., Kaliisa, R., Misiejuk, K., &
  Tikka, S. (2025). Transition Network Analysis: A Novel Framework for
  Modeling, Visualizing, and Identifying the Temporal Patterns of
  Learners and Learning Processes. Proceedings of the 15th International
  Learning Analytics and Knowledge Conference (LAK ’25), 351–361. ACM.
  <https://doi.org/10.1145/3706468.3706513>
- Tikka, S., Lopez-Pernas, S., & Saqr, M. (2025). tna: An R Package for
  Transition Network Analysis. Applied Psychological Measurement (online
  ahead of print). <doi:10.1177/01466216251348840>
- López-Pernas, S., Misiejuk, K., Kaliisa, R., & Saqr, M. (2025).
  Capturing the process of students’ AI interactions when creating and
  learning complex network structures. IEEE Transactions on Learning
  Technologies, 1–13. <https://doi.org/10.1109/tlt.2025.3568599>
- Törmänen, T., Saqr, M., López-Pernas, S., Mänty, K., Suoraniemi, J.,
  Heikkala, N., & Järvenoja, H. (2025). Emotional dynamics and
  regulation in collaborative learning. Learning and Instruction,
  100, 102188. <https://doi.org/10.1016/j.learninstruc.2025.102188>
- López-Pernas, S., Misiejuk, K., Oliveira, E., & Saqr, M. (2025). The
  dynamics of the self-regulation process in student-AI interactions:
  The case of problem-solving in programming education. Proceedings of
  the 25th Koli Calling International Conference on Computing Education
  Research, 1–12. <https://doi.org/10.1145/3769994.3770043>
- Misiejuk, K., Kaliisa, R., Lopez-Pernas, S., & Saqr, M. (2026).
  Expanding the Quantitative Ethnography Toolkit with Transition Network
  Analysis: Exploring Methodological Synergies and Boundaries. In
  Advances in Quantitative Ethnography. ICQE 2025, CCIS vol. 2677.
  Springer. <https://doi.org/10.1007/978-3-032-12229-2_10>
- Lopez-Pernas, S., Misiejuk, K., Tikka, S., & Saqr, M. (2026). Role
  Dynamics in Student-AI Collaboration: A Heterogeneous Transition
  Network Analysis Approach. SSRN preprint.
  <https://doi.org/10.2139/ssrn.6082190>

## ----include = FALSE-------------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.width = 6,
  fig.height = 4,
  out.width = "100%",
  dev = "jpeg",
  dpi = 100,
  comment = "#>"
)
suppressPackageStartupMessages({
  library("tna")
  library("tibble")
  library("dplyr")
  library("gt")
})
options(scipen = 99)
options(digits = 2)
options(max.print = 30)
options(width = 83)

## --------------------------------------------------------------------------------
library("tna")
data("group_regulation", package = "tna")

## ----message = FALSE, results = FALSE--------------------------------------------
model <- tna(group_regulation)
print(model)

## ----fig.height=6, fig.width=6, fig.align='center', layout = c(1,1)--------------
# TNA visualization
plot(model)

## ----fig.show='hold', fig.width=5, fig.height=5----------------------------------
layout(matrix(1:4, ncol = 2, byrow = TRUE))
# Pruning with different methods (using comparable parameters)
pruned_threshold <- prune(model, method = "threshold", threshold = 0.15)
pruned_lowest <- prune(model, method = "lowest", lowest = 0.15)
pruned_disparity <- prune(model, method = "disparity", level = 0.5)

# Plotting for comparison
plot(pruned_threshold)
plot(pruned_lowest)
plot(pruned_disparity)
plot(model)

## --------------------------------------------------------------------------------
layout(t(1:2))
# Identify 2-cliques (dyads) from the TNA model, excluding loops in the visualization
# A clique of size 2 is essentially a pair of connected nodes
cliques_of_two <- cliques(
  model, 
  size = 2, 
  threshold = 0.15  # Only consider edges with weight > 0.15
)
print(cliques_of_two)
plot(cliques_of_two, ask = FALSE)

## ----fig.width=6,fig.height=2----------------------------------------------------
layout(t(1:3))
# Identify 3-cliques (triads) from the TNA_Model
# A clique of size 3 means a fully connected triplet of nodes
cliques_of_three <- cliques(
  model, 
  size = 3, 
  threshold = 0.05 # Only consider edges with weight > 0.05
)
print(cliques_of_three)
plot(cliques_of_three, ask = FALSE)

## ----fig.width=5-----------------------------------------------------------------
# Identify 4-cliques (quadruples) from the TNA_Model
# A clique of size 4 means four nodes that are all mutually connected
cliques_of_four <- cliques(
  model, 
  size = 4, 
  threshold = 0.035 # Only consider edges with weight > 0.03
)
print(cliques_of_four)
plot(cliques_of_four, ask = FALSE)

## ----fig.height=8, fig.width=8, fig.align='center'-------------------------------
# Compute centrality measures for the TNA model
Centralities <- centralities(model)

# Visualize the centrality measures
plot(Centralities)

## --------------------------------------------------------------------------------
# Calculate hub scores and the authority scores for the network
hits_scores <- igraph::hits_scores(as.igraph(model))
hub_scores <- hits_scores$hub
authority_scores <- hits_scores$authority

# Print the calculated hub and authority scores for further analysis
print(hub_scores)
print(authority_scores)

## ----fig.align='center', fig.width=6, fig.height=5.5, out.width="60%"------------
# Edge betweenness
Edge_betweeness <- betweenness_network(model)
plot(Edge_betweeness)

## --------------------------------------------------------------------------------
communities <- communities(model)
print(communities)
plot(communities, method = "leading_eigen")

## --------------------------------------------------------------------------------
# Perform bootstrapping on the TNA model with a fixed seed for reproducibility
set.seed(265)
boot <- bootstrap(model, threshold = 0.05)

# Print a summary of the bootstrap results
print(summary(boot))

# Show the non-significant edges (p-value >= 0.05 in this case)
# These are edges that are less likely to be stable across bootstrap samples
print(boot, type = "nonsig")

## ----fig.align='center', fig.width = 5, fig.height = 5, layout = c(1,1)----------
# Create TNA for the high-achievers subset (rows 1 to 1000)
Hi <- tna(group_regulation[1:1000, ])

# Create TNA for the low-achievers subset (rows 1001 to 2000)
Lo <- tna(group_regulation[1001:2000, ])

# Plot a comparison of the "Hi" and "Lo" models
# The 'minimum' parameter is set to 0.001, so edges with weights >= 0.001 are shown
plot_compare(Hi, Lo, minimum = 0.01)

# Run a permutation test to determine statistical significance of differences
# between "Hi" and "Lo"
# The 'iter' argument is set to 1000, meaning 1000 permutations are performed
Permutation <- permutation_test(Hi, Lo, iter = 1000, measures = "Betweenness")

# Plot the significant differences identified in the permutation test
plot(Permutation, minimum = 0.01)

## --------------------------------------------------------------------------------
print(Permutation$edges$stats)
print(Permutation$centralities$stats)

## ----fig.align='center', fig.width=7, fig.height=4-------------------------------
# Results of the Case-Dropping Bootstrap for Centrality Indices
Centrality_stability <- estimate_centrality_stability(model, iter = 100)
plot(Centrality_stability)

#' Calculate Summary of Network Metrics for a Transition Network
#'
#' This function calculates a variety of network metrics for a `tna` object.
#' It computes key metrics such as node and edge counts, network density,
#' mean distance, strength measures, degree centrality, and reciprocity.
#'
#' @export
#' @family basic
#' @param object A `tna` object.
#' @param ... Ignored.
#' @details
#' The function extracts the `igraph` network  and
#' computes the following network metrics:
#'
#'   * Node count: Total number of nodes in the network.
#'   * Edge count: Total number of edges in the network.
#'   * Network density: Proportion of possible edges that
#'     are present in the network.
#'   * Mean distance: The average shortest path length between nodes.
#'   * Mean and standard deviation of out-strength and in-strength: Measures
#'     of the total weight of outgoing and incoming edges for each node.
#'   * Mean and standard deviation of out-degree: The number of outgoing
#'     edges from each node.
#'   * Centralization of out-degree and in-degree: Measures of how
#'     centralized the network is based on the degrees of nodes.
#'   * Reciprocity: The proportion of edges that are reciprocated
#'     (i.e., mutual edges between nodes).
#'
#' A summary of the metrics is printed to the console.
#'
#' @return A named `list` containing the following network metrics (invisibly):
#'
#'   * `node_count`: The total number of nodes.
#'   * `edge_count`: The total number of edges.
#'   * `network_Density`: The density of the network.
#'   * `mean_distance`: The mean shortest path length.
#'   * `mean_out_strength`: The mean out-strength of nodes.
#'   * `sd_out_strength`: The standard deviation of out-strength.
#'   * `mean_in_strength`: The mean in-strength of nodes.
#'   * `sd_in_strength`: The standard deviation of in-strength.
#'   * `mean_out_degree`: The mean out-degree of nodes.
#'   * `sd_out_degree`: The standard deviation of out-degree.
#'   * `centralization_out_degree`: The centralization of out-degree.
#'   * `centralization_in_degree`: The centralization of in-degree.
#'   * `reciprocity`: The reciprocity of the network.
#'
#' @examples
#' model <- tna(group_regulation)
#' summary(model)
#'
summary.tna <- function(object, ...) {
  check_missing(object)
  check_class(object, "tna")
  weights <- object$weights
  g <- as.igraph(object)
  in_strength <- igraph::strength(g, mode = "out")
  out_strength <- igraph::strength(g, mode = "in")
  out_degree <- igraph::degree(g, mode = "out")
  cent_out <- igraph::centr_degree(g, mode = "out", loops = FALSE)
  cent_in <- igraph::centr_degree(g, mode = "in", loops = FALSE)
  out <- c(
    node_count = nodes(object),
    edge_count = sum(weights > 0),
    network_density = min(igraph::edge_density(g), 1),
    mean_distance = igraph::mean_distance(g),
    mean_out_strength = mean(out_strength, na.rm = TRUE),
    sd_out_strength = stats::sd(out_strength, na.rm = TRUE),
    mean_in_strength = mean(in_strength, na.rm = TRUE),
    sd_in_strength = stats::sd(in_strength, na.rm = TRUE),
    mean_out_degree = mean(out_degree),
    sd_out_degree = stats::sd(out_degree),
    centralization_out_degree = cent_out$centralization,
    centralization_in_degree = cent_in$centralization,
    reciprocity = igraph::reciprocity(g)
  )
  out_names <- c(
    "Node Count",
    "Edge Count",
    "Network Density",
    "Mean Distance",
    "Mean Out-Strength",
    "SD Out-Strength",
    "Mean In-Strength",
    "SD In-Strength",
    "Mean Out-Degree",
    "SD Out-Degree",
    "Centralization (Out-Degree)",
    "Centralization (In-Degree)",
    "Reciprocity"
  )
  out <- tibble::tibble(metric = out_names, value = unname(out))
  structure(
    out,
    class = c("summary.tna", "tbl_df", "tbl", "data.frame")
  )
}

#' Summarize Bootstrap Results
#'
#' @export
#' @family validation
#' @param object A `tna_bootstrap` object.
#' @param ... Ignored.
#' @return A `summary.tna_bootstrap` object containing the weight,
#' estimated p-value and confidence interval of each edge.
#' @examples
#' model <- tna(group_regulation)
#' # Small number of iterations for CRAN
#' boot <- bootstrap(model, iter = 50)
#' summary(boot)
#'
summary.tna_bootstrap <- function(object, ...) {
  check_missing(object)
  check_class(object, "tna_bootstrap")
  structure(
    object$summary,
    class = c("summary.tna_bootstrap", "data.frame")
  )
}

#' Calculate Summary of Network Metrics for a grouped Transition Network
#'
#' This function calculates a variety of network metrics for a `tna` object.
#' It computes key metrics such as node and edge counts, network density,
#' mean distance, strength measures, degree centrality, and reciprocity.
#'
#' @export
#' @family basic
#' @param object A `group_tna` object.
#' @param combined A logical indicating whether the summary results should be
#' combined into a single data frame for all clusters (defaults to `TRUE`)
#' @param ... Ignored
#' @details
#' The function extracts the `igraph` network for each cluster and
#' computes the following network metrics:
#'
#'   * Node count: Total number of nodes in the network.
#'   * Edge count: Total number of edges in the network.
#'   * Network density: Proportion of possible edges that
#'     are present in the network.
#'   * Mean distance: The average shortest path length between nodes.
#'   * Mean and standard deviation of out-strength and in-strength: Measures
#'     of the total weight of outgoing and incoming edges for each node.
#'   * Mean and standard deviation of out-degree: The number of outgoing
#'     edges from each node.
#'   * Centralization of out-degree and in-degree: Measures of how
#'     centralized the network is based on the degrees of nodes.
#'   * Reciprocity: The proportion of edges that are reciprocated
#'     (i.e., mutual edges between nodes).
#'
#' @return A `summary.group_tna` object which is a `list` of `list`s or a
#' combined `data.frame`  containing the following network metrics:
#'
#'   * `node_count`: The total number of nodes.
#'   * `edge_count`: The total number of edges.
#'   * `network_Density`: The density of the network.
#'   * `mean_distance`: The mean shortest path length.
#'   * `mean_out_strength`: The mean out-strength of nodes.
#'   * `sd_out_strength`: The standard deviation of out-strength.
#'   * `mean_in_strength`: The mean in-strength of nodes.
#'   * `sd_in_strength`: The standard deviation of in-strength.
#'   * `mean_out_degree`: The mean out-degree of nodes.
#'   * `sd_out_degree`: The standard deviation of out-degree.
#'   * `centralization_out_degree`: The centralization of out-degree.
#'   * `centralization_in_degree`: The centralization of in-degree.
#'   * `reciprocity`: The reciprocity of the network.
#'
#' @examples
#' group <- c(rep("High", 1000), rep("Low", 1000))
#' model <- group_model(group_regulation, group = group)
#' summary(model)
#'
summary.group_tna <- function(object, combined = TRUE, ...) {
  check_missing(object)
  check_class(object, "group_tna")
  if (!combined) {
    out <- lapply(object, summary)
    out_class <- "summary.group_tna"
  } else {
    out <- dplyr::bind_rows(
      lapply(object, summary),
      .id = "group"
    ) |>
      tidyr::pivot_wider(names_from = "group", values_from = "value")
    out_class <- c("summary.group_tna", "tbl_df", "tbl", "data.frame")
  }
  structure(
    out,
    class = out_class,
    combined = combined
  )
}

#' Summarize Bootstrap Results for a Grouped Transition Network
#'
#' @export
#' @family validation
#' @param object A `group_tna_bootstrap` object.
#' @param ... Ignored.
#' @return A `summary.group_tna_bootstrap` object containing the weight,
#' estimated p-value and confidence interval of each edge for each cluster.
#' @examples
#' model <- group_tna(engagement_mmm)
#' # Small number of iterations for CRAN
#' boot <- bootstrap(model, iter = 10)
#' summary(boot)
#'
summary.group_tna_bootstrap <- function(object, ...) {
  check_missing(object)
  check_class(object, "group_tna_bootstrap")
  structure(
    dplyr::bind_rows(
      lapply(object, function(x) x$summary),
      .id = "group"
    ),
    class = c("summary.group_tna_bootstrap", "data.frame")
  )
}

test_that("bootstrap can be applied", {
  model <- tna(mock_sequence)
  expect_error(
    bootstrap(model, iter = 20, method = "stability"),
    NA
  )
  expect_error(
    bootstrap(model, iter = 20, method = "threshold"),
    NA
  )
})

test_that("bootstrap results can be summarized", {
  model <- tna(mock_sequence)
  boot <- bootstrap(model, iter = 20)
  expect_error(
    summary.tna_bootstrap(boot),
    NA
  )
})

test_that("bootstrap can be applied for clusters", {
  expect_error(
    bootstrap(mmm_model, iter = 20),
    NA
  )
})

test_that("bootstrap results can be summarized for clusters", {
  boot <- bootstrap(mmm_model, iter = 10)
  expect_error(
    summary.group_tna_bootstrap(boot),
    NA
  )
})

test_that("cliques can be bootstrapped", {
  model <- tna(engagement)
  expect_error(
    bootstrap_cliques(model, iter = 10),
    NA
  )
})

.onAttach <- function(libname, pkgname) {
  startup()
  invisible()
}

startup <- function() {
  packageStartupMessage(
    "'tna' package version ", utils::packageVersion("tna"), "\n",
    "------------------------------------------------------\n",
    "  Tikka, S., L\u00f3pez-Pernas, S., and Saqr, M. (2025). \n",
    "  tna: An R Package for Transition Network Analysis.\n",
    "  Applied Psychological Measurement.\n",
    "  https://doi.org/10.1177/01466216251348840\n",
    "------------------------------------------------------\n",
    "Please type 'citation(\"tna\")' for more citation information.\n",
    "See the package website at https://sonsoles.me/tna/\n"
  )
}

#' Community Detection for Transition Networks
#'
#' This function detects communities within the transition networks
#' (represented by the `tna` object).
#' It uses various algorithms to find communities in the graph representation
#' of transitions and returns a `list` of communities for each cluster or a
#' specified cluster. If multiple transition matrices exist, the function
#' iterates over each cluster in the `tna` object to find communities using
#' different algorithms. The function uses the `igraph` package to convert
#' the transition matrices into graphs and then applies community detection
#' algorithms (e.g., Walktrap, Fast Greedy, Label Propagation, Infomap,
#' Edge Betweenness, Leading Eigenvector, and Spin Glass).
#'
#' @export
#' @family communities
#' @rdname communities
#' @param x A `tna` or a `group_tna` object.
#' @param methods A `character` vector of community detection algorithms to
#' apply to the network. The supported options are:
#'
#'   * `"walktrap"`: A community detection method using short random walks.
#'   * `"fast_greedy"`: A method based on modularity optimization.
#'   * `"label_prop"`: A method that uses label propagation.
#'   * `"infomap"`: A method that uses information flow to detect communities.
#'   * `"edge_betweenness"`: A method that uses edge betweenness to find
#'     communities.
#'   * `"leading_eigen"`: A method using the leading eigenvector of the
#'     modularity matrix.
#'   * `"spinglass"`: A method based on the spinglass model.
#'
#' If not provided, all methods are applied.
#' @param gamma A `numeric` value depicting a parameter that affects the
#' behavior of certain algorithms like the Spin Glass method. Defaults to `1`.
#' @return An object of class `tna_communities` which is a `list` with an
#'   element for each cluster containing:
#'
#'   * `counts`: A `list` with the number of communities found
#'     by each algorithm.
#'   * `assignments`: A `data.frame` where each row corresponds to a
#'     node and each column to a community detection algorithm,
#'     with color-coded community assignments.
#'
#' If `x` is a `group_tna` object, a `group_tna_communities` object is returned
#' instead, which is a `list` of `tna_communities` objects.
#'
#' @examples
#' model <- tna(group_regulation)
#' comm <- communities(model)
#'
communities <- function(x, methods, gamma) {
  UseMethod("communities")
}

#' @export
#' @rdname communities
communities.tna <- function(x, methods, gamma = 1) {
  check_missing(x)
  check_class(x, "tna")
  methods <- methods %m% names(supported_communities)
  methods <- check_match(
    methods,
    names(supported_communities),
    several.ok = TRUE
  )
  stopifnot_(
    checkmate::test_number(x = gamma),
    "Argument {.arg gamma} must be a single {.cls numeric} value."
  )
  g <- as.igraph(x)
  g_un <- as.igraph(x, mode = "plus")
  communities <- list()
  mapping <- list()
  w <- igraph::E(g)$weight
  w_un <- igraph::E(g_un)$weight
  for (method in methods) {
    directed <- supported_communities[[method]]$directed
    args <- list(
      graph = ifelse_(directed, g, g_un),
      weights = ifelse_(directed, w, w_un)
    )
    args$gamma <- onlyif(
      !is.null(supported_communities[[method]]$gamma),
      gamma
    )
    if (!is.null(supported_communities[[method]]$e_weights)) {
      args$e.weights <- args$weights
      args$weights <- NULL
    }
    if (method == "edge_betweenness") {
      # TODO suppress for now
      communities[[method]] <- suppressWarnings(
        communities[[method]] <- do.call(
          supported_communities[[method]]$fun,
          args
        )
      )
    } else {
      communities[[method]] <- do.call(
        supported_communities[[method]]$fun,
        args
      )
    }
    mapping[[method]] <- igraph::membership(communities[[method]]) |>
      as.numeric()
  }
  structure(
    list(
      counts = lengths(communities),
      assignments = as.data.frame(
        c(
          list(state = igraph::V(g)$name),
          mapping
        )
      )
    ),
    class = "tna_communities",
    tna = x
  )
}

#' @export
#' @family clusters
#' @rdname communities
communities.group_tna <- function(x, methods, gamma = 1) {
  check_missing(x)
  check_class(x, "group_tna")
  methods <- methods %m% names(supported_communities)
  structure(
    stats::setNames(
      lapply(x, communities, methods = methods, gamma = gamma),
      names(x)
    ),
    class = "group_tna_communities"
  )
}


# Supported community detection algorithms --------------------------------

supported_communities <- list(
  `walktrap` = list(
    fun = igraph::cluster_walktrap,
    directed = TRUE
  ),
  `fast_greedy` = list(
    fun = igraph::cluster_fast_greedy,
    directed = FALSE
  ),
  `label_prop` = list(
    fun = igraph::cluster_label_prop,
    directed = TRUE
  ),
  `infomap` = list(
    fun = igraph::cluster_infomap,
    directed = TRUE,
    e_weights = TRUE
  ),
  `edge_betweenness` = list(
    fun = igraph::cluster_edge_betweenness,
    directed = TRUE
  ),
  `leading_eigen` = list(
    fun = igraph::cluster_leading_eigen,
    directed = FALSE
  ),
  `spinglass` = list(
    fun = igraph::cluster_spinglass,
    directed = FALSE,
    gamma = TRUE
  )
)

test_that("communities can be detected", {
  expect_error(
    communities(mock_tna),
    NA
  )
})

test_that("communities can be detected for clusters", {
  expect_error(
    communities(mmm_model),
    NA
  )
})

#' Print a TNA Summary
#'
#' @export
#' @family basic
#' @param x A `summary.tna` object.
#' @param ... Ignored.
#' @return A `summary.tna` object (invisibly) containing the TNA model network
#' metrics and values.
#' @examples
#' model <- tna(group_regulation)
#' print(summary(model))
#'
print.summary.tna <- function(x, ...) {
  NextMethod(generic = "print", object = x, ...)
}

#' Print a Bootstrap Summary
#'
#' @export
#' @family validation
#' @param x A `summary.tna_bootstrap` object.
#' @param ... Arguments passed to the generic `print` method.
#' @return A `summary.tna_bootstrap` object (invisibly) containing the weight,
#' estimated p-value and confidence interval of each edge.
#' @examples
#' model <- tna(group_regulation)
#' # Small number of iterations for CRAN
#' boot <- bootstrap(model, iter = 10)
#' print(summary(boot))
#'
print.summary.tna_bootstrap <- function(x, ...) {
  NextMethod(generic = "print", object = x, ...)
}

#' Print a `tna` Object
#'
#' @export
#' @family basic
#' @param x A `tna` object.
#' @param generic A `logical` value. If `TRUE`, use generic print method
#' instead. Defaults to `FALSE`.
#' @param ... Additional arguments passed to the generic `print` methods.
#' @return The `tna` object passed as argument `x` (invisibly).
#' @examples
#' model <- tna(group_regulation)
#' print(model)
#'
print.tna <- function(x, generic = FALSE, ...) {
  check_missing(x)
  check_class(x, "tna")
  check_flag(generic)
  if (generic) {
    NextMethod(generic = "print", object = x, ...)
    return()
  }
  type <- attr(x, "type")
  mat_type <- switch(type,
    `relative` = "Transition Probability",
    `frequency` = "Transition Frequency",
    `co-occurrence` = "Co-occurrence",
    `n-gram` = "N-gram Transition",
    `gap` = "Gap-allowed Transition",
    `window` = "Sliding Window Transition",
    `betweenness` = "Edge Betweenness",
    "Edge Weight"
  )
  cat("State Labels : \n\n")
  cat("  ", paste(x$labels, collapse = ", "), "\n")
  cat("\n", mat_type, " Matrix :\n\n", sep = "")
  print(x$weights, ...)
  inits <- stats::setNames(x$inits, x$labels)
  if (!is.null(x$inits)) {
    cat("\nInitial Probabilities : \n\n", sep = "")
    print(inits, ...)
  }
  invisible(x)
}

#' Print Bootstrap Results
#'
#' @export
#' @family validation
#' @param x A `tna_bootstrap` object.
#' @param digits An `integer` giving the minimal number of
#' *significant* digits to print.
#' @param type A `character` vector giving the type of edges to print.
#' The default option `"both"` prints both statistically significant and
#' non-significant edges, `"sig"` prints only significant edges, and `"nonsig"`
#' prints only the non-significant edges.
#' @param ... Ignored.
#' @return `x` (invisibly).
#' @examples
#' model <- tna(group_regulation)
#' # Small number of iterations for CRAN
#' boot <- bootstrap(model, iter = 10)
#' print(boot)
#'
print.tna_bootstrap <- function(x, digits = getOption("digits"),
                                type = "both", ...) {
  check_missing(x)
  check_class(x, "tna_bootstrap")
  check_values(digits)
  type <- check_match(type, c("both", "sig", "nonsig"))
  sig <- x$summary$sig
  edges <- x$summary |>
    dplyr::select(!sig)
  if (any(sig) && type %in% c("both", "sig")) {
    cat("Significant Edges\n\n")
    print(edges[which(sig), ], digits = digits)
  }
  if (any(sig) && any(!sig) && type == "both") {
    cat("\n")
  }
  if (any(!sig) && type %in% c("both", "nonsig")) {
    cat("Non-significant Edges\n\n")
    print(edges[which(!sig), ], digits = digits)
  }
  invisible(x)
}

#' Print Centrality Measures
#'
#' @export
#' @family centralities
#' @param x A `centralities` object.
#' @param ... Ignored.
#' @return `x` (invisibly).
#' @examples
#' model <- tna(group_regulation)
#' cm <- centralities(model)
#' print(cm)
#'
print.tna_centralities <- function(x, ...) {
  NextMethod(generic = "print", object = x, ...)
}

#' Print Detected Communities
#'
#' @export
#' @family communities
#' @param x A `tna_communities` object.
#' @param ... Additional arguments passed to the generic `print` method.
#' @return `x` (invisibly).
#' @examples
#' model <- tna(group_regulation)
#' comm <- communities(model)
#' print(comm)
#'
print.tna_communities <- function(x, ...) {
  check_missing(x)
  check_class(x, "tna_communities")
  cat("Number of communities found by each algorithm\n\n")
  print(x$counts, ...)
  cat("\nCommunity assignments\n\n")
  print(x$assignments, ...)
  invisible(x)
}

#' Print Comparison Results
#'
#' @export
#' @family comparison
#' @param x A `tna_comparison` object.
#' @param ... Additional arguments passed to the tibble `print` method.
#' @return `x` (invisibly).
#' @examples
#' model_x <- tna(group_regulation[1:200, ])
#' model_y <- tna(group_regulation[1001:1200, ])
#' comp <- compare(model_x, model_y)
#' print(comp)
#'
print.tna_comparison <- function(x, ...) {
  check_missing(x)
  check_class(x, "tna_comparison")
  cat("Edge difference metrics\n")
  print(tibble::as_tibble(x$edge_metrics), ...)
  cat("\nSummary metrics of differences\n")
  print(x$summary_metrics, ...)
  cat("\nNetwork metrics\n")
  print(tibble::as_tibble(x$network_metrics), ...)
  if (!is.null(x$centrality_differences)) {
    cat("\nCentrality differences\n")
    print(x$centrality_differences, ...)
  }
  if (!is.null(x$centrality_correlations)) {
    cat("\nCentrality correlations\n")
    print(x$centrality_correlations, ...)
  }
  invisible(x)
}

#' Print Found Cliques of a TNA Network
#'
#' @export
#' @family cliques
#' @param x A `tna_cliques` object.
#' @param n An `integer` defining the maximum number of cliques to show.
#' The defaults is `6`.
#' @param first An `integer` giving the index of the first clique to show.
#' The default index is `1`.
#' @param digits An `integer` giving the minimal number of
#' *significant* digits to print.
#' @param ... Ignored.
#' @return `x` (invisibly).
#' @examples
#' model <- tna(group_regulation)
#' cliq <- cliques(model, size = 2)
#' print(cliq)
#'
print.tna_cliques <- function(x, n = 6, first = 1,
                              digits = getOption("digits"), ...) {
  check_missing(x)
  check_class(x, "tna_cliques")
  n_cliques <- length(x$weights)
  size <- attr(x, "size")
  if (n_cliques == 0) {
    cat("No ", attr(x, "size"), "-cliques were found in the network.", sep = "")
    return(invisible(x))
  }
  check_values(n, strict = TRUE)
  check_values(first, strict = TRUE)
  check_values(digits)
  n <- min(n, n_cliques)
  threshold <- attr(x, "threshold")
  cluster <- attr(x, "cluster")
  cat(
    "Number of ", size, "-cliques = ", n_cliques, " ",
    "(weight threshold = ", threshold, ")\n",
    sep = ""
  )
  max_cliques <- min(first + n - 1L, n_cliques)
  cat(
    "Showing ", max_cliques, " cliques starting from clique number ", first,
    sep = ""
  )
  cat("\n")
  for (i in seq(first, max_cliques)) {
    cat("\nClique ", i, "\n", sep = "")
    print(x$weights[[i]], digits = digits)
  }
  invisible(x)
}

#' Print a TNA Data Object
#'
#' @export
#' @family data
#' @param x A `tna_data` object.
#' @param data A `character` string that defines the data to be printed
#' tibble. Accepts either `"sequence"` (default) for wide format sequence data,
#' `"meta"`, for the wide format metadata, or `"long"` for the long format
#' data.
#' @param ... Arguments passed to the tibble `print` method.
#' @return `x` (invisibly).
#' @examples
#' res <- prepare_data(group_regulation_long, action = "Action", actor = "Actor",
#' time = "Time")
#' print(res, which = "sequence")
#' print(res, which = "meta")
#' print(res, which = "long")
#'
print.tna_data <- function(x, data = "sequence", ...) {
  check_missing(x)
  check_class(x, "tna_data")
  data <- check_match(data, c("sequence", "meta", "long", "names"))
  idx <- paste0(data, "_data")
  print(x[[idx]])
}

#' Print Centrality Stability Results
#'
#' @export
#' @family validation
#' @param x A `tna_stability` object.
#' @param ... Additional arguments passed to the generic `print` method.
#' @return `x` (invisibly).
#' @examples
#' model <- tna(group_regulation)
#' # Small number of iterations and drop proportions for CRAN
#' cs <- estimate_cs(
#'   model,
#'   measures = c("InStrength", "OutStrength"),
#'   drop_prop = seq(0.3, 0.9, by = 0.2),
#'   iter = 10
#' )
#' print(cs)
#'
print.tna_stability <- function(x, ...) {
  check_missing(x)
  check_class(x, "tna_stability")
  cs_coefs <- unlist(lapply(x, "[[", "cs_coefficient"))
  names(cs_coefs) <- names(x)
  cat("Centrality Stability Coefficients\n\n")
  print(cs_coefs, ...)
  invisible(x)
}

#' Print Permutation Test Results
#'
#' @export
#' @family validation
#' @param x A `tna_permutation` object.
#' @param ... Additional arguments passed to the `tibble` print method.
#' @return `x` (invisibly).
#' @examples
#' model_x <- tna(group_regulation[1:200, ])
#' model_y <- tna(group_regulation[1001:1200, ])
#' # Small number of iterations for CRAN
#' perm <- permutation_test(model_x, model_y, iter = 20)
#' print(perm)
#'
print.tna_permutation <- function(x, ...) {
  check_missing(x)
  check_class(x, "tna_permutation")
  if (!is.null(x$centralities)) {
    cat("Edges\n\n")
  }
  print(tibble::as_tibble(x$edges$stats), ...)
  if (!is.null(x$centralities)) {
    cat("\nCentralities\n\n")
    print(x$centralities$stats, ...)
  }
  invisible(x)
}

#' Print the Results of Clustering
#'
#' @export
#' @family validation
#' @param x A `tna_clustering` object.
#' @param ... Additional arguments passed to the generic print method.
#' @return `x` (invisibly).
#' @examples
#' data <- data.frame(
#'   T1 = c("A", "B", "A", "C", "A", "B"),
#'   T2 = c("B", "A", "B", "A", "C", "A"),
#'   T3 = c("C", "C", "A", "B", "B", "C")
#' )
#'
#' # PAM clustering with optimal string alignment (default)
#' result <- cluster_sequences(data, k = 2)
#' print(result)
#' 
print.tna_clustering <- function(x, ...) {
  cat("Clustering method:", x$method, "\n")
  cat("Number of clusters:", x$k, "\n")
  cat("Silhouette score:", x$silhouette, "\n")
  cat("Cluster sizes:\n")
  print(x$sizes, ...)
  invisible(x)
}

#' Print a Comparison of Sequences
#'
#' @export
#' @family comparison
#' @param x A `tna_sequence_comparison` object.
#' @param ... Arguments passed to the generic `print` method.
#' @return `x` (invisibly).
#' @examples
#' \donttest{
#' idx <- c(1:500, 1001:1500)
#' group <- c(rep("High", 500), rep("Low", 500))
#' comp <- compare_sequences(group_regulation[idx, ], group)
#' print(comp)
#' }
#'
print.tna_sequence_comparison <- function(x, ...) {
  NextMethod(generic = "print", object = x, ...)
}

#' Print Reliability Analysis Results
#'
#' @export
#' @family validation
#' @param x A `tna_reliability` object.
#' @param summary_metrics A `character` vector of metrics to display.
#' @param ... Arguments passed to the generic `print` method.
#' @return `x` (invisibly).
#' @examples
#' # Small number of iterations for CRAN
#' model <- tna(engagement)
#' rel <- reliability(model, iter = 20)
#' print(rel)
#'
print.tna_reliability <- function(x, summary_metrics, ...) {
  summary_metrics <- summary_metrics %m% c(
    "Mean Abs. Diff.",
    "Median Abs. Diff.",
    "Pearson",
    "Bray-Curtis"
  )
  cat("Reliability summary\n")
  sumr <- x$summary |>
    dplyr::filter(!!rlang::sym("metric") %in% summary_metrics) |>
    dplyr::arrange(
      factor(!!rlang::sym("metric"), levels = summary_metrics)
    )
  print(sumr, ...)
}

# Groups ----------------------------------------------------------------

#' Print a `group_tna` Object
#'
#' @export
#' @family basic
#' @param x A `group_tna` object.
#' @param ... Arguments passed to [print.tna()].
#' @return `x` (invisibly).
#' @examples
#' model <- group_model(engagement_mmm)
#' print(model)
#'
print.group_tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  print.listof(x, ...)
  invisible(x)
}

#' Print `group_tna` Bootstrap Results
#'
#' @export
#' @family validation
#' @param x A `group_tna_bootstrap` object.
#' @param ... Arguments passed to [print.tna_bootstrap()].
#' @return `x` (invisibly).
#' @examples
#' model <- group_model(engagement_mmm)
#' # Low number of iteration for CRAN
#' boot <- bootstrap(model, iter = 10)
#' print(boot)
#'
print.group_tna_bootstrap <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna_bootstrap")
  print.listof(x, ...)
  invisible(x)
}

#' Print a Summary of a Grouped Transition Network Analysis Model
#'
#' @export
#' @family basic
#' @param x A `summary.group_tna` object.
#' @param ... Arguments passed to the `tibble` print method
#' @return `x` (invisibly).
#' @examples
#' model <- group_model(engagement_mmm)
#' print(summary(model))
#'
print.summary.group_tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "summary.group_tna")
  NextMethod(generic = "print", object = x, ...)
}

#' Print a Bootstrap Summary for a Grouped Transition Network Model
#'
#' @export
#' @family validation
#' @param x A `summary.group_tna_bootstrap` object.
#' @param ... Arguments passed to the generic `print` method.
#' @return `x` (invisibly).
#' @examples
#' model <- group_model(engagement_mmm)
#' # Low number of iteration for CRAN
#' boot <- bootstrap(model, iter = 10)
#' print(summary(boot))
#'
print.summary.group_tna_bootstrap <- function(x, ...) {
  check_missing(x)
  check_class(x, "summary.group_tna_bootstrap")
  NextMethod(generic = "print", object = x, ...)
}

#' Print Centrality Measures
#'
#' @export
#' @family centralities
#' @param x A `group_tna_centralities` object.
#' @param ... Ignored.
#' @return `x` (invisibly).
#' @examples
#' model <- group_model(engagement_mmm)
#' cm <- centralities(model)
#' print(cm)
#'
print.group_tna_centralities <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna_centralities")
  NextMethod(generic = "print", object = x, ...)
}

#' Print Detected Communities
#'
#' @export
#' @family communities
#' @param x A `group_tna_communities` object.
#' @param ... Arguments passed to [print.tna_communities()].
#' @return `x` (invisibly).
#' @examples
#' model <- group_model(engagement_mmm)
#' comm <- communities(model)
#' print(comm)
#'
print.group_tna_communities <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna_communities")
  print.listof(x, ...)
  invisible(x)
}

#' Print Found Cliques
#'
#' @export
#' @family cliques
#' @param x A `group_tna_cliques` object.
#' @param ... Arguments passed to [print.tna_cliques()].
#' @return `x` (invisibly).
#' @examples
#' model <- group_model(engagement_mmm)
#' cliq <- cliques(model, size = 2)
#' print(cliq)
#'
print.group_tna_cliques <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna_cliques")
  print.listof(x, ...)
  invisible(x)
}

#' Print Centrality Stability Results
#'
#' @export
#' @family validation
#' @param x A `group_tna_stability` object.
#' @param ... Arguments passed to [print.tna_stability()].
#' @return `x` (invisibly).
#' @examples
#' model <- group_model(engagement_mmm)
#' # Low number of iterations for CRAN
#' stability <- estimate_cs(
#'   model,
#'   drop_prop = c(0.3, 0.5, 0.7, 0.9),
#'   iter = 10
#' )
#' print(stability)
#'
print.group_tna_stability <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna_stability")
  print.listof(x, ...)
  invisible(x)
}

#' Print Permutation Test Results
#'
#' @export
#' @family validation
#' @param x A `group_tna_permutation` object.
#' @param ... Arguments passed to [print.tna_permutation()].
#' @return `x` (invisibly).
#' @examples
#' model <- group_model(engagement_mmm)
#' # Small number of iterations for CRAN
#' perm <- permutation_test(model, iter = 20)
#' print(perm)
#'
print.group_tna_permutation <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna_permutation")
  print.listof(x, ...)
  invisible(x)
}

test_that("models can be printed", {
  expect_error(
    capture.output(print.tna(mock_tna)),
    NA
  )
  expect_error(
    capture.output(print.tna(mock_tna, generic = TRUE)),
    NA
  )
})

test_that("model summary can be printed", {
  summ <- summary(mock_tna)
  expect_error(
    capture.output(print.summary.tna(summ)),
    NA
  )
})

test_that("bootstrap results can be printed", {
  set.seed(0)
  model <- tna(group_regulation)
  boot <- bootstrap(model, iter = 50)
  expect_error(
    capture.output(print.tna_bootstrap(boot)),
    NA
  )
})

test_that("bootstrap summary can be printed", {
  model <- tna(mock_sequence)
  boot <- bootstrap(model, iter = 20)
  summ <- summary.tna_bootstrap(boot)
  expect_error(
    capture.output(print.summary.tna_bootstrap(summ)),
    NA
  )
})

test_that("centralities can be printed", {
  cm <- centralities(mock_tna)
  expect_error(
    capture.output(print.tna_centralities(cm)),
    NA
  )
})

test_that("cliques can be printed", {
  cliq <- cliques(mock_tna, size = 2)
  expect_error(
    capture.output(print.tna_cliques(cliq)),
    NA
  )
})

test_that("zero clique case is considered", {
  cliq <- cliques(mock_tna, size = 2, threshold = 0.5)
  expect_output(
    print.tna_cliques(cliq),
    "No 2-cliques were found in the network\\."
  )
})

test_that("communities can be printed", {
  comm <- communities(mock_tna)
  expect_error(
    capture.output(print.tna_communities(comm)),
    NA
  )
})

test_that("centrality stability coefficients can be printed", {
  model <- tna(mock_sequence)
  out <- estimate_cs(model, drop_prop = seq(0.3, 0.9, by = 0.2), iter = 10)
  expect_error(
    capture.output(print.tna_stability(out)),
    NA
  )
})

test_that("permutation test results can be printed", {
  model_x <- tna(group_regulation[1:100, ])
  model_y <- tna(group_regulation[1001:1200, ])
  perm <- permutation_test(
    model_x,
    model_y,
    measures = "InStrength", iter = 20
  )
  expect_error(
    capture.output(print.tna_permutation(perm)),
    NA
  )
})

test_that("group model can be printed", {
  expect_error(
    capture.output(print.group_tna(mmm_model)),
    NA
  )
})

test_that("group model summary can be printed", {
  summ <- summary(mmm_model)
  expect_error(
    capture.output(print.summary.group_tna(summ)),
    NA
  )
})

test_that("bootstrap results can be printed for clusters", {
  boot <- bootstrap(mmm_model, iter = 10)
  expect_error(
    capture.output(print.group_tna_bootstrap(boot)),
    NA
  )
})

test_that("bootstrap summary can be printed for clusters", {
  boot <- bootstrap(mmm_model, iter = 10)
  summ <- summary.group_tna_bootstrap(boot)
  expect_error(
    capture.output(print.summary.group_tna_bootstrap(summ)),
    NA
  )
})

test_that("centralities can be printed for clusters", {
  cm <- centralities(mmm_model)
  expect_error(
    capture.output(print.group_tna_centralities(cm)),
    NA
  )
})

test_that("cliques can be printed for clusters", {
  cliq <- cliques(mmm_model, size = 2)
  expect_error(
    capture.output(print.group_tna_cliques(cliq)),
    NA
  )
})

test_that("communities can be printed", {
  comm <- communities(mmm_model)
  expect_error(
    capture.output(print.group_tna_communities(comm)),
    NA
  )
})

test_that("centrality stability coefficients can be printed for clusters", {
  out <- estimate_cs(mmm_model, drop_prop = seq(0.3, 0.9, by = 0.2), iter = 10)
  expect_error(
    capture.output(print.group_tna_stability(out)),
    NA
  )
})

test_that("tna_data objects can be printed", {
  data_single_session <- tibble::tibble(
    action = c(
      "view", "click", "add_cart", "view", "checkout", "view", "click", "share"
    )
  )
  rlang::local_options(rlib_message_verbosity = "quiet")
  data_out <- prepare_data(data_single_session, action = "action")
  expect_error(
    capture.output(print(data_out, data = "sequence")),
    NA
  )
  expect_error(
    capture.output(print(data_out, data = "meta")),
    NA
  )
  expect_error(
    capture.output(print(data_out, data = "long")),
    NA
  )
})

test_that("comparison results can be printed", {
  model_x <- tna(group_regulation[1:200, ])
  model_y <- tna(group_regulation[1001:1200, ])
  # Comparing models
  comp <- compare(model_x, model_y)
  expect_error(
    capture.output(print(comp)),
    NA
  )
})

test_that("comparison results with centralities can be printed", {
  model_x <- tna(group_regulation[1:200, ])
  model_y <- tna(group_regulation[1001:1200, ])
  # Comparing models with centrality measures included
  comp <- compare(model_x, model_y, measures = c("InStrength", "OutStrength"))
  expect_true(!is.null(comp$centrality_differences))
  expect_true(!is.null(comp$centrality_correlations))
  expect_error(
    capture.output(print(comp)),
    NA
  )
})

test_that("permutation test results can be printed for clusters", {
  perm <- permutation_test(mmm_model, iter = 50)
  expect_error(
    capture.output(print(perm)),
    NA
  )
})

test_that("clustering output can be printed", {
  clust <- cluster_sequences(mock_cluster_data, k = 2)
  expect_error(
    capture.output(print(clust)),
    NA
  )
})

test_that("sequence comparison output can be printed", {
  group <- c(rep("High", 1000), rep("Low", 1000))
  comp <- compare_sequences(group_regulation, group)
  expect_error(
    capture.output(print(comp)),
    NA
  )
})

test_that("reliability analysis can be printed", {
  rel <- reliability(mock_tna_seq, iter = 50)
  expect_error(
    capture.output(print(rel)),
    NA
  )
})

#' The `tna` Package.
#'
#' @name tna-package
#' @family basic
#' @description Provides tools for performing transition network analysis (TNA),
#' including functions for building TNA models, plotting transition networks,
#' and calculating centrality measures. The package relies on the  `igraph` for 
#' centrality measure calculations.
#'
#' @author Sonsoles López-Pernas, Santtu Tikka, Mohammed Saqr
#' @references
#' Saqr M., López-Pernas S., Törmänen T., Kaliisa R., Misiejuk K., Tikka S. (2025).
#' Transition Network Analysis: A Novel Framework for Modeling, Visualizing,
#' and Identifying the Temporal Patterns of Learners and Learning Processes.
#' In *Proceedings of the 15th International Learning Analytics and Knowledge Conference (LAK '25)*, 351-361.
#'
#' Banerjee A., Chandrasekhar A., Duflo E., Jackson M. (2014).
#' Gossip: Identifying Central Individuals in a Social Network.
#' Working Paper.
#'
#' Kivimaki, I., Lebichot, B., Saramaki, J., Saerens, M. (2016).
#' Two betweenness centrality measures based on Randomized Shortest Paths.
#' *Scientific Reports*, 6, 19668.
#'
#' Serrano, M. A., Boguna, M., Vespignani, A. (2009).
#' Extracting the multiscale backbone of complex weighted networks.
#' *Proceedings of the National Academy of Sciences*, 106, 6483-6488.
#'
#' Zhang, B., Horvath, S. (2005).
#' A general framework for weighted gene co-expression network analysis.
#' *Statistical Applications in Genetics and Molecular Biology*, 4(1).
#'
"_PACKAGE"

#' Example Data on Student Engagement
#'
#' Students' engagement states (Active / Average / Disengaged)
#' throughout a whole study program. The data was generated synthetically
#' based on the article "The longitudinal association between engagement and
#' achievement varies by time, students' profiles, and achievement state:
#' A full program study"
#'
#' @family datasets
#' @source \doi{10.1016/j.compedu.2023.104787}
#' @format A `stslist` object (sequence data).
"engagement"

#' Example Mixture Markov Model Object for the `engagement` Data
#'
#' A `tna_mmm` object structurally equivalent to a fitted seqHMM
#' mixture Markov model, used by the package's examples for
#' `group_model()` and `mmm_stats()`. Generated deterministically by
#' [random_tna_mmm()] with engagement-flavored state labels and
#' three clusters; carries no foreign class references so it is safe
#' to bundle.
#'
#' Re-generate via `data-raw/engagement_mmm.R` whenever the simulator
#' changes.
#'
#' @family datasets
#' @source Generated by `data-raw/engagement_mmm.R` using
#' [random_tna_mmm()].
#' @format A `tna_mmm` object containing three clusters over the
#'   states `"Engaged"`, `"Moderate"`, `"Disengaged"`.
"engagement_mmm"

#' Example Wide Data on Group Regulation
#'
#' Students' regulation during collaborative learning. Students' interactions
#' were coded as:  "adapt", "cohesion", "consensus", "coregulate", "discuss",
#' "emotion", "monitor", "plan", "synthesis"
#'
#' @family datasets
#' @source The data was generated synthetically.
#' @format A `data.frame` object.
"group_regulation"

#' Example Long Data on Group Regulation
#'
#' Students' regulation during collaborative learning. This is the same dataset
#' as `group_regulation` but in long format. In addition to students'
#' actions (`Action`), it contains the student identifier (`Actor`),
#' timestamp (`Time`), `Course` name, and collaboration `Group`. It
#' also includes a column (`Achiever`) indicating whether the student
#' is a high or low achiever.
#'
#' @family datasets
#' @source The data was generated synthetically from `group_regulation`
#' @format A `data.frame` object.
"group_regulation_long"

#' Compare Two Networks from Sequence Data using a Permutation Test
#'
#' This function compares two networks built from sequence data using
#' permutation tests. The function builds Markov models for two sequence
#' objects, computes the transition probabilities, and compares them by
#' performing permutation tests. It returns the differences in transition
#' probabilities, effect sizes, estimated p-values, and confidence intervals.
#'
#' @export
#' @family validation
#' @param x A `tna` object containing sequence data for the first `tna` model.
#' @param y A `tna` object containing sequence data for the second `tna` model.
#' @param adjust A `character` string for the method to adjust p-values with
#' for multiple comparisons. The default is `"none"` for no adjustment.
#' See the `method` argument of [stats::p.adjust()] for details and available
#' adjustment methods.
#' @param iter An `integer` giving the number of permutations to perform.
#' The default is 1000.
#' @param paired A `logical` value. If `TRUE`, perform paired permutation tests;
#' if `FALSE`, perform unpaired tests. The default is `FALSE`.
#' @param level A `numeric` value giving the significance level for the
#' permutation tests. The default is 0.05.
#' @param measures A `character` vector of centrality measures to test.
#' See [centralities()] for a list of available centrality measures.
#' @param ... Additional arguments passed to [centralities()].
#' @return A `tna_permutation` object which is a `list` with two elements:
#' `edges` and `centralities`, both containing the following elements:
#'
#'   * `stats`: A `data.frame` of original differences, effect sizes, and
#'     estimated p-values for each edge or centrality measure. The effect size
#'     is computed as the observed difference divided by the standard deviation
#'     of the differences of the permuted samples.
#'   * `diffs_true`: A `matrix` of differences in the data.
#'   * `diffs_sig`: A `matrix` showing the significant differences.
#'
#' @examples
#' model_x <- tna(group_regulation[1:200, ])
#' model_y <- tna(group_regulation[1001:1200, ])
#' # Small number of iterations for CRAN
#' permutation_test(model_x, model_y, iter = 20)
#'
permutation_test <- function(x, ...) {
  UseMethod("permutation_test")
}

#' @export
#' @rdname permutation_test
permutation_test.tna <- function(x, y, adjust = "none", iter = 1000,
                                 paired = FALSE, level = 0.05,
                                 measures = character(0), ...) {
  check_missing(x)
  check_missing(y)
  check_tna_seq(x)
  check_tna_seq(y)
  check_values(iter, strict = TRUE)
  check_flag(paired)
  check_range(level, lower = 0, upper = 1)
  adjust <- check_match(adjust, stats::p.adjust.methods, match_case = TRUE)
  permutation_test_(
    x = x,
    y = y,
    adjust = adjust,
    iter = iter,
    paired = paired,
    level = level,
    measures = measures,
    ...
  )
}

#' Compare Networks using a Permutation Test
#'
#' Test edge weight differences between all pairs or a subset of pairs of
#' a `group_tna` object. See [permutation_test.tna()] for more details.
#'
#' @export
#' @family validation
#' @param x A `group_tna` object
#' @param groups An `integer` vector or a `character` vector of group indices
#' or names, respectively, defining which groups to compare. When not provided,
#' all pairs are compared (the default).
#' @param consecutive A `logical` value. If `FALSE` (the default), all pairwise
#' comparisons are performed in lexicographic order with respect to the order
#' of the groups. If `TRUE`, only comparisons between consecutive pairs
#' of groups are performed.
#' @inheritParams permutation_test.tna
#' @examples
#' model <- group_model(engagement_mmm)
#' # Small number of iterations for CRAN
#' permutation_test(model, iter = 20)
#'
permutation_test.group_tna <- function(x, groups, adjust = "none",
                                       iter = 1000, paired = FALSE,
                                       level = 0.05, measures = character(0),
                                       consecutive = FALSE, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  stopifnot_(
    length(attr(x, "scaling")) == 0L || attr(x, "groupwise"),
    "Permutation test is not supported for
     grouped models with globally scaled edge weights."
  )
  check_values(iter, strict = TRUE)
  check_flag(paired)
  check_range(level, lower = 0, upper = 1)
  adjust <- check_match(adjust, stats::p.adjust.methods, match_case = TRUE)
  x_names <- names(x)
  groups <- groups %m% seq_along(x)
  check_cluster(x, groups)
  groups <- ifelse_(
    is.character(groups),
    which(x_names %in% groups),
    groups
  )
  n_groups <- length(groups)
  stopifnot_(
    n_groups >= 2L,
    "Argument {.arg groups} must contain at least two groups to compare."
  )
  if (consecutive) {
    out <- vector(mode = "list", length = n_groups - 1L)
    for (i in seq_len(n_groups - 1L)) {
      group_i <- groups[i]
      group_j <- groups[i + 1L]
      out[[i]] <- permutation_test_(
        x = x[[group_i]],
        y = x[[group_j]],
        adjust = adjust,
        iter = iter,
        paired = paired,
        level = level,
        measures = measures,
        ...
      )
      names(out)[i] <- paste0(x_names[group_i], " vs. ", x_names[group_j])
    }
  } else {
    n_pairs <- (n_groups * (n_groups - 1L)) %/% 2L
    out <- vector(mode = "list", length = n_pairs)
    idx <- 0L
    for (i in seq_len(n_groups - 1L)) {
      for (j in seq(i + 1L, n_groups)) {
        idx <- idx + 1L
        group_i <- groups[i]
        group_j <- groups[j]
        out[[idx]] <- permutation_test_(
          x = x[[group_i]],
          y = x[[group_j]],
          adjust = adjust,
          iter = iter,
          paired = paired,
          level = level,
          measures = measures,
          ...
        )
        names(out)[idx] <- paste0(x_names[group_i], " vs. ", x_names[group_j])
      }
    }
  }
  structure(
    out,
    class = "group_tna_permutation"
  )
}

permutation_test_ <- function(x, y, adjust, iter, paired, level,
                              measures, ...) {
  data_x <- x$data
  data_y <- y$data
  n_x <- nrow(data_x)
  n_y <- nrow(data_y)
  stopifnot_(
    !paired || n_x == n_y,
    "The number of observations must be the same in {.arg x} and {.arg y}
     for a paired test."
  )
  alph_x <- attr(data_x, "alphabet")
  alph_y <- attr(data_y, "alphabet")
  a <- length(alph_x)
  stopifnot_(
    a == length(alph_y),
    "The number of states of {.arg x} and {.arg y} must be the same."
  )
  stopifnot_(
    all(alph_x == alph_y),
    "The state labels of {.arg x} and {.arg y} must be the same
     and in the same order."
  )
  combined_data <- rbind(data_x, data_y)
  attr(combined_data, "alphabet") <- attr(data_x, "alphabet")
  attr(combined_data, "labels") <- attr(data_x, "labels")
  n_xy <- n_x + n_y
  weights_x <- x$weights
  weights_y <- y$weights
  type <- attr(x, "type")
  scaling <- attr(x, "scaling")
  params <- attr(x, "params")
  n_measures <- length(measures)
  include_centralities <- n_measures > 0L
  if (include_centralities) {
    cent_x <- centralities(x, measures = measures, ...)
    cent_y <- centralities(y, measures = measures, ...)
    cent_diffs_true <- as.matrix(cent_x[, -1L] - cent_y[, -1L])
    cent_diffs_true_abs <- abs(cent_diffs_true)
  }
  edge_diffs_true <- weights_x - weights_y
  edge_diffs_true_abs <- abs(edge_diffs_true)
  edge_names <- expand.grid(
    from = rownames(weights_x),
    to = colnames(weights_y)
  )
  edge_names <- paste0(edge_names$from, " -> ", edge_names$to)
  idx_x <- seq_len(n_x)
  idx_y <- seq(n_x + 1L, n_xy)
  combined_model <- initialize_model(
    combined_data,
    type,
    scaling,
    params,
    transitions = TRUE
  )
  combined_trans <- combined_model$trans
  edge_diffs_perm <- array(0L, dim = c(iter, a, a))
  cent_diffs_perm <- array(0L, dim = c(iter, a, n_measures))
  edge_p_values <- matrix(0L, a, a)
  cent_p_values <- matrix(0L, a, n_measures)
  for (i in seq_len(iter)) {
    if (paired) {
      # For paired data, permute within pairs
      pair_idx <- matrix(seq_len(n_xy), ncol = 2)
      permuted_pairs <- t(apply(pair_idx, 1, sample))
      perm_idx <- c(permuted_pairs)
    } else {
      # For unpaired data, perform complete randomization
      perm_idx <- sample(n_xy)
    }
    trans_perm_x <- combined_trans[perm_idx[idx_x], , , drop = FALSE]
    trans_perm_y <- combined_trans[perm_idx[idx_y], , , drop = FALSE]
    weights_perm_x <- compute_weights(trans_perm_x, type, scaling, a)
    weights_perm_y <- compute_weights(trans_perm_y, type, scaling, a)
    if (include_centralities) {
      cent_perm_x <- centralities(weights_perm_x, measures = measures, ...)
      cent_perm_y <- centralities(weights_perm_y, measures = measures, ...)
      cent_diffs_perm[i, , ] <- as.matrix(
        cent_perm_x[, -1L] - cent_perm_y[, -1L]
      )
      cent_p_values <- cent_p_values +
        1L * (abs(cent_diffs_perm[i, , ]) >= cent_diffs_true_abs)
    }
    edge_diffs_perm[i, , ] <- weights_perm_x - weights_perm_y
    edge_p_values <- edge_p_values +
      1L * (abs(edge_diffs_perm[i, , ]) >= edge_diffs_true_abs)
  }
  edge_p_values <- (edge_p_values + 1) / (iter + 1)
  edge_p_values[,] <- stats::p.adjust(
    p = edge_p_values,
    method = adjust
  )
  edge_diffs_sd <- apply(edge_diffs_perm, c(2, 3), stats::sd)
  edge_diffs_sig <- edge_diffs_true * (edge_p_values < level)
  edge_stats <- data.frame(
    edge_name = edge_names,
    diff_true = c(edge_diffs_true),
    effect_size = c(edge_diffs_true) / c(edge_diffs_sd),
    p_value = c(edge_p_values)
  )
  out <- list(
    edges = list(
      stats = edge_stats,
      diffs_true = edge_diffs_true,
      diffs_sig = edge_diffs_sig
    )
  )
  if (include_centralities) {
    cent_p_values <- (cent_p_values + 1) / (iter + 1)
    cent_p_values[,] <- stats::p.adjust(
      p = cent_p_values,
      method = adjust
    )
    cent_diffs_sd <- apply(cent_diffs_perm, c(2, 3), stats::sd)
    cent_diffs_sig <- cent_diffs_true * (cent_p_values < level)
    cent_stats <- expand.grid(state = cent_x$state, centrality = measures)
    cent_stats$diff_true <- c(cent_diffs_true)
    cent_stats$effect_size <- c(cent_diffs_true) / c(cent_diffs_sd)
    cent_stats$p_value <- c(cent_p_values)
    cent_diffs_true <- cbind(
      data.frame(state = cent_x$state),
      as.data.frame(cent_diffs_true)
    )
    cent_diffs_sig <- cbind(
      data.frame(state = cent_x$state),
      as.data.frame(cent_diffs_sig)
    )
    out$centralities <- list(
      stats = cent_stats,
      diffs_true = cent_diffs_true,
      diffs_sig = cent_diffs_sig
    )
  }
  structure(
    out,
    labels = x$labels,
    colors = attr(x$data, "colors"),
    class = "tna_permutation"
  )
}

permutation_test_patterns <- function(x, len, iter, adjust) {
  stopifnot_(
    length(attr(x, "scaling")) == 0L || attr(x, "groupwise"),
    "Permutation test is not supported for
     grouped models with globally scaled edge weights."
  )
  check_values(iter, strict = TRUE)
  data <- combine_data(x)
  group <- data$.group
  data$.group <- NULL
  m <- as.matrix(data)
  n <- nrow(m)
  k <- length(len)
  labels <- x[[1L]]$labels
  pattern_matrices <- extract_patterns(m, len, labels)
  patterns <- factorize_patterns(pattern_matrices, group)
  stat_true <- lapply(patterns, pattern_statistic)
  stat_perm <- vector(mode = "list", length = k)
  stat_p_value <- vector(mode = "list", length = k)
  for (j in seq_len(k)) {
    u <- length(pattern_matrices[[j]]$unique)
    stat_perm[[j]] <- matrix(0, u, iter)
    stat_p_value[[j]] <- numeric(u)
  }
  for (i in seq_len(iter)) {
    perm_idx <- sample(n)
    perm_group <- group[perm_idx]
    perm_patterns <- factorize_patterns(pattern_matrices, perm_group)
    for (j in seq_len(k)) {
      stat_perm[[j]][, i] <- pattern_statistic(perm_patterns[[j]])
      stat_p_value[[j]] <- stat_p_value[[j]] + 1L *
        (stat_perm[[j]][, i] >= stat_true[[j]])
    }
  }
  out <- vector(mode = "list", length = k)
  for (j in seq_len(k)) {
    stat_perm_mean <- apply(stat_perm[[j]], 1, mean)
    stat_perm_sd <- apply(stat_perm[[j]], 1, stats::sd)
    out[[j]] <- data.frame(
      effect_size = (stat_true[[j]] - stat_perm_mean) / stat_perm_sd,
      p_value = stats::p.adjust(
        (stat_p_value[[j]] + 1) / (iter + 1),
        method = adjust
      )
    )
  }
  do.call("rbind", out)
}

pattern_statistic <- function(x) {
  n <- sum(x)
  nr <- nrow(x)
  nc <- ncol(x)
  rs <- .rowSums(x, m = nr, n = nc)
  cs <- .colSums(x, m = nr, n = nc)
  expected <- outer(rs, cs) / n
  sqrt(.rowSums((x - expected)^2, m = nr, n = nc))
}

test_that("permutation test can be applied", {
  model_x <- tna(mock_sequence[1:2, ])
  model_y <- tna(mock_sequence[3:5, ])
  expect_error(
    permutation_test(model_x, model_y, iter = 20),
    NA
  )
})

test_that("paired permutation test can be applied", {
  model_x <- tna(mock_sequence[1:2, ])
  model_y <- tna(mock_sequence[4:5, ])
  expect_error(
    permutation_test(model_x, model_y, paired = TRUE, iter = 20),
    NA
  )
})

test_that("permutation test can be applied with groups", {
  expect_error(
    permutation_test(mmm_model, iter = 20),
    NA
  )
})

test_that("permutation test p-values can be adjusted", {
  expect_error(
    permutation_test(mmm_model, iter = 20, adjust = "holm"),
    NA
  )
  expect_error(
    permutation_test(mmm_model, iter = 20, adjust = "holm", adjust_pairwise = FALSE),
    NA
  )
})

test_that("paired permutation fails when data are incomparable", {
  model_x <- tna(mock_sequence[1:4, ])
  model_y <- tna(mock_sequence[3:5, ])
  model_z <- tna(mock_sequence[c(1, 4, 5), 1:2])
  model_w <- model_z
  attr(model_w$data, "alphabet") <- rev(attr(model_w$data, "alphabet"))
  expect_error(
    permutation_test(model_x, model_y, paired = TRUE),
    "The number of observations must be the same in `x` and `y` for a paired test\\."
  )
  expect_error(
    permutation_test(model_x, model_z),
    "The number of states of `x` and `y` must be the same\\."
  )
  expect_error(
    permutation_test(model_z, model_w),
    "The state labels of `x` and `y` must be the same and in the same order\\."
  )
})

test_that("permutation test p-values can be adjusted", {
  model_x <- tna(mock_sequence[1:2, ])
  model_y <- tna(mock_sequence[3:5, ])
  for (method in stats::p.adjust.methods) {
    expect_error(
      permutation_test(model_x, model_y, adjust = method),
      NA
    )
  }
})

test_that("permutation test pairs can be made consecutive", {
  model <- group_model(mock_sequence, group = c(1, 1, 2, 3, 3))
  expect_error(
    out <- permutation_test(model, consecutive = TRUE, iter = 20),
    NA
  )
  expect_true(length(out) == 2)
})

#' Check if argument is missing
#'
#' @param x An \R object.
#' @noRd
check_missing <- function(x) {
  arg <- deparse(substitute(x))
  stopifnot_(
    !missing(x),
    "Argument {.arg {arg}} is missing."
  )
}

#' Check if argument contains missing values
#'
#' @param x An \R object.
#' @noRd
check_na <- function(x) {
  arg <- deparse(substitute(x))
  stopifnot_(
    all(!is.na(x)),
    "Argument {.arg {arg}} must not contain missing values."
  )
}

#' Check Transition Network Type for Validity
#'
#' @param type Type of the transition network.
#' @noRd
check_model_type <- function(type) {
  check_match(
    type,
    c(
      "relative",
      "frequency",
      "co-occurrence",
      "n-gram",
      "gap",
      "window",
      "reverse",
      "attention"
    )
  )
}

#' Check Transition Network Weight Scaling for Validity
#'
#' @param scaling A `character` vector of scaling options to apply.
#' @noRd
check_model_scaling <- function(scaling) {
  if (length(scaling) == 0L) {
    return(character(0L))
  }
  check_match(
    scaling,
    c("minmax", "max", "rank"),
    several.ok = TRUE
  )
}

#' Check that `x` is of specific class
#'
#' @param x An \R object.
#' @inheritParams class
#' @noRd
check_class <- function(x, what) {
  arg <- deparse(substitute(x))
  stopifnot_(
    inherits(x, what),
    "Argument {.arg {arg}} must be a {.cls {what}} object."
  )
}

#' Check that `x` is a `tna` object created from sequence Data
#'
#' @param x An \R object.
#' @noRd
check_tna_seq <- function(x) {
  arg <- deparse(substitute(x))
  stopifnot_(
    is_tna(x) && !is.null(x$data),
    "Argument {.arg {arg}} must be a {.cls tna}
     object created from sequence data."
  )
}

#' Check that `x` contains valid centrality measures
#'
#' @param x An \R object expected to be a `character` vector.
#' @noRd
check_measures <- function(x) {
  stopifnot_(
    checkmate::test_character(
      x = x,
      any.missing = FALSE,
      unique = TRUE,
    ),
    "Argument {.arg measures} must be a {.cls character} vector."
  )
  available_measures <- names(centrality_funs)
  lower_measures <- tolower(x)
  lower_defaults <- tolower(available_measures)
  measures_match <- pmatch(lower_measures, lower_defaults)
  no_match <- is.na(measures_match)
  invalid_measures <- x[no_match]
  valid_measures <- x[!no_match]
  stopifnot_(
    length(invalid_measures) == 0L,
    c(
      "Argument {.arg measures} contains invalid centrality measures:",
      `x` = "Measure{?s} {.val {invalid_measures}} {?is/are} not recognized."
    )
  )
  available_measures[measures_match]
}

#' Check that `x` is numeric
#'
#' @param x An \R object expected to be a `numeric` value.
#' @noRd
check_numeric <- function(x) {
  arg <- deparse(substitute(x))
  stopifnot_(
    checkmate::test_number(x = x),
    "Argument {.arg {arg}} must be a single {.cls numeric} value."
  )
}

#' Check that `x` is a non-negative
#'
#' @param x An \R object expected to be a `numeric` or `integer`
#' value or a vector.
#' @param type A `character` string corresponding to
#' the type that `x` should be.
#' @param strict A `logical` value. If `FALSE` (the default), expects
#' non-negative values and positive otherwise.
#' @param scalar A `logical` value indicating if `x` should be expected
#' to be a single value.
#' @noRd
check_values <- function(x, type = "integer", strict = FALSE,
                         scalar = TRUE) {
  arg <- deparse(substitute(x))
  suffix <- ifelse_(
    scalar,
    ifelse_(type == "integer", "", " value"),
    " vector"
  )
  test_fun <- ifelse_(
    type == "numeric",
    ifelse_(scalar, checkmate::test_number, checkmate::test_numeric),
    ifelse_(scalar, checkmate::test_int, checkmate::test_integer)
  )
  strictness <- ifelse_(strict, "positive", "non-negative")
  stopifnot_(
    test_fun(x = x, lower = as.integer(strict && type == "integer")),
    "Argument {.arg {arg}} must be a {strictness} {.cls {type}}{suffix}."
  )
}

#' Check that `x` is between a minimum and a maximum value
#'
#' @param x An \R object expected to be within a specific range.
#' @noRd
check_range <- function(x, type = "numeric", scalar = TRUE,
                        lower = -Inf, upper = Inf) {
  arg <- deparse(substitute(x))
  prefix <- ifelse_(scalar, "be a single", "only contain")
  suffix <- ifelse_(
    scalar,
    ifelse_(type == "integer", "", " value"),
    " values"
  )
  test_fun <- ifelse_(
    type == "numeric",
    ifelse_(scalar, checkmate::test_number, checkmate::test_numeric),
    ifelse_(scalar, checkmate::test_int, checkmate::test_integerish)
  )
  bounds <- ""
  if (is.infinite(lower)) {
    bounds <- paste0("less than or equal to ", upper)
  } else if (is.infinite(upper)) {
    bounds <- paste0("greater than or equal to ", lower)
  } else {
    bounds <- paste0("between ", lower, " and ", upper)
  }
  stopifnot_(
    test_fun(x = x, lower = lower, upper = upper),
    "Argument {.arg {arg}} must {prefix}
    {.cls {type}} {suffix} {bounds}."
  )
}

#' Check That `x` is a Logical Value
#'
#' @param x An \R object expected to be a `logical` value.
#' @noRd
check_flag <- function(x) {
  arg <- deparse(substitute(x))
  stopifnot_(
    checkmate::test_flag(x = x),
    "Argument {.arg {arg}} must be a single {.cls logical} value."
  )
}

#' Check Edge Weight Matrix based on TNA Type
#'
#' @param x A `matrix` of edge weights.
#' @param type Type of the transition network as a `character` string.
#' @noRd
check_weights <- function(x, type) {
  d <- dim(x)[1L]
  if (type == "relative") {
    rs <- .rowSums(x = x, m = d, n = d)
    stopifnot_(
      all(rs > 0),
      "At least one element of each row of {.arg x} must be positive."
    )
    x[] <- x / rs
  } else if (type %in% c("frequency", "co-occurrence", "reverse", "window")) {
    x_int <- as.integer(x)
    stopifnot_(
      all(x == x_int) && all(x >= 0),
      "All elements of {.arg x} must be non-negative integers."
    )
    x[] <- x_int
  }
  x
}

#' Check if argument matches given choices ignoring case
#'
#' @param x A `character` string.
#' @inheritParams match.arg
#' @noRd
check_match <- function(x, choices, several.ok = FALSE, match_case = FALSE) {
  arg <- deparse(substitute(x))
  if (!match_case) {
    x <- onlyif(is.character(x), tolower(x))
    choices <- tolower(choices)
  }
  x <- try_(match.arg(arg = x, choices = choices, several.ok = several.ok))
  n_choices <- length(choices)
  prefix <- ifelse_(several.ok, "Elements of", "Argument")
  stopifnot_(
    !inherits(x, "try-error"),
    "{prefix} {.arg {arg}} must be either
    {cli::qty(n_choices)} {.or {.val {choices}}}."
  )
  x
}

#' Check if argument is a character string
#'
#' @param x An \R object.
#' @noRd
check_string <- function(x) {
  arg <- deparse(substitute(x))
  stopifnot_(
    checkmate::test_string(x = x),
    "Argument {.arg {arg}} must be a {.cls character} vector of length 1."
  )
}

#' #' Check if argument is a character vector
#' #'
#' #' @param x An \R object.
#' #' @noRd
#' check_character <- function(x) {
#'   if (missing(x)) {
#'     return()
#'   }
#'   arg <- deparse(substitute(x))
#'   stopifnot_(
#'     checkmate::test_character(
#'       x = x,
#'       min.len = 1L,
#'       any.missing = FALSE,
#'       unique = TRUE,
#'     ),
#'     "Argument {.arg {arg}} must be a {.cls character} vector."
#'   )
#' }

#' Check that argument is a valid cluster
#'
#' @param x A `group_tna` object.
#' @param i Index vector of clusters.
#' @noRd
check_cluster <- function(x, i) {
  i <- ifelse_(is.numeric(i), as.integer(i), i)
  arg <- deparse(substitute(i))
  n <- length(x)
  stopifnot_(
    is.integer(i) || all(i %in% names(x)),
    "Argument {.arg {arg}} must only contain names of {.arg x} when of type
     {.cls character}."
  )
  stopifnot_(
    is.character(i) || all(i >= 1 & i <= n),
    "Argument {.arg {arg}} must contain integers between 1 and {n} when of type
     {.cls numeric}."
  )
}

#' Check that indices/names are valid clusters
#'
#' @param x A `group_tna` object.
#' @param i Index of the first cluster.
#' @param j Index of the second cluster.
#' @noRd
check_clusters <- function(x, i, j) {
  i <- ifelse_(is.numeric(i), as.integer(i), i)
  j <- ifelse_(is.numeric(j), as.integer(j), j)
  stopifnot_(
    !identical(i, j),
    "Arguments {.arg i} and {.arg j} must be different."
  )
  n <- length(x)
  for (arg in c("i", "j")) {
    idx <- eval(rlang::sym(arg))
    stopifnot_(
      length(idx) == 1L && (is.integer(idx) || is.character(idx)),
      "Argument {.arg {arg}} must be a {.cls numeric} or a {.cls character}
      vector of length 1."
    )
    stopifnot_(
      is.integer(idx) || idx %in% names(x),
      "Argument {.arg {arg}} must be a name of {.arg x} when of type
      {.cls character}."
    )
    stopifnot_(
      is.character(idx) || (idx >= 1 && idx <= n),
      "Argument {.arg {arg}} must be between 1 and {n} when of type
      {.cls numeric}."
    )
  }
}

check_cols <- function(cols, single = TRUE, missing_ok = TRUE) {
  arg <- deparse(substitute(cols))
  if (missing(cols)) {
    stopifnot_(
      missing_ok,
      "Argument {.arg {arg}} is missing."
    )
    return()
  }
  if (single) {
    stopifnot_(
      length(cols) == 1L,
      "Argument {.arg {arg}} must provide a single column name."
    )
  }
}

#' Check `build_model` dots arguments
#'
#' @noRd
check_dots <- function(...) {
  dots <- list(...)
  dots_names <- names(dots)
  valid_args <- c(
    "cols",
    "inits",
    "begin_state",
    "end_state"
  )
  invalid_dots <- dots_names[!dots_names %in% valid_args]
  stopifnot_(
    length(invalid_dots) == 0L,
    "Argument{?/s} {.arg {invalid_dots}} {?is/are} not recognized."
  )
}

test_that("missing argument fails", {
  f <- function(z) {
    check_missing(z)
  }
  expect_error(
    f(),
    "Argument `z` is missing\\."
  )
})

test_that("missing values fail", {
  f <- function(z) {
    check_na(z)
  }
  expect_error(
    f(c(1, 2, NA)),
    "Argument `z` must not contain missing values\\."
  )
})

test_that("invalid model type fails", {
  expect_error(
    check_model_type("unknown"),
    "Argument `type` must be either"
  )
})

test_that("invalid scaling options fail", {
  expect_error(
    check_model_scaling("unknown"),
    "Elements of `scaling` must be either"
  )
})

test_that("invalid class fails", {
  f <- function(z) {
    check_class(z, "test_class")
  }
  expect_error(
    f(z = 1),
    "Argument `z` must be a <test_class> object\\."
  )
})

test_that("non-sequence data tna fails", {
  f <- function(z) {
    check_tna_seq(z)
  }
  expect_error(
    f(z = mock_tna),
    "Argument `z` must be a <tna> object created from sequence data\\."
  )
})

test_that("invalid centrality measures fail", {
  expect_error(
    check_measures("unknown"),
    "Argument `measures` contains invalid centrality measures:"
  )
})

test_that("negative values fail", {
  opts <- expand.grid(
    type = c("integer", "numeric"),
    strict = c(FALSE, TRUE),
    scalar = c(FALSE, TRUE)
  )
  f <- function(i, z) {
    check_values(
      z,
      type = opts[i, 1],
      strict = opts[i, 2],
      scalar = opts[i, 3]
    )
  }
  expect_error(
    f(i = 1, z = -1),
    "Argument `z` must be a non-negative <integer> vector\\."
  )
  expect_error(
    f(i = 2, z = -1),
    "Argument `z` must be a non-negative <numeric> vector\\."
  )
  expect_error(
    f(i = 3, z = -1),
    "Argument `z` must be a positive <integer> vector\\."
  )
  expect_error(
    f(i = 4, z = -1),
    "Argument `z` must be a positive <numeric> vector\\."
  )
  expect_error(
    f(i = 5, z = -1),
    "Argument `z` must be a non-negative <integer>\\."
  )
  expect_error(
    f(i = 6, z = -1),
    "Argument `z` must be a non-negative <numeric> value\\."
  )
  expect_error(
    f(i = 7, z = -1),
    "Argument `z` must be a positive <integer>\\."
  )
  expect_error(
    f(i = 8, z = -1),
    "Argument `z` must be a positive <numeric> value\\."
  )
})

test_that("invalid range check fails", {
  opts <- expand.grid(
    type = c("integer", "numeric"),
    scalar = c(FALSE, TRUE)
  )
  f <- function(i, z) {
    check_range(
      z, type = opts[i, 1], scalar = opts[i, 2], lower = -2, upper = 2
    )
  }
  expect_error(
    f(i = 1, z = 3),
    "Argument `z` must only contain <integer> values between -2 and 2\\."
  )
  expect_error(
    f(i = 2, z = 4),
    "Argument `z` must only contain <numeric> values between -2 and 2\\."
  )
  expect_error(
    f(i = 3, z = -4),
    "Argument `z` must be a single <integer> between -2 and 2\\."
  )
  expect_error(
    f(i = 4, z = -3),
    "Argument `z` must be a single <numeric> value between -2 and 2\\."
  )
})

test_that("invalid logical fails", {
  f <- function(z) {
    check_flag(z)
  }
  expect_error(
    f(data.frame()),
    "Argument `z` must be a single <logical> value\\."
  )
})

test_that("cluster check fails on invalid clusters", {
  expect_error(
    check_clusters(mmm_model, i = 1, j = 1),
    "Arguments `i` and `j` must be different\\."
  )
  expect_error(
    check_clusters(mmm_model, i = 1, j = 4),
    "Argument `j` must be between 1 and 3 when of type <numeric>\\."
  )
  expect_error(
    check_clusters(mmm_model, i = c(2, 3), j = 1),
    "Argument `i` must be a <numeric> or a <character> vector of length 1\\."
  )
  expect_error(
    check_clusters(mmm_model, i = 1, j = "Cluster 4"),
    "Argument `j` must be a name of `x` when of type <character>\\."
  )
})

test_that("range check variants are correct", {
  value <- 1
  expect_error(
    check_range(value, lower = 2),
    "Argument `value` must be a single <numeric> value greater than or equal to 2\\."
  )
  expect_error(
    check_range(value, upper = 0),
    "Argument `value` must be a single <numeric> value less than or equal to 0\\."
  )
  expect_error(
    check_range(value, lower = -1, upper = 0),
    "Argument `value` must be a single <numeric> value between -1 and 0\\."
  )
})

test_that("check_numeric validates correctly", {
  f <- function(z) {
    check_numeric(z)
  }
  expect_error(
    f("not numeric"),
    "must be a single <numeric> value"
  )
  expect_error(
    f(c(1, 2, 3)),
    "must be a single <numeric> value"
  )
  expect_error(f(1), NA)
})

test_that("check_string validates correctly", {
  f <- function(z) {
    check_string(z)
  }
  expect_error(
    f(123),
    "must be a <character> vector of length 1"
  )
  expect_error(
    f(c("a", "b")),
    "must be a <character> vector of length 1"
  )
  expect_error(f("valid"), NA)
})

test_that("check_cols validates correctly", {
  expect_error(
    check_cols(missing_ok = FALSE),
    "is missing"
  )
  expect_error(
    check_cols(c("a", "b"), single = TRUE),
    "must provide a single column name"
  )
})

test_that("check_cluster validates correctly", {
  # Invalid character cluster
  expect_error(
    check_cluster(mmm_model, "NonExistent"),
    "must only contain names of"
  )
  # Invalid numeric cluster
  expect_error(
    check_cluster(mmm_model, 99),
    "must contain integers between"
  )
  # Valid cases
  expect_error(check_cluster(mmm_model, 1), NA)
  expect_error(check_cluster(mmm_model, "Cluster 1"), NA)
})

test_that("check_dots validates correctly", {
  expect_error(
    check_dots(invalid_arg = 1),
    "not recognized"
  )
  expect_error(
    check_dots(cols = 1:3),
    NA
  )
})

#' Retrieve Statistics from a Mixture Markov Model (MMM)
#'
#' @export
#' @family clusters
#' @param x A `mhmm` object (from the `seqHMM` package) or a `tna_mmm`
#'   object built by [random_tna_mmm()].
#' @param level A `numeric` value representing the significance level for
#' hypothesis testing and confidence intervals. Defaults to `0.05`.
#' @return A `data.frame` object.
#' @examples
#' mmm_stats(engagement_mmm)
#'
mmm_stats <- function(x, level = 0.05) {
  UseMethod("mmm_stats")
}

#' @export
#' @rdname mmm_stats
mmm_stats.tna_mmm <- function(x, level = 0.05) {
  check_missing(x)
  check_range(level, lower = 0.0, upper = 1.0)
  mmm_stats_(x$coefficients, x$vcov, level)
}

#' @export
#' @rdname mmm_stats
mmm_stats.mhmm <- function(x, level = 0.05) {
  stopifnot_(
    requireNamespace("seqHMM", quietly = TRUE),
    "Please install the {.pkg seqHMM} package."
  )
  check_missing(x)
  check_range(level, lower = 0.0, upper = 1.0)
  stopifnot_(
    inherits(x, "mhmm"),
    c(
      "Argument {.arg x} must be a {.cls mhmm} object.",
      `i` = "See the {.pkg seqHMM} package for more information."
    )
  )
  sumr <- summary(x)
  mmm_stats_(sumr$coefficients, sumr$vcov, level)
}

#' Internal function for MMM statistics
#'
#' @param cf A `matrix` of regression coefficients from `coef`.
#' @param vc A variance-covariance matrix from `vcov`.
#' @param level The significance level
#' @noRd
mmm_stats_ <- function(cf, vc, level) {
  coef_flat <- c()
  se_flat <- c()
  cf <- as.matrix(cf)[, -1L, drop = FALSE]
  vcov_diag <- sqrt(diag(vc))
  num_vars <- nrow(cf)
  num_clusters <- ncol(cf)
  cluster_vec <- character(num_clusters)
  variable_vec <- character(num_vars)
  clusters <- colnames(cf)
  variables <- rownames(cf)
  idx <- 0L
  for (cluster in seq_len(num_clusters)) {
    for (var in seq_len(num_vars)) {
      idx <- idx + 1L
      coef_flat <- c(coef_flat, cf[var, cluster])
      se_flat <- c(se_flat, vcov_diag[(cluster - 1L) * num_vars + var])
      cluster_vec[idx] <- clusters[cluster]
      variable_vec[idx] <- variables[var]
    }
  }
  stopifnot_(
    length(coef_flat) == length(se_flat),
    "The lengths of the coefficients and standard errors do not match."
  )
  statistic <- coef_flat / se_flat
  p_value <- 2 * (1 - stats::pnorm(abs(statistic)))
  ci_margin <- stats::qnorm(1 - level / 2.0) * se_flat
  ci_lower <- coef_flat - ci_margin
  ci_upper <- coef_flat + ci_margin
  results <- data.frame(
    cluster = cluster_vec,
    variable = variable_vec,
    estimate = coef_flat,
    std_error = se_flat,
    ci_lower = ci_lower,
    ci_upper = ci_upper,
    z_value = statistic,
    p_value = p_value
  )
  rownames(results) <- NULL
  results
}

# Tests for mmm_stats generic function
test_that("mmm_stats is a generic function", {
  expect_true(is.function(mmm_stats))
  methods_info <- methods("mmm_stats")
  expect_true("mmm_stats.mhmm" %in% methods_info)
  expect_true("mmm_stats.tna_mmm" %in% methods_info)
})

# Tests for mmm_stats.tna_mmm method (dispatched via the bundled
# engagement_mmm, which is now a tna_mmm object).
test_that("mmm_stats.tna_mmm works with engagement_mmm data", {
  result <- mmm_stats(engagement_mmm)
  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) > 0)
  expect_named(
    result,
    c(
      "cluster", "variable", "estimate", "std_error",
      "ci_lower", "ci_upper", "z_value", "p_value"
    )
  )
})

test_that("mmm_stats.tna_mmm returns correct column types", {
  result <- mmm_stats(engagement_mmm)
  expect_type(result$cluster, "character")
  expect_type(result$variable, "character")
  expect_type(result$estimate, "double")
  expect_type(result$std_error, "double")
  expect_type(result$ci_lower, "double")
  expect_type(result$ci_upper, "double")
  expect_type(result$z_value, "double")
  expect_type(result$p_value, "double")
})

test_that("mmm_stats.tna_mmm respects custom level parameter", {
  result_default <- mmm_stats(engagement_mmm, level = 0.05)
  result_narrow <- mmm_stats(engagement_mmm, level = 0.01)

  # Narrower CI (lower level) should have wider margins
  ci_width_default <- result_default$ci_upper - result_default$ci_lower
  ci_width_narrow <- result_narrow$ci_upper - result_narrow$ci_lower

  # 99% CI should be wider than 95% CI

  expect_true(all(ci_width_narrow >= ci_width_default - 1e-10))
})

test_that("mmm_stats.tna_mmm validates level parameter", {
  expect_error(mmm_stats(engagement_mmm, level = -0.1))
  expect_error(mmm_stats(engagement_mmm, level = 1.5))
})

test_that("mmm_stats.tna_mmm returns p-values between 0 and 1", {
  result <- mmm_stats(engagement_mmm)
  expect_true(all(result$p_value >= 0))
  expect_true(all(result$p_value <= 1))
})

test_that("mmm_stats.tna_mmm returns confidence intervals containing estimate", {
  result <- mmm_stats(engagement_mmm)
  expect_true(all(result$ci_lower <= result$estimate))
  expect_true(all(result$ci_upper >= result$estimate))
})

test_that("mmm_stats.tna_mmm returns positive standard errors", {
  result <- mmm_stats(engagement_mmm)
  expect_true(all(result$std_error >= 0))
})

# Tests for mmm_stats_ internal helper function
test_that("mmm_stats_ computes statistics correctly", {
  # Create mock data
  cf <- matrix(
    c(0, 1, 2, 0, 0.5, 1.5),
    nrow = 2,
    ncol = 3,
    dimnames = list(c("(Intercept)", "x"), c("ref", "Cluster 2", "Cluster 3"))
  )
  vc <- diag(c(0.01, 0.04, 0.01, 0.04))

  result <- mmm_stats_(cf, vc, level = 0.05)

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 4) # 2 vars * 2 clusters (excluding ref)
  expect_named(
    result,
    c(
      "cluster", "variable", "estimate", "std_error",
      "ci_lower", "ci_upper", "z_value", "p_value"
    )
  )
})

test_that("mmm_stats_ computes z-values correctly", {
  cf <- matrix(
    c(0, 0, 0, 2),
    nrow = 2,
    ncol = 2,
    dimnames = list(c("(Intercept)", "x"), c("ref", "Cluster 2"))
  )
  vc <- diag(c(0.25, 0.25))

  result <- mmm_stats_(cf, vc, level = 0.05)

  # z = estimate / std_error = 2 / 0.5 = 4
  expect_equal(result$z_value[2], 4)
})

test_that("mmm_stats_ computes p-values correctly", {
  cf <- matrix(
    c(0, 0, 0, 0),
    nrow = 2,
    ncol = 2,
    dimnames = list(c("(Intercept)", "x"), c("ref", "Cluster 2"))
  )
  vc <- diag(c(1, 1))

  result <- mmm_stats_(cf, vc, level = 0.05)

  # When z = 0, p-value should be 1 (two-tailed)
  expect_equal(result$p_value, c(1, 1))
})

test_that("mmm_stats_ computes confidence intervals correctly", {
  cf <- matrix(
    c(0, 0, 0, 1),
    nrow = 2,
    ncol = 2,
    dimnames = list(c("(Intercept)", "x"), c("ref", "Cluster 2"))
  )
  vc <- diag(c(1, 1))

  result <- mmm_stats_(cf, vc, level = 0.05)

  # 95% CI: estimate +/- 1.96 * se
  # For estimate = 1, se = 1: CI = [1 - 1.96, 1 + 1.96]
  expected_margin <- qnorm(0.975) * 1
  expect_equal(result$ci_lower[2], 1 - expected_margin)
  expect_equal(result$ci_upper[2], 1 + expected_margin)
})

test_that("mmm_stats_ handles different level values", {
  cf <- matrix(
    c(0, 0, 0, 1),
    nrow = 2,
    ncol = 2,
    dimnames = list(c("(Intercept)", "x"), c("ref", "Cluster 2"))
  )
  vc <- diag(c(1, 1))

  result_90 <- mmm_stats_(cf, vc, level = 0.10)
  result_99 <- mmm_stats_(cf, vc, level = 0.01)

  # 90% CI should be narrower than 99% CI
  width_90 <- result_90$ci_upper[2] - result_90$ci_lower[2]
  width_99 <- result_99$ci_upper[2] - result_99$ci_lower[2]
  expect_true(width_90 < width_99)
})

test_that("mmm_stats_ preserves cluster and variable names", {
  cf <- matrix(
    c(0, 0, 0, 1, 2, 3),
    nrow = 3,
    ncol = 2,
    dimnames = list(c("(Intercept)", "age", "gender"), c("ref", "Active"))
  )
  vc <- diag(rep(1, 3))

  result <- mmm_stats_(cf, vc, level = 0.05)

  expect_equal(unique(result$cluster), "Active")
  expect_equal(result$variable, c("(Intercept)", "age", "gender"))
})

test_that("mmm_stats_ handles multiple clusters", {
  cf <- matrix(
    c(0, 0, 1, 2, 3, 4),
    nrow = 2,
    ncol = 3,
    dimnames = list(c("(Intercept)", "x"), c("ref", "Cluster A", "Cluster B"))
  )
  vc <- diag(rep(1, 4))

  result <- mmm_stats_(cf, vc, level = 0.05)

  expect_equal(nrow(result), 4) # 2 vars * 2 clusters
  expect_equal(result$cluster, c("Cluster A", "Cluster A", "Cluster B", "Cluster B"))
  expect_equal(result$variable, c("(Intercept)", "x", "(Intercept)", "x"))
})

test_that("mmm_stats_ returns default row names", {
  cf <- matrix(
    c(0, 0, 0, 1),
    nrow = 2,
    ncol = 2,
    dimnames = list(c("(Intercept)", "x"), c("ref", "Cluster 2"))
  )
  vc <- diag(c(1, 1))

  result <- mmm_stats_(cf, vc, level = 0.05)

  # Row names are reset to default sequential integers
  expect_equal(rownames(result), as.character(seq_len(nrow(result))))
})

# Edge case tests
test_that("mmm_stats.tna_mmm works with edge level values", {
  # Level close to 0 (very wide CI)
  result_wide <- mmm_stats(engagement_mmm, level = 0.001)
  expect_s3_class(result_wide, "data.frame")

  # Level close to 1 (very narrow CI)
  result_narrow <- mmm_stats(engagement_mmm, level = 0.999)
  expect_s3_class(result_narrow, "data.frame")
})

#' Build a Grouped Transition Network Analysis Model
#'
#' This function constructs a transition network analysis (TNA) model for
#' each group from a given sequence, wide-format dataframe
#' or a mixture Markov model.
#'
#' @export
#' @family clusters
#' @rdname group_model
#' @param x An `stslist` object describing a sequence of events or states to
#'   be used for building the Markov model. The argument `x` also accepts
#'   `data.frame` objects in wide format, and `tna_data` objects.
#'   This can also be the output of clustering from
#'   [cluster_sequences()].
#' @param group A `vector` indicating the group assignment of each
#'   row of the data/sequence. Must have the same length as the number of
#'   rows/sequences of `x`. Alternatively, a single `character` string giving
#'   the column name of the data that defines the group when `x` is a wide
#'   format `data.frame` or a `tna_data` object. If not provided, each row of
#'   the data forms a cluster. Not used when `x` is a mixture Markov model
#'   or a clustering result.
#' @param cols An `expression` giving a tidy selection of the
#'   columns that should be considered as sequence data.
#'   The default is all columns. The columns are
#'   automatically determined for `tna_data` objects. The `group` column
#'   is automatically removed from these columns if provided.
#' @param na.rm A `logical` value that determines if observations with `NA`
#'   value in `group` be removed. If `FALSE`, an additional category for `NA`
#'   values will be added. The default is `FALSE` and a warning is issued
#'   if `NA` values are detected.
#' @param groupwise A `logical` value that indicates whether scaling methods
#'   should be applied by group (`TRUE`) or globally (`FALSE`, the default).
#' @inheritParams build_model
#' @param ... Ignored.
#' @return An object of class `group_tna` which is a `list` containing one
#'   element per cluster. Each element is a `tna` object.
#'
#' @examples
#' # Manually specified groups
#' group <- c(rep("High", 1000), rep("Low", 1000))
#' model <- group_model(group_regulation, group = group)
#'
#' # Groups defined by a mixed Markov model
#' model <- group_model(engagement_mmm)
#'
group_model <- function(x, ...) {
  UseMethod("group_model")
}

#' @export
#' @rdname group_model
group_model.default <- function(x, group, type = "relative",
                                scaling = character(0L), groupwise = FALSE,
                                cols = tidyselect::everything(),
                                params = list(), concat = 1L,
                                na.rm = TRUE, ...) {
  check_missing(x)
  check_missing(group)
  check_flag(groupwise)
  check_flag(na.rm)
  stopifnot_(
    inherits(x, c("stslist", "data.frame", "tna_data", "tna_mmm")),
    "Argument {.arg x} must be {.cls stslist} (sequence object), a
    {.cls data.frame}, a {.cls tna_data} object."
  )
  if (inherits(x, "tna_data")) {
    wide <- cbind(x$sequence_data, x$meta_data)
    cols <- names(x$sequence_data)
    x <- wide
  } else {
    cols <- get_cols(rlang::enquo(cols), x)
  }
  type <- check_model_type(type)
  scaling <- check_model_scaling(scaling)
  n_group <- length(group)
  data <- NULL
  stopifnot_(
    n_group == nrow(x) || n_group == 1L,
    "Argument {.arg group} must be of length one or the same length as the
     number of rows/sequences in {.arg x}."
  )
  label <- "Cluster"
  prefix <- "Argument"
  if (n_group == 1L) {
    x_names <- names(x)
    stopifnot_(
      group %in% x_names,
      "Argument {.arg group} must be a column name of {.arg x}
       when of length one."
    )
    label <- group
    cols <- setdiff(cols, group)
    group <- as.factor(x[[group]])
    prefix <- "Column"
  }
  group_na <- any(is.na(group))
  if (group_na && na.rm) {
    warning_(
      c(
        "{prefix} {.arg group} contains missing values.",
        `i` = "The corresponding observations will be excluded. You can
               use {.code na.rm = FALSE} to keep missing values."
      )
    )
  }
  if (!is.factor(group)) {
    labels <- ifelse_(
      is.character(group),
      unique(group),
      paste0("Group ", seq_len(n_unique(group[!is.na(group)])))
    )
    group <- factor(group, labels = labels)
  }
  group <- ifelse_(group_na && !na.rm, addNA(group), group)
  levs <- levels(group)
  n_group <- length(levs)
  clusters <- stats::setNames(
    vector(mode = "list", length = n_group),
    levs
  )
  group_scaling <- ifelse_(groupwise, scaling, character(0L))
  groups <- vector(mode = "list", length = n_group)
  group <- as.integer(group)
  # TODO remove workaround if TraMineR is fixed
  vals <- ifelse_(
    inherits(x, "stslist"),
    sort(unique(unlist(x[, which(names(x) %in% cols)]))),
    vals <- sort(unique(unlist(x[, cols])))
  )
  alphabet <- ifelse_(
    inherits(x, "stslist"),
    attr(x, "alphabet"),
    vals[!is.na(vals)]
  )
  a <- length(alphabet)
  seq_data <- create_seqdata(
    x = x,
    cols = cols,
    alphabet = alphabet,
    concat = concat
  )
  trans <- compute_transitions(seq_data, a, type, params)
  for (i in seq_along(levs)) {
    groups[[i]] <- rep(i, sum(group == i, na.rm = TRUE))
    rows <- which(group == i)
    d <- seq_data[rows, , drop = FALSE]
    attr(d, "alphabet") <- alphabet
    attr(d, "labels") <- alphabet
    attr(d, "colors") <- attr(seq_data, "colors")
    inits <- factor(d[, 1L], levels = seq_len(a), labels = alphabet)
    inits <- as.vector(table(inits))
    weights <- compute_weights(
      trans[rows, , , drop = FALSE],
      type = type,
      scaling = group_scaling,
      a = a
    )
    dimnames(weights) <- list(alphabet, alphabet)
    clusters[[i]] <- build_model_(
      weights = weights,
      inits = inits / sum(inits),
      labels = alphabet,
      type = type,
      scaling = group_scaling,
      data = d,
      params = params
    )
  }
  names(groups) <- names(clusters)
  if (!groupwise && length(scaling > 0)) {
    weights <- scale_weights_global(
      weights = lapply(clusters, "[[", "weights"),
      type = type,
      scaling = scaling,
      a = a
    )
    for (i in seq_along(clusters)) {
      clusters[[i]]$weights <- weights[[i]]
    }
  }
  structure(
    clusters,
    groups = groups,
    label = label,
    levels = levs,
    na.rm = na.rm,
    cols = cols,
    groupwise = groupwise,
    type = type,
    scaling = scaling,
    class = "group_tna"
  )
}

#' @export
#' @rdname group_model
group_model.tna_mmm <- function(x, type = "relative", scaling = character(0L),
                                groupwise = FALSE, params = list(),
                                na.rm = TRUE, ...) {
  check_missing(x)
  check_class(x, "tna_mmm")
  group_model.default(
    x = x$observations,
    group = x$most_probable_cluster,
    type = type,
    scaling = scaling,
    groupwise = groupwise,
    params = params,
    na.rm = na.rm,
    ...
  )
}

#' @export
#' @rdname group_model
group_model.mhmm <- function(x, type = "relative", scaling = character(0L),
                             groupwise = FALSE, params = list(),
                             na.rm = TRUE, ...) {
  stopifnot_(
    requireNamespace("seqHMM", quietly = TRUE),
    "Please install the {.pkg seqHMM} package."
  )
  check_missing(x)
  check_class(x, "mhmm")
  group_model.default(
    x = x$observations,
    group = summary(x)$most_probable_cluster,
    type = type,
    scaling = scaling,
    groupwise = groupwise,
    params = params,
    na.rm = na.rm,
    ...
  )
}

#' @export
#' @rdname group_model
group_model.tna_clustering <- function(x, type = "relative",
                                       scaling = character(0L),
                                       groupwise = FALSE, params = list(),
                                       na.rm = TRUE, ...) {
  check_missing(x)
  check_class(x, "tna_clustering")
  group_model.default(
    x = x$data,
    group = x$assignment,
    type = type,
    scaling = scaling,
    groupwise = groupwise,
    params = params,
    na.rm = na.rm,
    ...
  )
}

#' @export
#' @rdname group_model
#' @examples
#' model <- group_tna(group_regulation, group = gl(2, 1000))
#'
group_tna <- function(x, ...) {
  check_missing(x)
  group_model(x = x, type = "relative", ...)
}

#' @export
#' @rdname group_model
#' @examples
#' model <- group_ftna(group_regulation, group = gl(2, 1000))
#'
group_ftna <- function(x, ...) {
  group_model(x = x, type = "frequency", ...)
}

#' @export
#' @rdname group_model
#' @examples
#' model <- group_ctna(group_regulation, group = gl(2, 1000))
#'
group_ctna <- function(x, ...) {
  group_model(x = x, type = "co-occurrence", ...)
}

#' @export
#' @rdname group_model
#' @examples
#' model <- group_atna(group_regulation, group = gl(2, 1000))
#'
group_atna <- function(x, ...) {
  group_model(x = x, type = "attention", ...)
}

#' Rename Groups
#'
#' @export
#' @family clusters
#' @param x A `group_tna` object.
#' @param new_names A `character` vector containing one name per cluster.
#' @return A renamed `group_tna` object.
#' @examples
#' model <- group_model(engagement_mmm)
#' model_renamed <- rename_groups(model, c("A", "B", "C"))
#'
rename_groups <- function(x, new_names) {
  check_missing(x)
  check_missing(new_names)
  check_class(x, "group_tna")
  stopifnot_(
    is.character(new_names),
    "Argument {.arg new_names} must be a {.cls character} vector."
  )
  stopifnot_(
    length(new_names) == length(x),
    "Argument {.arg new_names} must be the same length as {.arg x}"
  )
  names(x) <- new_names
  x
}

#' Scale Transition Network Weights
#'
#' @param weights A `list` of edge weights matrices
#' @param type Type of the transition network as a `character` string.
#' @param scaling Scaling methods to apply as a `character` vector.
#' @param a An `integer`, the number of states.
#' @noRd
scale_weights_global <- function(weights, type, scaling, a) {
  g <- length(weights)
  if (type == "relative") {
    for (i in seq_len(g)) {
      w <- weights[[i]]
      rs <- .rowSums(w, m = a, n = a)
      pos <- which(rs > 0)
      w[pos, ] <- w[pos, ] / rs[pos]
      w[!pos, ] <- NA
      weights[[i]] <- w
    }
  }
  weights_vec <- unlist(weights)
  for (i in seq_along(scaling)) {
    if (scaling[i] == "minmax") {
      weights_vec[] <- ranger(weights_vec)
    } else if (scaling[i] == "max") {
      weights_vec[] <- weights_vec / max(weights_vec, na.rm = TRUE)
    } else if (scaling[i] == "rank") {
      weights_vec[] <- rank(weights_vec, ties.method = "average")
    }
  }
  idx <- seq_len(a^2)
  for (i in seq_len(g)) {
    weights[[i]] <- matrix(weights_vec[idx], nrow = a, ncol = a)
    idx <- idx + a^2
  }
  weights
}

#' Combine data from clusters into a single dataset
#'
#' @param x A `group_tna` object.
#' @noRd
combine_data <- function(x) {
  cols <- attr(x, "cols")
  groups <- attr(x, "groups")
  data <- dplyr::bind_rows(
    lapply(x, function(y) as.data.frame(y$data))
  )
  data$.group <- attr(x, "levels")[unlist(groups)]
  data
}

test_that("group_model returns correct type", {
  expect_true(
    inherits(
      group_model(mock_sequence, group = c(1, 1, 2, 2, 2)),
      "group_tna"
    )
  )
})

test_that("group_ftna returns correct type", {
  expect_error(
    ftna_model <- group_ftna(mock_sequence, group = c(1, 1, 2, 2, 2)),
    NA
  )
  expect_equal(
    attr(ftna_model[[1]], "type"),
    "frequency"
  )
})

test_that("group_ctna returns correct type", {
  expect_error(
    ctna_model <- group_ctna(mock_sequence, group = c(1, 1, 2, 2, 2)),
    NA
  )
  expect_equal(
    attr(ctna_model[[1]], "type"),
    "co-occurrence"
  )
})

test_that("group_atna returns correct type", {
  expect_error(
    atna_model <- group_atna(mock_sequence, group = c(1, 1, 2, 2, 2)),
    NA
  )
  expect_equal(
    attr(atna_model[[1]], "type"),
    "attention"
  )
})

test_that("group_model returns correct type", {
  expect_true(
    inherits(
      group_model(mock_sequence, group = c(1, 1, 2, 2, 2))[[1]],
      "tna"
    )
  )
})

test_that("groups can be renamed", {
  expect_error(
    model <- rename_groups(mock_group_tna, c("A", "B")),
    NA
  )
  expect_equal(model$A, model[[1]])
  expect_equal(model$B, model[[2]])
})

test_that("group model can be summarized", {
  expect_error(
    summary(mmm_model),
    NA
  )
  expect_error(
    summary(mmm_model, combined = FALSE),
    NA
  )
})

test_that("grouped model can be constructed from tna_data objects", {
  data <- tibble::tibble(
    user = c("A", "A", "A", "B", "B", "C", "C", "C"),
    time = c(
      "2023-01-01 10:00:00", "2023-01-01 10:05:00",
      "2023-01-01 10:20:00", "2023-01-01 12:00:00",
      "2023-01-01 12:02:00", "2023-01-01 14:00:00",
      "2023-01-01 14:05:00", "2023-01-01 14:10:00"
    ),
    action = c(
      "view", "click", "add_cart", "view",
      "checkout", "view", "click", "share"
    ),
    group = c(rep("Group 1", 5), rep("Group 2", 3))
  )
  rlang::local_options(rlib_message_verbosity = "quiet")
  results <- prepare_data(
    data, actor = "user", time = "time", action = "action"
  )
  expect_error(
    model <- group_model(results, group = "group"),
    NA
  )
  expect_true(
    length(model) == 2L
  )
  expect_true(
    all(names(model) == c("Group 1", "Group 2"))
  )
})

test_that("missing values in group variable warns", {
  mock_tna_data_mis <- mock_tna_data
  mock_tna_data_mis$meta_data$group[4] <- NA
  expect_warning(
    model <- group_model(mock_tna_data_mis, group = "group"),
    "Column `group` contains missing values\\."
  )
})

test_that("groupwise models can be constructed with scaling", {
  expect_error(
    group_model(engagement_mmm, scaling = c("minmax", "rank", "max")),
    NA
  )
})

test_that("group model can be constructed from a clustering result", {
  clust <- cluster_sequences(engagement, k = 3)
  expect_error(
    group_model(clust),
    NA
  )
})

#' Map community assignments to a color palette.
#'
#' This function takes a vector of community assignments
#' (numeric or categorical) and maps them to corresponding colors
#' from a provided palette. If all values in the input vector are the same,
#' the function maps all of them to the first
#' color in the palette. Otherwise, it normalizes the values to ensure they
#' span across the entire palette.
#'
#' @param x A `numeric` vector representing the community assignments.
#' @param palette A `character` vector of colors to be used for mapping.
#' The length of  the palette defines the range of possible colors.
#'
#' @return A vector of colors corresponding to the input values, either
#' a single color or a gradient of colors based on the values in
#' the input vector.
#'
#' @details
#' - If the input vector `x` contains only one unique value,
#'   all elements will be mapped to the first color in the palette.
#' - If the input vector `x` contains multiple unique values,
#'   these values are scaled linearly to cover the entire range of
#'   the provided palette, ensuring that higher values in
#'   the input correspond to later colors in the palette.
#' - The normalization is performed using min-max scaling.
#'
#' @examples
#' # Example usage with numeric input
#' x <- c(1, 2, 3, 4, 5)
#' palette <- c("red", "green", "blue", "yellow", "purple")
#' colors <- map_to_color(x, palette)
#'
#' # Example with a single unique value
#' x <- c(1, 1, 1)
#' colors <- map_to_color(x, palette)
#'
#' @noRd
map_to_color <- function(x, palette) {
  if (n_unique(x) == 1) {
    return(rep(palette[1L], length(x)))
  }
  scaled_values <- as.integer(ranger(x) * (length(palette) - 1L)) + 1L
  palette[scaled_values]
}

#' Colors for Sequence Data
#'
#' This function creates colors analogous to those used by `TraMineR`.
#'
#' @param n_states The number of states as an `integer`.
#' @noRd
color_palette <- function(n_states) {
  color_group <- 4L -
    1L * (n_states <= 2) -
    1L * (n_states <= 8) -
    1L * (n_states <= 12)
  switch(color_group,
    RColorBrewer::brewer.pal(n = 3, name = "Accent")[seq_len(n_states)],
    RColorBrewer::brewer.pal(n = n_states, name = "Accent"),
    RColorBrewer::brewer.pal(n = n_states, name = "Set3"),
    colorspace::qualitative_hcl(n = n_states, palette = "Set 3")
  )
}

# Default Colors -------------------------------------------------------------

default_colors <- c(
  "#d1ea2c",
  "#fd5306",
  "#68b033",
  "#8601b0",
  "#fe2712",
  "#a7184d",
  "#3c02a6",
  "#fd9a01",
  "#0392ce"
)

test_that("color palette can be constructed", {
  for (n in 2:20) {
    expect_error(
      color_palette(n),
      NA
    )
  }
})

test_that("community assignments can be mapped to colors", {
  expect_equal(
    map_to_color(c(1), default_colors),
    default_colors[1]
  )
  expect_equal(
    map_to_color(c(1, 2, 3), default_colors),
    default_colors[c(1, 5, 9)]
  )
  expect_equal(
    map_to_color(c(10, 25, 100), default_colors),
    default_colors[c(1, 2, 9)]
  )
})

#' Simulate Method (Re-export of [stats::simulate])
#'
#' The [stats::simulate()] generic is re-exported so that the
#' `simulate.tna()` and `simulate.group_tna()` methods provided by this
#' package are dispatched correctly when the user only loads `tna`.
#' See the method-specific help pages for argument and return details.
#'
#' @name simulate
#' @rdname simulate
#' @importFrom stats simulate
#' @param object A statistical model object. The `tna` package provides
#'   methods for objects of class `tna` and `group_tna`.
#' @param nsim An `integer` giving the number of sequences to simulate.
#' @param seed An `integer` random seed for reproducibility, or `NULL`.
#' @param ... Further arguments passed to the dispatched method.
#' @return The value returned by the dispatched method (a `data.frame` of
#'   simulated sequences for `tna` and `group_tna` objects). See the
#'   method-specific help pages for details.
#' @examples
#' model <- tna(group_regulation)
#' sim <- simulate(model, nsim = 5, max_len = 10)
#' @export
stats::simulate

#' Simulate Data from a Transition Network Analysis Model
#'
#' @export
#' @family data
#' @inheritParams stats::simulate
#' @param object A `tna` object. The edge weights must be transition
#'   probabilities or frequencies, i.e., the model must have 
#'   `type = "relative"` or `type = "frequency"`.
#' @param nsim An `integer` giving the number of sequences to simulate.
#'   The default is 1.
#' @param max_len An `integer` giving the maximum length of the simulated
#'   sequences. When no missing values are generated, this is the length of
#'   all simulated sequences.
#' @param na_range An `integer` vector of length 2 giving the minimum and
#'   maximum number of missing values to generate for each sequence. The number
#'   of missing values is drawn uniformly from this range. If both values are
#'   zero (the default), no missing values are generated.
#' @param format A `character` string indicating whether the data should be
#'   returned in `wide` or `long` format.
#' @param zero_row A `character` string describing how to process zero rows
#'   in the weight matrix. The option `"self"` (the default) assigns
#'   probability 1 to the corresponding state (self loop) and option
#'   `"uniform"` assigns a uniform distribution.
#' @param ... Ignored.
#' @return A `data.frame` of the simulated sequence data.
#' @examples
#' model <- tna(group_regulation)
#' sim <- simulate(model, nsim = 10, max_len = 10)
#'
simulate.tna <- function(object, nsim = 1, seed = NULL, max_len = 100L,
                         na_range = c(0L, 0L), zero_row = "self", 
                         format = "wide", ...) {
  check_missing(object)
  check_class(object, "tna")
  type <- attr(object, "type")
  stopifnot_(
    type %in% c("relative", "frequency"),
    "Simulation is only supported for models with transition probabilities 
    ('relative') or transition frequencies ('frequency')."
  )
  check_values(nsim, strict = TRUE)
  check_values(max_len, strict = TRUE)
  zero_row <- check_match(zero_row, c("self", "uniform"))
  zero_row <- check_match(format, c("wide", "long"))
  na_range <- as.integer(na_range)
  stopifnot_(
    na_range[1L] >= 0L &&
      na_range[2L] >= na_range[1L] &&
      na_range[2L] < max_len,
    c(
      "Invalid {.arg na_range} values.",
      `x` = "The maximum number of missing values must be greater or equal
      to the minimum number of missing values. The maximum must be less than
      {.arg max_len}."
    )
  )
  if (!is.null(seed)) set.seed(seed)
  labels <- object$labels
  n <- nodes(object)
  nu <- nsim * n
  init <- object$inits %||% rep(1.0 / n, n)
  prob <- object$weights
  stopifnot_(
    is.matrix(prob) && nrow(prob) == n && ncol(prob) == n,
    "Invalid transition matrix dimensions."
  )
  if (type == "frequency") {
    prob <- freq_to_prob(prob, zero_row = zero_row)
  }
  out <- matrix(0L, nrow = nsim, ncol = max_len)
  out[, 1L] <- sample(seq_len(n), size = nsim, replace = TRUE, prob = init)
  for (i in seq(2L, max_len)) {
    # Gumbel softmax trick
    log_prob <- log(prob[out[, i - 1L], , drop = FALSE])
    out[, i] <- max.col(log_prob - log(-log(stats::runif(nu))))
  }
  out <- apply(out, 2L, function(y) labels[y], simplify = FALSE)
  out <- as.data.frame(out, col.names = paste0("T", seq_len(max_len)))
  if (na_range[2L] > 0L) {
    max_seq <- seq(na_range[1L], na_range[2L])
    nas <- sample(max_seq, size = nsim, replace = TRUE)
    seqs <- vector(mode = "list", length = na_range[2])
    seqs[[na_range[2L]]] <- seq(max_len - na_range[2L] + 1L, max_len)
    for (i in seq(na_range[2L] - 1L, na_range[1L])) {
      seqs[[i]] <- seqs[[i + 1]][-1L]
    }
    for (i in max_seq) {
      idx <- which(nas == i)
      out[idx, seqs[[i]]] <- NA
    }
  }
  ifelse_(
    format == "long",
    as_long_sim(out),
    out
  )
}

#' Simulate Data from a Group Transition Network Analysis Model
#'
#' @export
#' @family data
#' @param object A `group_tna` object. The edge weights must be transition
#'   probabilities or frequencies, i.e., the model must have 
#'   `type = "relative"` or `type = "frequency"`.
#' @param nsim An `integer` vector giving the number of sequences to simulate 
#'   per group. If a single integer is provided, the same number of sequences
#'   is generated per each group. The default is 1.
#' @param max_len An `integer` vector giving the maximum length of the simulated
#'   sequences per group. When no missing values are generated, this is the 
#'   length of all simulated sequences. If a single integer is provided, the 
#'   maximum length is the same for each group.
#' @inheritParams simulate.tna
#' @return A `data.frame` of the simulated sequence data.
#' @examples
#' model <- group_tna(
#'   group_regulation,
#'   group = rep(c("High", "Low"), each = 1000)
#' )
#' sim <- simulate(model, nsim = 10, max_len = 10)
#'
simulate.group_tna <- function(object, nsim = 1, seed = NULL,
                               max_len = 100L, na_range = c(0L, 0L),
                               zero_row = "self", format = "wide", ...) {
  check_missing(object)
  check_class(object, "group_tna")
  zero_row <- check_match(zero_row, c("self", "uniform"))
  G <- length(object)
  if (length(nsim) == 1L) {
    nsim <- rep(as.integer(nsim), G)
  } else {
    nsim <- as.integer(nsim)
    stopifnot_(
      length(nsim) == G,
      "{.arg nsim} must be length 1 or the same length as {.arg object}."
    )
  }
  if (length(max_len) == 1L) {
    max_len <- rep(as.integer(max_len), G)
  } else {
    max_len <- as.integer(max_len)
    stopifnot_(
      length(max_len) == G,
      "{.arg max_len} must be length 1 or the same length as {.arg object}."
    )
  }
  stopifnot_(all(max_len >= 1L), "All {.arg max_len} values must be >= 1.")
  grp_names <- names(object)
  if (is.null(grp_names)) {
    grp_names <- as.character(seq_along(object))
  }
  out_list <- vector("list", G)
  for (g in seq_len(G)) {
    mod <- object[[g]]
    out_g <- simulate.tna(
      object = mod,
      nsim = nsim[g],
      seed = seed,
      max_len = max_len[g],
      na_range = na_range,
      zero_row = zero_row,
      format = format,
      ...
    )
    if (format == "wide") {
      out_g <- cbind(group = grp_names[g], out_g)
    } else {
      out_g$group <- grp_names[g]
      out_g <- out_g[, c("group", setdiff(names(out_g), "group")), drop = FALSE]
    }
    out_list[[g]] <- out_g
  }
  if (format == "wide") {
    do.call(dplyr::bind_rows, out_list)
  } else {
    out <- do.call(base::rbind, out_list)
    rownames(out) <- NULL
    out
  }
}

freq_to_prob <- function(prob, zero_row) {
  stopifnot_(is.matrix(prob), "Transition weights must be a matrix.")
  stopifnot_(all(is.finite(prob)), "Non-finite values in transition matrix.")
  stopifnot_(all(prob >= 0), "Negative values in transition frequencies.")
  n <- nrow(prob)
  rs <- rowSums(prob)
  ok <- rs > 0
  if (any(ok)) {
    prob[ok, ] <- prob[ok, , drop = FALSE] / rs[ok]
  }
  if (any(!ok)) {
    if (zero_row == "uniform") {
      prob[!ok, ] <- 1 / n
    } else {
      prob[!ok, ] <- 0
      idx <- which(!ok)
      prob[cbind(idx, idx)] <- 1
    }
  }
  prob
}

as_long_sim <- function(wide_df) {
  tt <- names(wide_df)
  n_actor <- nrow(wide_df)
  n_time <- ncol(wide_df)
  id <- rep(seq_len(n_actor), each = n_time)
  time <- rep(seq_len(n_time), times = n_actor)
  state <- as.vector(t(as.matrix(wide_df)))
  out <- data.frame(
    id = id,
    time = time,
    state = state,
    stringsAsFactors = FALSE
  )
  out
}

test_that("transition networks can be simulated", {
  set.seed(0)
  # Special case with only one sequence
  expect_error(
    simulate(mock_tna, max_len = 10),
    NA
  )
  expect_error(
    simulate(mock_tna, nsim = 10, max_len = 10),
    NA
  )
})

test_that("missing values can be included", {
  set.seed(0)
  sim <- simulate(mock_tna, nsim = 100, max_len = 10, na_range = c(1, 3))
  nas <- apply(sim, 1L, function(y) sum(is.na(y)))
  expect_true(all(nas %in% c(1, 2, 3)))
})

test_that("transition matrix is recovered", {
  set.seed(0)
  sim <- simulate(mock_tna, nsim = 1e4, max_len = 100)
  tna_sim <- tna(sim)
  expect_equal(mock_tna$weights, tna_sim$weights, tolerance = 0.05)
})

test_that("transition networks can be simulated from `group_tna`", {
  set.seed(0)
  sim <- simulate(mock_group_tna, nsim = 100, max_len = 10, na_range = c(1, 3))
  nas <- apply(sim, 1L, function(y) sum(is.na(y)))
  expect_true(all(nas %in% c(1, 2, 3)))
})

test_that("simulate.tna returns expected shape for relative models", {
  set.seed(0)

  sim1 <- simulate(mock_tna, max_len = 10)
  expect_s3_class(sim1, "data.frame")
  expect_equal(dim(sim1), c(1L, 10L))
  expect_named(sim1, paste0("T", 1:10))

  sim10 <- simulate(mock_tna, nsim = 10, max_len = 10)
  expect_equal(dim(sim10), c(10L, 10L))
  expect_named(sim10, paste0("T", 1:10))

  # No missing values by default
  expect_true(all(!is.na(as.matrix(sim10))))
})

test_that("simulate.tna includes trailing missing values within na_range", {
  set.seed(0)

  sim <- simulate(mock_tna, nsim = 200, max_len = 10, na_range = c(1, 3))
  nas <- rowSums(is.na(sim))
  expect_true(all(nas %in% 1:3))

  # Missing values should be trailing only
  trailing_only <- apply(sim, 1L, function(y) {
    if (!any(is.na(y))) return(TRUE)
    first_na <- which(is.na(y))[1L]
    all(is.na(y[first_na:length(y)]))
  })
  expect_true(all(trailing_only))
})

test_that("simulate.tna approximately recovers the transition matrix (relative)", {
  set.seed(0)

  sim <- simulate(mock_tna, nsim = 2000, max_len = 200)
  tna_sim <- tna(sim)

  # A bit looser tolerance, but more stable across platforms/BLAS/R versions
  expect_equal(mock_tna$weights, tna_sim$weights, tolerance = 0.02)
})

test_that("simulate.tna works for frequency type by row-normalizing weights", {
  set.seed(0)

  mock_freq <- mock_tna
  attr(mock_freq, "type") <- "frequency"

  # Make sure weights are not already probabilities (simulate counts)
  mock_freq$weights <- round(mock_tna$weights * 100)

  sim <- simulate(mock_freq, nsim = 1000, max_len = 20)
  expect_equal(dim(sim), c(1000L, 20L))
  expect_named(sim, paste0("T", 1:20))

  # Recovered matrix should match the normalized version of the counts
  tna_sim <- tna(sim)
  target <- mock_freq$weights / rowSums(mock_freq$weights)
  expect_equal(target, tna_sim$weights, tolerance = 0.03)
})

test_that("simulate.group_tna supports scalar and vector nsim/max_len, wide default", {
  set.seed(0)

  # Scalar parameters (baseline)
  sim1 <- simulate(mock_group_tna, nsim = 10, max_len = 6, na_range = c(0, 0))
  expect_true("group" %in% names(sim1))
  expect_named(sim1, c("group", paste0("T", 1:6)))
  expect_equal(nrow(sim1), 10L * length(mock_group_tna))

  # Vector parameters imply different widths; wide output must fill with NA
  sim2 <- simulate(mock_group_tna, nsim = c(4, 8), max_len = c(4, 9), na_range = c(0, 0))
  expect_true("group" %in% names(sim2))
  expect_true(all(paste0("T", 1:9) %in% names(sim2)))
  expect_equal(nrow(sim2), 12L)  # 4 + 8

  # Rows from the shorter group should have NA beyond their max_len
  # Identify groups by name (or fallback to "1","2")
  grp_names <- names(mock_group_tna)
  if (is.null(grp_names)) grp_names <- as.character(seq_along(mock_group_tna))
  short_grp <- grp_names[1L]

  short_rows <- sim2[sim2$group == short_grp, , drop = FALSE]
  expect_true(all(is.na(short_rows[, paste0("T", 5:9), drop = FALSE])))
})

test_that("simulate.group_tna long format binds cleanly and has required columns", {
  set.seed(0)

  sim <- simulate(mock_group_tna, nsim = c(3, 5), max_len = c(4, 6), format = "long")
  expect_true(all(c("group", "id", "time", "state") %in% names(sim)))

  # t should be within [1, max_len] per group, actor should restart per group (simulate.tna default)
  expect_true(all(sim$time >= 1 & sim$time <= 6))
  #expect_true(all(grepl("^T\\d+$", sim$t_label)))
})

# Tests for freq_to_prob edge cases
test_that("freq_to_prob handles zero rows with uniform option", {
  # Create a 3x3 matrix with a zero row (row 2)
  prob <- matrix(c(1, 2, 3, 0, 0, 0, 4, 5, 6), nrow = 3, byrow = TRUE)
  result <- freq_to_prob(prob, zero_row = "uniform")

  # Zero row should have uniform distribution (1/3 each)
  expect_equal(result[2, ], rep(1/3, 3))
  # Non-zero rows should be normalized
  expect_equal(sum(result[1, ]), 1)
  expect_equal(sum(result[3, ]), 1)
})

test_that("freq_to_prob handles zero rows with self option", {
  # Create a 3x3 matrix with a zero row (row 2)
  prob <- matrix(c(1, 2, 3, 0, 0, 0, 4, 5, 6), nrow = 3, byrow = TRUE)
  result <- freq_to_prob(prob, zero_row = "self")

  # Zero row should have self-loop (1 on diagonal, 0 elsewhere)
  expect_equal(result[2, 1], 0)
  expect_equal(result[2, 2], 1)
  expect_equal(result[2, 3], 0)
})

test_that("freq_to_prob normalizes rows correctly", {
  # Row sums > 1 should be normalized
  prob <- matrix(c(10, 20, 30, 40), nrow = 2, byrow = TRUE)
  result <- freq_to_prob(prob, zero_row = "self")

  # Each row should sum to 1
  expect_equal(rowSums(result), c(1, 1))
  expect_equal(result[1, ], c(1/3, 2/3))
  expect_equal(result[2, ], c(3/7, 4/7))
})

test_that("simulate.group_tna handles unnamed groups", {
  set.seed(0)
  # Create group_tna without names
  unnamed_model <- mock_group_tna
  names(unnamed_model) <- NULL

  sim <- simulate(unnamed_model, nsim = 5, max_len = 5)
  expect_true("group" %in% names(sim))
  # Should use numeric group names
  expect_true(all(sim$group %in% c("1", "2")))
})

test_that("simulate.tna validates na_range correctly", {
  expect_error(
    simulate(mock_tna, nsim = 10, max_len = 10, na_range = c(5, 3)),
    "Invalid"
  )
  expect_error(
    simulate(mock_tna, nsim = 10, max_len = 10, na_range = c(-1, 5)),
    "Invalid"
  )
  expect_error(
    simulate(mock_tna, nsim = 10, max_len = 10, na_range = c(0, 15)),
    "Invalid"
  )
})

test_that("simulate.group_tna validates nsim length", {
  expect_error(
    simulate(mock_group_tna, nsim = c(1, 2, 3), max_len = 10),
    "must be length 1 or the same length"
  )
})

test_that("simulate.group_tna validates max_len length", {
  expect_error(
    simulate(mock_group_tna, nsim = 10, max_len = c(5, 10, 15)),
    "must be length 1 or the same length"
  )
})

test_that("simulate.tna rejects unsupported model types", {
  mock_co <- mock_tna
  attr(mock_co, "type") <- "co-occurrence"
  expect_error(
    simulate(mock_co, nsim = 10, max_len = 10),
    "Simulation is only supported"
  )
})

test_that("simulate.tna seed parameter produces reproducible results", {
  sim1 <- simulate(mock_tna, nsim = 5, max_len = 10, seed = 123)
  sim2 <- simulate(mock_tna, nsim = 5, max_len = 10, seed = 123)
  expect_equal(sim1, sim2)
})

#' Build a Random Transition Network Analysis Model
#'
#' @description
#' Construct a fully-functional `tna` object from synthetic parameters
#' without needing pre-existing sequence data. Random transition probabilities
#' are drawn from a Dirichlet distribution, sequences are simulated from the
#' resulting model, and a canonical `tna` object is fitted on those sequences.
#'
#' Calling `random_tna()` with no arguments returns a fresh, sticky network
#' with a coherent alphabet, drawn fresh on every call.
#'
#' @export
#' @family data
#' @param n_states An `integer` >= 2 giving the number of states. If `NULL`
#'   (the default), a value is drawn from `7:11` on each call.
#' @param states An optional `character` vector of state labels of length
#'   at least `n_states`. The first `n_states` are used. If `NULL` (the
#'   default), labels are taken from `category` or auto-picked from a
#'   built-in pool that fits `n_states`.
#' @param category An optional `character` string naming a built-in label
#'   pool. Available pools are returned by `list_random_state_pools()`.
#'   When `NULL` (the default), a pool whose size is at least `n_states`
#'   is sampled at random. Ignored when `states` is supplied.
#' @param alpha A positive `numeric` Dirichlet concentration parameter.
#'   Small values (e.g. `0.3`) produce sparse, peaked transitions; large
#'   values (e.g. `5`) produce near-uniform transitions. If `NULL` (the
#'   default), a value is drawn from `Uniform(0.5, 1.0)` on each call.
#' @param diag_boost A non-negative `numeric` added to the diagonal of the
#'   transition matrix before re-normalising rows. Larger values make
#'   states "stickier" (more self-transitions). If `NULL` (the default),
#'   a value is drawn from `Uniform(1.5, 3.0)` on each call.
#' @param trans_matrix An optional square `numeric` matrix of transition
#'   probabilities. When supplied, `alpha` and `diag_boost` are ignored
#'   for the transition matrix. Rows are renormalised to sum to one.
#' @param init_probs An optional `numeric` vector of initial state
#'   probabilities. Renormalised to sum to one. If `NULL`, drawn from a
#'   Dirichlet on the same alphabet.
#' @param n_sequences An `integer` giving the number of sequences to
#'   simulate from the true parameters. If `NULL` (the default), a value
#'   is drawn from `500:800` on each call.
#' @param seq_length An `integer` giving the length of each simulated
#'   sequence. If `NULL` (the default), a value is drawn from `6:20` on
#'   each call.
#' @param type A `character` string giving the model type. One of
#'   `"relative"` (the default), `"frequency"`, `"co-occurrence"`,
#'   `"attention"`.
#' @param return_params A `logical`. If `TRUE`, returns a `list` with the
#'   fitted model and the ground-truth parameters used to generate it.
#'   Default is `FALSE`.
#' @param seed An `integer` random seed for reproducibility, or `NULL`
#'   (the default) for fresh randomness on every call.
#' @return A `tna` object, or a `list` with elements `model`,
#'   `trans_matrix`, `init_probs`, `sequences`, `labels`, and `category`
#'   when `return_params = TRUE`.
#' @examples
#' # Fresh random model on every call
#' model <- random_tna()
#'
#' # Explicit small-state demo using the engagement pool
#' model <- random_tna(n_states = 3, category = "engagement")
#'
#' # Reproducible model
#' model <- random_tna(seed = 42)
#'
#' # Recover the ground-truth parameters
#' out <- random_tna(seed = 7, return_params = TRUE)
#' out$trans_matrix
#'
random_tna <- function(n_states = NULL, states = NULL, category = NULL,
                       alpha = NULL, diag_boost = NULL,
                       trans_matrix = NULL, init_probs = NULL,
                       n_sequences = NULL, seq_length = NULL,
                       type = "relative", return_params = FALSE,
                       seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  type <- check_match(type, c("relative", "frequency",
                              "co-occurrence", "attention"))
  n_states <- random_default_n_states(n_states)
  alpha <- random_default_alpha(alpha)
  diag_boost <- random_default_diag_boost(diag_boost)
  n_sequences <- random_default_n_sequences(n_sequences)
  seq_length <- random_default_seq_length(seq_length)
  picked <- random_resolve_states(n_states, states, category)
  labels <- picked$labels
  n <- length(labels)
  trans_matrix <- random_resolve_trans(trans_matrix, n, alpha, diag_boost)
  init_probs <- random_resolve_inits(init_probs, n, alpha)
  dimnames(trans_matrix) <- list(labels, labels)
  names(init_probs) <- labels

  true_model <- build_model_(
    weights = trans_matrix,
    inits = init_probs,
    labels = labels,
    type = "relative",
    scaling = character(0L),
    params = NULL
  )
  sim_data <- simulate.tna(
    true_model,
    nsim = n_sequences,
    max_len = seq_length,
    seed = NULL
  )
  fitter <- switch(type,
    "relative" = tna,
    "frequency" = ftna,
    "co-occurrence" = ctna,
    "attention" = atna
  )
  model <- fitter(sim_data)
  if (!return_params) {
    return(model)
  }
  list(
    model = model,
    trans_matrix = trans_matrix,
    init_probs = init_probs,
    sequences = sim_data,
    labels = labels,
    category = picked$category
  )
}

#' Build a Random Group Transition Network Analysis Model
#'
#' @description
#' Construct a fully-functional `group_tna` object from synthetic parameters.
#' Each group receives its own randomly drawn transition matrix and initial
#' probabilities over a shared alphabet, so groups have heterogeneous
#' dynamics by default. Per-group overrides allow custom group sizes,
#' sparsity, stickiness, or hand-supplied transition matrices.
#'
#' @export
#' @family data
#' @param n_groups An `integer` giving the number of groups. If `NULL`
#'   (the default), a value is drawn from `2:4` on each call.
#' @param group_names An optional `character` vector of group names of
#'   length `n_groups`. Defaults to `"Group 1"`, `"Group 2"`, ...
#' @param per_group An optional `list` of length `n_groups`. Each element is
#'   itself a `list` of overrides applied to that group only. Recognised
#'   override names: `alpha`, `diag_boost`, `trans_matrix`, `init_probs`,
#'   `n_sequences`, `seq_length`. Unset entries fall back to the top-level
#'   defaults (which may themselves be drawn at random when `NULL`).
#' @inheritParams random_tna
#' @return A `group_tna` object.
#' @examples
#' # Fresh random group model on every call
#' model <- random_group_tna()
#'
#' # Explicit two-group engagement demo with per-group differences
#' model <- random_group_tna(
#'   n_groups  = 2,
#'   n_states  = 3,
#'   category  = "engagement",
#'   per_group = list(
#'     list(n_sequences = 400, diag_boost = 3),
#'     list(n_sequences = 100, alpha = 0.3)
#'   ),
#'   seed = 42
#' )
#'
random_group_tna <- function(n_groups = NULL, group_names = NULL,
                             n_states = NULL, states = NULL, category = NULL,
                             alpha = NULL, diag_boost = NULL,
                             n_sequences = NULL, seq_length = NULL,
                             type = "relative", per_group = NULL,
                             seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  n_groups <- random_default_n_groups(n_groups)
  type <- check_match(type, c("relative", "frequency",
                              "co-occurrence", "attention"))
  if (is.null(group_names)) {
    group_names <- paste0("Group ", seq_len(n_groups))
  } else {
    stopifnot_(
      is.character(group_names) && length(group_names) == n_groups,
      "Argument {.arg group_names} must be a {.cls character} vector of length
      {.arg n_groups}."
    )
  }
  if (!is.null(per_group)) {
    stopifnot_(
      is.list(per_group) && length(per_group) == n_groups,
      "Argument {.arg per_group} must be a {.cls list} of length
      {.arg n_groups}."
    )
  }
  n_states <- random_default_n_states(n_states)
  picked <- random_resolve_states(n_states, states, category)
  labels <- picked$labels
  n <- length(labels)

  group_data <- vector("list", n_groups)
  for (g in seq_len(n_groups)) {
    ov <- if (is.null(per_group)) list() else per_group[[g]] %||% list()
    g_alpha <- random_default_alpha(ov$alpha %||% alpha)
    g_diag <- random_default_diag_boost(ov$diag_boost %||% diag_boost)
    g_nseq <- random_default_n_sequences(ov$n_sequences %||% n_sequences)
    g_slen <- random_default_seq_length(ov$seq_length %||% seq_length)
    g_trans <- random_resolve_trans(ov$trans_matrix, n, g_alpha, g_diag)
    g_inits <- random_resolve_inits(ov$init_probs, n, g_alpha)
    dimnames(g_trans) <- list(labels, labels)
    names(g_inits) <- labels
    true_model <- build_model_(
      weights = g_trans,
      inits = g_inits,
      labels = labels,
      type = "relative",
      scaling = character(0L),
      params = NULL
    )
    sim <- simulate.tna(
      true_model, nsim = g_nseq, max_len = g_slen, seed = NULL
    )
    sim[[".group"]] <- group_names[g]
    group_data[[g]] <- sim
  }
  combined <- do.call(dplyr::bind_rows, group_data)
  grp <- combined[[".group"]]
  combined[[".group"]] <- NULL
  group_fitter <- switch(type,
    "relative" = group_tna,
    "frequency" = group_ftna,
    "co-occurrence" = group_ctna,
    "attention" = group_atna
  )
  group_fitter(combined, group = grp)
}

#' Build a Random Mixture Markov Model Object
#'
#' @description
#' Construct a synthetic `tna_mmm` object that mirrors the *structure* a
#' fitted seqHMM mixed Markov model exposes to [tna] without depending on
#' seqHMM at runtime. The returned object can be passed to
#' [group_model()] (dispatching via `group_model.tna_mmm`) and to
#' [mmm_stats()] (dispatching via `mmm_stats.tna_mmm`).
#'
#' Real seqHMM `mhmm` objects continue to dispatch via the original
#' `*.mhmm` methods.
#'
#' @export
#' @family data
#' @param n_clusters An `integer` giving the number of mixture clusters.
#'   If `NULL` (the default), drawn from `2:4` on each call.
#' @param n_covariates An `integer` >= 1 giving the number of regression
#'   variables (including the intercept) used to predict cluster membership.
#'   Default is 1 (intercept only). When > 1, additional rows are added to
#'   the coefficient matrix.
#' @inheritParams random_tna
#' @return An object of class `tna_mmm` containing fields
#'   `observations`, `transition_probs`, `initial_probs`, `coefficients`,
#'   `vcov`, `most_probable_cluster`, `cluster_names`, `state_names`,
#'   `n_clusters`, `n_states`, `n_sequences`, `n_covariates`.
#' @examples
#' model <- random_tna_mmm(seed = 1)
#' mmm_stats(model)
#' grp <- group_model(model)
#'
random_tna_mmm <- function(n_clusters = NULL, n_states = NULL,
                           states = NULL, category = NULL,
                           alpha = NULL, diag_boost = NULL,
                           n_sequences = NULL, seq_length = NULL,
                           n_covariates = 1L, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  n_clusters <- random_default_n_groups(n_clusters)
  n_states <- random_default_n_states(n_states)
  alpha <- random_default_alpha(alpha)
  diag_boost <- random_default_diag_boost(diag_boost)
  n_sequences <- random_default_n_sequences(n_sequences)
  seq_length <- random_default_seq_length(seq_length)
  n_covariates <- as.integer(n_covariates)
  stopifnot_(
    length(n_covariates) == 1L && !is.na(n_covariates) && n_covariates >= 1L,
    "Argument {.arg n_covariates} must be a single positive integer."
  )

  picked <- random_resolve_states(n_states, states, category)
  state_names <- picked$labels
  cluster_names <- paste0("Cluster ", seq_len(n_clusters))

  per_cluster_share <- as.numeric(random_rdirichlet(1L, rep(2, n_clusters)))
  cluster_size <- as.integer(round(per_cluster_share * n_sequences))
  cluster_size[n_clusters] <- n_sequences - sum(cluster_size[-n_clusters])
  cluster_size[cluster_size < 1L] <- 1L

  transition_probs <- vector("list", n_clusters)
  initial_probs <- vector("list", n_clusters)
  observations_parts <- vector("list", n_clusters)
  for (k in seq_len(n_clusters)) {
    tm <- random_resolve_trans(NULL, n_states, alpha, diag_boost)
    ip <- random_resolve_inits(NULL, n_states, alpha)
    dimnames(tm) <- list(state_names, state_names)
    names(ip) <- state_names
    transition_probs[[k]] <- tm
    initial_probs[[k]] <- ip
    true_model <- build_model_(
      weights = tm, inits = ip, labels = state_names,
      type = "relative", scaling = character(0L), params = NULL
    )
    observations_parts[[k]] <- simulate.tna(
      true_model,
      nsim = cluster_size[k],
      max_len = seq_length,
      seed = NULL
    )
  }
  names(transition_probs) <- cluster_names
  names(initial_probs) <- cluster_names

  observations <- do.call(rbind, observations_parts)
  rownames(observations) <- NULL
  most_probable_cluster <- factor(
    rep(cluster_names, times = cluster_size),
    levels = cluster_names
  )

  variable_names <- if (n_covariates == 1L) {
    "(Intercept)"
  } else {
    c("(Intercept)", paste0("X", seq_len(n_covariates - 1L)))
  }
  coefficients <- matrix(
    stats::rnorm(n_covariates * n_clusters, mean = 0, sd = 1),
    nrow = n_covariates, ncol = n_clusters,
    dimnames = list(variable_names, cluster_names)
  )
  coefficients[, 1L] <- 0  # reference cluster fixed at zero

  vcov_dim <- n_covariates * (n_clusters - 1L)
  vcov_raw <- matrix(stats::rnorm(vcov_dim * vcov_dim), vcov_dim, vcov_dim)
  vcov <- crossprod(vcov_raw) / vcov_dim + diag(0.05, vcov_dim)
  vcov_labels <- paste0(
    rep(variable_names, n_clusters - 1L),
    ":",
    rep(cluster_names[-1L], each = n_covariates)
  )
  dimnames(vcov) <- list(vcov_labels, vcov_labels)

  structure(
    list(
      observations = observations,
      transition_probs = transition_probs,
      initial_probs = initial_probs,
      coefficients = coefficients,
      vcov = vcov,
      most_probable_cluster = most_probable_cluster,
      cluster_names = cluster_names,
      state_names = state_names,
      n_clusters = n_clusters,
      n_states = n_states,
      n_sequences = n_sequences,
      n_covariates = n_covariates
    ),
    class = "tna_mmm"
  )
}

#' List Built-in Label Pools for [random_tna()]
#'
#' @description
#' Return the names of the curated state-label pools available to
#' `random_tna()` and `random_group_tna()`, with their sizes.
#'
#' @export
#' @family data
#' @return A named `integer` vector mapping pool names to pool sizes.
#' @examples
#' list_random_state_pools()
#'
list_random_state_pools <- function() {
  vapply(random_state_pools, length, integer(1L))
}

# Internal --------------------------------------------------------------------

#' Built-in label pools used by [random_tna()].
#'
#' Two tiers:
#' \itemize{
#'   \item State-descriptor pools (3-5 labels): semantically ordered
#'     learning-state descriptors (engagement, motivation, ...).
#'   \item Action-verb pools (12 labels each): student learning actions
#'     by category (cognitive, metacognitive, ...).
#' }
#' @noRd
random_state_pools <- list(
  # State-descriptor pools (small, ordered)
  engagement   = c("Engaged", "Moderate", "Disengaged"),
  engagement5  = c("Highly Engaged", "Engaged", "Neutral", "Passive",
                   "Disengaged"),
  motivation   = c("Motivated", "Steady", "Demotivated"),
  motivation4  = c("Driven", "Motivated", "Hesitant", "Demotivated"),
  attention    = c("Focused", "Distracted", "Off task"),
  affect       = c("Positive", "Neutral", "Negative"),
  performance  = c("High", "Medium", "Low"),
  effort       = c("High effort", "Moderate effort", "Low effort"),
  # Action-verb pools (12 each)
  cognitive = c(
    "Read", "Study", "Analyze", "Summarize",
    "Apply", "Compare", "Synthesize", "Memorize",
    "Infer", "Interpret", "Recall", "Classify"
  ),
  metacognitive = c(
    "Plan", "Monitor", "Evaluate", "Reflect",
    "Adjust", "Strategize", "Assess", "Reconsider",
    "Set goals", "Track", "Calibrate", "Anticipate"
  ),
  behavioral = c(
    "Practice", "Annotate", "Research", "Review",
    "Revise", "Test", "Write", "Note",
    "Highlight", "Outline", "Draft", "Cite"
  ),
  social = c(
    "Discuss", "Collaborate", "Explain", "Share",
    "Help", "Teach", "Ask", "Listen",
    "Negotiate", "Critique", "Mentor", "Support"
  )
)

#' @noRd
random_default_n_states <- function(x) {
  if (is.null(x)) {
    return(sample(7:11, 1L))
  }
  x <- as.integer(x)
  stopifnot_(
    length(x) == 1L && !is.na(x) && x >= 2L,
    "Argument {.arg n_states} must be a single integer >= 2."
  )
  x
}

#' @noRd
random_default_n_groups <- function(x) {
  if (is.null(x)) {
    return(sample(2:4, 1L))
  }
  x <- as.integer(x)
  stopifnot_(
    length(x) == 1L && !is.na(x) && x >= 1L,
    "Argument {.arg n_groups} must be a single positive integer."
  )
  x
}

#' @noRd
random_default_n_sequences <- function(x) {
  if (is.null(x)) {
    return(sample(500:800, 1L))
  }
  x <- as.integer(x)
  stopifnot_(
    length(x) == 1L && !is.na(x) && x >= 1L,
    "Argument {.arg n_sequences} must be a single positive integer."
  )
  x
}

#' @noRd
random_default_seq_length <- function(x) {
  if (is.null(x)) {
    return(sample(6:20, 1L))
  }
  x <- as.integer(x)
  stopifnot_(
    length(x) == 1L && !is.na(x) && x >= 2L,
    "Argument {.arg seq_length} must be a single integer >= 2."
  )
  x
}

#' @noRd
random_default_alpha <- function(x) {
  if (is.null(x)) {
    return(stats::runif(1L, 0.5, 1.0))
  }
  stopifnot_(
    length(x) == 1L && is.finite(x) && x > 0,
    "Argument {.arg alpha} must be a single positive {.cls numeric}."
  )
  x
}

#' @noRd
random_default_diag_boost <- function(x) {
  if (is.null(x)) {
    return(stats::runif(1L, 1.5, 3.0))
  }
  stopifnot_(
    length(x) == 1L && is.finite(x) && x >= 0,
    "Argument {.arg diag_boost} must be a single non-negative {.cls numeric}."
  )
  x
}

#' Resolve the alphabet for a random TNA model.
#'
#' Returns a list with two elements:
#' \itemize{
#'   \item `labels`: the chosen `character` vector of length `n_states`.
#'   \item `category`: the pool name actually used, or `NA` if not from a
#'     pool (user-supplied `states` or fallback to `LETTERS`).
#' }
#' @noRd
random_resolve_states <- function(n_states, states, category) {
  if (!is.null(states)) {
    stopifnot_(
      is.character(states) && length(states) >= n_states,
      "Argument {.arg states} must be a {.cls character} vector of length
      at least {.arg n_states}."
    )
    return(list(labels = states[seq_len(n_states)], category = NA_character_))
  }
  if (!is.null(category)) {
    pool <- random_state_pools[[category]]
    stopifnot_(
      !is.null(pool),
      "Unknown {.arg category}. Available: {.val {names(random_state_pools)}}."
    )
    stopifnot_(
      length(pool) >= n_states,
      "Pool {.val {category}} only has {length(pool)} labels; reduce
      {.arg n_states} or supply {.arg states}."
    )
    return(list(labels = pool[seq_len(n_states)], category = category))
  }
  pool_sizes <- vapply(random_state_pools, length, integer(1L))
  is_descriptor <- pool_sizes <= 5L
  preferred <- if (n_states <= 5L) is_descriptor else !is_descriptor
  candidates <- names(pool_sizes)[preferred & pool_sizes >= n_states]
  if (length(candidates) == 0L) {
    candidates <- names(pool_sizes)[pool_sizes >= n_states]
  }
  if (length(candidates) > 0L) {
    chosen <- sample(candidates, 1L)
    pool <- random_state_pools[[chosen]]
    return(list(
      labels = sample(pool, n_states),
      category = chosen
    ))
  }
  fallback <- if (n_states <= length(LETTERS)) {
    LETTERS[seq_len(n_states)]
  } else {
    paste0("S", seq_len(n_states))
  }
  list(labels = fallback, category = NA_character_)
}

#' @noRd
random_resolve_trans <- function(trans_matrix, n, alpha, diag_boost) {
  if (!is.null(trans_matrix)) {
    stopifnot_(
      is.matrix(trans_matrix) && nrow(trans_matrix) == n &&
        ncol(trans_matrix) == n,
      "Argument {.arg trans_matrix} must be a {.cls matrix} of dimension
      {n} x {n}."
    )
    stopifnot_(
      all(is.finite(trans_matrix)) && all(trans_matrix >= 0),
      "Argument {.arg trans_matrix} must contain non-negative finite values."
    )
    rs <- rowSums(trans_matrix)
    stopifnot_(
      all(rs > 0),
      "Every row of {.arg trans_matrix} must have at least one positive value."
    )
    return(trans_matrix / rs)
  }
  m <- random_rdirichlet(n, rep(alpha, n))
  if (diag_boost > 0) {
    diag(m) <- diag(m) + diag_boost
    m <- m / rowSums(m)
  }
  m
}

#' @noRd
random_resolve_inits <- function(init_probs, n, alpha) {
  if (!is.null(init_probs)) {
    stopifnot_(
      is.numeric(init_probs) && length(init_probs) == n,
      "Argument {.arg init_probs} must be a {.cls numeric} vector of length
      {n}."
    )
    stopifnot_(
      all(is.finite(init_probs)) && all(init_probs >= 0) &&
        sum(init_probs) > 0,
      "Argument {.arg init_probs} must contain non-negative finite values
      with positive sum."
    )
    return(init_probs / sum(init_probs))
  }
  as.numeric(random_rdirichlet(1L, rep(alpha, n)))
}

#' Sample from a Dirichlet distribution (base R, no external deps).
#' @noRd
random_rdirichlet <- function(n, alpha) {
  k <- length(alpha)
  x <- matrix(
    stats::rgamma(n * k, shape = rep(alpha, each = n), rate = 1),
    nrow = n, ncol = k
  )
  x / rowSums(x)
}

# Model comparison --------------------------------------------------------

#' Compare Two Matrices or TNA Models with Comprehensive Metrics
#'
#' Various distances, measures of dissimilarity and similarity, correlations
#' and other metrics are computed to compare the models. Optionally, the weight
#' matrices of the models can be scaled before comparison. The resulting object
#' can be used to produce heatmap plots and scatterplots to further illustrate
#' the differences.
#'
#' @export
#' @family comparison
#' @param x A `tna` object or a `matrix` of weights.
#' @param y A `tna` object or a `matrix` of weights.
#' @param scaling A  `character` string naming a scaling method to
#' apply to the weights before comparing them. The supported options are:
#'
#' * `"none"`: No scaling is performed. The weights are used as is.
#' * `"minmax"`: Performs min-max normalization, i.e., the minimum value is
#'   subtracted and the differences are scaled by the range.
#' * `"max"`: Max-normalization: the values are divided by the maximum value.
#' * `"rank"`: Applies min-max normalization to the ranks of the weights
#'   (computed with `ties.method = "average"`).
#' * `"zscore"`: Computes the standard score, i.e. the mean weight is
#'   subtracted and the differences are scaled by the standard deviation.
#' * `"robust"`: Computes the robust z-score, i.e. the median weight is
#'   subtracted and the differences are scaled by the median absolute deviation
#'   (using [stats::mad]).
#' * `"log"`: Simply the natural logarithm of the weights.
#' * `"log1p"`: As above, but adds 1 to the values before taking the logarithm.
#'    Useful for scenarios with zero weights.
#' * `"softmax"`: Performs softmax normalization.
#' * `"quantile"`: Uses the empirical quantiles of the weights
#'   via [stats::ecdf].
#'
#' @param measures A `character` vector indicating which centrality
#'   measures should be computed. See [centralities()] for the
#'   available measures. No measures are included by default.
#' @param network A `logical` value indicating whether network metrics should
#'   be included in the comparison. The default is `TRUE`.
#' @param ... Ignored.
#' @return A `tna_comparison` object, which is a `list` containing the
#' following elements:
#'
#' * `matrices`: A `list` containing the scaled matrices of the input `tna`
#'   objects or the scaled inputs themselves in the case of matrices.
#' * `difference_matrix`: A `matrix` of differences `x - y`.
#' * `edge_metrics`: A `data.frame` of edge-level metrics about the differences.
#' * `summary_metrics`: A `data.frame` of summary metrics of the differences
#'   across all edges.
#' * `network_metrics`: A `data.frame` of network metrics for both `x` and `y`.
#' * `centrality_differences`: A `data.frame` of differences in centrality
#'   measures computes from `x` and `y`.
#' * `centrality_correlations`: A `numeric` vector of correlations of the
#'   centrality measures between `x` and `y`.
#'
#' @examples
#' # Comparing TNA models
#' model_x <- tna(group_regulation[1:200, ])
#' model_y <- tna(group_regulation[1001:1200, ])
#' comp1 <- compare(model_x, model_y)
#'
#' # Comparing matrices
#' mat_x <- model_x$weights
#' mat_y <- model_y$weights
#' comp2 <- compare(mat_x, mat_y)
#'
#' # Comparing a matrix to a TNA model
#' comp3 <- compare(mat_x, model_y)
#'
compare <- function(x, ...) {
  UseMethod("compare")
}

#' @export
#' @rdname compare
compare.tna <- function(x, y, scaling = "none", measures = character(0),
                        network = TRUE, ...) {
  compare_(x, y, scaling = scaling, measures = measures, network = network, ...)
}

#' @export
#' @rdname compare
compare.matrix <- function(x, y, scaling = "none", measures = character(0),
                           network = TRUE, ...) {
  compare_(x, y, scaling = scaling, measures = measures, network = network, ...)
}

#' Compare Grouped TNA Models with Comprehensive Metrics
#'
#' @export
#' @family comparison
#' @param x A `group_tna` object.
#' @param i An `integer` index or the name of the principal cluster as a
#' `character` string.
#' @param j An `integer` index or the name of the secondary cluster as a
#' `character` string.
#' @inheritParams compare
#' @param ... Additional arguments passed to [compare.tna()].
#' @return A `tna_comparison` object. See [compare.tna()] for details.
#' @examples
#' model <- group_model(engagement_mmm)
#' compare(model, i = 1, j = 2)
#'
compare.group_tna <- function(x, i = 1L, j = 2L, scaling = "none",
                              measures = character(0), network = TRUE, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  check_clusters(x, i, j)
  compare_(
    x = x[[i]],
    y = x[[j]],
    scaling = scaling,
    measures = measures,
    network = network,
    ...
  )
}

#' Internal compare function
#'
#' @inheritParams compare
#' @noRd
compare_ <- function(x, y, scaling = "none", measures, network, ...) {
  stopifnot_(
    is_tna(x) || is.matrix(x),
    "Argument {.arg x} must be a {.cls tna} object or a numeric {.cls matrix}."
  )
  stopifnot_(
    is_tna(y) || is.matrix(y),
    "Argument {.arg x} must be a {.cls tna} object or a numeric {.cls matrix}."
  )
  x <- ifelse_(is_tna(x), x$weights, x)
  y <- ifelse_(is_tna(y), y$weights, y)
  x_arg <- ifelse_(is_tna(x), "x$weights", "x")
  y_arg <- ifelse_(is_tna(y), "y$weights", "y")
  n <- ncol(x)
  stopifnot_(
    n == nrow(x),
    "The weight matrix {.arg {x_arg}} must be a square matrix."
  )
  stopifnot_(
    identical(dim(x), dim(y)),
    "Weight matrices {.arg {x_arg}} and
    {.arg {y_arg}} must have identical dimensions."
  )
  stopifnot_(
    all(!is.na(x)),
    "Weight matrix {.arg {x_arg}} must not contain missing values."
  )
  stopifnot_(
    all(!is.na(y)),
    "Weight matrix {.arg {y_arg}} must not contain missing values."
  )
  scaling_methods <- list(
    none = identity,
    minmax = ranger,
    max = function(w) w / max(w, na.rm = TRUE),
    rank = function(w) ranger(rank(w, ties.method = "average")),
    zscore = function(w) (w - mean(w)) / stats::sd(w),
    log = log,
    log1p = log1p,
    softmax = function(w) exp(w - log_sum_exp(w)),
    quantile = function(w) stats::ecdf(w)(w)
  )
  scaling <- check_match(scaling, names(scaling_methods))
  x[, ] <- scaling_methods[[scaling]](as.vector(x))
  y[, ] <- scaling_methods[[scaling]](as.vector(y))
  d <- x - y
  x_vec <- as.vector(x)
  y_vec <- as.vector(y)
  abs_diff <- abs(x_vec - y_vec)
  abs_x <- abs(x_vec)
  abs_y <- abs(y_vec)
  pos <- abs_x > 0 & abs_y > 0
  rn <- rownames(x)

  # Edge level metrics
  edges <- expand.grid(source = rn, target = rn)
  edges_x <- edges
  edges_y <- edges
  edges_x$weight <- as.vector(x)
  edges_y$weight <- as.vector(y)
  edges_combined <- edges
  edges_combined$weight_x <- edges_x$weight
  edges_combined$weight_y <- edges_y$weight
  edges_combined$raw_difference <- as.vector(d)
  edges_combined$absolute_difference <- abs_diff
  edges_combined$squared_difference <- abs_diff^2
  edges_combined$relative_difference <- abs_diff / (x_vec + y_vec)
  edges_combined$similarity_strength_index <- x_vec / y_vec
  edges_combined$difference_index <- (x_vec - y_vec) / y_vec
  edges_combined$rank_difference <-
    abs(rank(x_vec, na.last = "keep") - rank(y_vec, na.last = "keep"))
  edges_combined$percentile_difference <-
    abs(stats::ecdf(x_vec)(x_vec) - stats::ecdf(y_vec)(y_vec))
  edges_combined$logarithmic_ratio <- log1p(x_vec) - log1p(y_vec)
  edges_combined$standardized_weight_x <- (x_vec - mean(x)) / stats::sd(x)
  edges_combined$standardized_weight_y <- (y_vec - mean(y)) / stats::sd(y)
  edges_combined$standardized_score_inflation <-
    edges_combined$standardized_weight_x / edges_combined$standardized_weight_y

  # Summary metrics
  weight_dev <- list(
    category = "Weight Deviations",
    metric = c(
      "Mean Abs. Diff.",
      "Median Abs. Diff.",
      "RMS Diff.",
      "Max Abs. Diff.",
      "Rel. Mean Abs. Diff.",
      "CV Ratio"
    ),
    value = c(
      mean(abs_diff),
      stats::median(abs_diff),
      sqrt(mean(abs_diff^2)),
      max(abs_diff),
      mean(abs_diff) / mean(abs_y),
      stats::sd(x_vec) * mean(y_vec) / (mean(x_vec) * stats::sd(y_vec))
    )
  )
  correlations <- list(
    category = "Correlations",
    metric = c("Pearson", "Spearman", "Kendall", "Distance"),
    value = c(
      stats::cor(x_vec, y_vec, method = "pearson", use = "complete.obs"),
      stats::cor(x_vec, y_vec, method = "spearman", use = "complete.obs"),
      stats::cor(x_vec, y_vec, method = "kendall", use = "complete.obs"),
      distance_correlation(x_vec, y_vec)
    )
  )
  dissimilarities <- list(
    category = "Dissimilarities",
    metric = c(
      "Euclidean",
      "Manhattan",
      "Canberra",
      "Bray-Curtis",
      "Frobenius"
    ),
    value = c(
      sqrt(sum(abs_diff^2)),
      sum(abs_diff),
      sum(abs_diff[pos] / (abs_x[pos] + abs_y[pos])),
      sum(abs_diff) / sum(abs_x + abs_y),
      sqrt(sum(abs_diff^2)) / sqrt(n / 2)
    )
  )
  similarities <- list(
    category = "Similarities",
    metric = c("Cosine", "Jaccard", "Dice", "Overlap", "RV"),
    value = c(
      sum(x * y) / (sqrt(sum(x^2)) * sqrt(sum(y^2))),
      sum(pmin(abs_x, abs_y)) / sum(pmax(abs_x, abs_y)),
      2 * sum(pmin(abs_x, abs_y)) / (sum(abs_x) + sum(abs_y)),
      sum(pmin(abs_x, abs_y)) / min(sum(abs_x), sum(abs_y)),
      rv_coefficient(x, y)
    )
  )
  pattern_metrics <- list(
    category = "Pattern Similarities",
    metric = c("Rank Agreement", "Sign Agreement"),
    value = c(
      mean(sign(diff(x)) == sign(diff(y))),
      mean(sign(x) == sign(y))
    )
  )
  summary_metrics <- dplyr::bind_rows(
    weight_dev,
    correlations,
    dissimilarities,
    similarities,
    pattern_metrics
  )

  # Network metrics
  if (network) {
    metrics_x <- ifelse_(
      is.matrix(x),
      summary(build_model_(x)),
      summary(x)
    )
    metrics_y <- ifelse_(
      is.matrix(y),
      summary(build_model_(y)),
      summary(y)
    )
    network_metrics <- cbind(metrics_x, metrics_y[, -1L])
    names(network_metrics) <- c("metric", "x", "y")
  }

  # Centralities
  n_measures <- length(measures)
  include_centralities <- n_measures > 0L
  if (include_centralities) {
    cents_x <- centralities(x, measures = measures) |>
      tidyr::pivot_longer(
        cols = !(!!rlang::sym("state")),
        names_to = "centrality",
        values_to = "x"
      )
    cents_y <- centralities(y, measures = measures) |>
      tidyr::pivot_longer(
        cols = !(!!rlang::sym("state")),
        names_to = "centrality",
        values_to = "y"
      )
    cents_xy <- cents_x
    cents_xy$y <- cents_y$y
    cents_xy$difference <- cents_xy$x - cents_xy$y
    corr_fun <- function(x, y) {
      out <- try_(stats::cor(x, y, use = "complete.obs"))
      # Return NA in case of not insufficient pairs
      ifelse_(
        inherits(out, "try-error"),
        NA_real_,
        out
      )
    }
    cents_corr <- cents_xy |>
      dplyr::group_by(!!rlang::sym("centrality")) |>
      dplyr::summarize(
        Centrality = dplyr::first(!!rlang::sym("centrality")),
        correlation = corr_fun(!!rlang::sym("x"), !!rlang::sym("y"))
      )
  }
  out <- list(
    matrices = list(x = x, y = y),
    difference_matrix = d,
    edge_metrics = edges_combined,
    summary_metrics = summary_metrics
  )
  if (network) {
    out$network_metrics <- network_metrics
  }
  if (include_centralities) {
    out$centrality_differences = cents_xy
    out$centrality_correlations = cents_corr
  }
  structure(
    out,
    class = "tna_comparison"
  )
}

#' Distance correlation coefficient
#'
#' @param x A `numeric` vector.
#' @param y A `numeric` vector.
#' @noRd
distance_correlation <- function(x, y) {
  dist_x <- as.matrix(stats::dist(x, diag = TRUE, upper = TRUE))
  dist_y <- as.matrix(stats::dist(y, diag = TRUE, upper = TRUE))
  n <- ncol(dist_x)
  dist_row_means_x <- matrix(.rowMeans(dist_x, n, n), n, n)
  dist_row_means_y <- matrix(.rowMeans(dist_y, n, n), n, n)
  dist_col_means_x <- matrix(.colMeans(dist_x, n, n), n, n, byrow = TRUE)
  dist_col_means_y <- matrix(.colMeans(dist_y, n, n), n, n, byrow = TRUE)
  dist_mean_x <- mean(dist_x)
  dist_mean_y <- mean(dist_y)
  xx <- dist_x - dist_row_means_x - dist_col_means_x + dist_mean_x
  yy <- dist_y - dist_row_means_y - dist_col_means_y + dist_mean_y
  v_xy <- n^-2 * sum(xx * yy)
  v_x <- n^-2 * sum(xx^2)
  v_y <- n^-2 * sum(yy^2)
  v_xy / sqrt(v_x * v_y)
}

#' RV Coefficient
#'
#' @param x A `matrix`.
#' @param y A `matrix`.
#' @noRd
rv_coefficient <- function(x, y) {
  x <- scale(x, scale = FALSE)
  y <- scale(y, scale = FALSE)
  xx <- tcrossprod(x)
  yy <- tcrossprod(y)
  tr_xx_yy <- sum(diag(xx %*% yy))
  tr_xx_xx <- sum(diag(xx %*% xx))
  tr_yy_yy <- sum(diag(yy %*% yy))
  tr_xx_yy / sqrt(tr_xx_xx * tr_yy_yy)
}


# Sequence comparison -----------------------------------------------------


#' Compare Sequences Between Groups
#'
#' Performs comprehensive sequence comparison analysis between groups. All
#' patterns of the sequences (subsequences of specific length) are extracted
#' from all sequences in each group. The pattern frequencies are compared
#' between the groups using a permutation test. The reported effect size is the
#' difference between the observed test statistic (sum of squared differences
#' between the observed and expected counts) and the mean value over the
#' permutation samples divided by their standard deviation times square root of
#' the number of observations.
#'
#' @export
#' @family comparison
#' @param x A `group_tna` object or a `data.frame` containing sequence data in
#'   wide format.
#' @param group A `vector` indicating the group assignment of each
#'   row of the data/sequence. Must have the same length as the number of
#'   rows/sequences of `x`. Alternatively, a single `character` string giving
#'   the column name of the data that defines the group when `x` is a wide
#'   format `data.frame` or a `tna_data` object.
#' @param sub An `integer` vector of pattern lengths to analyze.
#'   The default is `2:5`.
#' @param min_freq An `integer` giving the minimum number of times that a
#'   specific pattern has to be observed in each group to be included in the
#'   analysis. The default is `5`.
#' @param test A `logical` value indicating whether to test the differences
#'   of pattern counts between the groups using a permutation test.
#'   The default is `TRUE`.
#' @param iter An `integer` giving the number of iterations for the permutation
#'   test. The default is `1000`.
#' @param adjust A `character` string naming the multiple comparison
#'   correction method (default: `"bonferroni"`). Supports all
#'   [stats::p.adjust] methods: `"holm"`, `"hochberg"`, `"hommel"`,
#'   `"bonferroni"`, `"BH"`, `"BY"`, `"fdr"`, `"none"`. The adjustment
#'   is carried out within sequences of the same length.
#' @param ... Not used.
#' @return A `tna_sequence_comparison` object, which is a `data.frame` with
#'   columns giving the names of the patterns, pattern frequencies, pattern
#'   proportions (within patterns of the same length), effect sizes,
#'   and p-values of the tests.
#'
#' @examples
#' \donttest{
#' # Use a subset to keep the example fast.
#' idx <- c(1:500, 1001:1500)
#' group <- c(rep("High", 500), rep("Low", 500))
#' comp <- compare_sequences(group_regulation[idx, ], group)
#'
#' # With permutation test (small number of iterations for CRAN)
#' comp_test <- compare_sequences(
#'   group_regulation[idx, ],
#'   group,
#'   test = TRUE,
#'   iter = 5
#' )
#' }
#'
#' @export
compare_sequences <- function(x, ...) {
  UseMethod("compare_sequences")
}

#' @export
#' @rdname compare_sequences
compare_sequences.default <- function(x, group, sub, min_freq = 5L,
                                      test = TRUE, iter = 1000L,
                                      adjust = "bonferroni", ...) {
  check_missing(group)
  model <- group_tna(x, group = group)
  compare_sequences.group_tna(
    x = model,
    sub = sub,
    min_freq = min_freq,
    test = test,
    iter = iter,
    adjust = adjust,
    ...
  )
}

#' @export
#' @rdname compare_sequences
compare_sequences.group_tna <- function(x, sub, min_freq = 5L,
                                        test = TRUE, iter = 1000L,
                                        adjust = "bonferroni", ...) {
  check_missing(x)
  check_class(x, "group_tna")
  sub <- sub %m% seq(1, min(5, ncol(x[[1L]]$data)))
  check_range(
    sub,
    type = "integer",
    scalar = FALSE,
    lower = 1L,
    upper = ncol(x[[1L]]$data)
  )
  check_values(min_freq)
  check_flag(test)
  check_values(iter, strict = TRUE)
  adjust <- check_match(adjust, stats::p.adjust.methods)
  compare_sequences_(
    x = x,
    len = sub,
    min_freq = min_freq,
    test = test,
    iter = iter,
    adjust = adjust
  )
}

# Internal sequence comparison
compare_sequences_ <- function(x, len, min_freq, test, iter, adjust) {
  data <- combine_data(x)
  group <- data$.group
  data$.group <- NULL
  groups <- unique(group)
  m <- as.matrix(data)
  n <- nrow(m)
  k <- length(len)
  labels <- x[[1L]]$labels
  pattern_matrices <- extract_patterns(m, len, labels)
  patterns <- factorize_patterns(pattern_matrices, group)
  perm <- onlyif(
    test,
    permutation_test_patterns(x, len, iter, adjust)
  )
  out <- vector(mode = "list", length = k)
  freq_cols <- paste0("freq_", groups)
  for (j in seq_len(k)) {
    pat <- patterns[[j]]
    cs <- colSums(pat)
    tmp <- cbind(pat, pat / cs[col(pat)])
    tmp[!is.finite(tmp)] <- NA_real_
    colnames(tmp) <- c(freq_cols, paste0("prop_", groups))
    out[[j]] <- cbind(
      data.frame(pattern = pattern_matrices[[j]]$unique),
      as.data.frame(tmp)
    )
  }
  out <- do.call(base::rbind, out)
  if (test) {
    out <- cbind(out, perm) |>
      dplyr::arrange(!!rlang::sym("p_value"))
  }
  rmin <- apply(out[, freq_cols], 1, min)
  valid_patterns <- rmin >= min_freq
  stopifnot_(
    any(valid_patterns),
    "No common patterns with a frequency greater than {min_freq} were found."
  )
  out <- out[valid_patterns, ]
  structure(
    out,
    groups = groups,
    freq_cols = freq_cols,
    class = c("tna_sequence_comparison", "data.frame")
  )
}

extract_patterns <- function(m, len, labels) {
  n <- nrow(m)
  p <- ncol(m)
  k <- length(len)
  mis <- is.na(m)
  out <- vector(mode = "list", length = k)
  for (l in seq_len(k)) {
    j <- len[l]
    tmp <- matrix("", n,  p - j + 1L)
    for (i in seq_len(p - j + 1L)) {
      idx <- i:(i + j - 1L)
      pattern <- labels[m[, idx]]
      dim(pattern) <- c(n, j)
      pattern_mis <- mis[, idx]
      valid <- .rowSums(pattern_mis, m = n, n = j) == 0L
      if (any(valid)) {
        subseq <- do.call(
          base::paste,
          c(
            as.data.frame(
              pattern[valid, , drop = FALSE],
              stringsAsFactors = FALSE
            ),
            sep = "->"
          )
        )
        tmp[valid, i] <- subseq
      }
    }
    out[[l]] <- list(
      patterns = t(tmp),
      unique = sort(unique(tmp[nzchar(tmp)]))
    )
  }
  out
}

factorize_patterns <- function(patterns, group) {
  groups <- unique(group)
  p <- length(patterns)
  g <- length(groups)
  out <- vector(mode = "list", length = p)
  group_idx <- vector(mode = "list", length = p)
  for (j in seq_len(g)) {
    group_idx[[j]] <- which(group == groups[j])
  }
  for (i in seq_len(p)) {
    pat <- patterns[[i]]$pat
    pat_u <- patterns[[i]]$unique
    n <- length(pat_u)
    tmp <- matrix(0L, nrow = n, ncol = g)
    for (j in seq_len(g)) {
      y <- pat[, group_idx[[j]]]
      idx <- match(y, pat_u, nomatch = 0)
      tmp[, j] <- tabulate(idx, nbins = n)
    }
    out[[i]] <- tmp
  }
  out
}

test_that("models can be compared", {
  model_x <- tna(group_regulation[1:200, ])
  model_y <- tna(group_regulation[1001:1200, ])
  # Comparing models
  expect_error(
    comp1 <- compare(model_x, model_y),
    NA
  )
  # Comparing matrices
  mat_x <- model_x$weights
  mat_y <- model_y$weights
  expect_error(
    comp2 <- compare(mat_x, mat_y),
    NA
  )
  # Comparing matric to a TNA model
  expect_error(
    comp3 <- compare(mat_x, model_y),
    NA
  )
})

test_that("centralities can be compared", {
  model_x <- tna(group_regulation[1:200, ])
  model_y <- tna(group_regulation[1001:1200, ])
  # Comparing models
  expect_error(
    comp <- compare(
      model_x,
      model_y,
      measures = c("InStrength", "OutStrength")
    ),
    NA
  )
})

test_that("clusters can be compared", {
  expect_error(
    comp <- compare(mmm_model, i = 1, j = 3),
    NA
  )
})

test_that("sequences can be compared", {
  expect_error(
    compare_sequences(mock_sequence, group = c(1, 1, 2, 2, 2), min_freq = 1L),
    NA
  )
  expect_error(
    compare_sequences(mock_group_tna, min_freq = 1L),
    NA
  )
})

test_that("comparison fails when minimum frequency is not met", {
  expect_error(
    compare_sequences(mock_group_tna),
    "No common patterns with a frequency greater than 5 were found\\."
  )
})

test_that("patterns can be tested", {
  expect_error(
    compare_sequences(mock_group_tna, min_freq = 1L, test = TRUE, iter = 10),
    NA
  )
})

#' Build and Visualize a Network with Edge Betweenness
#'
#' This function builds a network from a transition matrix in a `tna` object
#' and computes edge betweenness for the network.
#'
#' @export
#' @family centralities
#' @param x A `tna` object.
#' @param directed A `logical` value. If `TRUE`, the network is considered
#' directed.
#' @inheritParams centralities
#' @return A `tna` object where the edge weights are edge betweenness values.
#' @examples
#' model <- tna(group_regulation)
#' betweenness_network(model)
#'
betweenness_network <- function(x, directed = TRUE, invert = TRUE) {
  UseMethod("betweenness_network")
}

#' @rdname betweenness_network
#' @export
betweenness_network.tna <- function(x, directed = TRUE, invert = TRUE) {
  check_missing(x)
  check_class(x, "tna")
  g <- as.igraph(x)
  w <- ifelse_(invert, 1.0 / igraph::E(g)$weight, igraph::E(g)$weight)
  igraph::E(g)$weight <- w
  betweenness <- igraph::edge_betweenness(g, directed = directed)
  weights <- t(x$weights)
  weights[weights > 0] <- betweenness
  build_model_(
    weights = t(weights),
    type = "betweenness",
    inits = x$inits,
    labels = x$labels,
    data = x$data
  )
}

set.seed(0)

mock_matrix <- matrix(
  c(0, stats::runif(15)),
  nrow = 4,
  ncol = 4,
  dimnames = replicate(2, LETTERS[1:4], simplify = FALSE)
)

mock_freq_matrix <- matrix(
  c(0, rpois(24, lambda = 50)),
  nrow = 5,
  ncol = 5,
  dimnames = replicate(2, LETTERS[1:5], simplify = FALSE)
)

mock_sequence <- data.frame(
  T1 = c("A", "B", "C", "A", "A"),
  T2 = c("A", "C", "B", "B", "B"),
  T3 = c("B", "C", "A", "C", "A"),
  T4 = c("B", "A", "C", "A", "B"),
  T5 = c("C", "A", "B", "B", "A"),
  T6 = c("C", "C", "A", "A", "C")
)

mock_sequence_wide <- data.frame(
  ID = c("A", "A", "B", "B"),
  Time = c(1, 2, 1, 2),
  feature1 = c(10, 0, 15, 20),
  feature2 = c(5, 8, 0, 12),
  feature3 = c(2, 4, 6, 8),
  other_col = c("X", "Y", "Z", "W")
)

mock_tna <- tna(
  x = mock_matrix,
  inits = c(0.3, 0.2, 0.3, 0.2)
)

mock_tna_seq <- tna(mock_sequence)

mock_group_tna <- group_model(
  mock_sequence,
  group = c(1, 1, 2, 2, 2)
)

mmm_model <- group_tna(engagement_mmm)

mock_long <- data.frame(
  time = rep(1:6, 5),
  group = rep(1:5, each = 6),
  event =  as.vector(t(as.matrix(mock_sequence)))
)

mock_tsn <- structure(
  data.frame(
    id = gl(10, 100),
    value = c(
      replicate(
        10,
        stats::arima.sim(list(order = c(2, 1, 0), ar = c(0.5, 0.2)), n = 99)
      )
    ),
    state = factor(
      sample(3, 1000, replace = TRUE),
      labels = c("State 1", "State 2", "State 3")
    ),
    time = rep(seq_len(100), 10)
  ),
  class = c("tsn", "data.frame")
)

mock_cluster_data <- data.frame(
  T1 = c("A", "B", "A", "C", "A", "B"),
  T2 = c("B", "A", "B", "A", "C", "A"),
  T3 = c("C", "C", "A", "B", "B", "C")
)

{
  rlang::local_options(rlib_message_verbosity = "quiet")
  mock_tna_data <- prepare_data(
    mock_long,
    time = "time",
    actor = "group",
    action = "event"
  )
}

#' Normalize `x` to the unit interval from 0 to 1.
#'
#' @param x A `numeric` vector.
#' @param na.rm A `logical` value indicating whether missing values
#'   should be removed.
#' @noRd
ranger <- function(x, na.rm = FALSE) {
  mi <- min(x, na.rm = na.rm)
  ma <- max(x, na.rm = na.rm)
  (x - mi) / (ma - mi)
}

#' Check Weak Connectivity of an Adjacency Matrix
#'
#' This function checks if an adjacency matrix represents a weakly connected
#' graph. A graph is considered weakly connected if there is a path between
#' any two vertices when ignoring the direction of edges.
#'
#' @param mat A square adjacency `matrix` representing the graph.
#' @return A `logical` value indicating whether the graph is
#' weakly connected (`TRUE`) or not (`FALSE`).
#' @noRd
is_weakly_connected <- function(mat) {
  n <- nrow(mat)
  visited <- logical(n)
  stack <- integer(n)
  stack[1L] <- 1
  visited[1L] <- TRUE
  len <- 1L
  while (len > 0) {
    v <- stack[len]
    len <- len - 1L
    neighbors <- which(mat[v, ] > 0 | mat[, v] > 0)
    for (u in neighbors) {
      if (!visited[u]) {
        len <- len + 1L
        visited[u] <- TRUE
        stack[len] <- u
      }
    }
  }
  all(visited)
}

#' Shorthand for `try(., silent = TRUE)`
#'
#' @param expr An \R expression to try.
#' @noRd
try_ <- function(expr) {
  try(expr, silent = TRUE)
}

#' Check that argument is an object of class `tna`
#'
#' @param x An \R object.
#' @noRd
is_tna <- function(x) {
  inherits(x, "tna")
}

#' @importFrom igraph as.igraph
#' @export
igraph::as.igraph

#' Coerce a `tna` Object into an `igraph` Object.
#'
#' @export
#' @family helpers
#' @inheritParams igraph::graph_from_adjacency_matrix
#' @param x A `tna` object.
#' @param ... Ignored.
#' @return An `igraph` object.
as.igraph.tna <- function(x, mode = "directed", ...) {
  check_missing(x)
  check_class(x, "tna")
  igraph::graph_from_adjacency_matrix(
    adjmatrix = x$weights,
    mode = mode,
    weighted = TRUE
  )
}

#' Coerce a Weight Matrix into an `igraph` Object.
#'
#' @export
#' @family helpers
#' @inheritParams igraph::graph_from_adjacency_matrix
#' @param x A `matrix` of edge weights.
#' @param ... Ignored.
#' @return An `igraph` object.
as.igraph.matrix <- function(x, mode = "directed", ...) {
  check_missing(x)
  check_class(x, "matrix")
  igraph::graph_from_adjacency_matrix(
    adjmatrix = x,
    mode = mode,
    weighted = TRUE,
  )
}

#' Coerce a Specific Group from a `group_tna` Object into an `igraph` Object.
#'
#' @export
#' @family helpers
#' @inheritParams igraph::as.igraph
#' @param which The number or name of the group.
#' @return An `igraph` object.
as.igraph.group_tna <- function(x, which, ...) {
  check_missing(x)
  check_missing(which)
  check_class(x, "group_tna")
  stopifnot_(
    !is.null(x[[which]]),
    "There is no group named {which}."
  )
  as.igraph(x[[which]])
}

#' Log-sum-exp function
#'
#' @param x A `numeric` vector.
#' @noRd
log_sum_exp <- function(x) {
  n <- length(x)
  L <- x[1L]
  for (i in seq_len(n - 1L)) {
    L <- max(x[i + 1L], L) + log1p(exp(-abs(x[i + 1L] - L)))
  }
  L
}

#' Null coalescing operator
#'
#' Define the null coalescing operator for older R versions
#' @noRd
if (base::getRversion() < "4.4.0") {
  `%||%` <- function(x, y) {
    if (is.null(x)) y else x
  }
}

#' Default value operator for a missing argument
#'
#' @param x An \R object
#' @param y An \R object to assign if `x` is missing
#' @noRd
`%m%` <- function(x, y) {
  if (missing(x)) y else x
}

#' Number of unique elements in a vector
#'
#' @param x A `vector`.
#' @noRd
n_unique <- function(x) {
  length(unique(x))
}

#' Force values into bounds
#'
#' @param x A `numeric` vector of values
#' @param range A `numeric` vector of length 2 giving the bounds.
#' @noRd
bound <- function(x, range) {
  force(range)
  low <- range[1L]
  high <- range[2L]
  x[x < low] <- low
  x[x > high] <- high
  x
}

#' Get specific columns from data
#'
#' @param expr An `expression` for the columns to select
#' @param data A `data.frame` to select the columns from
#' @noRd
get_cols <- function(expr, data) {
  if (rlang::quo_is_missing(expr)) {
    return(rlang::missing_arg())
  }
  if (rlang::quo_is_symbolic(expr) && !rlang::quo_is_call(expr, "!!")) {
    pos <- tidyselect::eval_select(expr = expr, data = data)
    names(pos)
  } else {
    cols <- rlang::eval_tidy(expr = expr)
    if (is.character(cols)) {
      cols_mis <- setdiff(cols, names(data))
      stopifnot_(
        length(cols_mis) == 0L,
        c(
          "Can't select columns that don't exist.",
          `x` = "Column {.var {cols_mis[1L]}} doesn't exist."
        )
      )
      return(cols)
    }
    if (is.numeric(cols)) {
      nm <- names(data)
      k <- length(nm)
      cols_mis <- setdiff(cols, seq_len(k))
      stopifnot_(
        length(cols_mis) == 0L,
        c(
          "Can't select columns that don't exist.",
          `x` = "Attempted to select column {cols_mis[1]} from {k} columns."
        )
      )
      return(nm[cols])
    }
    stop_(
      "Columns must be selected using a tidy selection,
       a {.cls character} vector, or an {.cls integer} vector."
    )
  }
}

# Functions borrowed from the `dynamite` package --------------------------
# https://github.com/ropensci/dynamite

#' Shorthand for `if (test) yes else no`
#'
#' @param test A `logical` value of the condition to evaluate.
#' @param yes An \R object to return when `test` evaluates to `TRUE`.
#' @param no An \R object to return when `test` evaluates to `FALSE`.
#' @noRd
ifelse_ <- function(test, yes, no) {
  if (test) {
    yes
  } else {
    no
  }
}

#' Return `yes` if `test` is `TRUE`, otherwise return `NULL`
#'
#' @param test \[`logical(1)`] Condition to evaluate.
#' @param yes An \R object to return when `test` evaluates to `TRUE`.
#' @noRd
onlyif <- function(test, yes) {
  if (test) {
    yes
  } else {
    NULL
  }
}

#' Generate a Warning Message
#'
#' @param message See [cli::cli_warn()].
#' @param ... See [cli::cli_warn()].
#' @noRd
warning_ <- function(message, ...) {
  cli::cli_warn(message, ..., .envir = parent.frame())
}

#' Stop Function Execution Without Displaying the Call
#'
#' @param message See [cli::cli_abort()].
#' @param ... See [cli::cli_abort()].
#' @param call See [cli::cli_abort()].
#' @noRd
stop_ <- function(message, ..., call = rlang::caller_env()) {
  cli::cli_abort(message, ..., .envir = parent.frame(), call = call)
}

#' Stop function execution unless a condition is true
#'
#' @param message See [cli::cli_abort()].
#' @param ... See [cli::cli_abort()].
#' @param call See [cli::cli_abort()].
#' @noRd
stopifnot_ <- function(cond, message, ..., call = rlang::caller_env()) {
  if (!cond) {
    cli::cli_abort(message, ..., .envir = parent.frame(), call = call)
  }
}

#' Generate an Informative Message
#'
#' @param message See [cli::cli_inform()]
#' @param ... See [cli::cli_inform()]
#' @noRd
message_ <- function(message, ...) {
  cli::cli_inform(message, ..., .envir = parent.frame())
}

#' Create a Comma-separated Character String
#'
#' @param x A `character` vector.
#' @noRd
cs <- function(...) {
  paste0(c(...), collapse = ", ")
}

#' Identify Cliques in a Transition Network
#'
#' This function identifies cliques of a specified size in a transition network.
#' It searches for cliques, i.e., complete subgraphs where every pair of nodes
#' is connected, of size `n` in the transition matrix for the specified cluster
#' in the `tna` object.
#'
#' @export
#' @family cliques
#' @rdname cliques
#' @param x A `tna` or a `group_tna` object.
#' @param size An `integer` specifying the size of the cliques to identify.
#' Defaults to `2` (dyads).
#' @param threshold A `numeric` value that sets the minimum edge weight
#' for an edge to be considered in the clique. Edges below this value
#' are ignored. Defaults to `0`.
#' @param sum_weights A `logical` value specifying whether the sum of the
#' weights should be above the `threshold` instead of individual weights of the
#' directed edges. Defaults to `FALSE`.
#' @param ... Ignored.
#' @return A `tna_cliques` object which is a `list` of two elements:
#'
#'   * `weights` is a `matrix` of the edge weights in the clique.
#'   * `inits` is a `numeric` vector of initial weights for the clique.
#'
#' If `x` is a `group_tna` object, a `group_tna_cliques` object is returned
#' instead, which is a `list` or `tna_cliques` objects.
#'
#' @examples
#' model <- tna(group_regulation)
#'
#' # Find  2-cliques (dyads)
#' cliq <- cliques(model, size = 2)
#'
cliques <- function(x, ...) {
  UseMethod("cliques")
}

#' @export
#' @rdname cliques
cliques.tna <- function(x, size = 2, threshold = 0, sum_weights = FALSE, ...) {
  check_missing(x)
  check_class(x, "tna")
  check_values(threshold, type = "numeric")
  check_flag(sum_weights)
  stopifnot_(
    checkmate::test_int(x = size, lower = 2),
    "Argument {.arg size} must be a single {.cls integer}
    between 2 and {nodes(x)}."
  )
  weights <- x$weights
  labels <- x$labels
  inits <- x$inits
  clique_idx <- integer(0)
  mat <- weights
  mat[mat < threshold] <- 0
  if (sum_weights) {
    g <- as.igraph(mat, mode = "plus")
    cliq <- igraph::cliques(g, min = size, max = size)
    cliq_idx <- lapply(cliq, function(y) which(labels %in% attr(y, "names")))
  } else {
    g1 <- as.igraph(mat, mode = "upper")
    g2 <- as.igraph(mat, mode = "lower")
    cliq1 <- igraph::cliques(g1, min = size, max = size)
    cliq2 <- igraph::cliques(g2, min = size, max = size)
    nodes1 <- lapply(cliq1, function(y) which(labels %in% attr(y, "names")))
    nodes2 <- lapply(cliq2, function(y) which(labels %in% attr(y, "names")))
    cliq_idx <- intersect(nodes1, nodes2)
  }
  structure(
    list(
      weights = lapply(
        cliq_idx,
        function(y) weights[y, y]
      ),
      inits = lapply(
        cliq_idx,
        function(y) inits[y]
      ),
      indices = cliq_idx
    ),
    class = "tna_cliques",
    size = size,
    labels = labels,
    threshold = threshold,
    sum_weights = sum_weights,
    colors = attr(x$data, "colors")
  )
}

#' @export
#' @rdname cliques
#' @examples
#' model <- group_tna(engagement_mmm)
#' cliques(model)
#'
cliques.group_tna <- function(x, size = 2, threshold = 0,
                              sum_weights = FALSE, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  structure(
    lapply(
      x,
      cliques,
      size = size,
      threshold = threshold,
      sum_weights = sum_weights,
      ...
    ),
    class = "group_tna_cliques"
  )
}

test_that("cliques of different sizes can be found", {
  model <- tna(group_regulation)
  dyads <- cliques(model, size = 2)
  triads <- cliques(model, size = 3)
  expect_length(dyads$weights, 35)
  expect_length(triads$weights, 77)
})

test_that("cliques of can be found with sum of weights", {
  model <- tna(group_regulation)
  dyads <- cliques(model, size = 2, sum_weights = TRUE)
  triads <- cliques(model, size = 3, sum_weights = TRUE)
  expect_length(dyads$weights, 36)
  expect_length(triads$weights, 84)
})

test_that("cliques can be found for clusters", {
  expect_error(
    cliques(mmm_model, size = 2),
    NA
  )
})

#' Calculate Centrality Measures for a Transition Matrix
#'
#' Calculates several centrality measures. See 'Details' for
#' information about the measures.
#'
#' The following measures are provided:
#'
#'   * `OutStrength`: Outgoing strength centrality, calculated using
#'     [igraph::strength()] with `mode = "out"`. It measures the total weight
#'     of the outgoing edges from each node.
#'   * `InStrength`: Incoming strength centrality, calculated using
#'     [igraph::strength()] with `mode = "in"`. It measures the total weight
#'     of the incoming edges to each node.
#'   * `ClosenessIn`: Closeness centrality (incoming), calculated using
#'     [igraph::closeness()] with `mode = "in"`. It measures how close a node
#'     is to all other nodes based on the incoming paths.
#'   * `ClosenessOut`: Closeness centrality (outgoing), calculated using
#'     [igraph::closeness()] with `mode = "out"`. It measures how close a node
#'     is to all other nodes based on the outgoing paths.
#'   * `Closeness`: Closeness centrality (overall), calculated using
#'     [igraph::closeness()] with `mode = "all"`. It measures how close a node
#'     is to all other nodes based on both incoming and outgoing paths.
#'   * `Betweenness`: Betweenness centrality defined by the number of
#'     geodesics calculated using [igraph::betweenness()].
#'   * `BetweennessRSP`: Betweenness centrality based on randomized shortest
#'     paths (Kivimäki et al. 2016). It measures the extent to which a
#'     node lies on the shortest paths between other nodes.
#'   * `Diffusion`: Diffusion centrality of Banerjee et.al. (2014).
#'     It measures the influence of a node in spreading information through
#'     the network.
#'   * `Clustering`: Signed clustering coefficient of Zhang and Horvath (2005)
#'     based on the symmetric adjacency matrix (sum of the adjacency matrix
#'     and its transpose). It measures the degree to which nodes tend to
#'     cluster together.
#'
#' @export
#' @family centralities
#' @rdname centralities
#' @param x A `tna` object, a `group_tna` object, or
#' a square `matrix` representing edge weights.
#' @param loops A `logical` value indicating whether to include loops in the
#'   network when computing the centrality measures. The default is `FALSE`.
#' @param normalize A `logical` value indicating whether the centralities
#'   should be normalized. The default is `FALSE`.
#' @param invert A `logical` value indicating whether the weights should be
#'   inverted for distance-based measures. The default is `TRUE`.
#' @param measures A `character` vector indicating which centrality
#'   measures should be computed. If missing, all available measures are
#'   returned. See 'Details' for the available measures.
#' @return A `tna_centralities` object which is a tibble (`tbl_df`).
#'   containing centrality measures for each state.
#' @examples
#' model <- tna(group_regulation)
#'
#' # Centrality measures including loops in the network
#' centralities(model)
#'
#' # Centrality measures excluding loops in the network
#' centralities(model, loops = FALSE)
#'
#' # Centrality measures normalized
#' centralities(model, normalize = TRUE)
#'
centralities <- function(x, loops = FALSE, normalize = FALSE,
                         invert = TRUE, measures) {
  UseMethod("centralities")
}

#' @export
#' @rdname centralities
centralities.tna <- function(x, loops = FALSE, normalize = FALSE,
                             invert = TRUE, measures) {
  check_missing(x)
  check_class(x, "tna")
  out <- centralities_(
    x$weights,
    loops = loops,
    normalize = normalize,
    invert = invert,
    measures = measures
  )
  if (!is.null(x$data)) {
    attr(out, "colors") <- attr(x$data, "colors")
  }
  out
}

#' @export
#' @rdname centralities
centralities.matrix <- function(x, loops = FALSE, normalize = FALSE,
                                invert = TRUE, measures) {
  check_missing(x)
  stopifnot_(
    is.matrix(x),
    "Argument {.arg x} must be a {.cls matrix}."
  )
  centralities_(x, loops, normalize, invert, measures)
}

#' Internal function to calculate various centrality measures
#'
#' @param x An adjacency `matrix` of a directed weighted graph.
#' @inheritParams centralities
#' @noRd
centralities_ <- function(x, loops, normalize, invert, measures) {
  check_flag(loops)
  check_flag(normalize)
  check_flag(invert)
  measures <- measures %m% names(centrality_funs)
  measures <- check_measures(measures)
  diag(x) <- ifelse_(loops, diag(x), 0)
  g <- as.igraph(x)
  w <- ifelse_(invert, 1.0 / igraph::E(g)$weight, igraph::E(g)$weight)
  measures_out <- lapply(
    measures,
    function(y) {
      centrality_funs[[y]](g = g, x = x, w = w)
    }
  )
  names(measures_out) <- measures
  out <- as.data.frame(measures_out)
  if (normalize) {
    out[, measures] <- as.data.frame(
      lapply(out[, measures, drop = FALSE], ranger)
    )
  }
  rn <- rownames(out)
  out <- tibble::rownames_to_column(out, "state")
  out$state <- factor(out$state, levels = rn)
  structure(
    out,
    class = c("tna_centralities", "tbl_df", "tbl", "data.frame")
  )
}

#' Compute the Diffusion Centrality Measure
#'
#' @param mat A weight `matrix`.
#' @noRd
diffusion <- function(mat) {
  s <- 0
  n <- ncol(mat)
  p <- diag(1, n, n)
  for (i in seq_len(n)) {
    p <- p %*% mat
    s <- s + p
  }
  .rowSums(s, n, n)
}

#' @export
#' @rdname estimate_centrality_stability
estimate_cs <- function(x, loops, normalize, invert, measures, iter, method,
                        drop_prop, threshold, certainty, progressbar = NULL, 
                        detailed = NULL) {
  UseMethod("estimate_cs")
}

#' @export
#' @rdname estimate_centrality_stability
estimate_centrality_stability <- estimate_cs

#' Estimate Centrality Stability
#'
#' Estimates the stability of centrality measures in a network
#' using subset sampling without replacement. It allows for dropping varying
#' proportions of cases and calculates correlations between the original
#' centralities and those computed using sampled subsets.
#'
#' The function works by repeatedly resampling the data, dropping varying
#' proportions of cases, and calculating centrality measures on the subsets.
#' The correlation between the original centralities and the resampled
#' centralities is calculated for each drop proportion. The stability of each
#' centrality measure is then summarized using a centrality stability (CS)
#' coefficient, which represents the proportion of dropped cases at which
#' the correlations drop below a given threshold (default 0.7).
#'
#' The results can be visualized by plotting the output object showing the
#' stability of the centrality measures across different drop proportions,
#' along with confidence intervals. The CS-coefficients are displayed in the
#' subtitle.
#'
#' @export
#' @family validation
#' @rdname estimate_centrality_stability
#' @param x A `tna` or a `group_tna` object representing the temporal network
#'   analysis data. The object should be created from a sequence data object.
#' @param loops A `logical` value indicating whether to include loops in the
#'   network when computing the centrality measures. The default is `FALSE`.
#' @param normalize A `logical` value indicating whether to normalize
#'   the centrality measures. The default is `FALSE`.
#' @param invert A `logical` value indicating whether the weights should be
#'   inverted for distance-based measures. The default is `TRUE`.
#' @param measures A `character` vector of centrality measures to estimate.
#'   The default measures are `"InStrength"`, `"OutStrength"`,
#'   and `"Betweenness"`. See [centralities()] for a list of available
#'   centrality measures.
#' @param iter An `integer` specifying the number of resamples to draw.
#'   The default is 1000.
#' @param method A `character` string indicating the correlation coefficient
#'   type. The default is `"pearson"`. See [stats::cor()] for details.
#' @param drop_prop A `numeric` vector specifying the proportions of
#'   cases to drop in each sampling iteration. Default is a sequence from 0.1
#'   to 0.9 in increments of 0.1.
#' @param threshold A `numeric` value specifying the correlation threshold for
#'   calculating the CS-coefficient. The default is 0.7.
#' @param certainty A `numeric` value specifying the desired level of certainty
#'   for the CS-coefficient. Default is 0.95.
#' @param progressbar A `logical` value. If `TRUE`, a progress bar is displayed
#'   Defaults to `FALSE`
#' @param detailed Deprecated. This argument is ignored and will be removed in 
#'   a future version.
#' @return A `tna_stability` object which is a `list` with an element for each
#' `measure` with the following elements:
#'
#' * `cs_coefficient`: The centrality stability (CS) coefficient
#'   of the measure.
#' * `correlations`: A `matrix` of correlations between the original
#'   centrality and the resampled centralities for each drop proportion.
#'
#' If `x` is a `group_tna` object, a `group_tna_stability` object is returned
#' instead, which is a `list` of `tna_stability` objects.
#'
#' @examples
#' \donttest{
#' model <- tna(group_regulation)
#' # Small number of iterations and drop proportions for CRAN
#' estimate_cs(
#'   model,
#'   drop_prop = seq(0.3, 0.9, by = 0.2),
#'   measures = c("InStrength", "OutStrength"),
#'   iter = 10
#' )
#' }
#'
estimate_cs.tna <- function(x, loops = FALSE, normalize = FALSE, invert = TRUE,
                            measures = c(
                              "InStrength", "OutStrength", "Betweenness"
                            ), iter = 1000, method = "pearson",
                            drop_prop = seq(0.1, 0.9, by = 0.1),
                            threshold = 0.7, certainty = 0.95,
                            progressbar = FALSE, detailed = NULL) {
  check_tna_seq(x)
  check_flag(loops)
  check_flag(normalize)
  check_flag(invert)
  check_flag(progressbar)
  check_values(iter, strict = TRUE)
  check_range(threshold, lower = 0, upper = 1)
  check_range(certainty, lower = 0, upper = 1)
  check_measures(measures)
  if (!is.null(detailed)) {
    .Deprecated(msg = "'detailed' is deprecated and will be ignored.")
  }
  d <- x$data
  type <- attr(x, "type")
  scaling <- attr(x, "scaling")
  params <- attr(x, "params")
  model <- initialize_model(d, type, scaling, params, transitions = TRUE)
  trans <- model$trans
  a <- dim(trans)[2L]
  n <- nrow(d)
  n_seq <- seq_len(n)
  n_prop <- length(drop_prop)
  centralities_orig <- centralities_(
    x = x$weights,
    loops = loops,
    normalize = normalize,
    invert = invert,
    measures = measures
  )
  sds_orig <- centralities_orig |>
    dplyr::summarize(
      dplyr::across(
        dplyr::all_of(measures),
        stats::sd
      )
    )
  valid_measures <- which(sds_orig > 0)
  centralities_orig <- centralities_orig[, 1L + valid_measures]
  measures <- measures[valid_measures]
  n_measures <- length(measures)
  stability <- replicate(
    n_measures,
    matrix(NA, nrow = iter, ncol = n_prop),
    simplify = FALSE
  )
  names(stability) <- measures
  if (progressbar) {
    cli::cli_progress_bar(
      name = "Computing centrality stability",
      total = n_prop * iter
    )
  }
  for (i in seq_len(n_prop)) {
    prop <- drop_prop[i]
    n_drop <- floor(n * prop)
    if (n_drop == 0) {
      warning_(
        paste0("No cases dropped for proportion ", prop, ". Skipping...")
      )
      next
    }
    corr_prop <- matrix(nrow = iter, ncol = n_measures)
    for (j in seq_len(iter)) {
      keep <- sample(n_seq, n - n_drop, replace = FALSE)
      trans_sub <- trans[keep, , , drop = FALSE]
      weight_sub <- compute_weights(trans_sub, type, scaling, a)
      centralities_sub <- centralities_(
        x = weight_sub,
        loops = loops,
        invert = invert,
        normalize = normalize,
        measures = measures
      )
      sds_sub <- centralities_sub |>
        dplyr::summarize(
          dplyr::across(
            dplyr::all_of(measures),
            stats::sd
          )
        )
      corr_prop[j, ] <- vapply(
        measures,
        function(measure) {
          centrality_sub <- centralities_sub[[measure]]
          centrality_orig <- centralities_orig[[measure]]
          sd_sub <- sds_sub[measure]
          ifelse_(
            sd_sub > 0,
            stats::cor(
              centrality_sub,
              centrality_orig,
              method = method,
              use = "complete.obs"
            ),
            NA_real_
          )
        },
        numeric(1L)
      )
      if (progressbar) {
        cli::cli_progress_update()
      }
    }
    for (k in seq_len(n_measures)) {
      measure <- measures[k]
      stability[[measure]][, i] <- corr_prop[, k]
    }
  }
  out <- list()
  for (measure in measures) {
    cs_coef <- calculate_cs(
      stability[[measure]],
      threshold,
      certainty,
      drop_prop
    )
    out[[measure]] <- list(
      cs_coefficient = cs_coef,
      correlations = stability[[measure]]
    )
  }
  if (progressbar) {
    cli::cli_progress_done()
  }
  structure(
    out,
    class = "tna_stability",
    drop_prop = drop_prop,
    threshold = threshold
  )
}

#' @export
#' @rdname estimate_centrality_stability
estimate_centrality_stability.tna <- estimate_cs.tna

#' Calculate Centrality Stability
#'
#' @param corr_mat A `matrix` of correlation values
#' @inheritParams estimate_centrality_stability
#' @noRd
calculate_cs <- function(corr_mat, threshold, certainty, drop_prop) {
  prop_above <- apply(corr_mat, 2, function(x) {
    mean(x >= threshold, na.rm = TRUE)
  })
  valid_indices <- which(prop_above >= certainty)
  ifelse_(
    length(valid_indices) > 0,
    drop_prop[max(valid_indices)],
    0
  )
}

#' Compute the Randomized Shortest Path Betweenness Centrality Measure
#'
#' @param mat A weight `matrix`.
#' @param beta The beta parameter of the algorithm.
#' @noRd
rsp_bet <- function(mat, beta = 0.01) {
  n <- ncol(mat)
  D <- .rowSums(mat, m = n, n = n)
  if (any(D == 0)) {
    return(NA)
  }
  P_ref <- diag(D^-1) %*% mat
  C <- mat^-1
  C[is.infinite(C)] <- 0
  W <- P_ref * exp(-beta * C)
  Z <- solve(diag(1, n, n) - W)
  Z_recip <- Z^-1
  Z_recip[is.infinite(Z_recip)] <- 0
  Z_recip_diag <- diag(Z_recip) * diag(1, n, n)
  out <- diag(tcrossprod(Z, Z_recip - n * Z_recip_diag) %*% Z)
  out <- round(out)
  out <- out - min(out) + 1
  out
}

#' Compute the Signed Clustering Coefficient
#'
#' @param mat A weight `matrix`.
#' @noRd
wcc <- function(mat) {
  diag(mat) <- 0
  n <- ncol(mat)
  num <- diag(mat %*% mat %*% mat)
  den <- .colSums(mat, n, n)^2 - .colSums(mat^2, n, n)
  num / den
}


# Groups ----------------------------------------------------------------

#' @export
#' @rdname centralities
centralities.group_tna <- function(x, loops = FALSE, normalize = FALSE,
                                   invert = TRUE, measures) {
  check_missing(x)
  check_class(x, "group_tna")
  # missing() does not work with lapply, need to evaluate measures here.
  measures <- measures %m% names(centrality_funs)
  out <- dplyr::bind_rows(
    lapply(
      x,
      function(i) {
        data.frame(
          centralities.tna(
            x = i,
            loops = loops,
            normalize = normalize,
            invert = invert,
            measures = measures
          )
        )
      }
    ),
    .id = "group"
  )
  structure(
    out,
    class = c("group_tna_centralities", "tbl_df", "tbl", "data.frame")
  )
}

#' @export
#' @rdname estimate_centrality_stability
estimate_cs.group_tna <- function(x, loops = FALSE, normalize = FALSE,
                                  invert = TRUE,
                                  measures = c(
                                    "InStrength", "OutStrength", "Betweenness"
                                  ), iter = 1000, method = "pearson",
                                  drop_prop = seq(0.1, 0.9, by = 0.1),
                                  threshold = 0.7, certainty = 0.95,
                                  progressbar = FALSE, detailed = NULL) {
  check_missing(x)
  check_class(x, "group_tna")
  if (!is.null(detailed)) {
    .Deprecated(msg = "'detailed' is deprecated and will be ignored.")
  }
  structure(
    lapply(
      x,
      function(i) {
        estimate_centrality_stability.tna(
          i,
          loops = loops,
          normalize = normalize,
          invert = invert,
          measures = measures,
          iter = iter,
          method = method,
          drop_prop = drop_prop,
          threshold = threshold,
          certainty = certainty,
          progressbar = progressbar
        )
      }
    ),
    class = "group_tna_stability"
  )
}

#' @export
#' @rdname estimate_centrality_stability
estimate_centrality_stability.group_tna <- estimate_cs.group_tna

# Centrality measure function wrappers ----------------------------------------

centrality_funs <- list()

centrality_funs$OutStrength <- function(g, ...) {
  igraph::strength(g, mode = "out")
}

centrality_funs$InStrength <- function(g, ...) {
  igraph::strength(g, mode = "in")
}

centrality_funs$ClosenessIn <- function(g, w, ...) {
  igraph::closeness(g, mode = "in", weights = w)
}

centrality_funs$ClosenessOut <- function(g, w, ...) {
  igraph::closeness(g, mode = "out", weights = w)
}

centrality_funs$Closeness <- function(g, w, ...) {
  igraph::closeness(g, mode = "all", weights = w)
}

centrality_funs$Betweenness <- function(g, w, ...) {
  igraph::betweenness(g, weights = w)
}

centrality_funs$BetweennessRSP <- function(x, ...) {
  rsp_bet(x)
}

centrality_funs$Diffusion <- function(x, ...) {
  diffusion(x)
}

centrality_funs$Clustering <- function(x, ...) {
  wcc(x + t(x))
}

test_that("centralities computes correctly for a tna object", {
  result <- centralities(mock_tna)
  expect_s3_class(result, "tna_centralities")
  expect_true(all(names(centrality_funs) %in% colnames(result)))
})

test_that("centralities handles loops correctly in a tna object", {
  tna_model <- mock_tna
  result_tna <- centralities(tna_model, loops = FALSE)
  diag(tna_model$weights) <- 0
  result_manual <- centralities(tna_model, loops = TRUE)
  expect_equal(result_tna$OutStrength, result_manual$OutStrength)
  expect_equal(result_tna$InStrength, result_manual$InStrength)
})

test_that("centralities normalizes correctly for a tna object", {
  result_tna <- centralities(mock_tna, normalize = TRUE)
  result_manual <- centralities(mock_tna, normalize = FALSE)
  result_manual[, -1] <- apply(result_manual[, -1], 2, ranger)
  expect_equal(result_tna, result_manual)
})

test_that("centralities can be computed for a matrix", {
  expect_error(
    result_mat <- centralities(mock_matrix),
    NA
  )
})

test_that("centrality stability can be estimated", {
  model <- tna(mock_sequence)
  expect_error(
    estimate_cs(model, drop_prop = seq(0.3, 0.9, by = 0.1), iter = 20),
    NA
  )
})

test_that("centralities can be computed for clusters", {
  expect_error(
    centralities(mmm_model),
    NA
  )
})

test_that("centrality stability can be estimated for clusters", {
  expect_error(
    estimate_cs(
      mmm_model,
      drop_prop = seq(0.3, 0.9, by = 0.1),
      iter = 10
    ),
    NA
  )
})

test_that("progressbar works", {
  expect_error(
    suppressMessages(
      estimate_cs(
        mmm_model,
        drop_prop = seq(0.3, 0.9, by = 0.1),
        iter = 10,
        progressbar = TRUE
      )
    ),
    NA
  )
})

test_that("no cases dropped warns", {
  expect_warning(
    estimate_cs(
      mmm_model[[1]],
      drop_prop = 0.001,
      iter = 10
    ),
    "No cases dropped for proportion 0\\.001\\. Skipping\\.\\.\\."
  )
})

test_that("RSP is NA with no transitions", {
  mat <- mock_matrix
  mat[1, ] <- 0
  expect_equal(rsp_bet(mat), NA)
})

list(
  rd_family_title = list(
    basic = "Basic functions",
    centralities = "Centrality measure functions",
    communities = "Community detection functions",
    cliques = "Clique-related functions",
    comparison = "Model comparison functions",
    validation = "Validation functions",
    clusters = "Cluster-related functions",
    helpers = "Helper functions",
    datasets = "Datasets"
  )
)

#' Build a Transition Network Analysis Model
#'
#' Construct a transition network analysis (TNA) model from sequence data.
#' The function takes a data set of sequence of events or states as input and
#' builds a TNA model. It extracts the edge weights and initial probabilities
#' from the data along with the state labels. The function also accepts weight
#' matrices and initial state probabilities directly.
#'
#' @export
#' @family basic
#' @rdname build_model
#' @param x A `stslist` (from `TraMineR`), `data.frame`, a `matrix`, or
#'   a `tna_data` object (see [prepare_data()]).
#'   For `stslist` and `data.frame` objects `x`
#'   should describe a sequence of events or states to be used for building the
#'   Markov model.  If `x` is a matrix, it is assumed that the element on row
#'   `i` and column `j` is the weight of the edge representing the transition
#'   from state `i` to state `j`. If `x` is a `data.frame`, then
#'   it must be in wide format (see `cols` on how to define columns for the
#'   time points).
#' @param type A `character` string describing the weight matrix type.
#'   Currently supports the following types:
#'
#'   * `"relative"` for relative frequencies (probabilities, the default)
#'   * `"frequency"` for frequencies.
#'   * `"co-occurrence"` for co-occurrences.
#'   * `"n-gram"` for n-gram transitions. Captures higher-order transitions by
#'       considering sequences of n states, useful for identifying longer
#'       patterns.
#'   * `"gap"` allows transitions between non-adjacent states, with
#'       transitions weighted by the gap size.
#'   * `"reverse"` considers the sequences in reverse order
#'       (resulting in what is called a reply network in some contexts).
#'       The resulting weight matrix is the transpose of the `"frequency"`
#'       option.
#'   * `"attention"` aggregates all downstream pairs of states with an
#'       exponential decay for the gap between states. The parameter `lambda`
#'       can be used to control the decay rate (the default is 1)-
#'
#' @param scaling A `character` vector describing how to scale the weights
#'   defined by `type`. When a vector is provided, the scaling options are
#'   applied in the respective order. For example, `c("rank", "minmax")` would
#'   first compute the ranks, then scale them to the unit interval using
#'   min-max normalization. An empty vector corresponds to no scaling.
#'   Currently supports the following options:
#'
#'   * `"minmax"` performs min-max normalization to scale the weights to the
#'       unit interval. Note that if the smallest weight is positive, it
#'       will be zero after scaling.
#'   * `"max"` Multiplies the weights by the reciprocal of the largest weight
#'       to scale the weights to the unit interval. This options preserves
#'       positive ranks, unlike `"minmax"` when all weights are positive.
#'   * `"rank"` Computes the ranks of the weights using [base::rank()] with
#'       `ties.method = "average"`.
#'
#' @param cols An `expression` giving a tidy selection of columns that should
#'   be considered as sequence data. By default, all columns are used.
#' @param params A `list` of additional arguments for models of specific
#'   `type`. The potential elements of this list are:
#'
#'   * `n_gram`: An `integer` for n-gram transitions specifying the number of
#'     adjacent events. The default value is 2.
#'   * `max_gap`: An `integer` for the gap-allowed transitions specifying the
#'     largest allowed gap size. The default is 1.
#'   * `windowed`: Perform the model estimation by window. Supported for
#'     `relative`, `frequency` and `co-occurrence` `type`s.
#'   * `window_size`: An `integer` for the sliding window transitions
#'     specifying the window size. The default is 2.
#'   * `window_type`: A `character` string that defines the window type.
#'     Either `"rolling"` or `"tumbling"`.
#'   * `weighted`: A `logical` value. If `TRUE`, the transitions
#'     are weighted by the inverse of the sequence length. Can be used for
#'     frequency, co-occurrence and reverse model types. The default is
#'      `FALSE`.
#'   * `direction`: A `character` string specifying the direction of attention
#'     for models of `type = "attention"`. The available options are
#'     `"backward"`, `"forward"`, and `"both"`, for backward attention,
#'     forward attention, and bidirectional attention, respectively.
#'     The default is `"forward"`.
#'   * `decay`: A `function` that specifies the decay of the weights between
#'     two time points at a specific distance. The function should take three
#'     arguments: `i`, `j` and `lambda`, where `i` and `j` are `numeric`
#'     vectors of time values, and `lambda` is  a `numeric` value for the
#'     decay rate. The function should return a `numeric` vector of weights.
#'     The default is `function(i, j, lambda) exp(-abs(i - j) / lambda)`.
#'   * `lambda`: A `numeric` value for the decay rate. The default is 1.
#'   * `time`: A `matrix` or a `data.frame` providing the time values
#'     for each sequence and at time index. For `tna_data` objects, this can
#'     also be a logical value, where `TRUE` will use the `time_data` element
#'     of `x` for the time values. `Date` values are converted to `numeric`.
#'   * `duration`: A `matrix` or a `data.frame` providing the
#'     time spent in each state for each sequence and time index.
#'     This is an alternative to `time`.
#'
#' @param concat An `integer` for the number of consecutive sequences
#'   to concatenate. The default is 1 (no concatenation).
#' @param inits An optional `numeric` vector of initial state probabilities
#'   for each state. The vector will be scaled to unity.
#' @param begin_state A `character` string for an additional begin state.
#'   This state is added as the first observation for every sequence to
#'   signify the beginning of the sequence
#' @param end_state A `character` string for an additional end state.
#'   This state is added as the last observation for every sequence to
#'   signify the end of the sequence.
#' @param ... Ignored. For the `build_model` aliases (e.g., `tna`), this
#'   argument matches the actual arguments to `build_model` beside `x`.
#' @return An object of class `tna` which is a `list` containing the
#'   following elements:
#'
#'   * `weights`: An adjacency `matrix` of the model (weight matrix).
#'   * `inits`: A `numeric` vector of initial values for each state.
#'     For `matrix` type `x`, this element will be `NULL` if `inits` is not
#'     directly provided
#'   * `labels`: A `character` vector of the state labels, or `NULL` if
#'     there are no labels.
#'   * `data`: The original sequence data that has been converted to an
#'     internal format used by the package when `x` is a `stslist` or a
#'     `data.frame` object. Otherwise `NULL`.
#'
#' @examples
#' model <- build_model(group_regulation)
#' print(model)
#'
build_model <- function(x, ...) {
  UseMethod("build_model")
}

#' @export
#' @rdname build_model
build_model.default <- function(x, type = "relative", scaling = character(0L),
                                params = list(), inits, ...) {
  check_missing(x)
  check_dots(...)
  x <- try_(as.matrix(x))
  stopifnot_(
    !inherits(x, "try-error"),
    "Argument {.arg x} must be coercible to a {.cls matrix}."
  )
  build_model.matrix(
    x = x,
    type = type,
    scaling = scaling,
    params = params,
    inits = inits
  )
}

#' @export
#' @rdname build_model
build_model.matrix <- function(x, type = "relative", scaling = character(0L),
                               params = list(), inits, ...) {
  check_missing(x)
  check_dots(...)
  x <- try_(data.matrix(x))
  stopifnot_(
    !inherits(x, "try-error"),
    "Argument {.arg x} must be coercible to {.cls numeric}."
  )
  check_na(x)
  nc <- ncol(x)
  stopifnot_(
    nc == nrow(x),
    "Argument {.arg x} must be a square {.cls matrix}."
  )
  stopifnot_(
    nc >= 2L,
    "Argument {.arg x} must have at least two columns."
  )
  if (!missing(inits)) {
    stopifnot_(
      length(inits) >= nc,
      "Argument {.arg inits} must provide initial probabilities for all states."
    )
    inits <- try_(as.numeric(inits))
    stopifnot_(
      !inherits(inits, "try-error"),
      "Argument {.arg inits} must be coercible to {.cls numeric}."
    )
    stopifnot_(
      all(inits >= 0),
      "All elements of {.arg inits} must be non-negative."
    )
    if (length(inits) > ncol(x)) {
      warning_(
        c(
          "Argument {.arg inits} contains more values
           than the number of states.",
          `i` = "Only the first {nc} values will be used."
        )
      )
      inits <- inits[seq_len(nc)]
      inits <- inits / sum(inits)
    }
    names(inits) <- colnames(x)
  }
  type <- check_model_type(type)
  scaling <- check_model_scaling(scaling)
  x <- check_weights(x, type = type)
  x <- scale_weights(x, type = type, scaling = scaling, a = nc)
  if (is.null(colnames(x))) {
    dimnames(x) <- list(seq_len(nc), seq_len(nc))
  }
  build_model_(
    weights = x,
    inits = inits,
    labels = colnames(x),
    type = type,
    scaling = scaling
  )
}

#' @export
#' @rdname build_model

build_model.stslist <- function(x, type = "relative", scaling = character(0L),
                                cols = tidyselect::everything(),
                                params = list(), concat = 1L,
                                begin_state, end_state, ...) {
  check_missing(x)
  check_dots(...)
  check_class(x, "stslist")
  type <- check_model_type(type)
  scaling <- check_model_scaling(scaling)
  cols <- get_cols(rlang::enquo(cols), x)
  x <- create_seqdata(
    x = x,
    cols = cols,
    concat = concat,
    begin_state = begin_state,
    end_state = end_state
  )
  model <- initialize_model(x, type, scaling, params)
  build_model_(
    weights = model$weights,
    inits = model$inits,
    labels = attr(x, "labels"),
    type = type,
    scaling = scaling,
    data = x,
    params = params
  )
}

#' @export
#' @rdname build_model
build_model.data.frame <- function(x, type = "relative",
                                   scaling = character(0L),
                                   cols = tidyselect::everything(),
                                   concat = 1L, params = list(),
                                   begin_state, end_state, ...) {
  check_missing(x)
  check_class(x, "data.frame")
  check_dots(...)
  type <- check_model_type(type)
  scaling <- check_model_scaling(scaling)
  cols <- get_cols(rlang::enquo(cols), x)
  params$window_span <- attr(x, "window_span")
  params$window_size <- attr(x, "window_size")
  params$windowed <- !is.null(params$window_size) ||
    !is.null(params$window_span)
  x <- create_seqdata(
    x = x,
    cols = cols,
    concat = concat,
    begin_state = begin_state,
    end_state = end_state
  )
  model <- initialize_model(x, type, scaling, params)
  build_model_(
    weights = model$weights,
    inits = model$inits,
    labels = model$labels,
    type = type,
    scaling = scaling,
    data = x,
    params = params
  )
}

#' @export
#' @rdname build_model
build_model.tna_data <- function(x, type = "relative", scaling = character(0L),
                                 params = list(), concat = 1L,
                                 begin_state, end_state, ...) {
  check_missing(x)
  check_class(x, "tna_data")
  check_dots(...)
  type <- check_model_type(type)
  scaling <- check_model_scaling(scaling)
  wide <- cbind(x$sequence_data, x$meta_data)
  if (isTRUE(params$time)) {
    stopifnot_(
      !is.null(x$time_data),
      "No time data available in argument {.arg x}."
    )
    params$time <- as.matrix(
      as.data.frame(
        lapply(
          x$time_data,
          as.numeric
        )
      )
    )
  }
  x <- create_seqdata(
    x = wide,
    cols = names(x$sequence_data),
    concat = concat,
    begin_state = begin_state,
    end_state = end_state
  )
  model <- initialize_model(x, type, scaling, params)
  build_model_(
    weights = model$weights,
    inits = model$inits,
    labels = model$labels,
    type = type,
    scaling = scaling,
    data = x,
    params = params
  )
}

#' @export
#' @rdname build_model
build_model.tsn <- function(x, type = "relative", scaling = character(0L),
                            params = list(), concat = 1L,
                            begin_state, end_state, ...) {
  check_missing(x)
  check_class(x, "tsn")
  check_dots(...)
  type <- check_model_type(type)
  scaling <- check_model_scaling(scaling)
  wide <- x |>
    dplyr::select(
      c(!!rlang::sym("id"), !!rlang::sym("state"), !!rlang::sym("time"))
    ) |>
    tidyr::pivot_wider(
      id_cols = "id",
      values_from = !!rlang::sym("state"),
      names_from = !!rlang::sym("time"),
      names_prefix = "T"
    ) |>
    dplyr::select(!(!!rlang::sym("id")))
  x <- create_seqdata(
    x = wide,
    cols = names(wide),
    concat = concat,
    begin_state = begin_state,
    end_state = end_state
  )
  model <- initialize_model(x, type, scaling, params)
  build_model_(
    weights = model$weights,
    inits = model$inits,
    labels = model$labels,
    type = type,
    scaling = scaling,
    data = x,
    params = params
  )
}

#' #' @export
#' #' @rdname build_model
#' build_model.tsn_ews <- function(x, type = "relative", scaling = character(0L),
#'                                 params = list(), concat = 1L,
#'                                 begin_state, end_state, ...) { # nocov start
#'   check_missing(x)
#'   check_class(x, "tsn_ews")
#'   check_dots(...)
#'   type <- check_model_type(type)
#'   scaling <- check_model_scaling(scaling)
#'   id <- attr(x, "id_col")
#'   cls <- attr(x, "classification")
#'   wide <- cls |>
#'     dplyr::select(
#'       c(!!rlang::sym("state"), !!rlang::sym("time"))
#'     ) |>
#'     tidyr::pivot_wider(
#'       values_from = !!rlang::sym("state"),
#'       names_from = !!rlang::sym("time"),
#'       names_prefix = "T"
#'     )
#'   x <- create_seqdata(
#'     x = wide,
#'     cols = seq_len(ncol(wide)),
#'     concat = concat,
#'     begin_state = begin_state,
#'     end_state = end_state
#'   )
#'   model <- initialize_model(x, type, scaling, params)
#'   build_model_(
#'     weights = model$weights,
#'     inits = model$inits,
#'     labels = model$labels,
#'     type = type,
#'     scaling = scaling,
#'     data = x,
#'     params = params
#'   )
#' } # nocov end

# Aliases -----------------------------------------------------------------

#' @export
#' @rdname build_model
#' @examples
#' model <- tna(group_regulation)
#'
tna <- function(x, ...) {
  check_missing(x)
  build_model(x = x, type = "relative", ...)
}

#' @export
#' @rdname build_model
#' @examples
#' model <- ftna(group_regulation)
#'
ftna <- function(x, ...) {
  build_model(x = x, type = "frequency", ...)
}

#' @export
#' @rdname build_model
#' @examples
#' model <- ctna(group_regulation)
#'
ctna <- function(x, ...) {
  build_model(x = x, type = "co-occurrence", ...)
}

#' @export
#' @rdname build_model
#' @examples
#' model <- atna(group_regulation)
#'
atna <- function(x, ...) {
  build_model(x = x, type = "attention", ...)
}

#' @export
#' @rdname build_model
tsn <- function(x, ...) {
  build_model.tsn(x = x, ...)
}

#' Build a Social Network Analysis Model
#'
#' @export
#' @param x A `data.frame` or a `matrix` with three columns: the first two
#' representing the states and the third giving the weights.
#' @param aggregate A `function` to use for aggregating the weights. The
#' default is [sum()].
#' @param ... Additional arguments passed to `aggregate`.
#' @return A `tna` object representing the model.
#' @examples
#' set.seed(123)
#' d <- data.frame(
#'   from = sample(LETTERS[1:4], 100, replace = TRUE),
#'   to = sample(LETTERS[1:4], 100, replace = TRUE),
#'   weight = rexp(100)
#' )
#' model <- sna(d)
#'
sna <- function(x, aggregate = sum, ...) {
  check_missing(x)
  x <- try_(as.data.frame(x))
  stopifnot_(
    !inherits(x, "try-error"),
    "Argument {.arg x} must be coercible to a {.cls data.frame}."
  )
  check_na(x)
  nc <- ncol(x)
  stopifnot_(
    nc == 3L,
    "Argument {.arg x} must have three columns (from, to, weight)."
  )
  colnames(x) <- c("from", "to", "weight")
  stopifnot_(
    is.function(aggregate),
    "Argument {.arg aggregate} must be a function."
  )
  test_aggregate <- try_(aggregate(stats::runif(3L), ...))
  stopifnot_(
    !inherits(test_aggregate, "try-error") &&
      length(test_aggregate) == 1L &&
      is.numeric(test_aggregate),
    "Argument {.arg aggregate} must be a function that takes a {.cls numeric}
    vector and returns a single {.cls numeric} value."
  )
  x <- x |>
    dplyr::group_by(!!rlang::sym("from"), !!rlang::sym("to")) |>
    dplyr::summarize(weight = aggregate(!!rlang::sym("weight"), ...))
  lab <- unique(unlist(x[, c("from", "to")]))
  n <- length(lab)
  out <- matrix(0.0, n, n, dimnames = list(lab, lab))
  idx <- cbind(match(x$from, lab), match(x$to, lab))
  out[idx] <- x$weight
  build_model_(
    weights = out,
    labels = lab,
    type = "",
    scaling = character(0L)
  )
}

# Internal ----------------------------------------------------------------

#' Build a Transition Network Analysis object
#'
#' @param weights A `matrix` of edge weights.
#' @param inits A `numeric` vector of initial state probabilities.
#' @param labels A `character` vector of state labels.
#' @param type A `character` string defining the network type.
#' @param scaling A `character` string defining the scaling of the weights
#' @param data A `tna_seqdata` object when `weights` is
#'   created from sequence data.
#' @param params A `list` of parameters for computing the transitions
#' @return A `tna` object.
#' @noRd
build_model_ <- function(weights, inits = NULL, labels = NULL,
                         type = NULL, scaling = character(0L), data = NULL,
                         params = NULL) {
  structure(
    list(
      weights = weights,
      inits = onlyif(!missing(inits), inits),
      labels = labels,
      data = data
    ),
    type = type,
    scaling = scaling,
    params = params,
    class = "tna"
  )
}

#' Convert Sequence Data to an Internal Format
#'
#' @param x A `data.frame` or a `stslist` object.
#' @param cols An `character` vector of column names.
#' @param alphabet Optional `character` vector of the alphabet.
#' @param concat How many consecutive sequences should be concatenated?
#' @param begin_state Optional `character` string giving the begin state.
#' @param end_state Optional `character` string giving the end state.
#' @noRd
create_seqdata <- function(x, cols, alphabet, concat = 1L,
                           begin_state, end_state) {
  cols <- which(names(x) %in% cols)
  if (inherits(x, "stslist")) {
    alphabet <- attr(x, "alphabet")
    labels <- attr(x, "labels")
    colors <- ifelse_(
      is.null(attr(x, "cpal")),
      color_palette(length(labels)),
      attr(x, "cpal")
    )
    x <- as.data.frame(x)
  } else if (is.data.frame(x)) {
    if (missing(alphabet)) {
      vals <- sort(unique(unlist(x[, cols])))
      alphabet <- labels <- vals[!is.na(vals)]
    } else {
      labels <- alphabet
    }
    colors <- color_palette(length(labels))
    x[, cols] <- as.data.frame(
      lapply(x[, cols], function(y) factor(y, levels = alphabet))
    )
  }
  x <- as.matrix(
    as.data.frame(
      lapply(
        x[, cols],
        function(y) {
          as.integer(replace(y, which(!y %in% alphabet), NA))
        }
      )
    )
  )
  if (concat > 1L) {
    n <- nrow(x)
    k <- ncol(x)
    m <- n %/% concat
    modulus <- n %% concat
    extra <- modulus > 0L
    x_cc <- matrix(NA_integer_, nrow = m + extra, ncol = concat * ncol(x))
    for (i in seq_len(m)) {
      idx <- seq((i - 1L) * concat + 1L, i * concat)
      x_cc[i, ] <- c(t(x[idx, , drop = FALSE]))
    }
    if (extra) {
      idx <- seq(1L, k)
      for (i in seq_len(modulus)) {
        x_cc[m + 1L, idx] <- x[m * concat + i, , drop = FALSE]
        idx <- idx + k
      }
    }
    x <- x_cc
  }
  if (!missing(begin_state)) {
    x <- cbind(1L, x + 1L)
    alphabet <- c(begin_state, alphabet)
    labels <- c(begin_state, labels)
    colors <- c("darkgray", colors)
  }
  if (!missing(end_state)) {
    last_obs <- max.col(!is.na(x), ties.method = "last")
    new_max <- max(x, na.rm = TRUE) + 1L
    if (any(last_obs == ncol(x))) {
      x <- cbind(x, NA_integer_)
      idx <- cbind(seq_len(nrow(x)), last_obs + 1L)
      x[idx] <- new_max
    }
    alphabet <- c(alphabet, end_state)
    labels <- c(labels, end_state)
    colors <- c(colors, "darkgray")
  }
  structure(
    x,
    class = c("tna_seq_data", "matrix", "array"),
    alphabet = alphabet,
    labels = labels,
    colors = colors
  )
}


#' Compute Edge Weights from Sequence Data
#'
#' @param x A data object from `create_seqdata()`
#' @param type The type of transition network model to build.
#' @param scaling The scaling methods to apply to the weights.
#' @param params A `list` of parameters for the transition model.
#' @param transitions Should the individual-level transitions also be returned?
#' Defaults to `FALSE`.
#' @noRd
initialize_model <- function(x, type, scaling, params, transitions = FALSE) {
  alphabet <- attr(x, "alphabet")
  labels <- attr(x, "labels")
  a <- length(alphabet)
  inits <- factor(x[, 1L], levels = seq_len(a), labels = alphabet)
  inits <- as.vector(table(inits))
  trans <- compute_transitions(x, a, type, params)
  weights <- compute_weights(trans, type, scaling, a)
  inits <- inits / sum(inits)
  names(inits) <- alphabet
  dimnames(weights) <- list(alphabet, alphabet)
  list(
    weights = weights,
    inits = inits,
    labels = labels,
    trans = onlyif(transitions, trans)
  )
}

#' Compute Network Transitions Based on TNA Type
#'
#' @param m A `matrix` of sequences
#' @param a An `integer`, the number of states.
#' @param type Type of the transition network as a `character` string.
#' @param params Parameters for the transition model.
#' @noRd
compute_transitions <- function(m, a, type, params) {
  if (isTRUE(params$windowed)) {
    return(compute_transitions_windowed(m, a, type, params))
  }
  n <- nrow(m)
  p <- ncol(m)
  idx <- seq_len(n)
  trans <- array(0L, dim = c(n, a, a))
  seq_lengths <- .rowSums(!is.na(m), m = n, n = p)
  weight <- ifelse_(isTRUE(params$weighted), 1.0 / seq_lengths, rep(1L, n))
  if (type %in% c("relative", "frequency")) {
    weight <- ifelse_(type == "frequency", weight, rep(1L, n))
    for (i in seq_len(p - 1L)) {
      from <- m[, i]
      to <- m[, i + 1L]
      any_na <- is.na(from) | is.na(to)
      new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
      trans[new_trans] <- trans[new_trans] + weight[!any_na]
    }
  } else if (type == "reverse") {
    for (i in seq_len(p - 1L)) {
      from <- m[, i + 1L]
      to <- m[, i]
      any_na <- is.na(from) | is.na(to)
      new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
      trans[new_trans] <- trans[new_trans] + weight[!any_na]
    }
  } else if (type == "co-occurrence") {
    for (i in seq_len(p - 1L)) {
      for (j in seq(i + 1L, p)) {
        from <- m[, i]
        to <- m[, j]
        any_na <- is.na(from) | is.na(to)
        new_trans <- rbind(
          cbind(idx, from, to)[!any_na, , drop = FALSE],
          cbind(idx, to, from)[!any_na, , drop = FALSE]
        )
        trans[new_trans] <- trans[new_trans] + weight[!any_na]
      }
    }
  } else if (type == "n-gram") {
    n_gram <- params$n_gram %||% 2L
    for (i in seq_len(p - n_gram + 1L)) {
      for (j in seq(i, i + n_gram - 2L)) {
        from <- m[, j]
        to <- m[, j + 1L]
        any_na <- is.na(from) | is.na(to)
        new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
        trans[new_trans] <- trans[new_trans] + 1L
      }
    }
  } else if (type == "gap") {
    max_gap <- params$max_gap %||% 1L
    for (i in seq_len(p - 1L)) {
      max_j <- min(i + max_gap + 1L, p)
      for (j in seq(i + 1L, max_j)) {
        from <- m[, i]
        to <- m[, j]
        any_na <- is.na(from) | is.na(to)
        new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
        trans[new_trans] <- trans[new_trans] + 1.0 / (j - i)
      }
    }
  } else if (type == "attention") {
    lambda <- params$lambda %||% 1.0
    decay <- params$decay %||% function(i, j, lambda) exp(-abs(i - j) / lambda)
    direction <- params$direction %||% "forward"
    stopifnot_(
      is.null(params$time) || is.null(params$duration),
      "Both {.arg time} and {.arg duration} supplied via {.arg params}."
    )
    time <- matrix(rep(seq(1, p), each = n), nrow = n, ncol = p)
    if (!is.null(params$time)) {
      stopifnot_(
        nrow(params$time) == n && ncol(params$time) == p,
        "Argument {.arg params$time} must have the same dimensions as the
         sequence data."
      )
      time <- params$time
    }
    if (!is.null(params$duration)) {
      stopifnot_(
        nrow(params$duration) == n && ncol(params$duration) == p,
        "Argument {.arg params$duration} must have the same dimensions as the
         sequence data."
      )
      time <- cbind(0, t(apply(params$duration, 1L, cumsum))[, -p])
    }
    for (i in seq_len(p)) {
      if (direction %in% c("forward", "both")) {
        if (i < p) {
          for (j in seq(i + 1L, p)) {
            from <- m[, i]
            to <- m[, j]
            any_na <- is.na(from) | is.na(to)
            new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
            d <- decay(time[, i], time[, j], lambda)[!any_na]
            trans[new_trans] <- trans[new_trans] + d
          }
        }
      }
      if (direction %in% c("backward", "both")) {
        if (i > 1) {
          for (j in seq(1L, i)) {
            from <- m[, i]
            to <- m[, j]
            any_na <- is.na(from) | is.na(to)
            new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
            d <- decay(time[, i], time[, j], lambda)[!any_na]
            trans[new_trans] <- trans[new_trans] + d
          }
        }
      }
    }
  }
  trans
}

compute_transitions_windowed <- function(m, a, type, params) {
  n <- nrow(m)
  p <- ncol(m)
  idx <- seq_len(n)
  trans <- array(0L, dim = c(n, a, a))
  seq_lengths <- .rowSums(!is.na(m), m = n, n = p)
  weight <- ifelse_(isTRUE(params$weighted), 1.0 / seq_lengths, rep(1L, n))
  window_size <- params$window_size %||% 2L
  window_span <- params$window_span %||% 1L
  window_size <- window_size * window_span
  divides <- p %% window_size == 0L
  q <- p %/% window_size - 1L * divides
  if (type %in% c("relative", "frequency")) {
    for (i in seq_len(q)) {
      j_idx <- seq(
        (i - 1) * window_size + 1L,
        i * window_size
      )
      for (j in j_idx) {
        from <- m[, j]
        k_idx <- seq(
          i * window_size + 1L,
          min(p, (i + 1) * window_size)
        )
        for (k in k_idx) {
          to <- m[, k]
          any_na <- is.na(from) | is.na(to)
          new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
          trans[new_trans] <- trans[new_trans] + 1L
        }
      }
    }
  } else if (type == "co-occurrence") {
    for (i in seq_len(q + 1)) {
      j_idx <- seq(
        (i - 1) * window_size + 1L,
        min(p, i * window_size)
      )
      for (j in j_idx) {
        from <- m[, j]
        k_idx <- seq(
          (i - 1) * window_size + 1L,
          min(p, i * window_size)
        )
        for (k in k_idx) {
          to <- m[, k]
          any_na <- is.na(from) | is.na(to)
          new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
          trans[new_trans] <- trans[new_trans] + 1L
        }
      }
    }
    # for (i in seq_len(q)) {
    #   for (j in seq(i + 1, q + 1)) {
    #     k_idx <- seq(
    #       (i - 1) * window_size + 1L,
    #       i * window_size
    #     )
    #     for (k in k_idx) {
    #       from <- m[, k]
    #       l_idx <- seq(
    #         (j - 1) * window_size + 1L,
    #         min(p, j * window_size)
    #       )
    #       for (l in l_idx) {
    #         to <- m[, l]
    #         any_na <- is.na(from) | is.na(to)
    #         new_trans <- rbind(
    #           cbind(idx, from, to)[!any_na, , drop = FALSE],
    #           cbind(idx, to, from)[!any_na, , drop = FALSE]
    #         )
    #         trans[new_trans] <- trans[new_trans] + weight[!any_na]
    #       }
    #     }
    #   }
    # }
  }
  trans
}

# Internal function to get all transitions
get_transitions <- function(x) { # nocov start
  m <- x$data
  lab <- x$labels
  n <- nrow(m)
  p <- ncol(m)
  idx <- seq_len(n)
  trans <- vector(mode = "list", length = p - 1L)
  seq_lengths <- .rowSums(!is.na(m), m = n, n = p)
  for (i in seq_len(p - 1L)) {
    from <- m[, i]
    to <- m[, i + 1L]
    any_na <- is.na(from) | is.na(to)
    trans[[i]] <- as.data.frame(
      cbind(idx, from, to)[!any_na, , drop = FALSE]
    )
    names(trans[[i]]) <- c("id", "source", "target")
  }
  out <- dplyr::bind_rows(trans, .id = "time")
  out$source <- lab[out$source]
  out$target <- lab[out$target]
  out$weight <- 1.0
  out
} # nocov end

#' Compute Network Weights Based On TNA Type
#'
#' @param transitions An `array` of the individual-level transitions.
#' @param type Type of the transition network as a `character` string.
#' @param scaling Scaling methods to apply as a `character` vector.
#' @param a An `integer`, the number of states.
#' @return A `matrix` of transition probabilities or frequencies,
#' based on `type`.
#' @noRd
compute_weights <- function(transitions, type, scaling, a) {
  weights <- apply(transitions, c(2, 3), sum)
  scale_weights(weights, type, scaling, a)
}

#' Scale Transition Network Weights
#'
#' @param weights A `matrix` of edge weights
#' @param type Type of the transition network as a `character` string.
#' @param scaling Scaling methods to apply as a `character` vector.
#' @param a An `integer`, the number of states.
#' @noRd
scale_weights <- function(weights, type, scaling, a) {
  if (type == "relative" || "probability" %in% scaling) {
    rs <- .rowSums(weights, m = a, n = a)
    pos <- which(rs > 0)
    weights[pos, ] <- weights[pos, ] / rs[pos]
    weights[!pos, ] <- NA
  }
  for (i in seq_along(scaling)) {
    if (scaling[i] == "minmax") {
      weights[] <- ranger(weights)
    } else if (scaling[i] == "max") {
      weights[] <- weights / max(weights, na.rm = TRUE)
    } else if (scaling[i] == "rank") {
      weights[] <- rank(weights, ties.method = "average")
    }
  }
  weights
}

#' Get Network Node Count
#'
#' @param x A `tna` or a `group_tna` object, or a weight `matrix`.
#' @noRd
nodes <- function(x) {
  if (is_tna(x)) {
    dim(x$weights)[2L]
  } else if (inherits(x, "group_tna")) {
    dim(x[[1L]]$weights)[2L]
  } else {
    ncol(x)
  }
}

test_that("tna works with matrix data with inits", {
  inits <- c(0.25, 0.25, 0.25, 0.25)
  tna_model <- tna(mock_matrix, inits = inits)
  expect_s3_class(tna_model, "tna")
  expect_true(is.matrix(tna_model$weights))
  expect_true(is.vector(tna_model$inits))
})

test_that("tna fails with non-square matrix", {
  trans_matrix <- matrix(c(0.1, 0.2, 0.0, 0.0, 0.2, 0.3), nrow = 2, ncol = 3)
  expect_error(
    tna(trans_matrix),
    "Argument `x` must be a square <matrix>"
  )
})

test_that("tna fails with too few inits", {
  expect_error(
    tna(mock_matrix, inits = c(0.1, 0.2, 0.3)),
    "Argument `inits` must provide initial probabilities for all states."
  )
})

test_that("single element matrix fails", {
  expect_error(
    build_model.matrix(x = 0L),
    "Argument `x` must have at least two columns"
  )
})

test_that("non-square matrix fails", {
  expect_error(
    build_model.matrix(x = matrix(0, 3, 2)),
    "Argument `x` must be a square <matrix>"
  )
})

test_that("non-coercible arguments fail", {
  expect_error(
    tna(x = identity),
    "Argument `x` must be coercible to a <matrix>"
  )
})

test_that("tna warns with too many inits", {
  expect_warning(
    tna(mock_matrix, inits = c(0.1, 0.2, 0.3, 0.4, 0.5)),
    paste0(
      "Argument `inits` contains more values than the number of states\\.\n",
      "i Only the first 4 values will be used\\."
    )
  )
})

test_that("tna handles missing x argument", {
  expect_error(tna(), "Argument `x` is missing.")
})

test_that("tna handles default case", {
  expect_error(build_model.default(mock_matrix), NA)
})

test_that("unnamed matrix gains dimnames", {
  mat <- mock_matrix
  dimnames(mat) <- NULL
  model <- tna(mat)
  expect_equal(
    dimnames(model$weights),
    list(as.character(1:4), as.character(1:4))
  )
})

test_that("tna aliases work", {
  expect_error(ftna(mock_freq_matrix), NA)
  expect_error(ctna(mock_sequence), NA)
  expect_error(atna(mock_sequence), NA)
})

test_that("scaling options work", {
  model_minmax <- tna(mock_freq_matrix, scaling = "minmax")
  expect_equal(
    range(model_minmax$weights),
    c(0, 1)
  )
  model_max <- tna(mock_freq_matrix, scaling = "max")
  expect_equal(
    model_max$weights,
    model_minmax$weights
  )
  model_rank <- tna(mock_matrix, scaling = "rank")
  expect_equal(
    sort(model_rank$weights),
    seq_len(prod(dim(mock_matrix)))
  )
})

test_that("model summary can be extracted", {
  model <- tna(mock_sequence)
  expect_error(
    summary(model),
    NA
  )
})

test_that("igraph conversion works", {
  model <- tna(mock_sequence)
  expect_error(
    as.igraph(model),
    NA
  )
})

test_that("igraph conversion works for clusters", {
  expect_error(
    as.igraph(mmm_model, which = 1),
    NA
  )
})

test_that("different model types work", {
  expect_error(
    build_model(mock_sequence, type = "relative"),
    NA
  )
  expect_error(
    build_model(mock_sequence, type = "frequency"),
    NA
  )
  expect_error(
    build_model(mock_sequence, type = "co-occurrence"),
    NA
  )
  expect_error(
    build_model(mock_sequence, type = "reverse"),
    NA
  )
  expect_error(
    build_model(mock_sequence, type = "n-gram"),
    NA
  )
  expect_error(
    build_model(mock_sequence, type = "window"),
    NA
  )
  expect_error(
    build_model(mock_sequence, type = "gap"),
    NA
  )
  expect_error(
    build_model(mock_sequence, type = "attention"),
    NA
  )
})

test_that("models can be constructed from tna_data objects", {
  expect_error(
    tna(mock_tna_data),
    NA
  )
})

test_that("log-sum-exp is correct", {
  x <- c(1.05, 1.27, 1.33, -1.7, -1.34, -sqrt(2), sqrt(3))
  expect_equal(
    log_sum_exp(x),
    log(sum(exp(x)))
  )
})

test_that("number of nodes is correct", {
  expect_equal(nodes(mock_tna), 4)
  expect_equal(nodes(mmm_model), 3)
  expect_equal(nodes(mock_matrix), 4)
})

test_that("sna works", {
  set.seed(123)
  d <- data.frame(
    from = sample(LETTERS[1:4], 10, replace = TRUE),
    to = sample(LETTERS[1:4], 10, replace = TRUE),
    weight = rexp(10)
  )
  expect_error(
    sna(d),
    NA
  )
  expect_error(
    sna(d, aggregate = mean),
    NA
  )
})

test_that("sna fails with incorrect aggregate", {
  set.seed(123)
  d <- data.frame(
    from = sample(LETTERS[1:4], 10, replace = TRUE),
    to = sample(LETTERS[1:4], 10, replace = TRUE),
    weight = rexp(10)
  )
  expect_error(
    sna(d, aggregate = "not function"),
    "Argument `aggregate` must be a function\\."
  )
  expect_error(
    sna(d, aggregate = matrix),
    "Argument `aggregate` must be a function that takes a <numeric> vector and returns a single <numeric> value\\."
  )
})

test_that("tna from tsn works", {
  expect_error(
    tsn(mock_tsn),
    NA
  )
  expect_error(
    model <- build_model(mock_tsn),
    NA
  )
  expect_true(
    all(model$weights > 0)
  )
  expect_true(
    all(model$inits > 0)
  )
})


test_that("begin and end states can be included", {
  expect_error(
    tna(mock_sequence, begin_state = "begin"),
    NA
  )
  expect_error(
    tna(mock_sequence, end_state = "end"),
    NA
  )
  expect_error(
    tna(mock_sequence, begin_state = "begin", end_state = "end"),
    NA
  )
})

test_that("forward attention works with time measurements and durations", {
  durations <- matrix(
    1 + abs(rnorm(prod(dim(mock_sequence)))),
    nrow = nrow(mock_sequence),
  )
  times <- cbind(0, t(apply(durations, 1, cumsum))[, -ncol(durations)])
  expect_error(
    model_t <- atna(mock_sequence, params = list(time = times)),
    NA
  )
  expect_error(
    model_d <- atna(mock_sequence, params = list(duration = durations)),
    NA
  )
  expect_equal(
    model_t$weights,
    model_d$weights
  )
})

test_that("directional attention works", {
  expect_error(
    model_b <- atna(mock_sequence, params = list(direction = "backward")),
    NA
  )
  expect_error(
    model_f <- atna(mock_sequence, params = list(direction = "forward")),
    NA
  )
  expect_error(
    model_bf <- atna(mock_sequence, params = list(direction = "both")),
    NA
  )
  expect_equal(
    model_b$weights + model_f$weights,
    model_bf$weights
  )
})

test_that("decay can be customized", {
  durations <- matrix(
    1 + abs(rnorm(prod(dim(mock_sequence)))),
    nrow = nrow(mock_sequence),
  )
  my_decay <- function(i, j, lambda) (i - j)^-lambda
  expect_error(
    atna(
      mock_sequence,
      params = list(time = durations, decay = my_decay, lambda = 2)
    ),
    NA
  )
})

test_that("time data from tna_data objects can be used for attention models", {
  data_ordered <- tibble::tibble(
     user = c("A", "A", "A", "A", "B", "B", "B", "B"),
     time = c(1, 2, 3, 6, 2, 7, 10, 14),
     action = c(
       "view", "click", "add_cart", "view",
       "checkout", "view", "click", "share"
     )
  )
  rlang::local_options(rlib_message_verbosity = "quiet")
  data_timed <- prepare_data(
    data_ordered, actor = "user", time = "time", action = "action"
  )
  expect_error(
    atna(data_timed, params = list(time = TRUE)),
    NA
  )
})

test_that("sequence data can be concatenated", {
  expect_error(
    model_concat <- tna(mock_sequence, concat = 2L),
    NA
  )
  expect_true(
    ncol(model_concat$data) == 2L * ncol(mock_sequence)
  )
  data_concat <- c(t(model_concat$data))
  data_concat <- data_concat[!is.na(data_concat)]
  data_orig <- c(t(mock_tna_seq$data))
  expect_true(
    all.equal(data_concat, data_orig)
  )
})

# Tests for windowed transitions
test_that("windowed transitions work for relative type", {
  d <- data.frame(
    actor = gl(10, 10),
    feature1 = rbinom(100, 1, prob = 0.33),
    feature2 = rbinom(100, 1, prob = 0.25),
    feature3 = rbinom(100, 1, prob = 0.50)
  )
  onehot <- import_onehot(d, feature1:feature3, window_size = 2)
  expect_error(
    model <- tna(onehot),
    NA
  )
  expect_s3_class(model, "tna")
})

test_that("windowed transitions work for frequency type", {
  d <- data.frame(
    actor = gl(10, 10),
    feature1 = rbinom(100, 1, prob = 0.33),
    feature2 = rbinom(100, 1, prob = 0.25),
    feature3 = rbinom(100, 1, prob = 0.50)
  )
  onehot <- import_onehot(d, feature1:feature3, window_size = 2)
  expect_error(
    model <- ftna(onehot),
    NA
  )
  expect_s3_class(model, "tna")
})

test_that("windowed transitions work for co-occurrence type", {
  d <- data.frame(
    actor = gl(10, 10),
    feature1 = rbinom(100, 1, prob = 0.33),
    feature2 = rbinom(100, 1, prob = 0.25),
    feature3 = rbinom(100, 1, prob = 0.50)
  )
  onehot <- import_onehot(d, feature1:feature3, window_size = 2)
  expect_error(
    model <- ctna(onehot),
    NA
  )
  expect_s3_class(model, "tna")
})

test_that("sna validates input correctly", {
  # Wrong number of columns
  expect_error(
    sna(data.frame(a = 1, b = 2)),
    "Argument `x` must have three columns"
  )

  # NA values
  expect_error(
    sna(data.frame(from = "A", to = NA, weight = 1)),
    "must not contain missing values"
  )
})

test_that("build_model.matrix handles negative inits", {
  expect_error(
    tna(mock_matrix, inits = c(-1, 0.5, 0.5, 0)),
    "non-negative"
  )
})

#' Bootstrap Transition Networks from Sequence Data
#'
#' Perform bootstrapping on transition networks created from
#' sequence data stored in a `tna` object. Bootstrapped estimates
#' of edge weights are returned with confidence intervals and significance
#' testing.
#'
#' The function first computes the original edge weights for the specified
#' cluster from the `tna` object. It then performs bootstrapping by resampling
#' the sequence data and recalculating the edge weights for each
#' bootstrap sample. The mean and standard deviation of the transitions are
#' computed, and confidence intervals are derived. The function also estimates
#' p-values for each edge and identifies significant edges based on
#' the specified significance level. A matrix of significant edges
#' (those with estimated p-values below the significance level) is generated.
#' Additional statistics on removed edges (those not considered
#' significant) are provided.
#'
#' All results, including the original transition matrix, bootstrapped
#' estimates, and summary statistics for removed edges, are returned in a
#' structured list.
#'
#' @export
#' @family validation
#' @param x A `tna` or a `group_tna` object created from sequence data.
#' @param iter An `integer` specifying the number of bootstrap samples to
#' draw. Defaults to `1000`.
#' @param level A `numeric` value representing the significance level for
#' hypothesis testing and confidence intervals. Defaults to `0.05`.
#' @param method A `character` string. This argument defines the bootstrap
#' test statistic. The `"stability"` option (the default) compares edge weights
#' against a range of "consistent" values defined by `consistency_range`.
#' Weights that fall outside this range are considered insignificant. In other
#' words, an edge is considered significant if its value is within the range
#' in `(1 - level)` * 100% of the bootstrap samples. The `"threshold"` option
#' instead compares the edge weights against a user-specified `threshold` value.
#' @param consistency_range A `numeric` vector of length 2. Determines how much
#' the edge weights may deviate (multiplicatively) from their observed values
#' (below and above) before they are considered insignificant. The default is
#' `c(0.75, 1.25)` which corresponds to a symmetric 25% deviation range. Used
#' only when `method = "stability"`.
#' @param threshold A `numeric` value to compare edge weights against.
#' The default is the 10th percentile of the edge weights. Used only when
#' `method = "threshold"`.
#' @return A `tna_bootstrap` object which is a `list` containing the
#' following elements:
#
#'   * `weights_orig`: The original edge weight `matrix`.
#'   * `weights_sig`: The `matrix` of significant transitions
#'     (those with estimated p-values below the significance level).
#'   * `weights_mean`: The mean weight `matrix` from the bootstrap samples.
#'   * `weights_sd`: The standard deviation `matrix` from the bootstrap samples.
#'   * `cr_lower`: The lower bound `matrix` of the consistency range for the
#'     edge weights.
#'   * `cr_upper`: The upper bound `matrix` of the consistency range for the
#'     edge weights.
#'   * `ci_lower`: The lower bound `matrix` of the bootstrap confidence
#'     intervals for the edge weights.
#'   * `ci_upper`: The upper bound `matrix` of the bootstrap confidence
#'     intervals for the edge weights.
#'   * `p_values`: The `matrix` of estimated p-values for the edge weights.
#'   * `summary`: A `data.frame` summarizing the edges, their weights,
#'     p-values, statistical significance, consistency ranges, and
#'     confidence intervals.
#'
#' If `x` is a `group_tna` object, the output is a `group_tna_bootstrap`
#' object, which is a `list` of `tna_bootstrap` objects.
#'
#' @examples
#' model <- tna(group_regulation)
#' # Small number of iterations for CRAN
#' bootstrap(model, iter = 10)
#'
bootstrap <- function(x, iter, level, method, threshold, consistency_range) {
  UseMethod("bootstrap")
}

#' @export
#' @rdname bootstrap
bootstrap.tna <- function(x, iter = 1000, level = 0.05, method = "stability",
                          threshold, consistency_range = c(0.75, 1.25)) {
  check_missing(x)
  check_tna_seq(x)
  check_values(iter, strict = TRUE)
  check_range(level, lower = 0, upper = 1)
  method <- check_match(method, c("stability", "threshold"))
  threshold <- threshold %m% unname(stats::quantile(x$weights, probs = 0.1))
  check_values(threshold, type = "numeric")
  stopifnot_(
    checkmate::test_numeric(
      x = consistency_range,
      len = 2L,
      min = 0,
      any.missing = FALSE,
      sorted = TRUE
    ),
    "Argument {.arg consistency_range} must be a sorted {.cls numeric}
     vector of length 2 containing positive values."
  )
  d <- x$data
  type <- attr(x, "type")
  scaling <- attr(x, "scaling")
  params <- attr(x, "params")
  model <- initialize_model(d, type, scaling, params, transitions = TRUE)
  trans <- model$trans
  alphabet <- attr(d, "alphabet")
  dim_names <- list(alphabet, alphabet)
  n <- nrow(d)
  a <- length(alphabet)
  weights <- compute_weights(trans, type, scaling, a)
  dimnames(weights) <- dim_names
  weights_boot <- array(0.0, dim = c(iter, a, a))
  p_values <- matrix(0, a, a)
  idx <- seq_len(n)
  if (method == "stability") {
    for (i in seq_len(iter)) {
      trans_boot <- trans[sample(idx, n, replace = TRUE), , , drop = FALSE]
      weights_boot[i, , ] <- compute_weights(trans_boot, type, scaling, a)
      p_values[] <- p_values +
        1L * (weights_boot[i, , ] <= weights * consistency_range[1] | 
          weights_boot[i, , ] >= weights * consistency_range[2])
    }
  } else {
    for (i in seq_len(iter)) {
      trans_boot <- trans[sample(idx, n, replace = TRUE), , , drop = FALSE]
      weights_boot[i, , ] <- compute_weights(trans_boot, type, scaling, a)
      p_values <- p_values + 1L * (weights_boot[i, , ] < threshold)
    }
  }
  p_values <- (p_values + 1) / (iter + 1)
  weights_mean <- apply(weights_boot, c(2, 3), mean, na.rm = TRUE)
  weights_sd <- apply(weights_boot, c(2, 3), stats::sd, na.rm = TRUE)
  ci_lower <- apply(
    weights_boot,
    c(2, 3),
    stats::quantile,
    probs = level / 2,
    na.rm = TRUE
  )
  ci_upper <-  apply(
    weights_boot,
    c(2, 3),
    stats::quantile,
    probs = 1 - level / 2,
    na.rm = TRUE
  )
  weights_sig <- (p_values < level) * weights
  dimnames(p_values) <- dim_names
  dimnames(weights_mean) <- dim_names
  dimnames(weights_sd) <- dim_names
  dimnames(weights_sig) <- dim_names
  dimnames(ci_lower) <- dim_names
  dimnames(ci_upper) <- dim_names
  weights_vec <- as.vector(weights)
  combined <- data.frame(
    from = rep(alphabet, times = a),
    to = rep(alphabet, each = a),
    weight = weights_vec,
    p_value = as.vector(p_values),
    sig = as.vector(p_values < level),
    cr_lower = as.vector(weights * consistency_range[1]),
    cr_upper = as.vector(weights * consistency_range[2]),
    ci_lower = as.vector(ci_lower),
    ci_upper = as.vector(ci_upper)
  )[weights_vec > 0, ]
  model <- x
  tmp <- list(
    weights = weights_sig,
    method = "bootstrap",
    removed = combined[which(!combined$sig), c("from", "to", "weight")],
    num_removed = sum(!combined$sig),
    num_retained = sum(combined$sig)
  )
  tmp$original <- model$weights
  tmp$active <- TRUE
  model$weights <- tmp$weights
  attr(model, "pruning") <- tmp
  structure(
    list(
      weights_orig = weights,
      weights_sig = weights_sig,
      weights_mean = weights_mean,
      weights_sd = weights_sd,
      p_values = p_values,
      cr_lower = weights * consistency_range[1],
      cr_upper = weights * consistency_range[2],
      ci_lower = ci_lower,
      ci_upper = ci_upper,
      summary = combined,
      model = model
    ),
    class = "tna_bootstrap"
  )
}

#' @export
#' @rdname bootstrap
bootstrap.group_tna <- function(x, iter = 1000, level = 0.05,
                                method = "stability", threshold,
                                consistency_range = c(0.75, 1.25)) {
  check_missing(x)
  check_class(x, "group_tna")
  stopifnot_(
    length(attr(x, "scaling")) == 0L || attr(x, "groupwise"),
    "Bootstrapping is not supported for
     grouped models with globally scaled edge weights."
  )
  structure(
    stats::setNames(
      lapply(
        x,
        bootstrap,
        iter = iter,
        level = level,
        method = method,
        threshold = threshold,
        consistency_range = consistency_range
      ),
      names(x)
    ),
    class = "group_tna_bootstrap"
  )
}

#' Bootstrap Cliques of Transition Networks from Sequence Data
#'
#' Bootstrap the edge weights of all cliques of a given size in a `tna`
#' model, producing per-clique mean weights, p-values, confidence intervals,
#' and consistency-range bounds.
#'
#' @export
#' @inheritParams cliques
#' @inheritParams bootstrap
#' @return A `data.frame` (also of class `tna_bootstrap_cliques`) with
#'   one row per clique and the columns `clique`, `mean_weight`, `p_values`,
#'   `sig`, `cr_lower`, `cr_upper`, `ci_lower`, `ci_upper`.
#' @examples
#' model <- tna(group_regulation)
#' # Small number of iterations for CRAN
#' boot_cliq <- bootstrap_cliques(model, size = 2, iter = 10)
#'
bootstrap_cliques <- function(x, size, threshold,
                              iter, level, consistency_range) {
  UseMethod("bootstrap_cliques")
}

#' @export
#' @rdname bootstrap_cliques
bootstrap_cliques.tna <- function(x, size = 2L, threshold = 0,
                                  iter = 1000, level = 0.05,
                                  consistency_range = c(0.75, 1.25)) {
  check_missing(x)
  check_tna_seq(x)
  check_values(threshold, type = "numeric")
  stopifnot_(
    checkmate::test_int(x = size, lower = 2),
    "Argument {.arg size} must be a single {.cls integer}
    between 2 and {nodes(x)}."
  )
  check_values(iter, strict = TRUE)
  check_range(level, lower = 0, upper = 1)
  check_values(threshold, type = "numeric")
  stopifnot_(
    checkmate::test_numeric(
      x = consistency_range,
      len = 2L,
      min = 0,
      any.missing = FALSE,
      sorted = TRUE
    ),
    "Argument {.arg consistency_range} must be a sorted {.cls numeric}
     vector of length 2 containing positive values."
  )
  cliq <- cliques(
    x = x,
    size = size,
    threshold = threshold,
    sum_weights = TRUE
  )
  n <- length(cliq$weights)
  stopifnot_(
    length(cliq$weights) > 0,
    "No cliques of size {size} were found in the network."
  )
  d <- x$data
  type <- attr(x, "type")
  scaling <- attr(x, "scaling")
  params <- attr(x, "params")
  model <- initialize_model(d, type, scaling, params, transitions = TRUE)
  trans <- model$trans
  alphabet <- attr(d, "alphabet")
  dim_names <- list(alphabet, alphabet)
  a <- length(alphabet)
  weights <- compute_weights(trans, type, scaling, a)
  clique_weights <- vapply(
    cliq$indices,
    function(y) {
      mean(weights[y, y])
    },
    numeric(1L)
  )
  dimnames(weights) <- dim_names
  weights_boot <- array(0.0, dim = c(iter, a, a))
  clique_weights_boot <- matrix(0.0, iter, n)
  p_values <- numeric(n)
  idx <- seq_len(n)
  for (i in seq_len(iter)) {
    trans_boot <- trans[sample(idx, n, replace = TRUE), , , drop = FALSE]
    weights_boot[i, , ] <- compute_weights(trans_boot, type, scaling, a)
    clique_weights_boot[i, ] <- vapply(
      cliq$indices,
      function(y) {
        mean(weights_boot[i, y, y])
      },
      numeric(1L)
    )
    p_values <- p_values +
      1L * (clique_weights_boot[i, ] <= clique_weights * consistency_range[1] |
        clique_weights_boot[i, ] >= clique_weights * consistency_range[2])
  }
  p_values <- (p_values + 1) / (iter + 1)
  ci_lower <- apply(
    clique_weights_boot,
    2L,
    stats::quantile,
    probs = level / 2,
    na.rm = TRUE
  )
  ci_upper <-  apply(
    clique_weights_boot,
    2L,
    stats::quantile,
    probs = 1 - level / 2,
    na.rm = TRUE
  )
  clique_names <- vapply(
    cliq$indices,
    function(y) {
      paste0(alphabet[y], collapse = "-")
    },
    character(1L)
  )
  structure(
    data.frame(
      clique = clique_names,
      mean_weight = clique_weights,
      p_values = p_values,
      sig = (p_values < level),
      cr_lower = clique_weights * consistency_range[1L],
      cr_upper = clique_weights * consistency_range[2L],
      ci_lower = ci_lower,
      ci_upper = ci_upper
    ),
    class = c("tna_bootstrap_cliques", "data.frame")
  )
}

#' Compute User Sessions from Event Data
#'
#' Processes a dataset to create user sessions based on time gaps,
#' ordering columns, or actor groupings. It supports different ways to
#' understand order in user behavior and provides flexibility when
#' widening the data.
#'
#' @export
#' @family data
#' @param data A `data.frame` or containing the action/event data.
#' @param actor A `character` vector or an `expression` that represents
#' a tidy selection of the names of the columns that
#' represent a user/actor identifiers. If not provided and neither `time` nor
#' `order` is specified, the entire dataset is treated as a single session.
#' In the case of multiple actors, a new `.actor` column is added that
#' represents the interaction of the given columns.
#' @param time A `character` string or an `expression` giving the name of
#' the column representing timestamps of the action events.
#' @param action A `character` string or an `expression` giving the name of
#' the column holding the information about the action taken.
#' @param order A `character` string or an `expression` giving the name of a
#' column with sequence numbers or non-unique orderable values that indicate
#' order within an `actor` group, if not present it will be ordered with all
#' the data if no `actor` is available, used when widening the data.
#' If both `actor` and `time` are specified, then the sequence order should
#' be specified such that it determines the order of events within `actor`
#' and each session.
#' @param time_threshold An `integer` specifying the time threshold in seconds
#' for creating new time-based sessions. Defaults to 900 seconds.
#' @param custom_format A `character` string giving the format used to
#' parse the `time` column.
#' @param is_unix_time A `logical` value indicating whether the `time` column
#' is in Unix time. The default is `FALSE`.
#' @param unix_time_unit A `character` string giving the Unix time unit when
#' `is_unix_time` is `TRUE`. The default is `"seconds"`. Valid options are
#' `"seconds"`, `"milliseconds"`, or `"microseconds"`.
#' @param unused_fn How to handle extra columns when pivoting to wide format.
#' See [tidyr::pivot_wider()]. The default is to keep all columns and to
#' use the first value.
#' @return A `tna_data` object, which is a `list` with the following elements:
#'
#' * `long_data`: The processed data in long format.
#' * `sequence_data`: The processed data on the sequences in wide format,
#' with actions/events as different variables structured with sequences.
#' * `meta_data`: Other variables from the original data in wide format.
#' * `statistics`: A `list` containing summary statistics: total
#' sessions, total actions, unique users, time range (if applicable), and
#' top sessions and user by activities.
#'
#' @examples
#' results <- prepare_data(
#'   group_regulation_long, actor = "Actor", time = "Time", action = "Action"
#' )
#' print(results$sequence_data)
#' print(results$meta_data)
#' print(results$statistics)
#'
#' # Custom order column
#' data_ordered <- tibble::tibble(
#'    user = c("A", "A", "A", "B", "B", "C", "C", "C"),
#'    order = c(1, 2, 3, 1, 2, 1, 2, 3),
#'    action = c(
#'      "view", "click", "add_cart", "view",
#'      "checkout", "view", "click", "share"
#'    )
#' )
#' results_ordered <- prepare_data(
#'   data_ordered, actor = "user", order = "order", action = "action"
#' )
#' print(results_ordered$sequence_data)
#' print(results_ordered$meta_data)
#' print(results_ordered$statistics)
#'
#' # No actor scenario leading to a single session
#' data_single_session <- tibble::tibble(
#'   action = c(
#'     "view", "click", "add_cart", "view",
#'     "checkout", "view", "click", "share"
#'    )
#' )
#' results_single <- prepare_data(data_single_session, action = "action")
#' print(results_single$sequence_data)
#' print(results_single$meta_data)
#' print(results_single$statistics)
#'
#' # Multiple actors
#' data_multi_actor <- tibble::tibble(
#'   user = c("A", "A", "A", "A", "B", "B", "B", "B"),
#'   session = c(1, 1, 2, 2, 1, 1, 2, 2),
#'   action = c(
#'     "view", "click", "add_cart", "view",
#'     "checkout", "view", "click", "share"
#'   )
#' )
#' results_multi_actor <- prepare_data(
#'   data_multi_actor, actor = c("user", "session"), action = "action"
#' )
#' print(results_multi_actor$sequence_data)
#' print(results_multi_actor$meta_data)
#' print(results_multi_actor$statistics)
#'
prepare_data <- function(data, actor, time, action, order,
                         time_threshold = 900, custom_format = NULL,
                         is_unix_time = FALSE, unix_time_unit = "seconds",
                         unused_fn = dplyr::first) {
  check_missing(data)
  check_class(data, "data.frame")
  check_values(time_threshold, type = "numeric")
  check_flag(is_unix_time)
  unix_time_unit <- check_match(
    unix_time_unit,
    c("seconds", "milliseconds", "microseconds")
  )
  # Create some NULLs for R CMD Check
  .session_id <- .session_nr <- .new_session <- .time_gap <-
    .standardized_time <- .sequence <- n_sessions <- n_actions <- NULL
  rlang_verbose <- getOption("rlib_message_verbosity")
  onlyif(
    is.null(rlang_verbose) || isTRUE(rlang_verbose == "verbose"),
    cli::cli_rule(left = "Preparing Data")
  )
  message_(
    c(
      `i` = "Input data dimensions:
      {.val {nrow(data)}} rows, {.val {ncol(data)}} columns"
    )
  )
  action <- get_cols(rlang::enquo(action), data)
  actor <- get_cols(rlang::enquo(actor), data)
  time <- get_cols(rlang::enquo(time), data)
  order <- get_cols(rlang::enquo(order), data)
  check_cols(action, missing_ok = FALSE)
  check_cols(time)
  check_cols(order)
  data <- tibble::as_tibble(data)
  long_data <- data
  default_actor <- FALSE
  default_order <- FALSE
  multi_actor <- FALSE
  if (missing(actor)) {
    # Placeholder actor column
    actor <- ".actor"
    long_data$.actor <- "session"
    default_actor <- TRUE
  } else if (length(actor) > 1L) {
    actor_cols <- lapply(actor, function(x) long_data[[x]])
    long_data$.actor <- interaction(actor_cols, sep = "-")
    actor <- ".actor"
    multi_actor <- TRUE
  }
  if (missing(order)) {
    # Placeholder order column
    order <- ".order"
    long_data$.order <- seq_len(nrow(data))
    default_order <- TRUE
  }
  if (!missing(time)) {
    message_(
      c(`i` = "First few time values: {.val {utils::head(data[[time]], 3)}}")
    )
    if (is.numeric(data[[time]])) {
      message_(
        c(
          `i` = "Detected {.cls numeric} time values:
          treating as Unix timestamp."
        )
      )
      is_unix_time <- TRUE
    }
    parsed_times <- parse_time(
      time = data[[time]],
      custom_format = custom_format,
      is_unix_time = is_unix_time,
      unix_time_unit = unix_time_unit
    )
    message_(
      c(`i` = "Sample of parsed times: {.val {utils::head(parsed_times, 3)}}")
    )
    message_(
      c(`i` = "Time threshold for new session: {.val {time_threshold}} seconds")
    )
    long_data <- long_data |>
      dplyr::mutate(.standardized_time = parsed_times) |>
      dplyr::arrange(
        !!rlang::sym(actor),
        .standardized_time,
        !!rlang::sym(order)
      ) |>
      dplyr::group_by(!!rlang::sym(actor)) |>
      dplyr::mutate(
        .time_gap = as.numeric(
          difftime(
            .standardized_time,
            dplyr::lag(.standardized_time),
            units = "secs"
          )
        ),
        .new_session = is.na(.time_gap) | .time_gap > time_threshold,
        .session_nr = cumsum(.new_session),
        .session_id = ifelse_(
          default_actor,
          paste0("session", .session_nr),
          paste0(!!rlang::sym(actor), " session", .session_nr)
        )
      ) |>
      dplyr::group_by(.session_id) |>
      dplyr::mutate(.sequence = dplyr::row_number()) |>
      dplyr::ungroup()

    long_data$.time_gap <- NULL
    long_data$.new_session <- NULL
  } else {
    msg <- ifelse_(
      default_order,
      paste0(
        "No {.arg time} or {.arg order} column provided. ",
        ifelse_(
          default_actor,
          "Treating the entire dataset as one session.",
          "Using {.arg actor} as a session identifier."
        )
      ),
      "Using provided {.arg order} column to create sequences."
    )
    message_(c(`i` = msg))
    long_data <- long_data |>
      dplyr::arrange(
        !!rlang::sym(actor),
        !!rlang::sym(order)
      ) |>
      dplyr::group_by(!!rlang::sym(actor)) |>
      dplyr::mutate(
        .session_id = !!rlang::sym(actor),
        .sequence = dplyr::row_number()
      ) |>
      dplyr::ungroup()
  }
  if (default_actor) {
    long_data$.actor <- NULL
  }
  if (default_order) {
    long_data$.order <- NULL
  }
  if (!missing(time)) {
    wide_data <- long_data |>
      tidyr::pivot_wider(
        id_cols = .session_id,
        names_prefix = "T",
        names_from = .sequence,
        values_from = c(!!rlang::sym(action), .standardized_time),
        unused_fn = unused_fn
      ) |>
      dplyr::arrange(.session_id)
    sequence_cols <- grepl(
      paste0("^", action, "_T[0-9]+$"),
      names(wide_data),
      perl = TRUE
    )
    time_cols <- grepl(
      "^.standardized_time_T[0-9]+$",
      names(wide_data),
      perl = TRUE
    )
    sequence_data <- wide_data[, sequence_cols]
    time_data <- wide_data[, time_cols]
    meta_data <- wide_data[, !(sequence_cols | time_cols)]
  } else {
    wide_data <- long_data |>
      tidyr::pivot_wider(
        id_cols = .session_id,
        names_prefix = "T",
        names_from = .sequence,
        values_from = !!rlang::sym(action),
        unused_fn = unused_fn
      ) |>
      dplyr::arrange(.session_id)
    sequence_cols <- grepl("^T[0-9]+$", names(wide_data), perl = TRUE)
    sequence_data <- wide_data[, sequence_cols]
    meta_data <- wide_data[, !sequence_cols]
    time_data <- NULL
  }

  # Calculate statistics
  stats <- list(
    total_sessions = dplyr::n_distinct(long_data$.session_id),
    total_actions = nrow(long_data),
    max_sequence_length = max(long_data$.sequence)
  )
  if (!default_actor) {
    stats$unique_users <- dplyr::n_distinct(long_data[[actor]])
    stats$sessions_per_user <- long_data |>
      dplyr::group_by(!!rlang::sym(actor)) |>
      dplyr::summarize(n_sessions = dplyr::n_distinct(.session_id)) |>
      dplyr::arrange(dplyr::desc(n_sessions))
  }
  stats$actions_per_session <- long_data |>
    dplyr::group_by(.session_id) |>
    dplyr::summarize(n_actions = dplyr::n()) |>
    dplyr::arrange(dplyr::desc(n_actions))

  if (!missing(time)) {
    stats$time_range <- range(long_data$.standardized_time)
  }
  message_(c(`i` = "Total number of sessions: {.val {stats$total_sessions}}"))
  if (!default_actor) {
    message_(c(`i` = "Number of unique users: {.val {stats$unique_users}}"))
  }
  message_(c(`i` = "Total number of actions: {.val {stats$total_actions}}"))
  message_(
    c(
      `i` = "Maximum sequence length:
      {.val {stats$max_sequence_length}} actions"
    )
  )
  if (!missing(time) && default_order) {
    message_(
      c(
        `i` = "Time range: {.val {stats$time_range[1]}} to
        {.val {stats$time_range[2]}}"
      )
    )
  }
  structure(
    list(
      long_data = long_data,
      sequence_data = sequence_data,
      meta_data = meta_data,
      time_data = time_data,
      statistics = stats
    ),
    class = "tna_data"
  )
}

#' Robustly Parse Date and Time Values
#'
#' This function parses a variety of date and time formats into a standardized
#' POSIXct datetime object in R. It handles different separators,
#' time zone indicators, and partial dates. It also deals with missing
#' values and treats them as NA
#'
#' @param time A `vector` of time values.
#' @return A `POSIXct` object.
#' @inheritParams prepare_data
#' @noRd
parse_time <- function(time, custom_format, is_unix_time, unix_time_unit) {
  message_(c(`i` = "Number of values to parse: {.val {length(time)}}"))
  message_(c(`i` = "Sample values: {.val {utils::head(time, 3)}}"))
  # Handle Unix timestamps
  time_original <- time
  if (is.numeric(time) && is_unix_time) {
    parsed_time <- switch(
      unix_time_unit,
      "seconds" = as.POSIXct(time, origin = "1970-01-01"),
      "milliseconds" = as.POSIXct(time / 1000.0, origin = "1970-01-01"),
      "microseconds" = as.POSIXct(time / 1000000.0, origin = "1970-01-01")
    )
    return(parsed_time)
  }
  # If already datetime
  if (inherits(time, c("POSIXct", "POSIXlt"))) {
    return(time)
  }
  time_empty <- is.na(time) | !nzchar(time)
  if (any(time_empty)) {
    message_(
      c(
        `i` = "Found missing or empty time values; these will be treated as NA."
      )
    )
    time[time_empty] <- NA
  }
  # Remove whitespace
  time <- trimws(as.character(time))
  # Remove timezone indicators like Z
  # or other trailing chars and handle milliseconds
  time <- gsub("(\\.\\d{1,3})?[A-Za-z ]*$", "", time)
  # Try custom format first if provided
  if (!is.null(custom_format)) {
    parsed_time <- as.POSIXct(strptime(time, format = custom_format))
    if (!all(is.na(parsed_time))) {
      message_(c(`v` = "Successfully parsed using custom format."))
      return(parsed_time)
    }
  }

  # Comprehensive list of formats to try
  formats <- c(
    # Standard formats with different separators
    "%Y-%m-%d %H:%M:%S",
    "%Y-%m-%d %H:%M",
    "%Y/%m/%d %H:%M:%S",
    "%Y/%m/%d %H:%M",
    "%Y.%m.%d %H:%M:%S",
    "%Y.%m.%d %H:%M",

    "%Y-%m-%dT%H:%M:%S",  # ISO8601 formats
    "%Y-%m-%dT%H:%M",     # ISO8601 formats
    "%Y-%m-%dT%H:%M:%OS", # ISO with optional second fraction
    "%Y-%m-%d %H:%M:%S%z",  # timezone offset with colon like +00:00
    "%Y-%m-%d %H:%M%z", # timezone offset with colon like +00:00 without seconds
    "%Y-%m-%d %H:%M:%S %z", # timezone offset with space colon like  +00:00
    "%Y-%m-%d %H:%M %z", # timezone offset with space and colon like +00:00
    "%Y%m%d%H%M%S",      # compact without separators like 20240201204530
    "%Y%m%d%H%M",        # compact without separators like 202402012045

    # Day first formats
    "%d-%m-%Y %H:%M:%S",
    "%d-%m-%Y %H:%M",
    "%d/%m/%Y %H:%M:%S",
    "%d/%m/%Y %H:%M",
    "%d.%m.%Y %H:%M:%S",
    "%d.%m.%Y %H:%M",

    "%d-%m-%YT%H:%M:%S",  # ISO8601 dayfirst
    "%d-%m-%YT%H:%M",     # ISO8601 dayfirst

    # Month first formats
    "%m-%d-%Y %H:%M:%S",
    "%m-%d-%Y %H:%M",
    "%m/%d/%Y %H:%M:%S",
    "%m/%d/%Y %H:%M",
    "%m.%d.%Y %H:%M:%S",
    "%m.%d.%Y %H:%M",
    "%m-%d-%YT%H:%M:%S", # ISO8601 month first
    "%m-%d-%YT%H:%M",    # ISO8601 month first

    # Formats with month names
    "%d %b %Y %H:%M:%S",
    "%d %b %Y %H:%M",
    "%d %B %Y %H:%M:%S",
    "%d %B %Y %H:%M",
    "%b %d %Y %H:%M:%S",
    "%b %d %Y %H:%M",
    "%B %d %Y %H:%M:%S",
    "%B %d %Y %H:%M",

    # Date only formats
    "%Y-%m-%d",
    "%Y/%m/%d",
    "%Y.%m.%d",
    "%d-%m-%Y",
    "%d/%m/%Y",
    "%d.%m.%Y",
    "%m-%d-%Y",
    "%m/%d/%Y",
    "%m.%d.%Y",
    "%d %b %Y",
    "%d %B %Y",
    "%b %d %Y",
    "%B %d %Y"
  )

  # Try each format
  for (fmt in formats) {
    parsed_time <- as.POSIXct(strptime(time, format = fmt))
    if (!all(is.na(parsed_time))) {
      message_(c(`v` = "Successfully parsed using format: {.val {fmt}}"))
      return(parsed_time)
    }
  }

  # Finally, try Unix time
  message_(
    c(
      `x` = "Unable to parse using supported formats.
      Trying to convert to {.cls numeric} and assuming Unix time."
    )
  )
  # TODO suppress for now
  time <- suppressWarnings(try_(as.numeric(time)))
  if (!inherits(time, "try-error") && all(!is.na(time))) {
    parsed_time <- switch(
      unix_time_unit,
      "seconds" = as.POSIXct(time, origin = "1970-01-01"),
      "milliseconds" = as.POSIXct(time / 1000.0, origin = "1970-01-01"),
      "microseconds" = as.POSIXct(time / 1000000.0, origin = "1970-01-01")
    )
    return(parsed_time)
  }

  # If all attempts fail, provide helpful error message
  stop_(
    c(
      "Could not parse time values. Supported formats include:",
      "1. YYYY-MM-DD HH:MM:SS (e.g., 2023-01-09 18:44:00)",
      "2. YYYY/MM/DD HH:MM:SS (e.g., 2023/01/09 18:44:00)",
      "3. DD-MM-YYYY HH:MM:SS (e.g., 09-01-2023 18:44:00)",
      "4. MM-DD-YYYY HH:MM:SS (e.g., 01-09-2023 18:44:00)",
      "5. YYYY-MM-DDTHH:MM:SS (ISO8601 Format)",
      "6. YYYY-MM-DDTHH:MM:SS.milliseconds (ISO8601 with milliseconds)",
      "7. Compact Formats (YYYYMMDDHHMMSS)",
      "8. With or without Timezone offset (e.g, +00:00 or +01:00)",
      "9. Month names (e.g., 09 Jan 2023 18:44:00, January 09 2023 18:44:00)",
      "10. All above formats without seconds (HH:MM)",
      "11. Unix timestamps (numeric)",
      "Sample of problematic values: {.val {utils::head(time_original, 3)}}.",
      "Consider providing a custom format
       using the {.arg custom_format} argument."
    )
  )
}

#' Import Wide Format Sequence Data as Long Format Sequence Data
#'
#' This function transforms wide format data where features are in separate
#' columns into a long format suitable for sequence analysis. It creates
#' windows of data based on row order and generates sequence order within
#' these windows.
#'
#' @export
#' @family data
#' @param data A `data.frame` in wide format.
#' @param cols An `expression` giving a tidy selection of column names to be
#'   transformed into long format (actions). This can be a vector of column
#'   names (e.g., `c(feature1, feature2)`) or a range  specified as
#'   `feature1:feature6` (without quotes) to include all columns from
#'   'feature1' to 'feature6' in the order they appear in the data frame.
#'   For more information on tidy selections, see [dplyr::select()].
#' @param id_cols An `expression` giving a tidy selection of column names that
#'   uniquely identify each observation (IDs).
#' @param window_size An `integer` specifying the size of the window for
#'   sequence grouping. Default is 1 (each row is a separate window).
#' @param replace_zeros A `logical` value indicating whether to replace 0s
#'   in `cols` with `NA`. The default is `TRUE`.
#' @return A `data.frame` in long format with added columns for window and
#'   sequence order.
#' @examples
#' data <- data.frame(
#'   ID = c("A", "A", "B", "B"),
#'   Time = c(1, 2, 1, 2),
#'   feature1 = c(10, 0, 15, 20),
#'   feature2 = c(5, 8, 0, 12),
#'   feature3 = c(2, 4, 6, 8),
#'   other_col = c("X", "Y", "Z", "W")
#' )
#'
#' # Using a vector
#' long_data1 <- import_data(
#'   data = data,
#'   cols = c(feature1, feature2),
#'   id_cols = c("ID", "Time"),
#'   window_size = 2,
#'   replace_zeros = TRUE
#' )
#'
#' # Using a column range
#' long_data2 <- import_data(
#'   data = data,
#'   cols = feature1:feature3,
#'   id_cols = c("ID", "Time"),
#'   window_size = 2,
#'   replace_zeros = TRUE
#' )
#'
import_data <- function(data, cols, id_cols,
                        window_size = 1, replace_zeros = TRUE) {
  check_missing(data)
  check_class(data, "data.frame")
  check_flag(replace_zeros)
  cols <- get_cols(rlang::enquo(cols), data)
  id_cols <- get_cols(rlang::enquo(id_cols), data) %m% character(0L)
  check_cols(cols, single = FALSE, missing_ok = FALSE)
  out <- data
  n <- nrow(out)
  rownames(out) <- ifelse_(
    is.null(rownames(data)),
    seq_len(n),
    rownames(data)
  )
  # Create some NULLs for R CMD Check
  .original_row <- window_group <- action <- value <- order <- NULL
  out$.original_row <- as.numeric(rownames(out))
  if (replace_zeros) {
    out <- out |>
      dplyr::mutate(
        dplyr::across(
          dplyr::all_of(cols), ~ifelse(. == 0, NA, .)
        )
      )
  }
  out <- out |>
    dplyr::arrange(.original_row) |>
    dplyr::mutate(window_group = ceiling(seq_len(n) / window_size))
  out_names <- colnames(data)
  extra_cols <- out_names[!(out_names %in% c(cols, id_cols))]
  out |>
    tidyr::pivot_longer(
      cols = dplyr::all_of(cols),
      names_to = "action",
      values_to = "value"
    ) |>
    dplyr::filter(!is.na(value)) |>
    dplyr::arrange(.original_row, action) |>
    dplyr::group_by(
      dplyr::across(
        dplyr::all_of(c("window_group", id_cols))
      )
    ) |>
    dplyr::mutate(order = dplyr::row_number()) |>
    dplyr::ungroup() |>
    dplyr::arrange(
      dplyr::across(
        dplyr::all_of(c(id_cols, "window_group", "order"))
      )
    ) |>
    dplyr::select(
      dplyr::all_of(extra_cols),
      dplyr::all_of(id_cols),
      action,
      value,
      order
    )
}

#' Import One-Hot Data
#'
#' @export
#' @family data
#' @param data A `data.frame` in wide format.
#' @param cols An `expression` giving a tidy selection of columns to be
#'   considered as one-hot data.
#' @param actor An optional `character` string giving the column name of
#'   `data` containing the actor identifiers.
#' @param session An optional `character` string giving the column name of
#'   `data` containing the session identifiers.
#' @param interval An `integer` that defines how many windows can appear
#'   at most per one row of the output data. If not provided (the default),
#'   all windows are concatenated per actor/session combination.
#' @param window_size An `integer` specifying the window size for grouping.
#' @param window_type A `character` string. Either `"tumbling"` (the default)
#'   for non-overlapping windows or `"sliding"` for one-step sliding window.
#' @param aggregate A `logical` value that determines how multiple occurrences
#'   of the same event within a window are processed. Option `TRUE` aggregates
#'   multiple occurrences into a single occurrence. Option `FALSE` keeps
#'   all occurrences (the default).
#' @return The processed data as a `data.frame`.
#' @examples
#' d <- data.frame(
#'   actor = gl(100, 5),
#'   session = gl(10, 50),
#'   feature1 = rbinom(500, 1, prob = 0.33),
#'   feature2 = rbinom(500, 1, prob = 0.25),
#'   feature3 = rbinom(500, 1, prob = 0.50)
#' )
#' onehot1 <- import_onehot(d, feature1:feature3)
#' onehot2 <- import_onehot(d, feature1:feature3, "actor", "session")
#'
import_onehot <- function(data, cols, actor, session, interval,
                          window_size = 1L, window_type = "tumbling",
                          aggregate = FALSE) {
  check_missing(data)
  check_class(data, "data.frame")
  check_values(window_size, strict = TRUE)
  check_flag(aggregate)
  window_type <- check_match(window_type, c("sliding", "tumbling"))
  cols <- get_cols(rlang::enquo(cols), data)
  actor <- get_cols(rlang::enquo(actor), data)
  session <- get_cols(rlang::enquo(session), data)
  interval <- interval %m% (ncol(data) * nrow(data))
  check_cols(actor, missing_ok = TRUE)
  check_cols(session, missing_ok = TRUE)
  if (missing(actor)) {
    actor <- ".actor"
    data$.actor <- 1L
  }
  if (missing(session)) {
    session <- ".session"
    data$.session <- 1L
  }
  out <- data |>
    dplyr::select(tidyselect::all_of(c(actor, session, cols))) |>
    dplyr::mutate(
      dplyr::across(
        tidyselect::all_of(cols),
        ~ ifelse(.x == 1, dplyr::cur_column(), NA_character_)
      )
    ) |>
    dplyr::group_by(!!rlang::sym(actor), !!rlang::sym(session))
  if (window_type == "sliding") {
    for (w in seq(1, window_size - 1)) {
      out <- out |>
        dplyr::mutate(
          dplyr::across(
            tidyselect::all_of(cols),
            ~ ifelse(
              !is.na(.x) | !is.na(dplyr::lag(.x, n = w)),
              dplyr::cur_column(),
              NA_character_
            )
          )
        )
    }
    out <- out |>
      dplyr::slice(-1) |>
      dplyr::mutate(
        .window = floor(seq_len(dplyr::n()) - 1)
      )
  } else {
    out <- out |>
      dplyr::mutate(
        .window = floor((seq_len(dplyr::n()) - 1) / window_size)
      )
  }
  if (aggregate) {
    out <- out |>
      dplyr::group_by(
        !!rlang::sym(actor),
        !!rlang::sym(session),
        !!rlang::sym(".window")
      ) |>
      dplyr::summarise(
        dplyr::across(tidyselect::all_of(cols), ~ stats::na.omit(.x)[1L]),
        .groups = "drop"
      )
  }
  out <- out |>
    dplyr::group_by(!!rlang::sym(actor), !!rlang::sym(session)) |>
    dplyr::mutate(
      .window_grp = !!rlang::sym(".window") %/% interval,
      .window_idx = !!rlang::sym(".window") %% interval
    ) |>
    dplyr::ungroup() |>
    tidyr::pivot_longer(
      cols = tidyselect::all_of(cols)
    ) |> dplyr::group_by(
      !!rlang::sym(actor),
      !!rlang::sym(session),
      !!rlang::sym(".window_grp"),
      !!rlang::sym(".window_idx")
    ) |>
    dplyr::mutate(
      .obs = seq_len(dplyr::n())
    )
  out <- out |>
    dplyr::select(-!!rlang::sym("name")) |>
    dplyr::ungroup() |>
    tidyr::pivot_wider(
      id_cols = tidyselect::all_of(c(actor, session, ".window_grp")),
      names_from = tidyselect::all_of(c(".window_idx", ".obs")),
      names_glue = "W{.window_idx}_T{.obs}",
      values_from = "value"
    )
  out[[actor]] <- NULL
  out[[session]] <- NULL
  out[[".window_grp"]] <- NULL
  # Window properties for modeling
  attr(out, "windowed") <- TRUE
  attr(out, "window_size") <- window_size^(!aggregate)
  attr(out, "window_span") <- length(cols)
  out
}

test_that("data preparation works when actor and time are provided", {
  data <- tibble::tibble(
    user = c("A", "A", "A", "B", "B", "C", "C", "C"),
    time = c(
      "2023-01-01 10:00:00", "2023-01-01 10:05:00",
      "2023-01-01 10:20:00", "2023-01-01 12:00:00",
      "2023-01-01 12:02:00", "2023-01-01 14:00:00",
      "2023-01-01 14:05:00", "2023-01-01 14:10:00"
    ),
    action = c(
      "view", "click", "add_cart", "view",
      "checkout", "view", "click", "share"
    )
  )
  rlang::local_options(rlib_message_verbosity = "quiet")
  expect_error(
    prepare_data(
      data,
      actor = "user",
      time = "time",
      action = "action"
    ),
    NA
  )
})

test_that("data preparation works when actor and order are provided", {
  data_ordered <- tibble::tibble(
    user = c("A", "A", "A", "B", "B", "C", "C", "C"),
    order = c(1, 2, 3, 1, 2, 1, 2, 3),
    action = c(
      "view", "click", "add_cart", "view",
      "checkout", "view", "click", "share"
    )
  )
  rlang::local_options(rlib_message_verbosity = "quiet")
  expect_error(
    prepare_data(
      data_ordered,
      actor = "user",
      order = "order",
      action = "action"
    ),
    NA
  )
})

test_that("data preparation works when only action is provided", {
  data_single_session <- tibble::tibble(
    action = c(
      "view", "click", "add_cart", "view", "checkout", "view", "click", "share"
    )
  )
  rlang::local_options(rlib_message_verbosity = "quiet")
  expect_error(
    prepare_data(data_single_session, action = "action"),
    NA
  )
})

test_that("data preparation works when actor and order are provided", {
  data <- tibble::tibble(
    user = c("A", "A", "A", "B", "B", "C", "C", "C"),
    time = c(
      "2023-01-01 10:00:00", "2023-01-01 10:05:00",
      "2023-01-01 10:20:00", "2023-01-01 12:00:00",
      "2023-01-01 12:02:00", "2023-01-01 14:00:00",
      "2023-01-01 14:05:00", "2023-01-01 14:10:00"
    ),
    action = c(
      "view", "click", "add_cart", "view",
      "checkout", "view", "click", "share"
    )
  )
  data_unarranged <- dplyr::arrange(data, action)
  rlang::local_options(rlib_message_verbosity = "quiet")
  expect_error(
    prepare_data(
      data,
      actor = "user",
      order = "time",
      action = "action"
    ),
    NA
  )
  expect_equal(
    prepare_data(
      data,
      actor = "user",
      order = "time",
      action = "action"
    ),
    prepare_data(
      data_unarranged,
      actor = "user",
      order = "time",
      action = "action"
    )
  )
})

test_that("data preparation for multiple actors is supported", {
  data_multi_actor <- tibble::tibble(
    user = c("A", "A", "A", "A", "B", "B", "B", "B"),
    session = c(1, 1, 2, 2, 1, 1, 2, 2),
    action = c(
     "view", "click", "add_cart", "view",
     "checkout", "view", "click", "share"
    )
  )
  rlang::local_options(rlib_message_verbosity = "quiet")
  expect_error(
    prepare_data(
      data_multi_actor, actor = c("user", "session"), action = "action"
    ),
    NA
  )
})

test_that("unix time from character column works", {
  mock_long_unix <- mock_long
  mock_long_unix$time <- as.POSIXct("2023-01-01 00:00:00") |>
    as.numeric() |>
    as.character()
  rlang::local_options(rlib_message_verbosity = "quiet")
  expect_error(
    data_out <- prepare_data(
      mock_long_unix,
      time = "time",
      actor = "group",
      action = "event"
    ),
    NA
  )
})

test_that("unsupported date format fails", {
  time <- rep("2025#02#02", 5)
  rlang::local_options(rlib_message_verbosity = "quiet")
  expect_error(
    parse_time(
      time,
      custom_format = NULL,
      is_unix_time = FALSE,
      unix_time_unit = "secs"
    ),
    "Could not parse time values"
  )
})

test_that("datetime is unaffected", {
  time <- rep(Sys.time(), 5)
  rlang::local_options(rlib_message_verbosity = "quiet")
  expect_equal(
    time,
    parse_time(
      time,
      custom_format = NULL,
      is_unix_time = FALSE,
      unix_time_unit = "secs"
    )
  )
})

test_that("missing values informs", {
  time <- as.character(rep(Sys.time(), 5))
  time[c(4, 5)] <- NA
  out <- utils::capture.output(
    utils::capture.output(
      parse_time(
        time,
        custom_format = NULL,
        is_unix_time = FALSE,
        unix_time_unit = "secs"
      ),
      type = "message"
    ),
    type = "output"
  )
  out <- paste0(out, collapse = "")
  expect_true(
    grepl("Found missing or empty time values", out),
  )
})

test_that("parsing with custom time format works", {
  time_raw <- "27---2---2025"
  fmt <- "%d---%m---%Y"
  time <- as.POSIXct(strptime(time_raw, format = fmt))
  rlang::local_options(rlib_message_verbosity = "quiet")
  expect_equal(
    time,
    parse_time(
      time_raw,
      custom_format = fmt,
      is_unix_time = FALSE,
      unix_time_unit = "secs"
    )
  )
})

test_that("wide format sequence data can be imported", {
  expect_error(
    long_data <- import_data(
      data = mock_sequence_wide,
      cols = c(feature1, feature2),
      id_cols = c("ID", "Time"),
      window_size = 2,
      replace_zeros = TRUE
    ),
    NA
  )
  expect_equal(
    names(long_data),
    c("feature3", "other_col", "ID", "Time", "action", "value", "order")
  )
})

test_that("one-hot data can be imported", {
  d <- data.frame(
    actor = gl(100, 5),
    session = gl(10, 50),
    feature1 = rbinom(500, 1, prob = 0.33),
    feature2 = rbinom(500, 1, prob = 0.25),
    feature3 = rbinom(500, 1, prob = 0.50)
  )
  expect_error(
    import_onehot(d, feature1:feature3),
    NA
  )
  expect_error(
    import_onehot(d, feature1:feature3, window_size = 5),
    NA
  )
  expect_error(
    import_onehot(d, feature1:feature3, actor = "actor"),
    NA
  )
  expect_error(
    import_onehot(d, feature1:feature3, session = "session"),
    NA
  )
  expect_error(
    import_onehot(d, feature1:feature3, actor = "actor", session = "session"),
    NA
  )
})

test_that("valid column selection works", {
  expect_error(
    cols_1 <- get_cols(rlang::quo(T1:T3), mock_sequence),
    NA
  )
  expect_error(
    cols_2 <- get_cols(rlang::quo(c("T1", "T2", "T3")), mock_sequence),
    NA
  )
  expect_error(
    cols_3 <- get_cols(rlang::quo(1:3), mock_sequence),
    NA
  )
  expect_identical(cols_1, cols_2)
  expect_identical(cols_2, cols_3)
  expect_identical(
    get_cols(rlang::quo(1), mock_sequence),
    "T1"
  )
  expect_identical(
    get_cols(rlang::quo("T1"), mock_sequence),
    "T1"
  )
})

test_that("invalid column selection fails", {
  expect_error(
    get_cols(rlang::quo(TRUE), mock_sequence),
    "Columns must be selected using a tidy selection"
  )
  expect_error(
    get_cols(rlang::quo(1i), mock_sequence),
    "Columns must be selected using a tidy selection"
  )
  expect_error(
    get_cols(rlang::quo("T7"), mock_sequence),
    "Can't select columns that don't exist"
  )
  expect_error(
    get_cols(rlang::quo(7), mock_sequence),
    "Can't select columns that don't exist"
  )
})

# Tests for import_onehot with sliding window
test_that("import_onehot works with sliding window type", {
  d <- data.frame(
    actor = gl(10, 10),
    session = gl(5, 20),
    feature1 = rbinom(100, 1, prob = 0.33),
    feature2 = rbinom(100, 1, prob = 0.25),
    feature3 = rbinom(100, 1, prob = 0.50)
  )
  expect_error(
    result <- import_onehot(
      d,
      feature1:feature3,
      actor = "actor",
      session = "session",
      window_size = 3,
      window_type = "sliding"
    ),
    NA
  )
  expect_s3_class(result, "data.frame")
})

test_that("import_onehot works with aggregate = TRUE", {
  d <- data.frame(
    actor = gl(10, 10),
    session = gl(5, 20),
    feature1 = rbinom(100, 1, prob = 0.33),
    feature2 = rbinom(100, 1, prob = 0.25),
    feature3 = rbinom(100, 1, prob = 0.50)
  )
  expect_error(
    result <- import_onehot(
      d,
      feature1:feature3,
      actor = "actor",
      session = "session",
      window_size = 2,
      aggregate = TRUE
    ),
    NA
  )
  expect_s3_class(result, "data.frame")
})

test_that("import_onehot works with sliding window and aggregate", {
  d <- data.frame(
    actor = gl(10, 10),
    session = gl(5, 20),
    feature1 = rbinom(100, 1, prob = 0.33),
    feature2 = rbinom(100, 1, prob = 0.25),
    feature3 = rbinom(100, 1, prob = 0.50)
  )
  expect_error(
    result <- import_onehot(
      d,
      feature1:feature3,
      window_size = 2,
      window_type = "sliding",
      aggregate = TRUE
    ),
    NA
  )
  expect_s3_class(result, "data.frame")
})

test_that("import_onehot can split windows by interval", {
  d <- data.frame(
    actor = gl(10, 10),
    session = gl(5, 20),
    feature1 = rbinom(100, 1, prob = 0.33),
    feature2 = rbinom(100, 1, prob = 0.25),
    feature3 = rbinom(100, 1, prob = 0.50)
  )
  result_orig <- import_onehot(
    d,
    actor = "actor",
    session = "session",
    feature1:feature3,
    window_size = 2,
  )
  result1 <- import_onehot(
    d,
    actor = "actor",
    session = "session",
    feature1:feature3,
    interval = 2,
    window_size = 2,
  )
  result2 <- import_onehot(
    d,
    actor = "actor",
    session = "session",
    feature1:feature3,
    interval = 3,
    window_size = 2,
  )
  result3 <- import_onehot(
    d,
    actor = "actor",
    session = "session",
    feature1:feature3,
    interval = 4,
    window_size = 2,
  )
  result4 <- import_onehot(
    d,
    actor = "actor",
    session = "session",
    feature1:feature3,
    interval = 5,
    window_size = 2,
  )
  expect_equal(ncol(result1), 12)
  expect_equal(ncol(result2), 18)
  expect_equal(ncol(result3), 24)
  expect_equal(ncol(result4), 30)
  expect_equal(result_orig, result4)
})

test_that("parse_time handles milliseconds unix time", {
  time <- c(1609459200000, 1609459260000, 1609459320000)
  rlang::local_options(rlib_message_verbosity = "quiet")
  result <- parse_time(
    time,
    custom_format = NULL,
    is_unix_time = TRUE,
    unix_time_unit = "milliseconds"
  )
  expect_s3_class(result, "POSIXct")
})

test_that("parse_time handles microseconds unix time", {
  time <- c(1609459200000000, 1609459260000000, 1609459320000000)
  rlang::local_options(rlib_message_verbosity = "quiet")
  result <- parse_time(
    time,
    custom_format = NULL,
    is_unix_time = TRUE,
    unix_time_unit = "microseconds"
  )
  expect_s3_class(result, "POSIXct")
})

#' Prune a Transition Network based on Transition Probabilities
#'
#' Prunes a network represented by a `tna` object by removing
#' edges based on a specified threshold, lowest percent of non-zero edge
#' weights, or the disparity filter algorithm (Serrano et al., 2009).
#' It ensures the network remains weakly connected.
#'
#' @export
#' @family validation
#' @param x An object of class `tna` or `group_tna`.
#' @param ... Potential additional arguments passed to the pruning method.
prune <- function(x, ...) {
  UseMethod("prune")
}


#' Prune a Transition Network based on Transition Probabilities
#'
#' Prunes a network represented by a `tna` object by removing
#' edges based on a specified threshold, lowest percent of non-zero edge
#' weights, or the disparity filter algorithm (Serrano et al., 2009).
#' It ensures the network remains weakly connected.
#'
#' @export
#' @family validation
#' @rdname prune
#' @param x An object of class `tna` or `group_tna`
#' @param method A `character` string describing the pruning method.
#' The available options are `"threshold"`, `"lowest"`, `"bootstrap"` and
#' `"disparity"`, corresponding to the methods listed in Details. The default
#' is `"threshold"`.
#' @param threshold A numeric value specifying the edge weight threshold.
#' Edges with weights below or equal to this threshold will be considered for
#' removal.
#' @param lowest A `numeric` value specifying the lowest percentage
#' of non-zero edges. This percentage of edges with the lowest weights will be
#' considered for removal. The default is `0.05`.
#' @param level A `numeric` value representing the significance level for the
#' disparity filter. Defaults to `0.5`.
#' @param boot A `tna_bootstrap` object to be used for pruning with method
#' `"boot"`. The method argument is ignored if this argument is supplied.
#' @param ... Arguments passed to [bootstrap()] when
#' using `method = "bootstrap"` and when a `tna_bootstrap` is not supplied.
#' @return A pruned `tna` or `group_tna` object. Details on the pruning can be
#' viewed with [pruning_details()]. The original model can be restored with
#' [deprune()].
#' @examples
#' model <- tna(group_regulation)
#' pruned_threshold <- prune(model, method = "threshold", threshold = 0.1)
#' pruned_percentile <- prune(model, method = "lowest", lowest = 0.05)
#' pruned_disparity <- prune(model, method = "disparity", level = 0.5)
#'
prune.tna <- function(x, method = "threshold", threshold = 0.1, lowest = 0.05,
                      level = 0.5, boot = NULL, ...) {
  check_missing(x)
  check_class(x, "tna")
  method <- check_match(
    method,
    c("threshold", "lowest", "bootstrap", "disparity")
  )
  check_values(threshold, type = "numeric")
  check_range(lowest, lower = 0, upper = 1)
  check_range(level, lower = 0, upper = 1)
  stopifnot_(
    is.null(attr(x, "pruning")),
    "The model has already been pruned."
  )
  labels <- ifelse_(
    is.null(x$labels),
    seq_len(nodes(x)),
    x$labels
  )
  tmp <- switch(method,
    bootstrap = prune_bootstrap(x, boot, ...),
    disparity = prune_disparity(x, level, labels),
    prune_default(x, method, threshold, lowest, labels)
  )
  tmp$original <- x$weights
  tmp$active <- TRUE
  x$weights <- tmp$weights
  attr(x, "pruning") <- tmp
  x
}

prune_default <- function(x, method, threshold, lowest, labels) {
  weights <- x$weights
  pos_edges_idx <- weights > 0
  n_edges <- sum(pos_edges_idx)
  pos_edges <- weights[pos_edges_idx]
  cut_off <- switch(method,
    threshold = threshold,
    lowest = stats::quantile(pos_edges, probs = lowest)
  )
  to_remove_idx <- which(
    pos_edges_idx & weights <= cut_off,
    arr.ind = TRUE
  )
  n_remove <- nrow(to_remove_idx)
  to_remove <- data.frame(
    from = labels[to_remove_idx[, 1]],
    to = labels[to_remove_idx[, 2]],
    weight = weights[to_remove_idx]
  )
  removed_idx <- logical(n_remove)
  for (j in seq_len(n_remove)) {
    row <- to_remove_idx[j, 1]
    col <- to_remove_idx[j, 2]
    tmp <- weights
    tmp[row, col] <- 0
    if (is_weakly_connected(tmp)) {
      weights <- tmp
      removed_idx[j] <- TRUE
    }
  }
  pruned <- weights
  removed <- to_remove[removed_idx, ]
  num_removed <- sum(removed_idx)
  num_retained <- n_edges - num_removed
  list(
    weights = pruned,
    method = method,
    threshold = threshold,
    lowest = lowest,
    cut_off = cut_off,
    removed = removed,
    num_removed = num_removed,
    num_retained = num_retained
  )
}

prune_bootstrap <- function(x, boot, ...) {
  if (is.null(boot)) {
    boot <- bootstrap(x, ...)
  }
  attr(boot$model, "pruning")
}

prune_disparity <- function(x, level, labels) {
  weights <- x$weights
  n_edges <- sum(weights > 0)
  disparity_filtered <- disparity_filter(weights, level)
  weights_pruned <- disparity_filtered * weights
  dimnames(weights_pruned) <- dimnames(weights)
  pruned <- weights_pruned
  removed_idx <- which(
    weights_pruned == 0 & weights != 0,
    arr.ind = TRUE
  )
  num_removed <- nrow(removed_idx)
  num_retained <- n_edges - num_removed
  removed <- data.frame(
    from = labels[removed_idx[, 1]],
    to = labels[removed_idx[, 2]],
    weight = c(weights[removed_idx])
  )
  list(
    weights = pruned,
    method = "disparity",
    level = level,
    removed = removed,
    num_removed = num_removed,
    num_retained = num_retained
  )
}

#' Print Detailed Information on the Pruning Results
#'
#' @export
#' @family validation
#' @rdname pruning_details
#' @param x A `tna` or `group_tna` object.
#' @param ... Ignored.
#' @return A `data.frame` containing the removed edges if `x` is a `tna` object,
#' or a `list` of `data.frame` objects in the case of `group_tna` object.
#' @examples
#' model <- tna(group_regulation)
#' pruned_threshold <- prune(model, method = "threshold", threshold = 0.1)
#' pruning_details(pruned_threshold)
#'
pruning_details <- function(x, ...) {
  UseMethod("pruning_details")
}

#' @export
#' @rdname pruning_details
pruning_details.tna <- function(x, ...) {
  pruning <- attr(x, "pruning")
  stopifnot_(
    !is.null(pruning),
    "Argument {.arg x} must have been pruned."
  )
  method_txt <- switch(pruning$method,
    threshold = paste0("User-specified threshold (", pruning$threshold, ")"),
    lowest = paste0("Lowest ", pruning$lowest * 100, "% of non-zero edges"),
    bootstrap = "Bootstrapping",
    disparity = paste0("Disparity filter (sig. level = ", pruning$level, ")")
  )

  cat("**Pruning Details**\n")
  cat("\nMethod used:", method_txt)
  cat("\nNumber of removed edges:", cs(pruning$num_removed))
  cat("\nNumber of retained edges:", cs(pruning$num_retained))
  cat("\n\n**Removed edges**\n\n")
  print(pruning$removed)
}

#' Restore a Pruned Transition Network Analysis Model
#'
#' @export
#' @family validation
#' @rdname deprune
#' @param x A `tna` or `group_tna` object.
#' @param ... Ignored.
#' @return A `tna` or `group_tna` object that has not been pruned.
#' @examples
#' model <- tna(group_regulation)
#' pruned_model <- prune(model, method = "threshold", threshold = 0.1)
#' depruned_model <- deprune(pruned_model) # restore original model
#'
deprune <- function(x, ...) {
  UseMethod("deprune")
}

#' @export
#' @rdname deprune
deprune.tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "tna")
  tmp <- attr(x, "pruning")
  stopifnot_(
    !is.null(tmp),
    "Argument {.arg x} must have been pruned."
  )
  stopifnot_(
    tmp$active,
    "Pruning must be active for argument {.arg x}."
  )
  tmp <- attr(x, "pruning")
  tmp$active <- FALSE
  x$weights <- tmp$original
  attr(x, "pruning") <- tmp
  x
}

#' Restore Previous Pruning of a Transition Network Analysis Model
#'
#' @export
#' @family validation
#' @rdname reprune
#' @param x A `tna` or `group_tna` object.
#' @param ... Ignored.
#' @return A `tna` or `group_tna` object that has not been pruned. The previous
#' pruning result can be reactivated with [reprune()].
#' @examples
#' model <- tna(group_regulation)
#' pruned_model <- prune(model, method = "threshold", threshold = 0.1)
#' depruned_model <- deprune(pruned_model) # restore original model
#' repruned_model <- reprune(depruned_model) # reapply the previous pruning
#'
reprune <- function(x, ...) {
  UseMethod("reprune")
}

#' @export
#' @rdname deprune
reprune.tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "tna")
  tmp <- attr(x, "pruning")
  stopifnot_(
    !is.null(tmp),
    "Argument {.arg x} must have been pruned."
  )
  stopifnot_(
    !tmp$active,
    "Pruning must not be active for argument {.arg x}."
  )
  tmp$active <- TRUE
  x$weights <- tmp$weights
  attr(x, "pruning") <- tmp
}

#' Disparity Filter Algorithm
#'
#' @param mat A weighted adjacency `matrix` of a directed graph.
#' @param level A `numeric` value for the significance level.
#' @noRd
disparity_filter <- function(mat, level) {
  d <- dim(mat)[2]
  idx_mat <- 1L * (mat > 0)
  out_edges <- mat / .rowSums(mat, m = d, n = d)
  out_degree <- .rowSums(idx_mat, m = d, n = d)
  out_p_values <- (1 - out_edges)^(out_degree - 1)
  in_edges <- t(mat) / .colSums(mat, m = d, n = d)
  in_degree <- .colSums(idx_mat, m = d, n = d)
  in_p_values <- t((1 - in_edges)^(in_degree - 1))
  p_values <- pmin(out_p_values, in_p_values)
  sig <- 1 * (p_values < level)
  diag(sig) <- 0
  sig
}


# Groups ----------------------------------------------------------------


#' @export
#' @rdname prune
prune.group_tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  structure(
    stats::setNames(
      lapply(x, prune, ...),
      names(x)
    ),
    class = "group_tna"
  )
}

#' @export
#' @rdname pruning_details
pruning_details.group_tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  invisible(Map(
    function(y, i) {
      print(i)
      pruning_details.tna(y, ...)
    },
    x,
    names(x)
  ))
}

#' @export
#' @rdname deprune
deprune.group_tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  structure(
    stats::setNames(
      lapply(x, deprune, ...),
      names(x)
    ),
    class = "group_tna"
  )
}

#' @export
#' @rdname reprune
reprune.group_tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  structure(
    stats::setNames(
      lapply(x, reprune, ...),
      names(x)
    ),
    class = "group_tna"
  )
}

test_that("pruning works with user-specified threshold", {
  result <- prune(mock_tna, threshold = 0.1) |>
    attr("pruning")
  expect_true(is.list(result))
  expect_true(is.data.frame(result$removed))
  expect_true(result$num_removed > 0)
  expect_equal(result$cut_off, 0.1)
  expect_equal(result$method, "threshold")
})

test_that("pruning works with lowest percent", {
  result <- prune(mock_tna, method = "lowest", lowest = 0.25) |>
    attr("pruning")
  expect_true(is.list(result))
  expect_true(is.data.frame(result$removed))
  expect_true(result$num_removed > 0)
  expect_equal(result$method, "lowest")
})

test_that("pruning works with disparity filter", {
  result <- prune(mock_tna, method = "disparity", level = 0.5) |>
    attr("pruning")
  expect_true(is.list(result))
  expect_true(is.data.frame(result$removed))
  expect_true(result$num_removed > 7)
  expect_equal(result$method, "disparity")
})

test_that("pruning works with bootstrap", {
  set.seed(0)
  tna_object <- tna(group_regulation)
  result <- prune(tna_object, method = "bootstrap", iter = 100) |>
    attr("pruning")
  expect_true(is.list(result))
  expect_true(is.data.frame(result$removed))
  expect_true(result$num_removed > 0)
  expect_equal(result$method, "bootstrap")
})

test_that("pruning function ensures weak connectivity", {
  result <- prune(mock_tna, threshold = 0.2) |>
    attr("pruning")
  expect_true(is.list(result))
  expect_true(is.data.frame(result$removed))
  expect_true(sum(result$weights > 0) > 0)
  expect_true(is_weakly_connected(result$weights))
  expect_equal(result$method, "threshold")
})

test_that("pruning details can be obtained", {
  expect_error(
    pruned_model <- prune(mock_tna, threshold = 0.2),
    NA
  )
  expect_error(
    out <- capture.output(pruning_details(pruned_model)),
    NA
  )
})

test_that("pruning can be deactivated", {
  pruned_model <- prune(mock_tna, threshold = 0.2)
  expect_error(
    deprune(pruned_model),
    NA
  )
})

test_that("pruning can be reactivated", {
  pruned_model <- prune(mock_tna, threshold = 0.2)
  depruned_model <- deprune(pruned_model)
  expect_error(
    reprune(depruned_model),
    NA
  )
})

test_that("weights are restored by deprune", {
  pruned_model <- prune(mock_tna, threshold = 0.2)
  depruned_model <- deprune(pruned_model)
  expect_equal(
    mock_tna$weights,
    depruned_model$weights
  )
})

test_that("pruned weights are restored by reprune", {
  pruned_model <- prune(mock_tna, threshold = 0.2)
  depruned_model <- deprune(pruned_model)
  repruned_model <- reprune(depruned_model)
  expect_equal(
    pruned_model$weights,
    repruned_model$weights
  )
})

test_that("pruning can be applied for clusters", {
  expect_error(
    prune(mmm_model, threshold = 0.3),
    NA
  )
})

test_that("pruning details can be obtained for clusters", {
  pruned_model <- prune(mmm_model, threshold = 0.3)
  expect_error(
    out <- capture.output(pruning_details(pruned_model)),
    NA
  )
})

test_that("pruning can be deactivated for clusters", {
  pruned_model <- prune(mmm_model, threshold = 0.3)
  expect_error(
    deprune(pruned_model),
    NA
  )
})

test_that("pruning can be reactivated for clusters", {
  pruned_model <- prune(mmm_model, threshold = 0.3)
  depruned_model <- deprune(pruned_model)
  expect_error(
    reprune(depruned_model),
    NA
  )
})

test_that("weights are restored by deprune for clusters", {
  pruned_model <- prune(mmm_model, threshold = 0.3)
  depruned_model <- deprune(pruned_model)
  expect_equal(
    lapply(mmm_model, "[[", "weights"),
    lapply(depruned_model, "[[", "weights")
  )
})

test_that("pruned weights are restored by reprune for clusters", {
  pruned_model <- prune(mmm_model, threshold = 0.3)
  depruned_model <- deprune(pruned_model)
  repruned_model <- reprune(depruned_model)
  expect_equal(
    lapply(pruned_model, "[[", "weights"),
    lapply(repruned_model, "[[", "weights")
  )
})

test_that("pruning function fails with invalid tna object", {
  invalid_tna_object <- list()
  class(invalid_tna_object) <- "not_tna"
  expect_error(
    prune(invalid_tna_object, threshold = 0.1),
    "no applicable method for 'prune' applied to an object of class \"not_tna\""
  )
})

#' Plot a Histogram of Edge Weights in the Network
#'
#' @export
#' @family basic
#' @inheritParams graphics::hist
#' @param main A `character` string defining the title of the plot.
#' @param xlab A `character` string defining the vertical axis label.
#' @param ... Additional arguments passed to [graphics::hist()].
#' @return A `histogram` object of edge weights.
#' @examples
#' model <- tna(group_regulation)
#' hist(model)
#'
hist.tna <- function(x, breaks, col = "lightblue",
                     main, xlab, border = "white", ...) {
  check_missing(x)
  check_class(x, "tna")
  w <- c(x$weights)
  type <- attr(x, "type")
  xlab_missing <- missing(xlab)
  if (xlab_missing) {
    xlab <- paste0(
      "Edge Weights (",
      switch(type,
             `relative` = "Probabilities",
             `frequency` = "Frequencies",
             `co-occurrence` = "Co-occurrences"
      ),
      ")"
    )
  }
  if (missing(main)) {
    main <- ifelse_(
      xlab_missing,
      paste0("Histogram of ", xlab),
      "Histogram of Edge Weights"
    )
  }
  if (missing(breaks)) {
    breaks <- ifelse_(
      type == "relative",
      seq(0, 1, length.out = 20),
      seq(0, max(w), length.out = 20)
    )
  }
  graphics::hist(
    x = w,
    breaks = breaks,
    col = col,
    main = main,
    xlab = xlab,
    border = border,
    ...
  )
}

#' Plot a Transition Network Analysis Model
#'
#' This function plots a transition network analysis (TNA) model using
#' the `cograph` package. The nodes in the graph represent states, with node
#' sizes corresponding to initial state probabilities. Edge labels represent
#' the edge weights of the network. See [cograph::splot()] for details on
#' how to further configure the plot.
#'
#' @export
#' @family basic
#' @param x A `tna` object from [build_model()].
#' @param node_list An optional `list` of two `character` vectors that define
#'   two mutually exclusive groups of node labels.
#' @param use_list_order A `logical` value. If `node_list` is provided,
#'   defines how the order of the nodes in the plot is defined. A `TRUE` value
#'   uses the order in `node_list`. Otherwise, the nodes are ranked based on
#'   edge weights and ordered according to the rank.
#' @param scale_nodes A `character` string giving the name of a centrality
#'   measure to scale the node size by. See [centralities()] for valid names.
#'   If missing (the default), uses default [cograph::splot()] scaling.
#'   The value of `node_size` provided via `...` is used as baseline size.
#' @param scaling_factor A `numeric` value specifying how strongly to scale
#'   the nodes when `scale_nodes` is provided. Values
#'   between 0 and 1 will result in smaller differences and values larger
#'   than 1 will result in greater differences. The default is `0.5`.
#' @param ... Additional arguments passed to [cograph::splot()] or
#'   [cograph::plot_htna()].
#' @return A `cograph_network` plot of the transition network.
#' @examples
#' model <- tna(group_regulation)
#' plot(model)
#'
plot.tna <- function(x, node_list, use_list_order = TRUE,
                     scale_nodes, scaling_factor = 0.5, ...) {
  check_missing(x)
  check_class(x, "tna")
  n <- nodes(x)
  scale_nodes <- ifelse_(
    missing(scale_nodes) || isFALSE(scale_nodes),
    character(0L),
    scale_nodes
  )
  scale_nodes <- ifelse_(
    isTRUE(scale_nodes),
    "InStrength",
    scale_nodes
  )
  args <- list(...)
  args$node_size <- ifelse_(
    is.null(args$node_size) & is.null(args$vsize),
    rep(8 * exp(-n / 80)),
    args$node_size | args$vsize
  )
 
  if (length(scale_nodes) > 0) {
    check_string(scale_nodes)
    check_range(scaling_factor, lower = 0)
    cent <- centralities(x, measures = scale_nodes, normalize = TRUE)[[2L]]
    args$node_size <- args$node_size * (1 + cent)^scaling_factor
  }
  args$x <- x
  if (missing(node_list)) {
    return(do.call(cograph::splot, args))
  }
  stopifnot_(
    length(node_list) == 2L,
    "Argument {.arg node_list} must be a {.cls list} of length 2."
  )
  stopifnot_(
    is.character(node_list[[1L]]) && is.character(node_list[[2L]]),
    "Elements of {.arg node_list} must be {.cls character} vectors."
  )
  check_flag(use_list_order)
  lab <- x$labels
  lhs <- node_list[[1L]]
  rhs <- node_list[[2L]]
  common <- intersect(rhs, lhs)
  stopifnot_(
    length(common) == 0,
    "The groups defined by {.arg node_list} must not contain common states."
  )
  lhs_idx <- match(lhs, lab)
  rhs_idx <- match(rhs, lab)
  missing <- lhs[is.na(lhs_idx)]
  stopifnot_(
    length(missing) == 0,
    "Nodes {.val {missing}} are not present in the model."
  )
  missing <- rhs[is.na(rhs_idx)]
  stopifnot_(
    length(missing) == 0,
    "Nodes {.val {missing}} are not present in the model."
  )
  n_lhs <- length(lhs_idx)
  n_rhs <- length(rhs_idx)
  n <- length(lab)
  missing <- lab[!lab %in% union(lhs, rhs)]
  stopifnot_(
    n_lhs + n_rhs == n,
    c(
      "Every state must belong to one of the
        groups defined by {.arg node_list}.",
      `x` = "No group has been specified for node{?s} {.val {missing}}."
    )
  )
  
  args$vsize <- args$node_size
  args$node_size <- NULL
  args$use_list_order <- use_list_order
  args$node_list <- node_list
  do.call(cograph::plot_htna, args)
}

#' Plot a Bootstrapped Transition Network Analysis Model
#'
#' @export
#' @family validation
#' @param x A `tna_bootstrap` object.
#' @param ... Additional arguments passed to [plot.tna()].
#' @examples
#' model <- tna(group_regulation)
#' # Small number of iterations for CRAN
#' boot <- bootstrap(model, iter = 50)
#' plot(boot)
#'
plot.tna_bootstrap <- function(x, ...) {
  check_missing(x)
  check_class(x, "tna_bootstrap")
  if (requireNamespace("cograph", quietly = TRUE)) {
    cograph::splot(x, ...)
  } else {
    plot(x$model, ...)
  }
}

#' Plot Centrality Measures
#'
#' Plots the centrality measures of a `tna_centralities` object as a
#' lollipop chart. The resulting plot includes facets for each centrality
#' measure, showing the values for each state. The returned plot is a
#' `ggplot2` object, so it can be easily modified and styled. See
#' [centralities()] for details on the centrality measures.
#'
#' @export
#' @family centralities
#' @param x An object of class `tna_centralities`.
#' @param ncol Number of columns to use for the facets. The default is 3.
#' @param scales Either `"fixed"` or `"free_x"` (the default). If `"free_x"`,
#'   the horizontal axis is scaled individually in each facet. If `"fixed"`,
#'   the same values are used for all axes.
#' @param reorder A `logical` value indicating whether to reorder the values
#'   for each centrality in a descending order. The default is `TRUE`.
#' @param colors The colors for each node (default is the model colors
#'  if the `tna` model object is passed, otherwise `"black"`).
#' @param labels A `logical` value indicating whether to show the centrality
#'   numeric values. The default is `TRUE`.
#' @param ... Ignored.
#' @return A `ggplot` object displaying the lollipop charts for each centrality
#'   measure.
#' @examples
#' tna_model <- tna(group_regulation)
#' cm <- centralities(tna_model)
#' plot(cm, ncol = 3, reorder = TRUE)
#'
plot.tna_centralities <- function(x, reorder = TRUE, ncol = 3,
                                  scales = c("free_x", "fixed"),
                                  colors, labels = TRUE, ...) {
  check_class(x, "tna_centralities")
  plot_centralities_(x, reorder, ncol, scales, colors, NULL, labels)
}

#' Plot Cliques of a TNA Network
#'
#' @export
#' @family cliques
#' @inheritParams print.tna_cliques
#' @inheritParams plot.tna
#' @param colors Optional `character` vector of colors values to use
#'   for the nodes.
#' @param show_loops A `logical` value indicating whether to include loops
#'   in the plots or not.
#' @param ask A `logical` value. When `TRUE`, show plots one by one and asks
#'   to plot the next plot in interactive mode.
#' @return `NULL` (invisibly).
#' @examples
#' model <- tna(group_regulation)
#' cliq <- cliques(model, size = 2)
#' plot(cliq, n = 1, ask = FALSE)
#'
plot.tna_cliques <- function(x, n = 6, first = 1, show_loops = FALSE,
                             colors, ask = TRUE, ...) {
  check_class(x, "tna_cliques")
  n_cliques <- length(x$weights)
  size <- attr(x, "size")
  if (n_cliques == 0) {
    warning_("No {size}-cliques were found in the network.")
    return(invisible(NULL))
  }
  colors <- colors %m% attr(x, "colors")
  labels <- attr(x, "labels")
  max_cliques <- min(first + n - 1L, n_cliques)
  if (interactive()) { # nocov start
    op <- graphics::par(ask = ask)
    on.exit(graphics::par(op))
  } # nocov end
  for (i in seq(first, max_cliques)) {
    clique_weights <- x$weights[[i]]
    diag(clique_weights) <- ifelse_(
      show_loops,
      diag(clique_weights),
      0
    )
    plot_args <- list(
      x = clique_weights,
      labels = colnames(clique_weights),
      node_fill = colors[match(rownames(clique_weights), labels)],
      donut_fill = x$inits[[i]],
      edge_labels = TRUE
    )
    plot_args <- utils::modifyList(plot_args, list(...))
    do.call(cograph::splot, args = plot_args)
  }
  invisible(NULL)
}

#' Plot Communities
#'
#' This function visualizes the communities detected within a `tna` object
#' based on different community detection algorithms and their corresponding
#' color mappings.
#'
#' @export
#' @family communities
#' @param x A `communities` object generated by the `find_communities` method.
#'   Each community detection method maps nodes or points in to a specific
#'   communities.
#' @param colors A `character` vector of color values used for visualizing
#'   community assignments.
#' @param method A `character` string naming a community detection method to
#'   use for coloring the plot. The default is to use the first available
#'   method in `x`. See [communities()] for details.
#' @param ... Additional arguments passed to [cograph::splot()].
#' @return A `cograph_network` object in which the nodes are colored by
#'   community.
#' @examples
#' model <- tna(group_regulation)
#' comm <- communities(model)
#' plot(comm, method = "leading_eigen")
#'
plot.tna_communities <- function(x, colors, method, ...) {
  check_class(x, "tna_communities")
  available_methods <- intersect(
    names(x$assignment),
    names(supported_communities)
  )
  method <- method %m% available_methods[1L]
  check_string(method)
  stopifnot_(
    method %in% available_methods,
    "The {.val {method}} method is not available in {.arg x}."
  )
  y <- attr(x, "tna")
  colors <- colors %m% default_colors
  plot(y, node_fill = map_to_color(x$assignment[[method]], colors), ...)
}

#' Plot the Comparison of Two TNA Models or Matrices
#'
#' @export
#' @family comparison
#' @param x A `tna_comparison` object.
#' @param type A `character` string naming the type of plot to produce. The
#'   available options are `"heatmap"` (the default), `"scatterplot"`,
#'   `"centrality_heatmap"`, and `"weight_density"`.
#' @param population A `"character"` string naming the population for which
#'   to produce the heatmaps, i.e, one of `"x"`, `"y"`, or `"difference"` for
#'   the differences. Ignored for `type = "scatterplot"`. Defaults to `"diff"`.
#' @param method A `character` string naming the correlation coefficient to
#'   use when plotting a scatterplot. The available options are `"pearson"`
#'   (the default), `"kendall"`, `"spearman"`, and `"distance"`. The final
#'   option is the distance correlation coefficient of
#'   Szekely, Rizzo, and Bakirov (2007). See also the `energy` package for
#'   further information on this measure.
#' @param name_x An optional `character` string to use as the name of the
#'   first population in the plots. The default is `"x"`.
#' @param name_y An optional `character` string to use as the name of the
#'   second population in the plots. The default is `"y"`.
#' @param ... Ignored.
#' @return A `ggplot` object.
#' @references
#' Szekely, G.J., Rizzo, M.L., and Bakirov, N.K. (2007),
#' Measuring and Testing Dependence by Correlation of Distances,
#' *Annals of Statistics*, 35(6), 2769-2794.
#' doi:10.1214/009053607000000505
#' @examples
#' model_x <- tna(group_regulation[1:200, ])
#' model_y <- tna(group_regulation[1001:1200, ])
#' comp <- compare(model_x, model_y)
#' plot(comp)
#'
plot.tna_comparison <- function(x, type = "heatmap",
                                population = "difference", method = "pearson",
                                name_x = "x", name_y = "y", ...) {
  check_class(x, "tna_comparison")
  check_string(name_x)
  check_string(name_y)
  type <- check_match(
    type,
    c("heatmap", "scatterplot", "centrality_heatmap", "weight_density")
  )
  if (type == "heatmap") {
    population <- check_match(population, c("x", "y", "difference"))
    weight_col <- switch(
      population,
      x = "weight_x",
      y = "weight_y",
      difference = "raw_difference"
    )
    title <- switch(
      population,
      x = paste0("Heatmap ", name_x),
      y = paste0("Heatmap ", name_y),
      difference = paste0(
        "Difference Matrix Heatmap (", name_x, " vs. ", name_y, ")"
      )
    )
    edges <- x$edge_metrics[, c("source", "target", weight_col)]
    names(edges)[3] <- "value"
    return(create_heatmap(edges, title))
  }
  if (type == "scatterplot") {
    method <- check_match(
      method,
      c("pearson", "kendall", "spearman", "distance")
    )
    edges <- x$edge_metrics[, c("source", "target", "weight_x", "weight_y")]
    metric_idx <- tolower(x$summary_metrics$metric) == method
    corr <- round(x$summary_metrics$value[metric_idx], 3)
    # switch tracking does not seem to work here
    corr_subtitle <- switch( # nocov start
      method,
      pearson = bquote("Pearson's" ~ {rho} ~~ "=" ~~ .(corr)),
      spearman = bquote("Spearman's" ~ {rho} ~~ "=" ~~ .(corr)),
      kendall = bquote("Kendall's" ~ {tau} ~~ "=" ~~ .(corr)),
      distance = paste0("Distance correlation = ", corr)
    ) # nocov end
    out <-
      ggplot2::ggplot(
        edges,
        ggplot2::aes(
          !!rlang::sym("weight_x"),
          !!rlang::sym("weight_y")
        )
      ) +
      ggplot2::geom_point(alpha = 0.6, color = "blue") +
      ggplot2::geom_smooth(
        formula = y ~ x, method = "lm", color = "red", linetype = "dashed"
      ) +
      ggplot2::labs(
        title = paste0("Correlation between ", name_x, " and ", name_y),
        subtitle = corr_subtitle,
        x = paste0("Weights (", name_x, ")"),
        y = paste0("Weights (", name_y, ")")
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        plot.title = ggplot2::element_text(size = 12, face = "bold"),
        plot.subtitle = ggplot2::element_text(size = 10),
        axis.title = ggplot2::element_text(size = 10))
    return(out)
  }
  if (type == "centrality_heatmap") {
    out <-
      ggplot2::ggplot(
        x$centrality_differences,
        ggplot2::aes(
          x = !!rlang::sym("centrality"),
          y = !!rlang::sym("state"),
          fill = !!rlang::sym("difference")
        )
      ) +
      ggplot2::geom_tile() +
      ggplot2::scale_fill_gradient2(
        low = "blue",
        high = "red",
        mid = "white"
      ) +
      ggplot2::labs(
        title = paste0(
          "Centrality Difference Heatmap (", name_x, " vs ", name_y, ")"
        ),
        x = "Centrality Measure",
        y = "Node"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 8),
        axis.text.y = ggplot2::element_text(size = 8),
        plot.title = ggplot2::element_text(size = 12, face = "bold")
      )
    return(out)
  }
  if (type == "weight_density") {
    edges <- x$edge_metrics[, c("source", "target", "weight_x", "weight_y")]
    out <-
      ggplot2::ggplot(
        edges,
        ggplot2::aes(x = !!rlang::sym("weight_x"), color = name_x)
      ) +
      ggplot2::geom_density(alpha = 0.5) +
      ggplot2::geom_density(
        ggplot2::aes(x = !!rlang::sym("weight_y"), color = name_y),
        alpha = 0.5
      ) +
      ggplot2::labs(
        title = "Density Plot of Model Weights",
        x = "Weight",
        y = "Density",
        color = "Model"
      ) +
      ggplot2::theme_minimal()
    return(out)
  }
}

#' Plot the Significant Differences from a Permutation Test
#'
#' @export
#' @family validation
#' @param x A `tna_permutation` object.
#' @param colors See [cograph::splot()].
#' @param ... Arguments passed to [plot_model()].
#' @param posCol Color for plotting edges
#'   the difference in edge weights is positive. See [cograph::splot()].
#' @param negCol Color for plotting edges when
#'   the the difference in edge weights is negative. See [cograph::splot()].
#' @param edge_labels Boolean indicating whether edge labels should be 
#'   ploted. Defaults to `TRUE`
#' @return A `cograph_network` object containing only the significant edges
#'   according to the permutation test.
#' @examples
#' model_x <- tna(group_regulation[1:200, ])
#' model_y <- tna(group_regulation[1001:1200, ])
#' # Small number of iterations for CRAN
#' perm <- permutation_test(model_x, model_y, iter = 20)
#' plot(perm)
#'
plot.tna_permutation <- function(x, colors,
                                 posCol = "#009900", 
                                 negCol = "red", 
                                 edge_labels = TRUE, ...) {
  check_missing(x)
  check_class(x, "tna_permutation")
  colors <- colors %m% attr(x, "colors")
  plot_model(
    x$edges$diffs_sig,
    labels = attr(x, "labels"),
    colors = colors,
    edge_positive_color = posCol,
    edge_negative_color = negCol,
    edge_labels = edge_labels,
    ...
  )
}

#' Plot Centrality Stability Results
#'
#' This function visualizes the centrality stability results produced by the
#' `estimate_centrality_stability` function. It shows how different centrality
#' measures' correlations change as varying proportions of cases are dropped,
#' along with their confidence intervals (CIs).
#'
#' @export
#' @family validation
#' @param x A `tna_stability` object produced by `estimate_cs`.
#' @param level A `numeric` value representing the significance level for
#'   the confidence intervals. Defaults to `0.05`.
#' @param ... Ignored.
#'
#' @details
#' The function aggregates the results for each centrality measure across
#' multiple proportions of dropped cases (e.g., 0.1, 0.2, ..., 0.9) and
#' calculates the mean and the desired quantiles for each proportion.
#' The confidence intervals (CIs) are computed based on the quantiles
#' and displayed in the plot.
#'
#' If no valid data is available for a centrality measure
#' (e.g., missing or NA values), the function skips that measure with a warning.
#'
#' The plot includes:
#'
#' * The mean correlation for each centrality measure as a function of the
#'   proportion of dropped cases.
#' * Shaded confidence intervals representing CIs for each centrality measure.
#' * A horizontal dashed line at the threshold value used for calculating
#'   the CS-coefficient.
#' * A subtitle listing the CS-coefficients for each centrality measure.
#'
#' @return A `ggplot` object displaying the stability analysis plot.
#' @examples
#' model <- tna(group_regulation)
#' cs <- estimate_cs(model, iter = 10)
#' plot(cs)
#'
plot.tna_stability <- function(x, level = 0.05, ...) {
  check_class(x, "tna_stability")
  check_range(level, lower = 0, upper = 1)
  x$detailed_results <- NULL
  x_names <- names(x)
  drop_prop <- attr(x, "drop_prop")
  threshold <- attr(x, "threshold")
  measure_data <- vector(mode = "list", length = length(x))
  cs_subtitle <- character(length(x))
  for (i in seq_along(x)) {
    measure <- x_names[i]
    corr <- x[[measure]]$correlations
    means <- apply(corr, 2L, mean, na.rm = TRUE)
    ci_lower <- apply(
      corr,
      2L,
      stats::quantile,
      probs = level / 2.0,
      na.rm = TRUE
    )
    ci_upper <- apply(
      corr,
      2L,
      stats::quantile,
      probs = 1.0 - level / 2.0,
      na.rm = TRUE
    )
    measure_data[[i]] <- data.frame(
      measure = measure,
      proportion = drop_prop,
      correlation = means,
      lower = ci_lower,
      upper = ci_upper
    )
    cs_coef <- x[[measure]]$cs_coefficient
    cs_subtitle[i] <- paste0(
      measure,
      " CS = ",
      round(cs_coef, 2L)
    )
  }
  plot_data <- dplyr::bind_rows(measure_data)
  stopifnot_(
    nrow(plot_data) > 0,
    "No valid data to plot."
  )
  cs_subtitle <- paste0(cs_subtitle, collapse = "; ")
  ggplot2::ggplot(
    plot_data,
    ggplot2::aes(
      x = !!rlang::sym("proportion"),
      y = !!rlang::sym("correlation"),
      color = !!rlang::sym("measure")
    )
  ) +
    ggplot2::geom_ribbon(
      ggplot2::aes(
        ymin = !!rlang::sym("lower"),
        ymax = !!rlang::sym("upper"),
        fill = !!rlang::sym("measure")
      ),
      alpha = 0.2
    ) +
    ggplot2::geom_line() +
    ggplot2::geom_hline(
      yintercept = threshold,
      linetype = "dashed",
      color = "gray50"
    ) +
    ggplot2::geom_hline(
      yintercept = 0,
      linetype = "solid",
      color = "gray25"
    ) +
    ggplot2::labs(
      title = "Centrality Stability Analysis",
      subtitle = paste0("CS-Coeficients: ", cs_subtitle),
      x = "Proportion of Cases Dropped",
      y = "Correlation with Original Centrality",
      color = "Centrality Measure",
      fill = "Centrality Measure"
    ) +
    ggplot2::scale_x_continuous(breaks = drop_prop) +
    ggplot2::theme_minimal() +
    ggplot2::ylim(-1, 1)
}

#' Plot a Sequence Comparison
#'
#' Visualize the differences in pattern counts by plotting the standardized
#' residuals by pattern and group.
#'
#' @export
#' @family comparison
#' @param x A `tna_sequence_comparison` object.
#' @param n An `integer` giving the number of patterns to plot.
#'   The default is `10`.
#' @param legend  A `logical` value indicating whether to show the color scale
#'   legend. The default is `TRUE`.
#' @param cells A `logical` value indicating whether to display the
#'   numeric values in each cell. The default is `TRUE`.
#' @param text_color A `character` string specifying the text color to use for
#'   the cell values. The default is `"white"`.
#' @param digits An `integer` specifying the number of digits for the cell
#'   values.
#' @param ... Not used.
#' @return A `ggplot` object.
#' @examples
#' \donttest{
#' idx <- c(1:500, 1001:1500)
#' group <- c(rep("High", 500), rep("Low", 500))
#' comp <- compare_sequences(group_regulation[idx, ], group)
#' plot(comp)
#' }
#'
plot.tna_sequence_comparison <- function(x, n = 10, legend = TRUE,
                                         cells = TRUE, text_color = "white",
                                         digits = 2L, ...) {
  check_missing(x)
  check_class(x, "tna_sequence_comparison")
  check_values(n, strict = TRUE)
  check_flag(legend)
  check_flag(cells)
  check_string(text_color)
  check_values(digits)
  n <- min(n, length(x$pattern))
  pat <- x$pattern[seq_len(n)]
  freq_cols <- attr(x, "freq_cols")
  groups <- attr(x, "groups")
  x <- x |>
    dplyr::filter(!!rlang::sym("pattern") %in% pat) |>
    dplyr::select(dplyr::all_of(freq_cols))
  n <- sum(x)
  rs <- rowSums(x)
  cs <- colSums(x)
  expected <- outer(rs, cs) / n
  v <- function(r, c, n) c * r * (n - r) * (n - c) / n^3
  cell_var <- outer(rs, cs, v, n)
  resid <- (x - expected) / sqrt(cell_var)
  n <- nrow(resid)
  m <- ncol(resid)
  d <- data.frame(xmin = rep(0.0, n * m), xmax = 0.0, ymin = 0.0, ymax = 0.0)
  for (i in seq_len(n)) {
    for (j in seq_len(m)) {
      row <- (i - 1) * m + j
      d[row, "xmin"] <- j
      d[row, "xmax"] <- j + 1.0
      d[row, "ymin"] <- -i
      d[row, "ymax"] <- -i - 1.0
      d[row, "resid"] <- resid[i, j]
      d[row, "label"] <- round(resid[i, j], digits = digits)
    }
  }
  d$xcent <- (d$xmin + d$xmax) / 2.0
  d$ycent <- (d$ymin + d$ymax) / 2.0
  p <-
    ggplot2::ggplot(
      d,
      ggplot2::aes(
        xmin = !!rlang::sym("xmin"),
        xmax = !!rlang::sym("xmax"),
        ymin = !!rlang::sym("ymin"),
        ymax = !!rlang::sym("ymax"),
        fill = !!rlang::sym("resid")
      )
    ) +
    ggplot2::geom_rect(color = "white", show.legend = legend) +
    ggplot2::scale_fill_gradient2(
      name = "Standardized\nresidual",
      oob = bound,
      low = "#D33F6A",
      high = "#4A6FE3",
      limits = c(-4, 4),
      breaks = c(-4, -2, 0, 2, 4)
    ) +
    ggplot2::scale_x_continuous(
      breaks = unique(d$xcent),
      labels = groups,
      position = "bottom",
      expand = c(0.01, 0)
    ) +
    ggplot2::scale_y_continuous(
      breaks = unique(d$ycent),
      labels = pat,
      expand = c(0.01, 0)
    ) +
    ggplot2::theme_classic() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(hjust = 0.5),
      plot.subtitle = ggplot2::element_text(hjust = 0.5),
      axis.ticks = ggplot2::element_blank(),
      axis.line = ggplot2::element_blank(),
      axis.text.x =  ggplot2::element_text(
        angle = ifelse(n > 3, 90, 0),
        hjust = ifelse(n > 3, 0, 0.5),
        vjust = ifelse(n > 3, 0.5, 0)
      ),
      legend.key.size = ggplot2::unit(1, "cm"),
      axis.text.y = ggplot2::element_text(hjust = 1, vjust = 0.40)
    ) +
    ggplot2::labs(x = "Groups", y = "")
  if (cells) {
    p <- p +
      ggplot2::geom_text(
        mapping = ggplot2::aes(
          x = !!rlang::sym("xcent"),
          y = !!rlang::sym("ycent"),
          label = !!rlang::sym("label")
        ),
        fontface = "bold",
        color = text_color
      )
  }
  p
}

#' Plot Reliability Analysis Results
#'
#' @export
#' @family validation
#' @param x A `tna_reliability` object.
#' @param type A `character` string specifying the plot type. The options are:
#'   `"histogram"` (default), `"density"`, or `"boxplot"`.
#' @param metric A `character` string specifying the metric to plot.
#'   The default is the median absolute difference (`"Median Abs. Diff."`).
#' @param ... Ignored
#' @examples
#' # Small number of iterations for CRAN
#' model <- tna(engagement)
#' rel <- reliability(model, iter = 20)
#' plot(rel)
#'
plot.tna_reliability <- function(x, type = "histogram",
                                 metric = "Median Abs. Diff.", ...) {
  check_missing(x)
  check_class(x, "tna_reliability")
  check_string(metric)
  type <- check_match(type, c("histogram", "density", "boxplot"))
  stopifnot_(
    !metric %in% unique(x$metrics),
    "Metric {.val {metric}} was not found in {.arg x}"
  )
  met <- metric
  d <- x$metrics |>
    dplyr::filter(metric == met)
  has_model <- "model_type" %in% names(x$metrics)
  if (has_model) {
    stats <- d |>
      dplyr::group_by(!!rlang::sym("model_type")) |>
      dplyr::summarize(
        mean = mean(!!rlang::sym("value"), na.rm = TRUE),
        sd = stats::sd(!!rlang::sym("value"), na.rm = TRUE),
        .groups = "drop"
      ) |>
      dplyr::mutate(
        label = paste0(
          !!rlang::sym("model_type"),
          " (Mean: ",
          round(!!rlang::sym("mean"), 3),
          ", SD: ",
          round(!!rlang::sym("sd"), 3),
          ")"
        )
      )
    d <- d |> dplyr::left_join(stats, by = "model_type")
    group_var <- "label"
  } else {
    stats <- d |>
      dplyr::summarize(
        mean = mean(!!rlang::sym("value"), na.rm = TRUE),
        sd = stats::sd(!!rlang::sym("value"), na.rm = TRUE)
      )
    group_var <- NULL
  }
  sub <- paste0(
    "Mean: ", round(stats$mean, 3),
    " | SD: ", round(stats$sd, 3)
  )
  p <- ggplot2::ggplot(d, ggplot2::aes(x = !!rlang::sym("value"))) +
    ggplot2::labs(
      title = paste("Reliability Distribution:", metric),
      x = metric,
      y = "Frequency"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      plot.title = ggplot2::element_text(face = "bold"),
      legend.position = "top",
      legend.title = ggplot2::element_blank()
    )
  if (type == "histogram") {
    if (has_model) {
      p <- p +
        ggplot2::geom_histogram(
          ggplot2::aes(
            fill = !!rlang::sym(group_var)
          ),
          position = "identity",
          alpha = 0.5,
          bins = 30,
          color = "white"
        ) +
        ggplot2::geom_vline(
          data = stats,
          ggplot2::aes(
            xintercept = !!rlang::sym("mean"),
            color = !!rlang::sym("label"),
          ),
          linetype = "dashed",
          linewidth = 1,
          show.legend = FALSE
        )
    } else {
      p <- p +
        ggplot2::geom_histogram(
          fill = "cadetblue",
          color = "white",
          alpha = 0.7,
          bins = 30
        ) +
        ggplot2::geom_vline(
          ggplot2::aes(xintercept = stats$mean),
          linetype = "dashed",
          color = "firebrick",
          linewidth = 1
        ) +
        ggplot2::labs(y = "Frequency", subtitle = sub)
    }
  } else if (type == "density") {
    if (has_model) {
      p <- p +
        ggplot2::geom_density(
          ggplot2::aes(fill = !!rlang::sym(group_var)),
          alpha = 0.4
        ) +
        ggplot2::geom_vline(
          data = stats,
          ggplot2::aes(
            xintercept = !!rlang::sym("mean"),
            color = !!rlang::sym("label")
          ),
          linetype = "dashed",
          linewidth = 1,
          show.legend = FALSE
        ) +
        ggplot2::labs(y = "Density")
    } else {
      p <- p +
        ggplot2::geom_density(fill = "cadetblue", alpha = 0.5) +
        ggplot2::geom_vline(
          ggplot2::aes(xintercept = stats$mean),
          linetype = "dashed",
          color = "firebrick",
          linewidth = 1
        ) +
        ggplot2::labs(y = "Density", subtitle = sub)
    }

  } else if (type == "boxplot") {
    if (has_model) {
      p <- ggplot2::ggplot(
        d,
        ggplot2::aes(
          x = !!rlang::sym(group_var),
          y = !!rlang::sym("value"),
          fill = !!rlang::sym(group_var))
      ) +
        ggplot2::geom_boxplot(alpha = 0.7) +
        ggplot2::labs(x = "", title = metric) +
        ggplot2::theme_minimal() +
        ggplot2::theme(legend.position = "none")
    } else {
      p <- ggplot2::ggplot(
        d,
        ggplot2::aes(y = !!rlang::sym("value"))
      ) +
        ggplot2::geom_boxplot(fill = "cadetblue", alpha = 0.7) +
        ggplot2::labs(title = metric, subtitle = sub) +
        ggplot2::theme_minimal()
    }
  }
  p
}


#' Plot Centrality Measures
#'
#' @inheritParams plot.tna_centralities
#' @noRd
plot_centralities_ <- function(x, reorder, ncol, scales, colors,
                               palette, labels) {
  check_flag(reorder)
  check_flag(labels)
  scales <- check_match(scales, c("free_x", "fixed"))
  scales <- ifelse_(scales == "free_x", "free", "free_y")
  n <- n_unique(x$state)
  colors <- ifelse_(
    missing(colors),
    attr(x, "colors") %||% rep("black", n),
    rep(colors, length.out = n)
  )
  ifelse_(
    inherits(x, "tna_centralities"),
    plot_centralities_single(x, reorder, ncol, scales, colors, labels),
    plot_centralities_multiple(x, reorder, ncol, scales, colors, palette, labels)
  )
}

#' Plot Centralities for a Single Cluster
#'
#' @inheritParams plot.tna_centralities
#' @noRd
plot_centralities_single <- function(x, reorder, ncol, scales, colors, labels) {
  # Create some NULLs for R CMD Check
  name <- value <- NULL
  levs <- names(x)[-1L]
  x <- stats::reshape(
    as.data.frame(x),
    idvar = "state",
    ids = x[["state"]],
    times = levs,
    timevar = "name",
    drop = "state",
    varying = list(levs),
    direction = "long",
    v.names = "value"
  ) |>
    dplyr::group_by(name) |>
    dplyr::mutate(
      prop = value / sum(value, na.rm = TRUE),
      name = factor(name, levels = levs)
    )
  n_measures <- length(levs)
  state_within_name <- paste(x$state, x$name, sep = "___")
  x$state_within <- ifelse_(
    reorder,
    stats::reorder(state_within_name, x$value, FUN = identity),
    x$state
  )
  ggplot2::ggplot(x) +
    ggplot2::scale_fill_manual(values = colors) +
    ggplot2::geom_col(
      ggplot2::aes(
        fill = !!rlang::sym("state"),
        x = !!rlang::sym("state_within"),
        y = !!rlang::sym("value")
      ),
      linewidth = 4
    ) +
    ggplot2::coord_flip(clip = "off") +
    onlyif(
      labels,
      ggplot2::geom_text(
        ggplot2::aes(
          label = paste0(" ", round(!!rlang::sym("value"), 2), " "),
          x = !!rlang::sym("state_within"),
          y = !!rlang::sym("value"),
          hjust = !!rlang::sym("prop") > 0.05
        ),
        vjust = 0.35,
        size = 3
      )
    ) +
    ggplot2::facet_wrap(~name, ncol = ncol, scales = scales) +
    ggplot2::scale_x_discrete(
      labels = function(y) gsub("___.+$", "", y)
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      legend.position = "none",
      panel.grid.major.y = ggplot2::element_blank(),
      panel.grid.minor.y = ggplot2::element_blank(),
      panel.grid.minor.x = ggplot2::element_blank(),
      strip.text = ggplot2::element_text(face = "bold", size = 12),
      axis.text.y = ggplot2::element_text(size = 8, vjust = 0.2),
      panel.spacing = ggplot2::unit(2, "lines"),
      plot.margin = ggplot2::margin(5.5, 11, 5.5, 5.5, "points")
    ) +
    ggplot2::xlab("") +
    ggplot2::ylab("")
}

plot_centralities_multiple <- function(x, reorder, ncol, scales,
                                       colors, palette = "Set2", labels) {
  measures <- names(x)[3:ncol(x)]
  n_clusters <- n_unique(x$group)
  x$state <- factor(x$state)
  x <- x |>
    data.frame() |>
    stats::reshape(
      varying = measures,
      v.names = "value",
      timevar = "name",
      times = measures,
      direction = "long"
    )
  x$name <- factor(x$name, levels = measures)
  ggplot2::ggplot(
    x,
    ggplot2::aes(
      x = !!rlang::sym("value"),
      y = !!rlang::sym("state"),
      color = !!rlang::sym("group"),
      fill = !!rlang::sym("group"),
      group = !!rlang::sym("group")
    )
  ) +
    ggplot2::facet_wrap("name", ncol = ncol, scales = scales) +
    ggplot2::geom_path() +
    ifelse_(
      !is.null(colors) & (n_unique(colors) == n_clusters),
      ggplot2::scale_color_manual(values = colors),
      ggplot2::scale_color_brewer(palette = palette)
    ) +
    ggplot2::geom_point(size = 2, shape = 21, stroke = NA) +
    ifelse_(
      !is.null(colors) & (n_unique(colors) == n_clusters),
      ggplot2::scale_fill_manual(values = colors),
      ggplot2::scale_fill_brewer(palette = palette)
    ) +
    ggplot2::theme_minimal() +
    ggplot2::xlab("Centrality") +
    ggplot2::ylab("") +
    ggplot2::labs(color = "", fill = "") +
    ggplot2::theme(
      strip.text = ggplot2::element_text(face = "bold", size = 12),
      panel.spacing = ggplot2::unit(1, "lines"),
      axis.text.y = ggplot2::element_text(size = 8, vjust = 0.2),
      legend.position = "bottom"
    )
}

#' Plot the Difference Network Between Two Models
#'
#' Plots the difference network between model `x` and model `y`. The edges are
#' computed from subtracting the two models. The pie chart is the difference in
#' initial probabilities between model `x` and model `y`. Green color indicates
#' that `x`is greater than `y`and red indicates otherwise.
#'
#' @export
#' @family comparison
#' @param x A `tna` object. This is the the principal model.
#' @param y A `tna` object. This is the model subtracted from the
#'   principal model.
#' @param posCol Color for plotting edges and pie when
#'   the first group has a higher value. See [cograph::splot()].
#' @param negCol Color for plotting edges and pie when
#'   the second group has a higher value. See [cograph::splot()].
#' @param ... Additional arguments passed to [cograph::splot()].
#' @return A `cograph_network` object displaying the difference network
#'   between the two models.
#' @examples
#' model_x <- tna(group_regulation[group_regulation[, 1] == "plan", ])
#' model_y <- tna(group_regulation[group_regulation[, 1] != "plan", ])
#' plot_compare(model_x, model_y)
#'
plot_compare <- function(x, ...) {
  UseMethod("plot_compare")
}

#' @export
#' @rdname plot_compare
plot_compare.tna <- function(x, y, posCol = "#009900", negCol = "red", ...) {
  check_class(x, "tna")
  check_class(y, "tna")
  stopifnot_(
    all(x$labels == y$labels),
    "{.arg x} and {.arg y} must have the same labels."
  )
  cograph::plot_compare(
    x = x, y = y, pos_color = posCol, neg_color = negCol, ...
  )
}

#' Plot the Frequency Distribution of States
#'
#' @export
#' @family basic
#' @param x A `tna` object created from sequence data.
#' @param colors A `character` vector of colors to be used in the plot
#'   (one per label) or a single color.
#' @param width A `numeric` value for the Width of the bars. Default is 0.7,
#' @param hjust A `numeric` value for the horizontal adjustment of the labels.
#'   Default is 1.2.
#' @param show_label A `logical` value indicating whether to show a label with
#'   the frequency counts. Default is `TRUE`.
#' @param ... Ignored.
#' @return A `ggplot` object.
#' @examples
#' model <- tna(group_regulation)
#' plot_frequencies(model)
#' plot_frequencies(model, width =  0.5, colors = "pink")
#'
plot_frequencies <- function(x, ...) {
  UseMethod("plot_frequencies")
}

#' @export
#' @rdname plot_frequencies
plot_frequencies.tna <- function(x, width = 0.7, hjust = 1.2,
                                 show_label = TRUE, colors, ...) {
  check_missing(x)
  check_tna_seq(x)
  check_values(width, type = "numeric")
  check_numeric(hjust)
  check_flag(show_label)
  colors <- ifelse_(
    missing(colors),
    attr(x$data, "colors") %||% "black",
    colors
  )
  n_colors <- length(colors)
  n_labels <- length(x$labels)
  stopifnot_(
    n_colors == 1L || n_colors == n_labels,
    "The number of {.arg colors} does not match
     the number of labels in {.arg x}."
  )
  colors <- ifelse_(
    n_colors == 1L,
    rep(colors, n_labels),
    colors
  )
  tab <- table(unlist(x$data))
  d <- as.data.frame(tab)
  names(d) <- c("state", "freq")
  d[[1L]] <- factor(x$labels[d[[1L]]], levels = rev(x$labels))
  p <- ggplot2::ggplot(
    d,
    ggplot2::aes(y = !!rlang::sym("state"), x = !!rlang::sym("freq"))
  ) +
    ggplot2::geom_bar(
      ggplot2::aes(fill = !!rlang::sym("state")),
      stat = "identity",
      width = width
    )
  if (show_label) {
    p <- p +
      ggplot2::geom_text(
        ggplot2::aes(label = !!rlang::sym("freq")),
        position = ggplot2::position_dodge(width = width),
        hjust = hjust
      )
  }
  p +
    ggplot2::theme_minimal() +
    ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0, .1))) +
    ggplot2::labs(y = "State", x = "Frequency") +
    ggplot2::theme(
      axis.title =  ggplot2::element_text(face = "bold"),
      axis.text = ggplot2::element_text(color = "black"),
      text = ggplot2::element_text(color = "black")
    ) +
    ggplot2::scale_fill_manual(values = colors) +
    ggplot2::theme(legend.position = "none")
}

#' Plot a Transition Network Model from a Matrix of Edge Weights
#'
#' @export
#' @param x A square `matrix` of edge weights.
#' @param labels Optional `character` vector of node labels.
#' @param colors An optional `character` vector of node colors to use.
#' @inheritParams plot.tna
#' @keywords internal
#' @return See [plot.tna()].
#' @examples
#' m <- matrix(rexp(25), 5, 5)
#' plot_model(m)
#'
plot_model <- function(x, labels, colors, ...) {
  stopifnot_(
    is.matrix(x) && ncol(x) == nrow(x),
    "Argument {.arg x} must be a square matrix."
  )
  nc <- ncol(x)
  labels <- labels %m% seq_len(nc)
  colors <- colors %m% color_palette(nc)
  cograph::splot(
    x = x,
    node_fill = colors,
    labels = labels,
    ...
  )
}

#' Create a Mosaic Plot of Transitions or Events
#'
#' @export
#' @family basic
#' @param x A `tna` or a `group_tna` object.
#' @param ... Ignored.
#' @return A `ggplot` object.
#' @examples
#' ftna_model <- ftna(group_regulation)
#' plot_mosaic(ftna_model)
#'
plot_mosaic <- function(x, ...) {
  UseMethod("plot_mosaic")
}

#' @export
#' @rdname plot_mosaic
plot_mosaic.tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "tna")
  stopifnot_(
    attr(x, "type") %in% c("frequency", "co-occurrence"),
    "Mosaic plots are supported only for integer-valued weight matrices."
  )
  plot_mosaic_(
    as.table(t(x$weights)),
    xlab = "Incoming edges",
    ylab = "Outgoing edges"
  )
}

# from https://stackoverflow.com/questions/19233365/how-to-create-a-marimekko-mosaic-plot-in-ggplot2,
# Based on the code by Jake Fisher and cpsyctc.
plot_mosaic_ <- function(tab, xlab, ylab) {
  n <- nrow(tab)
  m <- ncol(tab)
  rs <- .rowSums(tab, n, m)
  widths <- c(0, cumsum(rs)) / sum(tab)
  heights <- apply(tab, 1L, function(y) c(0, cumsum(y / sum(y))))
  d <- data.frame(xmin = rep(0, n * m), xmax = 0, ymin = 0, ymax = 0)
  for (i in seq_len(n)) {
    for (j in seq_len(m)) {
      row <- (i - 1) * m + j
      row_offset <- (i - 1) * n * 0.0025
      col_offset <- (j - 1) * m * 0.0025
      d[row, "xmin"] <- widths[i] + row_offset
      d[row, "xmax"] <- widths[i + 1] + row_offset
      d[row, "ymin"] <- heights[j, i] + col_offset
      d[row, "ymax"] <- heights[j + 1, i] + col_offset
      d[row, "freq"] <- tab[i, j]
    }
  }
  d$row <- rep(dimnames(tab)[[1]], m)
  d$col <- rep(dimnames(tab)[[2]], each = n)
  chisq <- suppressWarnings(stats::chisq.test(tab))
  df <- chisq$parameter
  chisqval <- chisq$statistic
  d$xcent <- (d$xmin + d$xmax) / 2
  d$ycent <- (d$ymin + d$ymax) / 2
  d$stdres <- as.vector(t(chisq$stdres))
  ggplot2::ggplot(
    d,
    ggplot2::aes(
      xmin = !!rlang::sym("xmin"),
      xmax = !!rlang::sym("xmax"),
      ymin = !!rlang::sym("ymin"),
      ymax = !!rlang::sym("ymax"),
      fill = !!rlang::sym("stdres"),
    )
  ) +
  ggplot2::geom_rect(color = "black", show.legend = TRUE) +
  ggplot2::scale_fill_gradient2(
    name = "Standardized\nresidual",
    oob = bound,
    low = "#D33F6A",
    high = "#4A6FE3",
    limits = c(-4, 4),
    breaks = c(-4, -2, 0, 2, 4)
  ) +
  ggplot2::scale_x_continuous(
    breaks = unique(d$xcent),
    labels = dimnames(tab)[[1]],
    position = "top",
    expand = c(0.01, 0)
  ) +
  ggplot2::scale_y_continuous(
    breaks = d$ycent[d$xmin == 0],
    labels = dimnames(tab)[[2]],
    expand = c(0.01, 0)
  ) +
  ggplot2::theme_classic() +
  ggplot2::theme(
    plot.title = ggplot2::element_text(hjust = 0.5),
    plot.subtitle = ggplot2::element_text(hjust = 0.5),
    axis.ticks = ggplot2::element_blank(),
    axis.line = ggplot2::element_blank(),
    axis.text.x =  ggplot2::element_text(
      angle =  ifelse(n > 3, 90, 0),
      hjust =  ifelse(n > 3, 0, 0.5),
      vjust =  ifelse(n > 3, 0.5, 0)
    ),
    axis.text.y = ggplot2::element_text(hjust = 1, vjust = 0.40)
  ) +
  ggplot2::labs(x = xlab, y = ylab)
}

#' Create a Sequence Index Plot or a Distribution Plot
#'
#' @export
#' @rdname plot_sequences
#' @param x A `tna`, `group_tna`, `tna_data` or a `data.frame` object with
#'   sequence data in wide format.
#' @param cols An `expression` giving a tidy selection of column names to be
#'   treated as time points. By default, all columns will be used.
#' @param group A vector indicating the group assignment of each
#'   row of the data. Must have the same length as the
#'   number of rows of `x`. Alternatively, a single `character` string giving
#'   the column name of the data that defines the group when `x` is a wide
#'   format `data.frame` or a `tna_data` object. Used for faceting the plot.
#' @param type A `character` string for the type of plot to generate. The
#'   available options are  `"index"` (the default) for a sequence index plot,
#'   and `"distribution"` showing the distribution of the states over time.
#' @param scale A `character` string that determines the scaling of the
#'   vertical axis for distribution plots. The options are `"proportion"`
#'   (the default) and `"count"` for proportions and raw counts of states,
#'   respectively.
#' @param geom A `character` string for the type of geom to use for
#'   distribution plots. The options are `"bar"` (the default) and `"area"`.
#' @param include_na A `logical` value for whether to include missing values
#'   for distribution plots. The default is `FALSE`. If `TRUE`, the missing
#'   values are converted to a new state and included in the plot.
#' @param colors A named `character` vector mapping states to colors, or an
#'   unnamed `character` vector. If missing, a default palette is used.
#' @param na_color A `character` string giving the color to use for missing
#'   values. The default is `"white"`.
#' @param sort_by An optional `expression` giving a tidy selection of column
#' names of `x` to sort by or `"everything"`.
#' @param show_n A `logical` value for whether to add the number of
#'   observations (total or by group) to the plot title.
#' @param border A `character` string giving the color for borders. For index
#'   plots, this is the color of borders between cells (tiles). For
#'   distribution plot with `geom = "bar"`, this is the color of bar outlines.
#'   Not applicable to `geom = "area"`.
#' @param title An optional `character` string providing a title for the plot.
#' @param legend_title An optional `character` string providing a title for the
#'   legend.
#' @param xlab A `character` string giving the label for the horizontal axis.
#'   The default is `"Time"`.
#' @param ylab A `character` string giving the label for the vertical axis.
#'   The default is `"Sequence"` for index plots, and `"Proportion"` or
#'   `"Count"` based on `scale` for distribution plots.
#' @param tick An `integer` specifying the horizontal axis label interval. The
#' default value `tick = 5` shows every 5th label. Setting this to 1 will show
#' every label.
#' @param ncol Number of columns to use for the facets. The default is 2.
#' @param ... Ignored.
#' @return A `ggplot` object containing either a sequence index plot or
#'   a state distribution plot, faceted by `group` when supplied.
#' @examples
#' # Sequence index plot (default)
#' plot_sequences(
#'   group_regulation,
#'   group = rep(1:2, each = 1000),
#' )
#' # State distribution plot
#' plot_sequences(
#'   group_regulation,
#'   group = rep(1:2, each = 1000),
#'   type = "distribution",
#' )
#'
plot_sequences <- function(x, ...) {
  UseMethod("plot_sequences")
}

#' @export
#' @rdname plot_sequences
plot_sequences.tna <- function(x, group, type = "index",
                               scale = "proportion", geom = "bar",
                               include_na = FALSE, na_color = "white", sort_by,
                               show_n = TRUE, border, title, legend_title,
                               xlab, ylab, tick = 5, ncol = 2L, ...) {
  check_missing(x)
  check_tna_seq(x)
  d <- as.data.frame(x$data)
  cols <- names(d)
  colors <- attr(x$data, "colors")
  lab <- x$labels
  lev <- seq_along(lab)
  if (!missing(group)) {
    stopifnot_(
      length(group) == nrow(d),
      "Argument {.arg group} must the same length as the number of rows in
       the sequence data of {.arg x}."
    )
    d$.group <- group
    group <- ".group"
  }
  plot_sequences_(
    d, lev, lab, cols, group, type, scale, geom, include_na, colors,
    na_color, rlang::enquo(sort_by), show_n, border, title, legend_title,
    xlab, ylab, tick, ncol
  )
}

#' @export
#' @rdname plot_sequences
plot_sequences.tna_data <- function(x, group, type = "index",
                                    scale = "proportion",
                                    geom = "bar", include_na = FALSE,
                                    colors, na_color = "white", sort_by,
                                    show_n = TRUE, border, title,
                                    legend_title, xlab, ylab, tick = 5,
                                    ncol = 2L, ...) {
  check_missing(x)
  check_class(x, "tna_data")
  wide <- cbind(x$sequence_data, x$meta_data)
  plot_sequences.default(
    x = wide,
    cols = dplyr::all_of(names(x$sequence_data)),
    group = group,
    type = type,
    scale = scale,
    geom = geom,
    include_na = include_na,
    colors = colors,
    na_color = na_color,
    sort_by = rlang::enquo(sort_by),
    show_n = show_n,
    border = border,
    title = title,
    legend_title = legend_title,
    xlab = xlab,
    ylab = ylab,
    tick = tick,
    ncol = ncol,
    ...
  )
}

#' @export
#' @rdname plot_sequences
plot_sequences.default <- function(x, cols = tidyselect::everything(),
                                   group, type = "index",
                                   scale = "proportion", geom = "bar",
                                   include_na = FALSE, colors,
                                   na_color = "white", sort_by,
                                   show_n = TRUE, border, title,
                                   legend_title, xlab, ylab, tick = 5,
                                   ncol = 2L, ...) {
  check_missing(x)
  stopifnot_(
    inherits(x, "stslist") || inherits(x, "data.frame"),
    "Argument {.arg x} must be {.cls stslist} (sequence data) or a
    {.cls data.frame} or object."
  )
  # TODO can remove this if TraMineR is fixed
  x <- as.data.frame(x)
  cols <- get_cols(rlang::enquo(cols), x)
  if (!missing(group)) {
    n_group <- length(group)
    stopifnot_(
      n_group == nrow(x) || n_group == 1L,
      "Argument {.arg group} must be of length one or the same length as the
       number of rows in {.arg x}."
    )
    if (n_group == 1L) {
      stopifnot_(
        group %in% names(x),
        "Argument {.arg group} must be a column name of {.arg x}
         when of length one."
      )
    } else {
      x$.group <- group
      group <- ".group"
    }
  }
  lab <- ifelse_(
    inherits(x, "stslist"),
    attr(x, "alphabet"),
    sort(unique(unlist(x[, cols])))
  )
  lev <- seq_along(lab)
  x[cols] <- lapply(x[cols], factor, labels = lab, levels = lab)
  x[cols] <- lapply(x[cols], as.integer)
  sort_by <- sort_by %m% rlang::enquo(sort_by)
  sort_by <- rlang::as_quosure(sort_by, rlang::caller_env())
  plot_sequences_(
    x, lev, lab, cols, group, type, scale, geom, include_na, colors,
    na_color, sort_by, show_n, border, title, legend_title,
    xlab, ylab, tick, ncol
  )
}

plot_sequences_ <- function(x, lev, lab, cols, group, type, scale,
                            geom, include_na, colors, na_color, sort_by,
                            show_n, border, title, legend_title,
                            xlab, ylab, tick, ncol) {
  type <- check_match(type, c("distribution", "index"))
  scale <- check_match(scale, c("count", "proportion"))
  geom <- check_match(geom, c("area", "bar"))
  check_flag(include_na)
  check_flag(show_n)
  check_values(tick)
  has_group <- TRUE
  if (missing(group)) {
    x$.group <- 1L
    group <- ".group"
    has_group <- FALSE
  }
  x <- x |>
    dplyr::select(c(dplyr::all_of(cols), !!rlang::sym(group))) |>
    dplyr::group_by(!!rlang::sym(group))
  sort_by <- get_cols(sort_by, x)
  if (!missing(sort_by)) {
    x <- x |>
      dplyr::arrange(dplyr::across(dplyr::all_of(sort_by)), .by_group = TRUE)
  }
  x$.seq_id <- rev(seq_len(nrow(x)))
  # Remove temporary grouping used in sorting
  group <- ifelse_(has_group, group, rlang::missing_arg())
  x <- x |> dplyr::ungroup()
  long_data <- x |>
    tidyr::pivot_longer(
      cols = dplyr::all_of(cols),
      names_to = "time",
      values_to = "state"
    ) |>
    dplyr::mutate(
      .seq_id = factor(!!rlang::sym(".seq_id")),
      time = factor(!!rlang::sym("time"), levels = cols),
      state = factor(!!rlang::sym("state"), levels = lev, labels = lab)
    )
  colors <- colors %m% color_palette(n_unique(long_data$state))
  if (show_n) {
    if (!missing(group)) {
      group_n <- x |>
        dplyr::group_by(!!rlang::sym(group)) |>
        dplyr::summarize(n = dplyr::n())
      title_n <- paste0(
        "{n[", seq_along(group_n$n), "] == ", group_n$n, "}",
        collapse = " * \", \" * "
      )
      title_n <- paste0("\"(\" * ", title_n, " * \")\"")
    } else {
      title_n <- paste0("\"(\" * n == ", nrow(x), "* \")\"")
    }
  }
  if (type == "index") {
    create_index_plot(
      long_data, group, colors, na_color, border,
      title, include_na, title_n, legend_title, xlab, ylab, tick, ncol
    )
  } else {
    create_distribution_plot(
      long_data, group, scale, geom, include_na, colors, na_color,
      border, title, title_n, legend_title, xlab, ylab, tick, ncol
    )
  }
}

create_index_plot <- function(x, group, colors, na_color, border,
                              title, include_na, title_n, legend_title,
                              xlab, ylab, tick, ncol) {
  title <- title %m% "Sequence Index Plot"
  xlab <- xlab %m% "Time"
  ylab <- ylab %m% "Sequence"
  legend_title <- legend_title %m% NULL
  title <- str2expression(paste0("\"", title, " \" * ", title_n))
  every_nth <- function(y) y[(seq_along(y) - 1L) %% tick == 0]
  if (!include_na) {
    x <- x |> tidyr::drop_na()
  }
  p <- ggplot2::ggplot(
    x,
    ggplot2::aes(
      x = !!rlang::sym("time"),
      y = !!rlang::sym(".seq_id")
    )
  ) +
    ggplot2::geom_raster(ggplot2::aes(fill = !!rlang::sym("state"))) +
    ggplot2::scale_fill_manual(
      values = colors,
      name = legend_title,
      na.value = na_color
    ) +
    ggplot2::scale_x_discrete(breaks = every_nth) +
    ggplot2::labs(title = title, x = xlab, y = ylab) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      panel.grid = ggplot2::element_blank(),
      axis.text.y = ggplot2::element_blank(),
      axis.ticks.y = ggplot2::element_blank(),
      legend.position = "right"
    )
  if (!missing(border)) {
    p <- p + ggplot2::geom_tile(
      ggplot2::aes(fill = !!rlang::sym("state")),
      color = border,
      linewidth = 0.1,
      na.rm = FALSE
    )
  }
  if (!missing(group)) {
    p <- p + ggplot2::facet_wrap(
      ggplot2::vars(!!rlang::sym(group)),
      ncol = ncol,
      scales = "free_y",
    )
  }
  p
}

create_distribution_plot <- function(x, group, scale, geom, include_na,
                                     colors, na_color, border,
                                     title, title_n, legend_title,
                                     xlab, ylab, tick, ncol) {
  title <- title %m% "Sequence Distribution Plot"
  xlab <- xlab %m% "Time"
  ylab <- ifelse_(
    missing(ylab),
    ifelse_(scale == "proportion", "Proportion", "Count"),
    ylab
  )
  legend_title <- legend_title %m% NULL
  title <- str2expression(paste0("\"", title, " \" * ", title_n))
  every_nth <- function(y) y[(seq_along(y) - 1L) %% tick == 0]
  position <- ifelse_(scale == "proportion", "fill", "stack")
  if (!include_na) {
    x <- x |> tidyr::drop_na()
  }
  if (geom == "bar") {
    p <- ggplot2::ggplot(
      x,
      ggplot2::aes(
        x = !!rlang::sym("time"),
        fill = !!rlang::sym("state")
      )
    ) +
      ggplot2::geom_bar(na.rm = FALSE, width = 1, position = position) +
      ggplot2::scale_x_discrete(breaks = every_nth)
  } else if (geom == "area") {
    time_levels <- levels(x$time)
    x$time <- as.numeric(x$time)
    p <- ggplot2::ggplot(
      x,
      ggplot2::aes(
        x = !!rlang::sym("time"),
        fill = !!rlang::sym("state")
      )
    ) +
      ggplot2::geom_area(position = position, stat = "count") +
      ggplot2::scale_x_continuous(
        breaks = every_nth(seq_along(time_levels)),
        labels = every_nth(time_levels)
      )
  }
  p <- p +
    ggplot2::scale_fill_manual(
      values = colors,
      name = legend_title,
      na.value = na_color
    ) +
    ggplot2::labs(title = title, x = xlab, y = ylab) +
    ggplot2::theme_minimal() +
    ggplot2::scale_y_reverse() +
    ggplot2::theme(
      panel.grid = ggplot2::element_blank(),
      axis.text.y = ggplot2::element_blank(),
      axis.ticks.y = ggplot2::element_blank(),
      legend.position = "right"
    )
  if (!missing(group)) {
    p <- p + ggplot2::facet_wrap(
      ggplot2::vars(!!rlang::sym(group)),
      ncol = ncol,
      scales = "free_y"
    )
  }
  p
}

#' Create a heatmap from edgelist data
#'
#' @param data A `data.frame` with source and target columns and edge weights.
#' @param title A `character` string giving the plot title.
#' @noRd
create_heatmap <- function(data, title) {
  ggplot2::ggplot(
    data,
    ggplot2::aes(
      x = !!rlang::sym("target"),
      y = !!rlang::sym("source"),
      fill = !!rlang::sym("value")
    )) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_gradient2(
      low = "blue",
      high = "red",
      mid = "white",
      limits = c(-1, 1),
      na.value = "grey50"
    ) +
    ggplot2::labs(title = title, x = "Target", y = "Source") +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1, size = 8),
      axis.text.y = ggplot2::element_text(size = 8),
      plot.title = ggplot2::element_text(size = 12, face = "bold")
    )
}

# Groups ----------------------------------------------------------------

#' Plot a Histogram of Edge Weights for a `group_tna` Object.
#'
#' @export
#' @family basic
#' @param x A `group_tna` object.
#' @param ... Additional arguments passed to [graphics::hist()].
#' @return A `list` (invisibly) of `histogram` objects of the edge weights of
#'   each cluster.
#' @examples
#' model <- group_model(engagement_mmm)
#' hist(model)
#'
hist.group_tna <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  invisible(lapply(x, hist.tna, ...))
}

#' Plot a Grouped Transition Network Analysis Model
#'
#' Plots a transition network of each cluster using `cograph`.
#'
#' @export
#' @family basic
#' @param x A `group_model` object.
#' @param title A title for each plot. It can be a single string (the same one
#'   will be used for all plots) or a list (one per group)
#' @param which An optional `integer` vector of groups to plot. By default, all
#'   groups are plotted.
#' @param ... Same as [plot.tna()].
#' @return `NULL` (invisibly).
#' @inheritDotParams plot.tna
#' @examples
#' model <- group_model(engagement_mmm)
#' plot(model, which = 1)
#'
plot.group_tna <- function(x, title, which, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  title <- title %m% names(x)
  title <- rep_len(title, length(x))
  which <- which %m% seq_along(x)
  stopifnot_(
    all(which %in% seq_along(x)),
    "Argument {.arg which} must only contain valid cluster indices."
  )
  for (i in which) {
    plot(x[[i]], title = title[i], ...)
  }
  invisible(NULL)
}

#' Plot a Bootstrapped Grouped Transition Network Analysis Model
#'
#' @export
#' @family validation
#' @param x A `group_tna_bootstrap` object.
#' @param title A `character` vector of titles to use for each plot.
#' @param ... Additional arguments passed to [plot.tna()].
#' @examples
#' model <- group_model(engagement_mmm)
#' # Small number of iterations for CRAN
#' boot <- bootstrap(model, iter = 10)
#' plot(boot)
#'
plot.group_tna_bootstrap <- function(x, title, ...) {
  check_missing(x)
  check_class(x, "group_tna_bootstrap")
  title <- title %m% names(x)
  title <- rep_len(title, length(x))
  invisible(
    Map(function(y, i) plot.tna_bootstrap(y, title = i, ...), x, title)
  )
}

#' Plot Centrality Measures
#'
#' @export
#' @family centralities
#' @param x A `group_tna_centralities` object.
#' @param palette A color palette to be applied if `colors` is not specified.
#' @inheritParams plot.tna_centralities
#' @return A `ggplot` object displaying a line chart for each centrality
#'   with one line per cluster.
#' @examples
#' model <- group_model(engagement_mmm)
#' cm <- centralities(model)
#' plot(cm)
#'
plot.group_tna_centralities <- function(x, reorder = TRUE, ncol = 3,
                                        scales = c("free_x", "fixed"),
                                        colors, palette = "Set2",
                                        labels = TRUE, ...) {
  check_missing(x)
  check_class(x, "group_tna_centralities")
  plot_centralities_(x, reorder, ncol, scales, colors, palette, labels)
}

#' Plot Found Cliques
#'
#' @export
#' @family cliques
#' @param x A `group_tna_cliques` object.
#' @param title A `character` vector of titles to use for each plot.
#' @param ... Arguments passed to [plot.tna_cliques()].
#' @return A `list` (invisibly) with one element per cluster. Each element
#'   contains a `cograph_network` plot when only one clique is present per
#'   cluster, otherwise the element is `NULL`.
#' @examples
#' model <- group_model(engagement_mmm)
#' cliq <- cliques(model, size = 2)
#' plot(cliq, ask = FALSE)
#'
plot.group_tna_cliques <- function(x, title, ...) {
  check_missing(x)
  check_class(x, "group_tna_cliques")
  title <- title %m% names(x)
  title <- rep_len(title, length(x))
  invisible(
    Map(function(y, i) plot.tna_cliques(y, title = i, ...), x, title)
  )
}

#' Plot Detected Communities
#'
#' @export
#' @family communities
#' @param x A `group_tna_communities` object.
#' @param title A `character` vector of titles to use for each plot.
#' @param colors A `character` vector of colors to use.
#' @param ... Arguments passed to [plot.tna_communities()].
#' @return A `list` (invisibly) of `cograph_network` objects in which the
#'   nodes are colored by community for each cluster.
#' @examples
#' model <- group_model(engagement_mmm)
#' comm <- communities(model)
#' plot(comm)
#'
plot.group_tna_communities <- function(x, title, colors, ...) {
  check_missing(x)
  check_class(x, "group_tna_communities")
  n <- length(x)
  colors <- colors %m% default_colors
  colors <- rep_len(colors, n)
  title <- title %m% names(x)
  title <- rep_len(title, n)
  invisible(
    Map(
      function(y, i, j) {
        plot(y, title = i, colors = j, ...)
      },
      x,
      title,
      colors
    )
  )
}

#' Plot Centrality Stability Results
#'
#' @export
#' @family validation
#' @param x A `group_tna_stability` object.
#' @param ... Arguments passed to [plot.tna_stability()].
#' @return A `list` (invisibly) of `ggplot` objects displaying the stability
#'   analysis plot.
#' @examples
#' model <- group_model(engagement_mmm)
#' # Low number of iterations for CRAN
#' stability <- estimate_cs(
#'   model,
#'   drop_prop = c(0.3, 0.5, 0.7, 0.9),
#'   iter = 10
#' )
#' plot(stability)
#'
plot.group_tna_stability <- function(x, ...) {
  check_missing(x)
  check_class(x, "group_tna_stability")
  invisible(lapply(x, plot.tna_stability, ...))
}

#' Plot Permutation Test Results
#'
#' @export
#' @family validation
#' @param x A `group_tna_permutation` object.
#' @param title An optional `character` vector of titles for each plot.
#'   When not provided, the title shows the names of the clusters being
#'   contrasted.
#' @param ... Arguments passed to [plot.tna_permutation()].
#' @return A `list` (invisibly) of `cograph_network` objects depicting the
#'   significant difference between each pair.
#' @examples
#' model <- group_tna(engagement_mmm)
#' # Small number of iterations for CRAN
#' perm <- permutation_test(model, iter = 20)
#' plot(perm)
#'
plot.group_tna_permutation <- function(x, title, ...) {
  check_missing(x)
  check_class(x, "group_tna_permutation")
  title <- title %m% names(x)
  invisible(
    lapply(
      seq_along(x),
      function(i) plot.tna_permutation(x[[i]], title = title[i], ...)
    )
  )
}

#' Plot the Difference Network Between Two Groups
#'
#' @export
#' @family comparison
#' @param x A `group_tna` object.
#' @param i An `integer` index or the name of the principal cluster as a
#'   `character` string. When `NULL`, defaults are chosen automatically (see
#'   [cograph::plot_compare()]).
#' @param j An `integer` index or the name of the secondary cluster as a
#'   `character` string. When `NULL`, defaults are chosen automatically (see
#'   [cograph::plot_compare()]).
#' @param ... Additional arguments passed to [cograph::plot_compare()].
#' @return A `cograph_network` object displaying the difference network between
#'   the two clusters
#' @examples
#' model <- group_model(engagement_mmm)
#' plot_compare(model)
#'
plot_compare.group_tna <- function(x, i = NULL, j = NULL, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  cograph::plot_compare(x = x, i = i, j = j, ...)
}

#' Plot the Frequency Distribution of States
#'
#' @export
#' @family basic
#' @param x A `group_tna` object.
#' @param label An optional `character` string that can be provided to specify
#'   the grouping factor name if `x` was not constructed using a column name of
#'   the original data.
#' @param colors A `character` vector of colors to be used in the plot
#'   (one per group).
#' @param palette A `character` string that specifies the palette to be used if
#'   `colors` are not passed.
#' @param width A `numeric` value for the width of the bars.
#'   The default is `0.7`.
#' @param hjust A `numeric` value for the horizontal adjustment of the labels.
#'   The default is `1.2`.
#' @param position Position of the bars:`"dodge"`, `"dodge2"`, `"fill"` or
#'   `"stack"`.
#' @param show_label A `logical` value indicating whether to show a label with
#'   the frequency counts. Default is `TRUE`.
#' @param ... Ignored.
#' @return A `ggplot` object.
#' @examples
#' \donttest{
#' model <- group_model(engagement_mmm)
#' # Default
#' plot_frequencies(model)
#' # Default labels outside and custom colors
#' plot_frequencies(
#'   model,
#'   width = 0.9,
#'   hjust = -0.3,
#'   colors = c("#218516", "#f9c22e", "#53b3cb")
#' )
#' # Stacked with no labels
#' plot_frequencies(model, position = "stack", show_label = FALSE)
#' # Fill
#' plot_frequencies(model, position = "fill", hjust = 1.1)
#' }
#'
plot_frequencies.group_tna <- function(x, label, colors, width = 0.7,
                                       palette = "Set2",
                                       show_label = TRUE, position = "dodge",
                                       hjust = 1.2, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  label <- label %m% attr(x, "label")
  combined <- combine_data(x)
  long <- tidyr::pivot_longer(combined, cols = !(!!rlang::sym(".group")))
  check_values(width, type = "numeric")
  check_numeric(hjust)
  check_flag(show_label)
  long$value <- factor(x[[1L]]$labels[long$value], levels = rev(x[[1L]]$labels))
  long$.group <- factor(long$.group)
  position <- check_match(position, c("dodge", "dodge2", "fill", "stack"))
  position <- switch(position,
    dodge = ggplot2::position_dodge(width = width),
    dodge2 = ggplot2::position_dodge2(width = width),
    stack = "stack",
    fill = "fill"
  )
  d <- long |>
    dplyr::group_by(!!rlang::sym(".group"), !!rlang::sym("value")) |>
    dplyr::summarize(freq = dplyr::n()) |>
    dplyr::ungroup() |>
    dplyr::filter(!is.na(!!rlang::sym("value")))

  p <- ggplot2::ggplot(
    d,
    ggplot2::aes(
      y = !!rlang::sym("value"),
      x = !!rlang::sym("freq"),
      fill = !!rlang::sym(".group"))
  ) +
    ggplot2::geom_bar(
      stat = "identity",
      colour = "black",
      position = position,
      width = width
    )
  if(show_label) {
    p <- p +
      ggplot2::geom_text(
        ggplot2::aes(label = !!rlang::sym("freq")),
        position = position,
        hjust = hjust
      )
  }
  if (!missing(colors)) {
    colors <- rep(colors, length.out = length(x))
    p <- p + ggplot2::scale_fill_manual(name = label, values = colors)
  } else {
    p <- p + ggplot2::scale_fill_brewer(name = label, palette = palette)
  }
  p +
    ggplot2::theme_minimal() +
    ggplot2::scale_x_continuous(
      expand = ggplot2::expansion(mult = c(0, 0.1))
    ) +
    ggplot2::labs(y = "State", x = "Frequency") +
    ggplot2::theme(
      legend.position =  "bottom",
      axis.title =  ggplot2::element_text(face = "bold"),
      axis.text = ggplot2::element_text(color = "black"),
      text = ggplot2::element_text(color = "black"))
}

#' Plot State Frequencies as a Mosaic Between Two Groups
#'
#' @export
#' @family basic
#' @param x A `tna_data` object.
#' @param group A `character` string giving the column name of the (meta) data
#'   to contrast the frequencies with or a vector of group indicators with the
#'   the same length as the number of rows in the sequence data.
#' @param label An optional `character` string that specifies a label for the
#'   grouping variable when `group` is not a column name of the data.
#' @param ... Ignored.
#' @return A `ggplot` object.
#' @examples
#' d <- data.frame(
#'   time = rep(1:5, rep = 4),
#'   group = rep(1:4, each = 5),
#'   event = sample(LETTERS[1:3], 20, replace = TRUE)
#' )
#' sequence_data <- prepare_data(
#'   d,
#'   time = "time",
#'   actor = "group",
#'   action = "event"
#' )
#' plot_mosaic(sequence_data, group = "group")
#'
plot_mosaic.tna_data <- function(x, group, label = "Group", ...) {
  check_missing(x)
  check_class(x, "tna_data")
  check_missing(group)
  check_string(label)
  n_group <- length(group)
  stopifnot_(
    n_group == nrow(x$sequence_data) || n_group == 1L,
    "Argument {.arg group} must be of length one or the same length as the
     number of rows/sequences in {.arg x}."
  )
  if (n_group == 1L) {
    stopifnot_(
      group %in% names(x$meta_data),
      "Argument {.arg group} must be a column name of the input data
       when of length one."
    )
    group <- x$meta_data[[group]]
  }
  group <- ifelse_(is.factor(group), group, factor(group))
  wide <- cbind(x$sequence_data, group)
  names(wide) <- c(names(x$sequence_data), ".group")
  long <- wide |>
    tidyr::pivot_longer(cols = !(!!rlang::sym(".group"))) |>
    tidyr::drop_na()
  tab <- table(long$.group, long$value)
  plot_mosaic_(
    tab,
    xlab = label,
    ylab = "State"
  )
}

#' Plot State Frequencies as a Mosaic Between Two Groups
#'
#' @export
#' @family basic
#' @param x A `group_tna` object.
#' @param label An optional `character` string that can be provided to specify
#'   the grouping factor name if `x` was not constructed using a column name of
#'   the original data.
#' @param ... Ignored.
#' @return A `ggplot` object.
#' @examples
#' model <- group_model(engagement, group = rep(1:3, length.out = 1000))
#' plot_mosaic(model)
#'
plot_mosaic.group_tna <- function(x, label, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  label <- label %m% attr(x, "label")
  combined <- combine_data(x)
  long <- tidyr::pivot_longer(combined, cols = !(!!rlang::sym(".group")))
  labels <- x[[1L]]$labels
  use_na <- ifelse_(attr(x, "na.rm"), "no", "ifany")
  tab <- table(long$.group, long$value, useNA = use_na)
  dimnames(tab) <- list(attr(x, "levels"), labels)
  plot_mosaic_(
    tab,
    xlab = label,
    ylab = "State"
  )
}

#' @export
#' @rdname plot_sequences
plot_sequences.group_tna <- function(x, type = "index", scale = "proportion",
                                     geom = "bar", include_na = FALSE,
                                     na_color = "white", sort_by, show_n = TRUE,
                                     border, title, legend_title, xlab, ylab,
                                     tick = 1, ncol = 2L, ...) {
  check_missing(x)
  check_class(x, "group_tna")
  d <- combine_data(x)
  cols <- setdiff(names(d), ".group")
  group <- ".group"
  colors <- attr(x[[1L]]$data, "colors")
  lab <- x[[1L]]$labels
  lev <- seq_along(lab)
  plot_sequences_(
    d, lev, lab, cols, group, type, scale, geom, include_na, colors,
    na_color, rlang::enquo(sort_by), show_n, border, title, legend_title,
    xlab, ylab, tick, ncol
  )
}

# Associations ------------------------------------------------------------

#' Plot an Association Network
#'
#' @export
#' @rdname plot_associations
#' @param x A `tna` object.
#' @param edge_color An optional `character` vector of colors for the edges.
#'   By default, the colors are specified by the magnitude of the
#'   standardized residual.
#' @param ... Additional arguments passed to [plot_model()].
#' @return A `cograph_network` object.
#' @examples
#' model <- ftna(group_regulation)
#' plot_associations(model)
#'
plot_associations <- function(x, ...) {
  UseMethod("plot_associations")
}

#' @export
#' @rdname plot_associations
plot_associations.tna <- function(x, edge_color, ...) {
  check_missing(x)
  check_class(x, "tna")
  stopifnot_(
    attr(x, "type") %in% c("frequency", "co-occurrence"),
    "Association network plots are supported only for
     integer-valued weight matrices."
  )
  tab <- as.table(t(x$weights))
  chisq <- suppressWarnings(stats::chisq.test(tab))
  res <- as.numeric(chisq$stdres)
  dim(res) <- dim(tab)
  if (missing(edge_color)) {
    edge_color <- matrix(NA_character_, ncol = ncol(res))
    edge_color[res > 4] <- "#4A6FE3"
    edge_color[res > 2 & res <= 4] <- "#9DA8E2"
    edge_color[res <= 2 & res >= -2] <- "#E2E2E2"
    edge_color[res < -2 & res >= -4] <- "#E495A5"
    edge_color[res < -4] <- "#D33F6A"
  }
  plot_model(res, edge_color = edge_color, labels = x$labels, ...)
}

test_that("tna models can be plotted", {
  pdf(NULL)
  expect_error(
    plot.tna(mock_tna),
    NA
  )
  dev.off()
})

test_that("histogram of edge weights can be plotted", {
  pdf(NULL)
  expect_error(
    hist.tna(mock_tna),
    NA
  )
  dev.off()
})

test_that("bootstrapped model can be plotted", {
  set.seed(0)
  boot <- bootstrap(mock_tna_seq, iter = 50)
  expect_error(
    plot.tna_bootstrap(boot),
    NA
  )
})

test_that("centralities can be plotted", {
  cm <- centralities(mock_tna)
  pdf(NULL)
  expect_error(
    plot.tna_centralities(cm),
    NA
  )
  expect_error(
    plot.tna_centralities(cm, colors = color_palette(4)),
    NA
  )
  expect_error(
    plot.tna_centralities(cm, colors = "red"),
    NA
  )
  dev.off()
})

test_that("cliques can be plotted", {
  cliq <- cliques(mock_tna, size = 2)
  pdf(NULL)
  expect_error(
    plot.tna_cliques(cliq),
    NA
  )
  dev.off()
})

test_that("communities can be plotted", {
  comm <- communities(mock_tna)
  pdf(NULL)
  expect_error(
    plot.tna_communities(comm),
    NA
  )
  dev.off()
})

test_that("centrality stability can be plotted", {
  model <- tna(mock_sequence)
  out <- estimate_cs(model, drop_prop = seq(0.3, 0.9, by = 0.2), iter = 10)
  pdf(NULL)
  expect_error(
    plot.tna_stability(out),
    NA
  )
  dev.off()
})

test_that("permutation test significant edges can be plotted", {
  model_x <- tna(mock_sequence[1:3, ])
  model_y <- tna(mock_sequence[3:5, ])
  perm <- permutation_test(model_x, model_y, iter = 20)
  pdf(NULL)
  expect_error(
    plot.tna_permutation(perm),
    NA
  )
  dev.off()
})

test_that("permutation test significant edges can be plotted with groups", {
  perm <- permutation_test(mmm_model, iter = 20)
  pdf(NULL)
  expect_error(
    plot.group_tna_permutation(perm),
    NA
  )
  dev.off()
})

test_that("model comparison can be plotted", {
  model_x <- tna(mock_sequence[1:3, ])
  model_y <- tna(mock_sequence[3:5, ])
  pdf(NULL)
  expect_error(
    plot_compare(model_x, model_y),
    NA
  )
  expect_error(
    plot_compare(mmm_model),
    NA
  )
  dev.off()
})

test_that("edge weight matrix can be plotted", {
  pdf(NULL)
  expect_error(
    plot_model(mock_matrix),
    NA
  )
  dev.off()
})

test_that("frequencies can be plotted", {
  expect_error(
    plot_frequencies(mock_tna_seq),
    NA
  )
})

test_that("warning is issued by plot if no cliques are found", {
  cliq <- cliques(mock_tna, size = 2, threshold = 0.5)
  expect_warning(
    plot.tna_cliques(cliq),
    "No 2-cliques were found in the network\\."
  )
})

test_that("group model can be plotted", {
  expect_error(
    plot(mmm_model),
    NA
  )
  expect_error(
    plot(mmm_model, title = "Groups"),
    NA
  )
})

test_that("bootstrapped model can be plotted for clusters", {
  set.seed(0)
  boot <- bootstrap(mmm_model, iter = 20)
  expect_error(
    plot(boot),
    NA
  )
})

test_that("centralities can be plotted for clusters", {
  cm <- centralities(mmm_model)
  expect_error(
    plot(cm),
    NA
  )
})

test_that("centrality stability results can be plotted for clusters", {
  stability <- estimate_cs(
    mmm_model,
    drop_prop = seq(0.3, 0.9, by = 0.1),
    iter = 10
  )
  expect_error(
    plot(stability),
    NA
  )
})

test_that("cliques can be plotted clusters", {
  cliq <- cliques(mmm_model, size = 2)
  expect_error(
    plot(cliq),
    NA
  )
  expect_error(
    plot(cliq, title = "Groups"),
    NA
  )
})

test_that("communities can plotted for clusters", {
  comm <- communities(mmm_model)
  expect_error(
    plot(comm),
    NA
  )
  expect_error(
    plot(comm, title = "Community detection"),
    NA
  )
})

test_that("histogram of edge weights can be plotted", {
  pdf(NULL)
  expect_error(
    hist(mmm_model),
    NA
  )
  dev.off()
})

test_that("comparison results can be plotted", {
  # Needs sufficiently large data for correlations
  model_x <- tna(engagement[1:20, ])
  model_y <- tna(engagement[21:40, ])
  # Comparing models
  comp <- compare(model_x, model_y)
  expect_error(
    plot(comp, type = "heatmap"),
    NA
  )
  expect_error(
    plot(comp, type = "scatterplot"),
    NA
  )
  expect_error(
    plot(comp, type = "centrality_heatmap"),
    NA
  )
  expect_error(
    plot(comp, type = "weight_density"),
    NA
  )
})

test_that("pruned models can be plotted", {
  model_pruned <- prune(mock_tna)
  expect_error(
    plot(model_pruned),
    NA
  )
})

test_that("mosaic can be plotted", {
  ftna_model <- ftna(mock_sequence)
  group_ftna_model <- group_ftna(mock_sequence, group = c(1, 1, 2, 2, 2))
  expect_error(
    plot_mosaic(ftna_model),
    NA
  )
  expect_error(
    plot_mosaic(group_ftna_model),
    NA
  )
  expect_error(
    plot_mosaic(mock_tna_data, group = "group"),
    NA
  )
})

test_that("permutation test results can be plotted for clusters", {
  perm <- permutation_test(mmm_model, iter = 20)
  expect_error(
    plot(perm),
    NA
  )
})

test_that("frequencies can be plotted for groups", {
  expect_error(
    plot_frequencies(mmm_model),
    NA
  )
  expect_error(
    plot_frequencies(mmm_model, colors = c("red", "green", "blue")),
    NA
  )
})

test_that("sequence index plots can be created", {
  model1 <- tna(engagement)
  model2 <- group_tna(engagement_mmm)
  expect_error(
    plot_sequences(mock_tna_seq, group = c(1, 1, 2, 2, 2)),
    NA
  )
  expect_error(
    plot_sequences(mock_group_tna),
    NA
  )
  expect_error(
    plot_sequences(mock_sequence, group = c(1, 1, 2, 2, 2)),
    NA
  )
  expect_error(
    plot_sequences(mock_tna_data),
    NA
  )
  expect_error(
    plot_sequences(mock_tna_data, group = "group"),
    NA
  )
})

test_that("sequence distribution plots can be created", {
  expect_error(
    plot_sequences(mock_tna_seq, type = "distr", group = c(1, 1, 2, 2, 2)),
    NA
  )
  expect_error(
    plot_sequences(mock_group_tna, type = "distr"),
    NA
  )
  expect_error(
    plot_sequences(
      mock_sequence,
      type = "distr",
      group = c(1, 1, 2, 2, 2)
    ),
    NA
  )
  expect_error(
    plot_sequences(mock_tna_data, type = "distr", geom = "area"),
    NA
  )
})

test_that("sequences can be sorted in index plot", {
  expect_error(
    plot_sequences(mock_tna_data, sort_by = tidyselect::everything()),
    NA
  )
  expect_error(
    plot_sequences(mock_tna_data, sort_by = "event_T1"),
    NA
  )
  expect_error(
    plot_sequences(mock_tna_data, sort_by = c(4, 1, 3, 2)),
    NA
  )
})

test_that("border can be added for sequence index plot", {
  expect_error(
    plot_sequences(mock_tna_data, border = "black"),
    NA
  )
})

test_that("association plots can be created", {
  model <- ftna(mock_sequence)
  expect_error(
    plot_associations(model),
    NA
  )
})

test_that("node size can be scaled based on centrality measures", {
  expect_error(
    plot(mock_tna, scale_nodes = "OutStrength"),
    NA
  )
})

test_that("sequence comparison can be plotted", {
  comp <- compare_sequences(
    mock_sequence,
    group = c(1, 1, 2, 2, 2),
    min_freq = 1L
  )
  expect_error(
    plot(comp, cells = TRUE),
    NA
  )
})

test_that("observations can be bounded", {
  x <- rnorm(100, sd = 10)
  y <- bound(x, c(-2, 2))
  expect_equal(max(y), 2)
  expect_equal(min(y), -2)
})

test_that("heterogenous tna can be plotted", {
  groups <- list(
    `A` = "A",
    `B` = c("C", "B")
  )
  expect_error(
    plot(mock_tna_seq, node_list = groups),
    NA
  )
  expect_error(
    plot(mock_tna_seq, node_list = groups, use_list_order = FALSE),
    NA
  )
})

test_that("invalid node lists fail", {
  node_list1 <- list(c("A", "B"), c("A", "B"), c("C"))
  node_list2 <- list(c(1, 2), c(3))
  node_list3 <- list(c("A", "B"), c("A", "B"))
  node_list4 <- list(c("A"), c("B"))
  expect_error(
    plot(mock_tna_seq, node_list = node_list1),
    "Argument `node_list` must be a <list> of length 2"
  )
  expect_error(
    plot(mock_tna_seq, node_list = node_list2),
    "Elements of `node_list` must be <character> vectors"
  )
  expect_error(
    plot(mock_tna_seq, node_list = node_list3),
    "The groups defined by `node_list` must not contain common states"
  )
  expect_error(
    plot(mock_tna_seq, node_list = node_list4),
    "Every state must belong to one of the groups defined by `node_list`"
  )
})

test_that("reliability analysis can be plotted", {
  rel <- reliability(mock_tna_seq, iter = 50)
  expect_error(
    plot(rel, type = "histogram"),
    NA
  )
  expect_error(
    plot(rel, type = "density"),
    NA
  )
  expect_error(
    plot(rel, type = "boxplot"),
    NA
  )
  rel2 <- reliability(
    mock_tna_seq,
    types = c("relative", "frequency"),
    iter = 50
  )
  expect_error(
    plot(rel2, type = "histogram"),
    NA
  )
  expect_error(
    plot(rel2, type = "density"),
    NA
  )
  expect_error(
    plot(rel2, type = "boxplot"),
    NA
  )
})

#' Clustering via Dissimilarity Matrix based on String Distances
#'
#' Performs clustering using specified dissimilarity measures
#' and clustering methods. The rows of the data are first converted to strings
#' and compared using the dissimilarity measures available in the
#' `stringdist` package.
#'
#' @param data A `data.frame`, a `matrix` in wide format, or a `tna_data`
#'   object (in which case, the sequence data is extracted automatically).
#' @param k An `integer` giving the number of clusters.
#' @param dissimilarity A `character` string specifying the
#'   dissimilarity measure. The available options are: `"osa"`, `"lv"`, `"dl"`,
#'   `"hamming"`, `"lcs"`, `"qgram"`, `"cosine"`, `"jaccard"`, and `"jw"`.
#'   See [stringdist::stringdist-metrics] for more information on
#'   these measures.
#' @param method A `character` string specifying clustering method.
#'   The available methods are `"pam"`, `"ward.D"`, `"ward.D2"`,
#'   `"complete"`,`"average"`, `"single"`, `"mcquitty"`, `"median"`, and
#'   `"centroid"`. See [cluster::pam()] and [stats::hclust()] for more
#'   information on these methods.
#' @param na_syms A `character` vector of symbols or factor levels to convert
#'   to explicit missing values.
#' @param weighted A `logical` value indicating whether the dissimilarity
#'   measure should be weighted (the default is `FALSE` for no weighting).
#'   If `TRUE`, earlier observations of the sequences receive a greater weight
#'   in the distance calculation with an exponential decay. Currently only
#'   supported for the Hamming distance.
#' @param lambda A `numeric` value defining the strength of the decay when
#'   `weighted = TRUE`. The default is `1.0`.
#' @param ... Additional arguments passed to [stringdist::stringdist()].
#' @return A `tna_clustering` object which is a  `list` containing:
#'
#'   * `data`: The original data.
#'   * `k`: The number of clusters.
#'   * `assignments`: An `integer` vector of cluster assignments.
#'   * `silhouette`: Silhouette score measuring clustering quality.
#'   * `sizes`: An `integer` vector of cluster sizes.
#'   * `method`: The clustering method used.
#'   * `distance`: The distance matrix.
#'
#' @examples
#' data <- data.frame(
#'   T1 = c("A", "B", "A", "C", "A", "B"),
#'   T2 = c("B", "A", "B", "A", "C", "A"),
#'   T3 = c("C", "C", "A", "B", "B", "C")
#' )
#'
#' # PAM clustering with optimal string alignment (default)
#' result <- cluster_sequences(data, k = 2)
#'
#' @export
cluster_data <- function(data, k, dissimilarity = "hamming",
                         method = "pam", na_syms = c("*", "%"),
                         weighted = FALSE, lambda = 1.0, ...) {
  if (inherits(data, "tna_data")) {
    data <- data$sequence_data
  }
  stopifnot_(
    is.data.frame(data) || is.matrix(data),
    "Argument {.arg data} must be a {.cls data.frame}, a {.cls matrix},
    or a {.cls tna_data} object."
  )
  check_range(k, type = "integer", lower = 2L, upper = nrow(data) - 1L)
  dissimilarity <- check_match(dissimilarity, available_clustering_metrics)
  method <- check_match(method, available_clustering_methods)
  stopifnot_(
    !weighted || dissimilarity == "hamming",
    "Weighting is only supported for the Hamming distance."
  )
  lambda <- ifelse_(weighted, lambda, 0.0)
  strings <- seq2chr(data, na_syms)
  dist_mat <- dissimilarity_matrix(strings, dissimilarity, lambda, ...)
  if (method == "pam") {
    clust_result <- cluster::pam(dist_mat, diss = TRUE, k = k)
    assignments <- clust_result$clustering
    silhouette_score <- clust_result$silinfo$avg.width
  } else {
    hc <- stats::hclust(dist_mat, method = method)
    assignments <- stats::cutree(hc, k = k)
    sil <- cluster::silhouette(assignments, dist = dist_mat)
    silhouette_score <- mean(sil[, 3L])
  }
  sizes <- c(table(assignments))
  structure(
    list(
      data = data,
      k = k,
      assignments = assignments,
      silhouette = silhouette_score,
      sizes = sizes,
      method = method,
      distance = dist_mat
    ),
    class = "tna_clustering"
  )
}

#' @export
#' @rdname cluster_data
cluster_sequences <- cluster_data

#' Convert a matrix of sequences to a character matrix of symbols
#'
#' @param data A `data.frame` or a `matrix`
#' @param na_syms An optional `vector` of values to convert to `NA` values.
#' @noRd
seq2chr <- function(data, na_syms) {
  data <- as.matrix(data)
  chr_mat <- matrix(NA_character_, nrow = nrow(data), ncol = ncol(data))
  data[data %in% na_syms] <- NA
  obs <- !is.na(data)
  last_obs <- max.col(obs, ties.method = "last")
  u_vals <- unique(c(data))
  base <- c(letters, LETTERS, as.character(0:9))
  obs_vals <- data[obs]
  idx <- match(obs_vals, u_vals)
  chr_mat[obs] <- base[idx]
  chr_mat[!obs] <- base[max(idx) + 1L]
  list(mat = chr_mat, len = last_obs)
}

get_qgram <- function(x, n, q) {
  if (n < q) {
    return(integer())
  }
  grams <- character(n - q + 1L)
  for (i in seq_along(grams)) {
    grams[i] <- paste0(x[i:(i + q - 1L)], collapse = "")
  }
  tab <- table(grams)
  counts <- as.integer(tab)
  names(counts) <- names(tab)
  counts
}

# Dissimilarity measures --------------------------------------------------

available_clustering_metrics <- c(
  "osa",
  "lv",
  "dl",
  "hamming",
  "lcs",
  "qgram",
  "cosine",
  "jaccard",
  "jw"
)

#' Dissimilarity matrix
#'
#' @param strings Output of `seq2str`.
#' @param lambda A `numeric` value for the decay rate.
#' @noRd
dissimilarity_matrix <- function(strings, dissimilarity, lambda, ...) {
  mat <- strings$mat
  len <- strings$len
  n <- nrow(mat)
  weights <- 1.0
  if (lambda > 0) {
    k <- ncol(mat)
    weights <- exp(-lambda * seq(0, k - 1))
    weights <- weights / max(weights)
  }
  qgrams <- vector(mode = "list", length = n)
  if (dissimilarity %in% c("qgram", "cosine", "jaccard")) {
    q <- list(...)$q
    q <- ifelse_(is.null(q), 2L, q)
    for (i in seq_len(n)) {
      qgrams[[i]] <- get_qgram(x = mat[i, ], n = len[i], q = q)
    }
  }
  d_fun <- switch(dissimilarity,
    osa = osa_dist,
    lv = levenshtein_dist,
    dl = damerau_levenshtein_dist,
    hamming = hamming_dist,
    lcs = lcs_dist,
    qgram = qgram_dist,
    cosine = cosine_dist,
    jaccard = jaccard_dist,
    jw = jaro_winkler_dist
  )
  d <- matrix(0.0, n, n)
  for (i in seq_len(n - 1L)) {
    for (j in seq(i + 1L, n)) {
      d[i, j] <- d_fun(
        x = mat[i, ],
        y = mat[j, ],
        n = len[i],
        m = len[j],
        qx = qgrams[[i]],
        qy = qgrams[[j]],
        weights = weights,
        ...
      )
      d[j, i] <- d[i, j]
    }
  }
  stats::as.dist(d)
}

osa_dist <- function(x, y, n, m, ...) {
  d <- matrix(0L, n + 1L, m + 1L)
  d[, 1L] <- 0:n
  d[1L, ] <- 0:m
  for (i in seq_len(n)) {
    for (j in seq_len(m)) {
      cost <- 0L + 1L * (x[i] != y[j])
      d[i + 1L, j + 1L] <- min(
        d[i, j + 1L] + 1L,
        d[i + 1L, j] + 1L,
        d[i, j] + cost
      )
      if (i > 1 && j > 1 && x[i] == y[j - 1L] && x[i - 1L] == y[j]) {
        d[i + 1L, j + 1L] <- min(d[i + 1L, j + 1L], d[i - 1L, j - 1L] + 1L)
      }
    }
  }
  d[n+1L, m+1L]
}

levenshtein_dist <- function(x, y, n, m, ...) {
  d <- matrix(0L, n + 1L, m + 1L)
  d[, 1L] <- 0:n
  d[1L, ] <- 0:m
  for (i in seq_len(n)) {
    for (j in seq_len(m)) {
      cost <- 0L + 1L * (x[i] != y[j])
      d[i + 1L, j + 1L] <- min(
        d[i, j + 1L] + 1L,
        d[i + 1L, j] + 1L,
        d[i, j] + cost
      )
    }
  }
  d[n + 1L, m + 1L]
}

damerau_levenshtein_dist <- function(x, y, n, m, ...) {
  if (n == 0) {
    return(m)
  }
  if (m == 0) {
    return(n)
  }
  alphabet <- unique(c(x, y))
  last <- stats::setNames(rep(0L, length(alphabet)), alphabet)
  d <- matrix(0L, n + 2L, m + 2L)
  maxdist <- n + m
  cost <- 0L
  d[1L, ] <- maxdist
  d[, 1L] <- maxdist
  for (i in 0:n) {
    d[i + 2L, 2L] <- i
  }
  for (j in 0:m) {
    d[2L, j + 2L] <- j
  }
  for (i in seq_len(n)) {
    db <- 0L
    for (j in seq_len(m)) {
      k <- last[[y[j]]] %||% 0L
      l <- db
      if (x[i] == y[j]) {
        cost <- 0L
        db <- j
      } else {
        cost <- 1L
      }
      d[i + 2L, j + 2L] <- min(
        d[i + 1L, j + 2L] + 1L,
        d[i + 2L, j + 1L] + 1L,
        d[i + 1L, j + 1L] + cost,
        d[k + 1L, l + 1L] + (i - k - 1L) + (j - l - 1L) + 1L
      )
    }
    last[[x[i]]] <- i
  }
  d[n + 2L, m + 2L]
}

hamming_dist <- function(x, y, n, m, weights, ...) {
  sum((x != y) * weights)
}

lcs_dist <- function(x, y, n, m, ...) {
  lcs_len <- matrix(0L, n + 1L, m + 1L)
  for (i in seq_len(n)) {
    for (j in seq_len(m)) {
      if (x[i] == y[j]) {
        lcs_len[i + 1L, j + 1L] <- lcs_len[i, j] + 1L
      } else {
        lcs_len[i + 1L, j + 1L] <- max(lcs_len[i, j + 1L], lcs_len[i + 1L, j])
      }
    }
  }
  n + m - 2L * lcs_len[n + 1L, m + 1L]
}

qgram_dist <- function(x, y, qx, qy, ...) {
  grams <- union(names(qx), names(qy))
  x_all <- qx[grams]
  y_all <- qy[grams]
  x_all[is.na(x_all)] <- 0L
  y_all[is.na(y_all)] <- 0L
  sum(abs(x_all - y_all))
}

cosine_dist <- function(x, y, qx, qy, ...) {
  grams <- union(names(qx), names(qy))
  x_all <- qx[grams]
  y_all <- qy[grams]
  x_all[is.na(x_all)] <- 0L
  y_all[is.na(y_all)] <- 0L
  num <- sum(x_all * y_all)
  den <- sqrt(sum(x_all^2)) * sqrt(sum(y_all^2))
  if (den == 0) {
    return(1)
  }
  1.0 - num / den
}

jaccard_dist <- function(qx, qy, ...) {
  qx <- names(qx)
  qy <- names(qy)
  if (length(qx) == 0 && length(qy) == 0) {
    return(0)
  }
  1.0 - length(intersect(qx, qy)) / length(union(qx, qy))
}

jaro_dist <- function(x, y, n, m, ...) {
  if (n == 0 && m == 0) return(0)
  if (n == 0 || m == 0) return(1)
  match_dist <- max(floor(max(n, m) / 2) - 1L, 0L)
  x_match <- rep(FALSE, n)
  y_match <- rep(FALSE, m)
  matches <- 0L
  for (i in seq_len(n)) {
    start <- max(1L, i - match_dist)
    end <- min(m, i + match_dist)
    if (start > end) {
      next
    }
    for (j in start:end) {
      if (!y_match[j] && x[i] == y[j]) {
        x_match[i] <- TRUE
        y_match[j] <- TRUE
        matches <- matches + 1L
        break
      }
    }
  }
  if (matches == 0L) {
    return(1)
  }
  trans <- 0L
  k <- 1L
  for (i in seq_len(n)) {
    if (x_match[i]) {
      while (k <= m && !y_match[k]) {
        k <- k + 1L
      }
      if (k <= m && x[i] != y[k]) {
        trans <- trans + 1L
      }
      k <- k + 1L
    }
  }
  trans <- trans / 2L
  jaro_sim <- (matches / n + matches / m + (matches - trans) / matches) / 3.0
  1.0 - jaro_sim
}

jaro_winkler_dist <- function(x, y, n, m, p = 0.1, max_l = 4L, ...) {
  jaro_sim <- 1 - jaro_dist(x, y, n, m)
  if (jaro_sim == 0) {
    return(1.0)
  }
  pref <- min(n, m, max_l)
  l <- 0L
  for (k in seq_len(pref)) {
    if (x[k] == y[k]) {
      l <- l + 1L
    } else {
      break
    }
  }
  p <- min(p, 0.25)
  jw <- jaro_sim + l * p * (1.0 - jaro_sim)
  1.0 - jw
}

# Clustering methods ------------------------------------------------------

available_clustering_methods <- c(
  "pam",
  "ward.D2",
  "ward.D",
  "complete",
  "average",
  "single",
  "mcquitty",
  "median",
  "centroid"
)

# Test data
mock_cluster_data <- data.frame(
  T1 = c("A", "B", "A", "C", "A", "B"),
  T2 = c("B", "A", "B", "A", "C", "A"),
  T3 = c("C", "C", "A", "B", "B", "C")
)

# Tests for cluster_data / cluster_sequences ----------------------------------

test_that("cluster_data works with PAM method", {
  result <- cluster_data(mock_cluster_data, k = 2, method = "pam")
  expect_s3_class(result, "tna_clustering")
  expect_equal(result$k, 2)
  expect_equal(result$method, "pam")
  expect_length(result$assignments, nrow(mock_cluster_data))
  expect_true(is.numeric(result$silhouette))
  expect_equal(sum(result$sizes), nrow(mock_cluster_data))
})

test_that("cluster_data works with hclust methods", {
  # Note: check_match lowercases methods, so only lowercase methods work with hclust
  methods <- c("complete", "average", "single", "mcquitty", "median", "centroid")
  for (m in methods) {
    result <- cluster_data(mock_cluster_data, k = 2, method = m)
    expect_s3_class(result, "tna_clustering")
    expect_equal(result$method, m)
    expect_length(result$assignments, nrow(mock_cluster_data))
  }
})

test_that("cluster_data returns correct structure", {
  result <- cluster_data(mock_cluster_data, k = 2)
  expect_named(
    result,
    c("data", "k", "assignments", "silhouette", "sizes", "method", "distance")
  )
  expect_equal(result$data, mock_cluster_data)
  expect_true(inherits(result$distance, "dist"))
})

test_that("cluster_data validates input arguments", {
  # Invalid data type
  expect_error(cluster_data("not a data frame", k = 2), "data.frame")

  # k out of range
  expect_error(cluster_data(mock_cluster_data, k = 0))
  expect_error(cluster_data(mock_cluster_data, k = 100))

  # Invalid dissimilarity
  expect_error(cluster_data(mock_cluster_data, k = 2, dissimilarity = "invalid"))

  # Invalid method
  expect_error(cluster_data(mock_cluster_data, k = 2, method = "invalid"))

  # Weighted only for hamming
  expect_error(
    cluster_data(mock_cluster_data, k = 2, dissimilarity = "lv", weighted = TRUE),
    "Hamming"
  )
})

test_that("cluster_data works with matrix input", {
  mat_data <- as.matrix(mock_cluster_data)
  result <- cluster_data(mat_data, k = 2)
  expect_s3_class(result, "tna_clustering")
})

test_that("cluster_data handles weighted hamming distance", {
  result <- cluster_data(
    mock_cluster_data,
    k = 2,
    dissimilarity = "hamming",
    weighted = TRUE,
    lambda = 0.5
  )
  expect_s3_class(result, "tna_clustering")
})

test_that("cluster_sequences is alias for cluster_data", {
  result1 <- cluster_data(mock_cluster_data, k = 2)
  result2 <- cluster_sequences(mock_cluster_data, k = 2)
  expect_equal(result1$k, result2$k)
  expect_equal(result1$method, result2$method)
})

# Tests for seq2chr -----------------------------------------------------------

test_that("seq2chr converts data to character matrix", {
  result <- seq2chr(mock_cluster_data, na_syms = character(0))
  expect_type(result, "list")
  expect_named(result, c("mat", "len"))
  expect_true(is.matrix(result$mat))
  expect_equal(nrow(result$mat), nrow(mock_cluster_data))
  expect_equal(ncol(result$mat), ncol(mock_cluster_data))
})

test_that("seq2chr handles NA symbols", {
  data_with_na <- mock_cluster_data
  data_with_na[1, 1] <- "*"
  data_with_na[2, 2] <- "%"
  result <- seq2chr(data_with_na, na_syms = c("*", "%"))
  expect_type(result, "list")
})

test_that("seq2chr computes correct sequence lengths", {
  # All complete sequences should have length equal to ncol
  result <- seq2chr(mock_cluster_data, na_syms = character(0))
  expect_equal(result$len, rep(ncol(mock_cluster_data), nrow(mock_cluster_data)))
})

# Tests for get_qgram ---------------------------------------------------------

test_that("get_qgram returns empty for short sequences", {
  result <- get_qgram(c("a", "b"), n = 2, q = 3)
  expect_length(result, 0)
})

test_that("get_qgram computes q-grams correctly", {
  result <- get_qgram(c("a", "b", "c", "d"), n = 4, q = 2)
  expect_type(result, "integer")
  expect_true(length(result) > 0)
  expect_true(all(names(result) %in% c("ab", "bc", "cd")))
})

test_that("get_qgram counts repeated q-grams", {
  result <- get_qgram(c("a", "b", "a", "b"), n = 4, q = 2)
  expect_equal(result["ab"], c(ab = 2L))
})

# Tests for dissimilarity_matrix ----------------------------------------------

test_that("dissimilarity_matrix works with all methods", {
  strings <- seq2chr(mock_cluster_data, na_syms = character(0))
  methods <- c("osa", "lv", "dl", "hamming", "lcs", "qgram", "cosine", "jaccard", "jw")
  for (d in methods) {
    result <- dissimilarity_matrix(strings, d, lambda = 0)
    expect_true(inherits(result, "dist"))
  }
})

test_that("dissimilarity_matrix handles weighted distances", {
  strings <- seq2chr(mock_cluster_data, na_syms = character(0))
  result <- dissimilarity_matrix(strings, "hamming", lambda = 1.0)
  expect_true(inherits(result, "dist"))
})

test_that("dissimilarity_matrix computes q-gram based distances", {
  strings <- seq2chr(mock_cluster_data, na_syms = character(0))
  # Test with custom q parameter
  result <- dissimilarity_matrix(strings, "qgram", lambda = 0, q = 2)
  expect_true(inherits(result, "dist"))
})

# Tests for individual distance functions -------------------------------------

test_that("osa_dist computes optimal string alignment distance", {
  x <- c("a", "b", "c")
  y <- c("a", "b", "d")

  # Function executes and returns numeric
  result_same <- osa_dist(x, x, 3, 3)
  expect_type(result_same, "integer")

  result_diff <- osa_dist(x, y, 3, 3)
  expect_type(result_diff, "integer")

  # Transposition case (adjacent swap)
  x2 <- c("a", "b", "c", "d")
  y2 <- c("a", "c", "b", "d")
  result_trans <- osa_dist(x2, y2, 4, 4)
  expect_type(result_trans, "integer")
})

test_that("levenshtein_dist computes edit distance", {
  x <- c("a", "b", "c")
  y <- c("a", "b", "d")

  # Function executes and returns numeric
  result_same <- levenshtein_dist(x, x, 3, 3)
  expect_type(result_same, "integer")

  result_diff <- levenshtein_dist(x, y, 3, 3)
  expect_type(result_diff, "integer")

  # Completely different strings
  z <- c("d", "e", "f")
  result_all_diff <- levenshtein_dist(x, z, 3, 3)
  expect_type(result_all_diff, "integer")
})

test_that("hamming_dist computes hamming distance", {
  x <- c("a", "b", "c")
  y <- c("a", "b", "c")
  expect_equal(hamming_dist(x, y, 3, 3, weights = 1), 0)

  # One difference
  y <- c("a", "b", "d")
  expect_equal(hamming_dist(x, y, 3, 3, weights = 1), 1)

  # All different
  y <- c("d", "e", "f")
  expect_equal(hamming_dist(x, y, 3, 3, weights = 1), 3)

  # With weights
  weights <- c(1, 0.5, 0.25)
  y <- c("d", "e", "f")
  expect_equal(hamming_dist(x, y, 3, 3, weights = weights), sum(weights))
})

test_that("qgram_dist computes q-gram distance", {
  qx <- c(ab = 1L, bc = 1L)
  qy <- c(ab = 1L, bc = 1L)
  expect_equal(qgram_dist(NULL, NULL, qx, qy), 0)

  # Different q-grams
  qy <- c(ab = 2L, cd = 1L)
  result <- qgram_dist(NULL, NULL, qx, qy)
  expect_true(result > 0)
})

test_that("cosine_dist computes cosine distance", {
  qx <- c(ab = 1L, bc = 1L)
  qy <- c(ab = 1L, bc = 1L)
  expect_equal(cosine_dist(NULL, NULL, qx, qy), 0)

  # Orthogonal vectors
  qx <- c(ab = 1L)
  qy <- c(cd = 1L)
  expect_equal(cosine_dist(NULL, NULL, qx, qy), 1)

  # Empty vectors (edge case)
  qx <- integer(0)
  names(qx) <- character(0)
  qy <- integer(0)
  names(qy) <- character(0)
  expect_equal(cosine_dist(NULL, NULL, qx, qy), 1)
})

test_that("jaccard_dist computes jaccard distance", {
  qx <- c(ab = 1L, bc = 1L)
  qy <- c(ab = 1L, bc = 1L)
  expect_equal(jaccard_dist(qx, qy), 0)

  # No overlap
  qx <- c(ab = 1L)
  qy <- c(cd = 1L)
  expect_equal(jaccard_dist(qx, qy), 1)

  # Partial overlap
  qx <- c(ab = 1L, bc = 1L)
  qy <- c(ab = 1L, cd = 1L)
  expect_equal(jaccard_dist(qx, qy), 1 - 1/3)

  # Empty sets
  qx <- integer(0)
  names(qx) <- character(0)
  qy <- integer(0)
  names(qy) <- character(0)
  expect_equal(jaccard_dist(qx, qy), 0)
})

test_that("jaro_dist computes jaro distance", {
  x <- c("a", "b", "c")
  y <- c("a", "b", "c")
  expect_equal(jaro_dist(x, y, 3, 3), 0)

  # Completely different
  y <- c("d", "e", "f")
  expect_equal(jaro_dist(x, y, 3, 3), 1)

  # Empty strings
  expect_equal(jaro_dist(character(0), character(0), 0, 0), 0)
  expect_equal(jaro_dist(c("a"), character(0), 1, 0), 1)
  expect_equal(jaro_dist(character(0), c("a"), 0, 1), 1)
})

test_that("jaro_dist handles transpositions", {
  x <- c("a", "b", "c", "d")
  y <- c("a", "c", "b", "d")
  result <- jaro_dist(x, y, 4, 4)
  expect_true(result > 0 && result < 1)
})

test_that("jaro_winkler_dist computes jaro-winkler distance", {
  x <- c("a", "b", "c")
  y <- c("a", "b", "c")
  expect_equal(jaro_winkler_dist(x, y, 3, 3), 0)

  # Common prefix should reduce distance
  x <- c("a", "b", "c", "d")
  y <- c("a", "b", "x", "y")
  jaro <- jaro_dist(x, y, 4, 4)
  jw <- jaro_winkler_dist(x, y, 4, 4)
  # JW distance should be <= Jaro distance when there's a common prefix
  expect_true(jw <= jaro)
})

test_that("jaro_winkler_dist handles no common prefix", {
  x <- c("a", "b", "c")
  y <- c("d", "e", "f")
  result <- jaro_winkler_dist(x, y, 3, 3)
  expect_equal(result, 1)
})

test_that("distance implementations match stringdist", {
  skip_if_not_installed("stringdist")
  dist_funs <- list(
    osa = osa_dist,
    lv = levenshtein_dist,
    dl = damerau_levenshtein_dist,
    lcs = lcs_dist,
    qgram = qgram_dist,
    cosine = cosine_dist,
    jaccard = jaccard_dist,
    jw = jaro_winkler_dist
  )
  set.seed(0)
  k <- 100
  p <- length(dist_funs)
  nm <- names(dist_funs)
  dist_a <- matrix(NA, nrow = k, ncol = p)
  dist_b <- matrix(NA, nrow = k, ncol = p)
  for (i in 1:k) {
    n <- 5 + sample.int(15, 1)
    m <- 5 + sample.int(15, 1)
    x <- letters[sample.int(26, n, replace = TRUE)]
    y <- letters[sample.int(26, m, replace = TRUE)]
    z <- letters[sample.int(26, n, replace = TRUE)]
    qx <- get_qgram(x, n, q = 2)
    qy <- get_qgram(y, m, q = 2)
    x_str <- paste0(x, collapse = "")
    y_str <- paste0(y, collapse = "")
    weights <- 1.0
    for (j in 1:p) {
      dist <- nm[j]
      dist_a[i, j] <- dist_funs[[dist]](
        x = x, y = y, n = n, m = m, qx = qx, qy = qy, weights = weights
      )
      dist_b[i, j] <- stringdist::stringdist(
        a = x_str, b = y_str, method = dist, q = 2, p = 0.1
      )
    }
  }
  expect_equal(dist_a, dist_b)
  # Hamming
  set.seed(0)
  dist_a <- numeric(k)
  dist_b <- numeric(k)
  for (i in 1:k) {
    n <- 5 + sample.int(15, 1)
    x <- letters[sample.int(26, n, replace = TRUE)]
    y <- letters[sample.int(26, n, replace = TRUE)]
    x_str <- paste0(x, collapse = "")
    y_str <- paste0(y, collapse = "")
    weights <- 1.0
    dist_a[i] <- hamming_dist(x = x, y = y, n = n, m = n, weights = 1.0)
    dist_b[i] <- stringdist::stringdist(a = x_str, b = y_str, method = "hamming")
  }
  expect_equal(dist_a, dist_b)
})

# Edge cases ------------------------------------------------------------------

test_that("distance functions handle single element sequences", {
  x <- c("a")
  y <- c("a")
  # Functions execute on single element
  result_lev <- levenshtein_dist(x, y, 1, 1)
  expect_type(result_lev, "integer")

  result_ham <- hamming_dist(x, y, 1, 1, weights = 1)
  expect_type(result_ham, "double")
})

test_that("cluster_data works with minimum k", {
  result <- cluster_data(mock_cluster_data, k = 2)
  expect_equal(result$k, 2)
})

test_that("cluster_data handles all dissimilarity measures", {
  methods <- c("osa", "lv", "dl", "hamming", "lcs", "qgram", "cosine", "jaccard", "jw")
  for (d in methods) {
    result <- cluster_data(mock_cluster_data, k = 2, dissimilarity = d)
    expect_s3_class(result, "tna_clustering")
  }
})

#' Assess Model Reliability
#'
#' Performs reliability analysis and outputs a concise summary of key
#' metrics. The results can also be visualized.
#'
#' @export
#' @family validation
#' @param x A `tna` object.
#' @param types A `character` vector giving the model types to fit.
#'   See [build_model()] for available options.
#' @param split A `numeric` value between `0` and `1` specifying the proportion
#'   of data for the split. The default is `0.5` for an even split.
#' @param iter An `integer` specifying number of iterations (splits). The
#'   default is `1000`.
#' @param scaling See [compare()].
#' @param ... Ignored.
#' @return A `tna_reliability` object.
#' @examples
#' # Small number of iterations for CRAN
#' model <- tna(engagement)
#' rel <- reliability(model, iter = 20)
#'
reliability <- function(x, ...) {
  UseMethod("reliability")
}

#' @export
#' @rdname reliability
reliability.tna <- function(x, types = "relative", split = 0.5, iter = 1000,
                            scaling = "none", ...) {
  check_tna_seq(x)
  check_values(iter, strict = TRUE)
  check_range(split, lower = 0.0, upper = 1.0)
  d <- x$data
  type <- attr(x, "type")
  scaling_ <- attr(x, "scaling")
  params <- attr(x, "params")
  model <- initialize_model(d, type, scaling_, params, transitions = TRUE)
  trans <- model$trans
  weights_a <- model$weights
  weights_b <- model$weights
  alphabet <- attr(d, "alphabet")
  n <- nrow(d)
  m <- length(types)
  n_sample <- round(n * split)
  a <- length(alphabet)
  idx <- seq_len(n)
  res <- vector(mode = "list", length = length(types))
  for (i in seq_len(m)) {
    check_model_type(types[i])
    comparisons <- vector(mode = "list", length = iter)
    for (j in seq_len(iter)) {
      idx_a <- sample(idx, n_sample, replace = FALSE)
      idx_b <- setdiff(idx, idx_a)
      trans_a <- trans[idx_a, , , drop = FALSE]
      trans_b <- trans[idx_b, , , drop = FALSE]
      weights_a[] <- compute_weights(trans_a, types[i], scaling_, a)
      weights_b[] <- compute_weights(trans_b, types[i], scaling_, a)
      comparisons[[j]] <- compare(
        x = weights_a,
        y = weights_b,
        scaling = scaling,
        measures = character(0),
        network = FALSE
      )$summary_metrics
    }
    metrics <- dplyr::bind_rows(comparisons)
    metrics$model_type <- types[i]
    res[[i]] <- metrics
  }
  metrics <- dplyr::bind_rows(res)
  metrics_summary <- metrics |>
    dplyr::group_by(!!rlang::sym("model_type"), !!rlang::sym("metric")) |>
    dplyr::summarize(
      mean = mean(!!rlang::sym("value"), na.rm = TRUE),
      sd = stats::sd(!!rlang::sym("value"), na.rm = TRUE),
      median = stats::median(!!rlang::sym("value"), na.rm = TRUE),
      min = min(!!rlang::sym("value"), na.rm = TRUE),
      max = max(!!rlang::sym("value"), na.rm = TRUE),
      q25 = stats::quantile(!!rlang::sym("value"), 0.25, na.rm = TRUE),
      q75 = stats::quantile(!!rlang::sym("value"), 0.75, na.rm = TRUE)
    ) |>
    dplyr::ungroup()
  if (m == 1) {
    metrics$model_type <- NULL
    metrics_summary$model_type <- NULL
  }
  structure(
    list(
      metrics = metrics,
      summary = metrics_summary
    ),
    class = "tna_reliability"
  )
}

test_that("reliability can be analyzed", {
  expect_error(
    reliability(mock_tna_seq, iter = 50),
    NA
  )
  expect_error(
    reliability(mock_tna_seq, types = c("relative", "frequency"), iter = 50),
    NA
  )
})

---
title: "A showcase of the main tna functions"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{A showcase of the main tna functions}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.width = 6,
  fig.height = 4,
  out.width = "100%",
  dev = "jpeg",
  dpi = 100,
  comment = "#>"
)
suppressPackageStartupMessages({
  library("tna")
  library("tibble")
  library("dplyr")
  library("gt")
})
options(scipen = 99)
options(digits = 2)
options(max.print = 30)
options(width = 83)
```


# Tutorial of TNA with R

This is a short tutorial of the `tna` package. We begin by loading the package and the example data set `group_regulation`.

```{r}
library("tna")
data("group_regulation", package = "tna")
```

## Building `tna` Model

TNA models can be built with the `tna` function, which accepts several types of data such a sequence data, data frames or matrices.
```{r, message = FALSE, results = FALSE}
model <- tna(group_regulation)
print(model)
```


## Plotting and interpreting `tna` models

### Interpretation of the model

```{r fig.height=6, fig.width=6, fig.align='center', layout = c(1,1)}
# TNA visualization
plot(model)
```

### Pruning and retaining edges that "matter"


```{r, fig.show='hold', fig.width=5, fig.height=5 }
layout(matrix(1:4, ncol = 2, byrow = TRUE))
# Pruning with different methods (using comparable parameters)
pruned_threshold <- prune(model, method = "threshold", threshold = 0.15)
pruned_lowest <- prune(model, method = "lowest", lowest = 0.15)
pruned_disparity <- prune(model, method = "disparity", level = 0.5)

# Plotting for comparison
plot(pruned_threshold)
plot(pruned_lowest)
plot(pruned_disparity)
plot(model)
```

## Patterns

```{r }
layout(t(1:2))
# Identify 2-cliques (dyads) from the TNA model, excluding loops in the visualization
# A clique of size 2 is essentially a pair of connected nodes
cliques_of_two <- cliques(
  model, 
  size = 2, 
  threshold = 0.15  # Only consider edges with weight > 0.15
)
print(cliques_of_two)
plot(cliques_of_two, ask = FALSE)
```

```{r, fig.width=6,fig.height=2}
layout(t(1:3))
# Identify 3-cliques (triads) from the TNA_Model
# A clique of size 3 means a fully connected triplet of nodes
cliques_of_three <- cliques(
  model, 
  size = 3, 
  threshold = 0.05 # Only consider edges with weight > 0.05
)
print(cliques_of_three)
plot(cliques_of_three, ask = FALSE)
```

```{r, fig.width=5}
# Identify 4-cliques (quadruples) from the TNA_Model
# A clique of size 4 means four nodes that are all mutually connected
cliques_of_four <- cliques(
  model, 
  size = 4, 
  threshold = 0.035 # Only consider edges with weight > 0.03
)
print(cliques_of_four)
plot(cliques_of_four, ask = FALSE)
```


### Centralities

#### Node-level measures

```{r fig.height=8, fig.width=8, fig.align='center'}
# Compute centrality measures for the TNA model
Centralities <- centralities(model)

# Visualize the centrality measures
plot(Centralities)
```


```{r}
# Calculate hub scores and the authority scores for the network
hits_scores <- igraph::hits_scores(as.igraph(model))
hub_scores <- hits_scores$hub
authority_scores <- hits_scores$authority

# Print the calculated hub and authority scores for further analysis
print(hub_scores)
print(authority_scores)
```


#### Edge-level measures


```{r, fig.align='center', fig.width=6, fig.height=5.5, out.width="60%"}
# Edge betweenness
Edge_betweeness <- betweenness_network(model)
plot(Edge_betweeness)
```

### Community finding

```{r}
communities <- communities(model)
print(communities)
plot(communities, method = "leading_eigen")
```

## Network inference

### Bootstrapping

```{r}
# Perform bootstrapping on the TNA model with a fixed seed for reproducibility
set.seed(265)
boot <- bootstrap(model, threshold = 0.05)

# Print a summary of the bootstrap results
print(summary(boot))

# Show the non-significant edges (p-value >= 0.05 in this case)
# These are edges that are less likely to be stable across bootstrap samples
print(boot, type = "nonsig")
```

### Permutation

```{r, fig.align='center', fig.width = 5, fig.height = 5, layout = c(1,1)}
# Create TNA for the high-achievers subset (rows 1 to 1000)
Hi <- tna(group_regulation[1:1000, ])

# Create TNA for the low-achievers subset (rows 1001 to 2000)
Lo <- tna(group_regulation[1001:2000, ])

# Plot a comparison of the "Hi" and "Lo" models
# The 'minimum' parameter is set to 0.001, so edges with weights >= 0.001 are shown
plot_compare(Hi, Lo, minimum = 0.01)

# Run a permutation test to determine statistical significance of differences
# between "Hi" and "Lo"
# The 'iter' argument is set to 1000, meaning 1000 permutations are performed
Permutation <- permutation_test(Hi, Lo, iter = 1000, measures = "Betweenness")

# Plot the significant differences identified in the permutation test
plot(Permutation, minimum = 0.01)
```

```{r}
print(Permutation$edges$stats)
print(Permutation$centralities$stats)
```

### Interpreting the Results of the Case-Dropping Bootstrap for Centrality Indices

```{r, fig.align='center', fig.width=7, fig.height=4}
# Results of the Case-Dropping Bootstrap for Centrality Indices
Centrality_stability <- estimate_centrality_stability(model, iter = 100)
plot(Centrality_stability)
```

---
title: "How to prepare data for tna"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{How to prepare data for tna}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.width = 6,
  fig.height = 4,
  out.width = "100%",
  message = FALSE,
  fig.show = TRUE,
  dev = "jpeg",
  dpi = 100,
  results = "hide",  
  comment = "#>"
)
suppressPackageStartupMessages({
  library("tna")
  library("tibble")
  library("dplyr")
})
```

The `tna` package accepts sequence data (a `stslist` object created by the `TraMineR` package), a wide `data.frame` where each row represents a sequence and each column represents a timepoint, or a transition matrix. When our data does not follow any of those formats, we can use the `prepare_data` function from `tna` to get it in the right shape. 

Let's start by creating an example dataframe containing event logs where each row represents an action performed by a `user` at a specific `timestamp.` The `achievement` column indicates whether the user was associated with high or low achievement.


```{r}
df <- tribble(
  ~user, ~timestamp, ~event, ~achievement, ~order,
  2, "2025-02-27 18:01:32", "Plan",           "High", 1,
  2, "2025-02-27 18:03:32", "Goals",          "High", 2,
  1, "2025-02-27 18:08:32", "Goals",          "High", 1,
  3, "2025-02-27 18:15:32", "Plan",           "High", 1,
  5, "2025-02-27 18:16:32", "Help",           "Low",  1,
  3, "2025-02-27 18:19:32", "Goals",          "High", 2,
  5, "2025-02-27 18:19:32", "Plan",           "Low",  2,
  2, "2025-02-27 18:20:32", "Environment",    "High", 3,
  1, "2025-02-27 18:25:32", "Task",           "High", 2,
  4, "2025-02-27 18:25:32", "Help",           "Low",  1,
  5, "2025-02-27 18:26:32", "Task",           "Low",  3,
  3, "2025-02-27 18:32:32", "Metacognition",  "High", 3,
  4, "2025-02-27 18:33:32", "Goals",          "Low",  2,
  1, "2025-02-27 18:36:32", "Environment",    "High", 3,
  1, "2025-02-27 18:44:32", "Task",           "High", 4,
  1, "2025-02-27 18:45:32", "Task",           "High", 5,
  5, "2025-02-27 18:46:32", "Help",           "Low",  4,
  1, "2025-02-27 19:01:32", "Plan",           "High", 6,
  2, "2025-02-27 19:06:32", "Environment",    "High", 4,
  4, "2025-02-27 19:06:32", "Plan",           "Low",  3,
  1, "2025-02-27 19:13:32", "Metacognition",  "High", 7,
  3, "2025-02-27 19:13:32", "Metacognition",  "High", 4,
  4, "2025-02-27 19:15:32", "Goals",          "Low",  4,
  4, "2025-02-27 19:20:32", "Metacognition",  "Low",  5,
  4, "2025-02-27 19:20:32", "Environment",    "Low",  6,
  5, "2025-02-27 19:23:32", "Metacognition",  "Low",  5,
  3, "2025-02-27 19:25:32", "Help",           "High", 5,
  2, "2025-02-27 19:27:32", "Metacognition",  "High", 5,
  2, "2025-02-27 19:33:32", "Environment",    "High", 6,
  4, "2025-02-27 19:46:32", "Environment",    "Low",  7,
  5, "2025-02-27 19:49:32", "Plan",           "Low",  6,
  2, "2025-02-27 19:55:32", "Goals",          "High", 7
)

```

When we are interested in the whole dataset as a single sequence, and the data is already ordered chronologically, we just need to specify the `action` column, which in this case is called `event`. We can pass the result of calling `prepare_data` directly to the `tna` function to create a `tna` model.

```{r}
by_classroom <- prepare_data(df, action = "event")
tna_by_classroom <- tna(by_classroom)
plot(tna_by_classroom)
```
When we want to create a sequence per user, in addition to `action`, we need to specify the `actor` column, which in this case is `user`. If the data is already ordered, we do not need to provide any additional arguments. If it is not ordered, we can provide a column to order the data by, in this case `order`.

```{r}
by_user <- prepare_data(df, actor = "user", action = "event", order = "order")
tna_by_user <- tna(by_user)
plot(tna_by_user)
```

If rather than the order we only have the timestamps, we can provide it as `time` column. By default, events happening less than 15 minutes apart will be grouped in the same sequence, while events that happen after a longer time, will mark the start of a new sequence (i.e., session). If both `time` and `order` are provided, the data will be first ordered by `time`, and in case of a tie, by `order`. 

```{r}
by_session <- prepare_data(df, actor = "user", time = "timestamp", action = "event")
tna_by_session <- tna(by_session)
plot(tna_by_session)
```

If we want to customize the time gap that marks the start of a new sequence, we can do so by customizing the `time_threshold` argument (in minutes). 
```{r}
by_session_custom <- prepare_data(
  df, actor = "user", time = "timestamp", 
  action = "event", time_threshold = 10 * 60 # 10 minutes
)
tna_by_session_custom <- tna(by_session_custom)
plot(tna_by_session_custom)
```
Another advantage of using `prepare_data` prior to constructing the `tna` model is that we get to keep other variables of the data and use them in our analysis. For instance, we can use `group_tna` to create a `tna` model by achievement group just by passing the result of `prepare_data` as a first argument, and indicating the name of the column in the data that we want to group by.
```{r,fig.width=8,fig.height=3, echo=2:3}
layout(t(1:2))
gtna <- group_tna(by_user, group = "achievement")
plot(gtna)
```

---
title: "Getting started with the tna package"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with the tna package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.width = 6,
  fig.height = 4,
  dev = "jpeg",
  dpi = 100,
  out.width = "100%",
  comment = "#>"
)
suppressPackageStartupMessages({
  library("tna")
  library("tibble")
  library("dplyr")
  library("gt")
})
```

This vignette showcases some basic usage of the `tna` package. For more tutorials, please visit [the package website](https://sonsoles.me/tna/).

First we load the package that we will use for this example.

```{r, eval = FALSE}
library("tna")
library("tibble")
library("dplyr")
library("gt")
```

We also load the `group_regulation` data available in the package (see `?group_regulation` for further information)
```{r}
data("group_regulation", package = "tna")
```

We build a TNA model using this data with the `tna()` function .
```{r}
tna_model <- tna(group_regulation)
```

To visualize the model, we can use the standard `plot()` function.
```{r, fig.width=5, fig.height=5}
plot(tna_model)
```

The initial state probabilities are
```{r}
data.frame(`Initial prob.` = tna_model$inits, check.names = FALSE) |>
  rownames_to_column("Action") |>
  arrange(desc(`Initial prob.`)) |>
  gt() |>
  fmt_percent()
```

and the transitions probabilities are
```{r}
tna_model$weights |>
  data.frame() |>
  rownames_to_column("From\\To") |>
  gt() |>
  fmt_percent()
```

The function `centralities()` can be used to compute various centrality measures (see `?centralities` for more information).
These measures can also be visualized with the `plot()` function.
```{r}
centrality_measures <- c("BetweennessRSP", "Closeness", "InStrength", "OutStrength")
cents_withoutloops <- centralities(
  tna_model,
  measures = centrality_measures,
  loops = FALSE,
  normalize = TRUE
)
plot(cents_withoutloops, ncol = 2, model = tna_model)
```

---
title: "Finding cliques and communities with tna"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Finding cliques and communities with tna}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.width = 6,
  fig.height = 4,
  out.width = "100%",
  dev = "jpeg",
  dpi = 100,
  comment = "#>"
)
suppressPackageStartupMessages({
  library("tna")
})
```

The `tna` package includes functionalities for finding cliques of the transition network as well as discovering communities. We begin by loading the package and the example data set `group_regulation`.
```{r}
library("tna")
data("group_regulation", package = "tna")
```

We fit a TNA model to the data.
```{r}
tna_model <- tna(group_regulation)
print(tna_model)
plot(tna_model)
```

Next, we apply several community finding algorithms to the model (see `?communities` for more details), and plot the results for the `leading_eigen` algorithm.
```{r, warning = FALSE}
cd <- communities(tna_model)
plot(cd, method = "leading_eigen")
```

Cliques can be obtained with the `cliques` function. Here we look for dyads and triads by setting `size = 2` and `size = 3`, respectively. Finally, we plot the results.
```{r, figures-side, fig.show="hold", fig.height=6, fig.width=8}
layout(matrix(1:4, ncol = 2, byrow = TRUE))
dyads <- cliques(tna_model, size = 2, threshold = 0.2)
triads <- cliques(tna_model, size = 3, threshold = 0.05)
plot(dyads, ask = FALSE)
plot(triads, ask = FALSE)
```

---
title: "tna — R Package Documentation"
subtitle: "v1.1.0 — Transition Network Analysis"
output:
  html_document:
    toc: true
    toc_float:
      collapsed: true
    theme: flatly
    highlight: tango
    code_folding: hide
    number_sections: false
---
```{r setup, include=FALSE}
knitr::opts_chunk$set(
  message = FALSE, warning = FALSE,
  fig.width = 8, fig.height = 6,
  comment = "#>"
)
suppressPackageStartupMessages({
  library("cograph")
  library("tna")
})
```

---

**tna** provides tools for performing Transition Network Analysis (TNA) to study relational dynamics. Build Markov models from sequence data, compute centrality measures, detect communities and cliques, and validate findings with bootstrapping and permutation tests.

**Install:**

```{r eval=FALSE}
install.packages("tna")
# or dev version:
devtools::install_github("sonsoleslp/tna")
```

Requires R >= 4.1.0. All analysis objects have `print()`, `summary()`, and `plot()` methods.

---

# Building Models

## `build_model()`

Construct a transition network analysis model from sequence data. Requires specifying `type`. For convenience, use the shortcut functions: `tna()` = relative, `ftna()` = frequency, `ctna()` = co-occurrence, `atna()` = attention.

```
build_model(x, type = "relative", scaling = character(0L), cols = everything(),
            params = list(), inits, begin_state, end_state)
```

| Parameter | Description | Default |
|-----------|-------------|---------|
| `x` | `data.frame` (wide), `stslist`, `matrix`, or `tna_data` | — |
| `type` | `"relative"`, `"frequency"`, `"co-occurrence"`, `"n-gram"`, `"gap"`, `"window"`, `"reverse"`, `"attention"` | `"relative"` |
| `scaling` | `"minmax"`, `"max"`, `"rank"`, or empty vector | `character(0)` |
| `params` | List: `n_gram`, `max_gap`, `window_size`, `weighted`, `direction`, `decay`, `lambda`, `time`, `duration` | `list()` |

**Returns:** A `tna` object with `$weights`, `$inits`, `$labels`, `$data`.

```{r}
# Using build_model with explicit type
model <- build_model(group_regulation, type = "relative")

# Shortcut functions (preferred)
model <- tna(group_regulation)       # type = "relative"
model_f <- ftna(group_regulation)    # type = "frequency"
model_c <- ctna(group_regulation)    # type = "co-occurrence"
model_a <- atna(group_regulation)    # type = "attention"

print(model)
summary(model)
```

### Model types

| Type | Description |
|------|-------------|
| `"relative"` | Transition probabilities (rows sum to 1). Default |
| `"frequency"` | Raw transition counts |
| `"co-occurrence"` | Co-occurrence within sequences |
| `"n-gram"` | Higher-order transitions |
| `"gap"` | Non-adjacent transitions, weighted by gap |
| `"window"` | Transitions within sliding window |
| `"reverse"` | Reverse order (reply networks) |
| `"attention"` | Exponential decay-weighted downstream pairs |

## `group_model()` / `group_tna()`

Build a TNA model for each group. Accepts manual group assignments, `mhmm` objects, or `tna_clustering` results. Most tna functions accept `group_tna` directly — no need to specify groups again.

```
group_model(x, group, type = "relative", scaling = character(0L),
            groupwise = FALSE, cols = everything(), params = list(), na.rm = TRUE, ...)
```

```{r}
# From mixture Markov model
mmm_model <- group_model(engagement_mmm)
print(mmm_model)
summary(mmm_model)
```

```{r}
# Manual groups
group <- c(rep("High", 1000), rep("Low", 1000))
gmodel <- group_model(group_regulation, group = group)
```

## `sna()`

Build a social network analysis model from edge-list data (from, to, weight).

```{r}
set.seed(42)
d <- data.frame(
  from = sample(LETTERS[1:4], 100, replace = TRUE),
  to = sample(LETTERS[1:4], 100, replace = TRUE),
  weight = rexp(100)
)
model_sna <- sna(d)
```

---

# Plotting

All TNA analysis objects have `plot()` methods returning `cograph_network` or `ggplot2` objects.

## `plot()`

Plot a TNA model as a transition network.

```
plot(x, labels, colors, pie, cut, show_pruned = TRUE, pruned_edge_color = "pink",
     edge.color = NA, edge.labels = TRUE, edge.label.position = 0.65,
     layout = "circle", layout_args = list(), scale_nodes, scaling_factor = 0.5,
     mar = rep(5, 4), theme = "colorblind", ...)
```

```{r}
plot(model)
```

```{r}
plot(model, layout = "spring", scale_nodes = "OutStrength")
```

```{r,fig.width=12}
layout(t(1:2))
# Group model — side-by-side panels
plot(gmodel)
```

## `plot_frequencies()`

Bar plot of state frequency distribution.

```{r}
plot_frequencies(model)
```

```{r}
# Group comparison
plot_frequencies(gmodel)
```

## `plot_mosaic()`

Mosaic plot with chi-square test results. Requires frequency model (`ftna()`).

```{r}
plot_mosaic(model_f)
```

## `plot_sequences()`

Sequence index plots or state distribution plots.

```
plot_sequences(x, cols = everything(), group, type = "index", scale = "proportion",
               geom = "bar", include_na = FALSE, colors, na_color = "white",
               sort_by, show_n = TRUE, border, title, legend_title, xlab, ylab,
               tick = 5, ncol = 2L, ...)
```

```{r}
plot_sequences(group_regulation)
```

```{r}
plot_sequences(group_regulation, type = "distribution")
```

```{r}
# Group comparison — pass group_tna directly
plot_sequences(gmodel)
```

## `plot_compare()`

Difference network between groups. Just pass the group model object.
```{r}
plot_compare(gmodel)
```

Difference network between two models. Green = x greater, red = y greater.

```{r}
model_a <- tna(group_regulation[1:1000, ])
model_b <- tna(group_regulation[1001:2000, ])
plot_compare(model_a, model_b)
```

## `plot_associations()`

Association network. Requires frequency model (`ftna()`).

```{r}
plot_associations(model_f)
```

## `hist()`

```{r}
hist(model)
```

### Additional plot methods

| Object | plot() produces |
|--------|----------------|
| `tna_centralities` | Lollipop charts by measure |
| `tna_communities` | Network colored by community |
| `tna_cliques` | Individual clique subnetworks |
| `tna_bootstrap` | Significant edges only |
| `tna_permutation` | Significant edge differences |
| `tna_stability` | Stability curves with CS-coefficients |
| `tna_comparison` | Heatmap or difference viz |
| `tna_sequence_comparison` | Pattern comparison chart |
| `group_tna` | Side-by-side network plots |

---

# Data Preparation

## `prepare_data()`

Convert long-format event logs into wide-format sequences.

```
prepare_data(data, actor, time, action, order, time_threshold = 900,
             custom_format = NULL, is_unix_time = FALSE,
             unix_time_unit = "seconds", unused_fn = dplyr::first)
```

```{r}
results <- prepare_data(
  group_regulation_long,
  actor = "Actor", time = "Time", action = "Action"
)
print(results$statistics)
```

## `import_data()`

Transform wide-format feature data into long-format sequence data.

```
import_data(data, cols, id_cols, window_size = 1, replace_zeros = TRUE)
```

## `import_onehot()`

Import one-hot encoded data as co-occurrence network.

```
import_onehot(data, cols, window = 1L)
```

## `simulate()`

Simulate sequence data from a TNA model (requires `type = "relative"`).

```{r}
sim <- simulate(model, nsim = 5, seed = 123, max_len = 10)
print(sim)
```

---

# Centrality Measures

## `centralities()`

Calculate centrality measures. Works on `tna`, `group_tna`, and `matrix`.

```
centralities(x, loops = FALSE, normalize = FALSE, measures)
```

| Measure | Description |
|---------|-------------|
| `OutStrength` | Total weight of outgoing edges |
| `InStrength` | Total weight of incoming edges |
| `ClosenessIn` / `ClosenessOut` / `Closeness` | Closeness centrality variants |
| `Betweenness` | Geodesic betweenness |
| `BetweennessRSP` | Randomized shortest paths betweenness |
| `Diffusion` | Diffusion centrality |
| `Clustering` | Signed clustering coefficient |

```{r}
cm <- centralities(model)
print(cm)
plot(cm, ncol = 3, reorder = TRUE)
```

```{r}
# On group_tna directly
centralities(gmodel)
```

## `betweenness_network()`

Build network with edge betweenness as weights.

```{r}
bn <- betweenness_network(model)
print(bn)
```

## `estimate_cs()`

Centrality stability via subset sampling.

```
estimate_cs(x, loops = FALSE, normalize = FALSE,
            measures = c("InStrength", "OutStrength", "Betweenness"),
            iter = 1000, method = "pearson",
            drop_prop = seq(0.1, 0.9, by = 0.1),
            threshold = 0.7, certainty = 0.95, progressbar = FALSE)
```

```{r}
cs <- estimate_cs(model, measures = c("InStrength", "OutStrength"), iter = 100)
print(cs)
plot(cs)
```

---

# Network Structure

## `communities()`

Detect communities using 7 igraph algorithms. Works on `tna` and `group_tna`.

```
communities(x, methods, gamma = 1)
```

```{r}
comm <- communities(model)
print(comm)
plot(comm, method = "spinglass")
```

```{r}
# On group_tna directly
communities(gmodel)
```

## `cliques()`

Identify cliques (complete subgraphs) of a given size.

```
cliques(x, size = 2, threshold = 0, sum_weights = FALSE, ...)
```

```{r, fig.width=6, fig.height=2}
layout(t(1:3))
cliq <- cliques(model, size = 2)
print(cliq)
plot(cliq, n = 3, ask = FALSE)
```

---

# Comparison

## `compare()`

Compare two TNA models with comprehensive metrics.

```{r}
comp <- compare(gmodel, i = 1, j = 2)
print(comp)
plot(comp)
```

## `compare_sequences()`

Compare subsequence patterns between groups. Pass `group_tna` directly.

```
# For group_tna (groups already defined)
compare_sequences(x, sub, min_freq = 5L, correction = "bonferroni", ...)
```

```{r}
comp_seq <- compare_sequences(gmodel)
print(head(comp_seq, 10))
plot(comp_seq)
```

## `permutation_test()`

Permutation tests for edge weight and centrality differences.

```
permutation_test(x, y, adjust = "none", iter = 1000, paired = FALSE,
                 level = 0.05, measures = character(0), ...)
```

```{r}
model_x <- tna(group_regulation[1:200, ])
model_y <- tna(group_regulation[1001:1200, ])
perm <- permutation_test(model_x, model_y, iter = 100)
print(perm)
plot(perm)
```

It also accepts a group tna model object.
```{r}
permg <- permutation_test(gmodel, iter = 100)
```

---

# Validation

## `bootstrap()`

Bootstrap transition networks for confidence intervals and significance.

```
bootstrap(x, iter = 1000, level = 0.05, method = "stability",
          threshold, consistency_range = c(0.75, 1.25))
```

```{r}
boot <- bootstrap(model, iter = 100)
plot(boot)
```
```{r, fig.height=11}
plot_bootstrap_forest(boot)
```

## `bootstrap_cliques()`

Bootstrap cliques to assess stability.

```{r, results=F, message = F}
bc <- bootstrap_cliques(model, size = 2, iter = 100)
print(bc)
```

## `prune()`

Remove weak edges. Four methods available.

```
prune(x, method = "threshold", threshold = 0.1, lowest = 0.05,
      level = 0.5, boot = NULL, ...)
```

```{r, results=F, message = F}
pruned_t <- prune(model, method = "threshold", threshold = 0.1)
pruned_p <- prune(model, method = "lowest", lowest = 0.05)
pruned_d <- prune(model, method = "disparity", level = 0.5)
pruning_details(pruned_t)
```

```{r}
plot(pruned_t)
```

```{r}
# Restore and reapply
restored <- deprune(pruned_t)
repruned <- reprune(restored)
```

---

# Clustering

## `cluster_sequences()`

Cluster sequences using string distance-based dissimilarity.

```
cluster_sequences(data, k, dissimilarity = "hamming", method = "pam",
                  na_syms = c("*", "%"), weighted = FALSE, lambda = 1, ...)
```

```{r}
result <- cluster_sequences(group_regulation[1:200, ], k = 3, dissimilarity = "osa")
print(result)

# Pass directly to group_model
gmodel_clust <- group_model(result)
plot(gmodel_clust)
```

## `rename_groups()`

```{r}
gmodel_mmm <- group_model(engagement_mmm)
gmodel_mmm_renamed <- rename_groups(gmodel_mmm, c("A", "B", "C"))
cat("Original:", names(group_model(engagement_mmm)), "\n")
cat("Renamed:", names(gmodel_mmm_renamed), "\n")
```

## `mmm_stats()`

```{r}
mmm_stats(engagement_mmm)
```

---

# Summary & Conversion

## `summary()`

```{r}
summary(model)
```

```{r}
summary(gmodel)
```

## `as.igraph()`

```{r}
g <- as.igraph(model)
print(g)
```

---

**tna** v1.1.0 | MIT License | Mohammed Saqr, Santtu Tikka, Sonsoles Lopez-Pernas

Reference: Saqr M., Lopez-Pernas S., et al. (2025). Transition Network Analysis. *Proc. LAK '25*, 351-361. [doi:10.1145/3706468.3706513](https://doi.org/10.1145/3706468.3706513)

---
title: "Using grouped sequence data with tna"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using grouped sequence data with tna}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.width = 6,
  fig.height = 4,
  out.width = "100%",
  dev = "jpeg",
  dpi = 100,
  comment = "#>"
)
suppressPackageStartupMessages({
  library("tna")
  library("tibble")
  library("dplyr")
  library("gt")
})
```

TNA supports the analysis of transition networks constructed from grouped sequence data. Groups can be defined in several ways, but mainly: using a pre-existing grouping variable in the data (e.g., a demographic or experimental condition), or by clustering the sequences themselves based on their similarity. This vignette demonstrates both approaches using the `group_regulation_long` dataset.

First, we load the packages we will use for this example.

```{r, eval = FALSE}
library("tna")
library("tibble")
library("dplyr")
library("gt")
```

## Data preparation

We import the data in long format and prepare it for analysis. The `prepare_data()` function converts the long-format event log into wide-format sequences. The unused columns in the dataset are stored in the metadata of `prepared` and we can use them later on.

```{r, message = F}
data("group_regulation_long", package = "tna")
prepared <- prepare_data(group_regulation_long,
                         actor = "Actor",
                         action = "Action",
                         time = "Time")
```

## Groups from a pre-existing variable

When the data contains a grouping variable, we can build separate TNA models for each group directly. Here, the `"Achiever"` column in the metadata splits the sequences into two groups (high vs. low achievers).

```{r, fig.width=9, fig.height=4}
layout(t(1:2))
achievers <- group_tna(prepared, group = "Achiever")
plot(achievers)
```

### Comparing groups

The `plot_compare()` function visualizes the difference network between the two groups. Green edges and donut segments indicate that the first group (High achievers) has higher values, while red indicates the opposite (Low achievers have higher values).

```{r}
plot_compare(achievers)
```

### Permutation test

A permutation test can be used to assess whether the observed differences between the two groups are statistically significant.

```{r}
permutation_test_results <- permutation_test(achievers)
plot(permutation_test_results)
```

### Subsequence comparison

We can also compare the frequency of subsequences across groups. Here we look at subsequences of length 3 to 5, keeping only those that appear at least 5 times, and apply FDR correction for multiple comparisons.

```{r}
subsequence_comparison  <- compare_sequences(achievers,
                                                  sub = 3:5,
                                                  min_freq = 5,
                                                  correction = "fdr")
plot(subsequence_comparison, cells = TRUE)
```

## Groups from sequence clustering

When no pre-existing grouping variable is available, we can cluster the sequences based on their pairwise dissimilarity. The `cluster_sequences()`.

```{r}
clustering_results <- cluster_sequences(prepared, k = 3)
```

To choose an appropriate number of clusters, we can plot the silhouette score for different values of *k*. Higher silhouette values indicate better-separated clusters.

```{r}
plot(
  2:8,
  sapply(2:8, \(k) cluster_sequences(prepared, k = k)$silhouette),
  type = "b",
  xlab = "Number of clusters (k)",
  ylab = "Silhouette",
  xaxt = "n"
)
```

Once we have chosen *k*, we build the grouped TNA model using the cluster assignments.

```{r}
tna_model_clus <- group_tna(prepared, group = clustering_results$assignments)
```

```{r, fig.width=9, fig.height=9}
layout(matrix(1:4, byrow = T, ncol = 2))
plot(tna_model_clus)
```

### Summarizing the cluster-specific models

We can summarize the cluster-specific models to compare their overall characteristics.

```{r}
summary(tna_model_clus) |>
  gt() |>
  fmt_number(decimals = 2)
```

Initial probabilities show which states are most common at the start of the sequences in each cluster.

```{r}
mat <- sapply(
  tna_model_clus,
  \(x) setNames(x$inits, x$labels)
)

df <- data.frame(label = rownames(mat), mat, row.names = NULL)

gt(df, rowname_col = "label") |> fmt_percent(columns = -label)
```

The full transition probability matrices can also be inspected for each cluster.

```{r}
transitions <- lapply(
  tna_model_clus,
  function(x) {
    x$weights |>
      data.frame() |>
      rownames_to_column("From\\To") |>
      gt() |>
      fmt_percent()
  }
)

transitions[[1]] |> tab_header(title = names(tna_model_clus)[1])
transitions[[2]] |> tab_header(title = names(tna_model_clus)[2])
transitions[[3]] |> tab_header(title = names(tna_model_clus)[3])
```

### Pruning with bootstrap

Just like ordinary TNA models, we can retain only the statistically robust edges.

```{r}
cluster_boot <- bootstrap(tna_model_clus)
```

```{r, fig.width=9, fig.height=9, message = F}
layout(matrix(1:4, byrow = T, ncol = 2))
plot(cluster_boot)
```

### Centrality measures

Centrality measures can be computed for each cluster to identify which states play central roles in each group's transition dynamics.

```{r, fig.width=9, fig.height=4}
centrality_measures <- c(
  "BetweennessRSP",
  "Closeness",
  "InStrength",
  "OutStrength"
)
centralities_per_cluster <- centralities(
  tna_model_clus,
  measures = centrality_measures
)
plot(
  centralities_per_cluster, ncol = 4,
  colors = c("purple", "orange", "pink")
)
```

### Subsequence comparison across clusters

Finally, we can compare subsequence frequencies across the clusters, just as we did for the pre-existing groups above.

```{r}
subsequence_comparison  <- compare_sequences(tna_model_clus, sub = 3:5, min_freq = 5, correction = "fdr")
plot(subsequence_comparison, cells = TRUE)
```

---
title: "Attention TNA"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Attention TNA}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.width = 6,
  fig.height = 4,
  out.width = "100%",
  dev = "png",
  dpi = 150,
  comment = "#>"
)
suppressPackageStartupMessages({
  library("tna")
  library("tibble")
  library("dplyr")
  library("ggplot2")
})
options(scipen = 9999)
options(digits = 2)
options(max.print = 30)
options(width = 83)
```


## What Problem Does ATNA Solve?

Regular TNA looks at pairs of states that happen one right after the other. If a student does Plan at time 3 and Monitor at time 4, TNA counts that as a transition. But it ignores the relationship between Plan at time 3 and Adapt at time 7. Those two events are four steps apart, so TNA treats them as unrelated.

That is a problem when earlier events still matter later on. A student who plans at the start of a session is still influenced by that planning when they adapt several steps later. A user's first prompt in a chatbot conversation shapes the responses several turns down the line. In these cases, the connection between two events is real — it's just not immediate.

ATNA fills this gap. Instead of only counting adjacent transitions, it looks at *all* pairs of states across the whole sequence and weights them by how far apart they are. Close events get high weight. Distant events get low weight. The weight drops off following an exponential curve, so nearby things matter a lot and far-away things matter a little — but they still matter.

---

## The Intuition

Take this sequence of student actions:

```
Plan → Monitor → Discuss → Emotion → Adapt → Monitor → Plan
```

Regular TNA draws a line between each pair of neighbors: Plan→Monitor, Monitor→Discuss, Discuss→Emotion, and so on. Seven events produce six edges.

ATNA draws lines between *every* pair, but with decreasing weight:

- Plan→Monitor (1 step apart): strong connection
- Plan→Discuss (2 steps apart): weaker
- Plan→Emotion (3 steps apart): weaker still
- Plan→Adapt (4 steps apart): faint
- Plan→Monitor (5 steps apart): very faint
- Plan→Plan (6 steps apart): barely there

It does this starting from every event in the sequence. All these weighted connections get added up across every sequence in the data. The result is a weight matrix where each cell captures how often state A appears before state B, weighted by how close together they tend to be.

---

## Building an ATNA Model

If you've used `tna()` before, this works the same way with a different function name.

```{r}
library("tna")
data("group_regulation")

model_atna <- atna(group_regulation, scaling = "minmax")

print(model_atna)
```

You get a weight matrix and initial probabilities, just like regular TNA. The difference is in what the weights represent. In TNA, they are transition probabilities (they add up to 1 per row). In ATNA, they are accumulated attention weights across all temporal distances, scaled to the 0–1 range by the `scaling = "minmax"` argument.

---

## Plotting the Network

```{r, fig.width=4, fig.height=4}
plot(model_atna)
```

This produces the same style of network graph you get from `plot()` on a regular TNA model. Nodes are states, edges are weighted connections, and thicker arrows mean stronger relationships.

The difference is what the arrows mean. In a regular TNA plot, a thick arrow from Plan to Monitor means "when Plan happens, Monitor tends to come next." In an ATNA plot, a thick arrow from Plan to Monitor means "when Plan happens, Monitor tends to show up somewhere afterward — not just in the very next step, but across the whole sequence, weighted by distance."

The `minimum` argument hides edges below that weight. The `cut` argument removes edges that are too weak to be meaningful. Raising these values thins out the network so you can focus on the strongest connections. Start with `minimum = 0.05` and adjust from there.

You can also look at the distribution of edge weights:

```{r}
hist(model_atna)
```

This histogram shows how the attention weights are spread out. If most weights cluster near zero with a few high ones, the network has a clear backbone of strong connections. If weights are spread evenly, the network is more diffuse and harder to interpret — you may want to prune it more aggressively.

---

## Centrality Analysis

Centrality tells you which states play the biggest role in the network.

```{r}
cents <- centralities(model_atna, normalize = TRUE)
print(cents)
```

This gives you three measures for each state:

- **InStrength**: how much attention flows *into* this state from other states. A state with high InStrength is a common destination — other events tend to be followed by it, even several steps later.
- **OutStrength**: how much attention flows *out* of this state toward other states. A state with high OutStrength has a long reach — when it happens, it shapes what comes after it across multiple future steps.
- **Betweenness**: how often this state sits on the shortest path between other states. A state with high betweenness acts as a bridge connecting different parts of the network.

To plot all three:

```{r, fig.width=8, fig.height=8}
plot(cents)
```

This makes a bar chart with states on the x-axis and centrality scores on the y-axis, one panel per measure.

Reading the results: compare what ATNA centralities tell you against what you'd get from regular TNA. A state might have low OutStrength in TNA (it doesn't strongly predict the very next step) but high OutStrength in ATNA (it has a broad, lasting influence across many future steps). That kind of state matters more than a first-order analysis would suggest.

---

## Patterns: Communities and Cliques

### Communities

Communities are groups of states that are more tightly connected to each other than to the rest of the network.

```{r, fig.width=4, fig.height=4}
comms <- communities(model_atna)
plot(comms)
```

The plot colors each community differently and draws the network with community boundaries visible. States within the same community tend to co-occur in temporal proximity — they form clusters of related activity.

In ATNA, communities can look different from what you'd find in regular TNA, because ATNA captures longer-range relationships. Two states that rarely follow each other directly but often appear within a few steps of each other can end up in the same community under ATNA even though TNA would put them in separate ones.

### Cliques

Cliques are small groups of states where every state has a strong connection to every other state in the group.

```{r, fig.width=3,fig.height=1}
layout(t(1:3)); par(mar = c(0,0,0,0))
cliq <- cliques(model_atna, size = 2, threshold = 0.2)
print(cliq)
plot(cliq)
```

The `size` argument sets the minimum clique size (2 means pairs, 3 means triangles, etc.). The `threshold` argument sets the minimum edge weight needed for two states to count as connected.

In ATNA, a clique means those states all have strong long-range attention toward each other. A 3-clique of {Plan, Monitor, Adapt} means that whenever any one of those three shows up, the other two tend to appear nearby (within the attention window), no matter the exact step distance.

---

## Validation

Before interpreting results, check that they are stable.

### Bootstrap

```{r, fig.width=4, fig.height=4}
boot <- bootstrap(model_atna, threshold = 0.05)
plot(boot)
```

This resamples your sequences, rebuilds the model each time, and checks how often each edge exceeds the threshold. Edges that appear in a high percentage of bootstrap samples (say 95%+) are stable. Edges that come and go are not reliable.

### Centrality Stability

```{r, fig.width=8, fig.height=5}
stab <- estimate_centrality_stability(model_atna)
print(stab)
plot(stab)
```

This drops increasing fractions of your data and recalculates centrality each time. The stability coefficient tells you how much data you can lose before the centrality rankings start to change. A coefficient above 0.5 is generally considered stable. Below that, your centrality rankings may not hold up with less data.

---

## Group Comparisons

### Building Group Models

To compare groups (e.g., high- vs. low-achieving students), use `group_atna()`:

```{r, message = F}
data("group_regulation_long")
prepared <- prepare_data(
  group_regulation_long,
  actor = "Actor",
  time = "Time",
  action = "Action" 
)

gtna <- group_atna(prepared, group = "Achiever")
```

### Plotting Group Networks

```{r, echo = -1,fig.width=8, fig.height=4}
layout(t(1:2))
plot(gtna)
```

This puts the network for each group side by side. You can visually compare which edges are thick or thin in each group. If one group has a strong Plan→Monitor connection and the other doesn't, that difference will be visible.

### Comparing Centralities Across Groups

```{r,fig.width=7,fig.height=7}
plot(centralities(gtna))
```

This makes centrality bar charts for each group. Look for states where the bars are very different across groups — those are the states whose role changes depending on group membership.

### Difference Network

```{r, fig.width=4, fig.height=4}
plot_compare(gtna)
```

This draws a single network where each edge shows the *difference* between groups. Positive edges (one color) are stronger in the first group; negative edges (another color) are stronger in the second group. It is the fastest way to see where the two groups diverge.
 

### Permutation Test

```{r, fig.width=4,fig.height=4}
perm <- permutation_test(gtna, iter = 1000)
print(perm)
plot(perm, minimum = 0.01)
```

This tests whether the observed differences between groups are bigger than what you'd expect by chance. It shuffles the group labels 1000 times, recomputes the group models each time, and compares the real difference to the shuffled differences. Edges with small p-values are where the two groups differ in a way that is unlikely to be random.

The plot shows the network of significant differences. Only edges that passed the permutation test are visible.

---

## When to Use ATNA vs. Regular TNA

| Situation | Use |
|-----------|-----|
| You care about what happens in the very next step | `tna()` |
| You think events influence other events several steps later | `atna()` |
| Conversational data where early turns shape later turns | `atna()` |
| You want to find states that connect distant parts of the sequence | `atna()` |

ATNA and TNA can give different answers about which states are most central. A state might have weak immediate transitions (low importance in TNA) but strong long-range influence (high importance in ATNA). If the two methods agree on what matters, that's good evidence the finding is solid. If they disagree, the disagreement itself is informative — it tells you the state's role depends on the time scale you're looking at.

---
title: "Frequency-based TNA"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Frequency-based TNA}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  fig.width = 6,
  fig.height = 4,
  out.width = "100%",
  dev = "png",
  dpi = 150,
  comment = "#>"
)
suppressPackageStartupMessages({
  library("tna")
  library("tibble")
  library("dplyr")
  library("ggplot2")
})
options(scipen = 99)
options(digits = 2)
options(max.print = 30)
options(width = 83)
```


# FTNA tutorial

```{r}
# Install 'tna' package from CRAN if needed (uncomment if required).
# install.packages("tna")

# Load packages
library("tna")

# Load example data provided within the 'tna' package, 
# representing group regulatory interactions
data(group_regulation)

# Run FTNA on 'group_regulation' data using raw counts of 
# transitions ("absolute" type) and print the result
model <- ftna(group_regulation)

# Print the output to inspect the model
print(model)
```

```{r, fig.align='center'}
# Calculate the Transition Network Analysis (TNA) on the group_regulation 
# data with scaled weights between 0 and 1
model_scaled <- ftna(group_regulation, scaling = "minmax")
print(model_scaled) # Print the FTNA model with scaled weights
```

## Plotting

```{r, fig.align='center'}
# Plotting the two weights together to see if the scaling distorts the data

# Combine weights from absolute and scaled models into a data frame for plotting
weights_data <- data.frame(
  Absolute = as.vector(model$weights), # Extract absolute weights as a vector
  Scaled = as.vector(model_scaled$weights) # Extract scaled weights as a vector
)
corr <- cor(weights_data$Absolute, weights_data$Scaled, method = c("pearson")) |>
  round(digits = 2)

# Create a scatter plot comparing absolute vs. scaled weights
plot_abs_scaled <- ggplot(weights_data, aes(x = Absolute, y = Scaled)) +
  geom_point(color = "steelblue", alpha = 0.6, size = 2) +  # Add points with specified aesthetics
  geom_smooth(formula = y ~ x, method = "lm", color = "red", linetype = "dashed") + # Add a linear trend line
  # geom_smooth(method = lm, formula = y ~ x, se = FALSE) +
  geom_text(x = 0.1, y = 0.9, label = paste0('r = ', corr), color = 'red')
  # stat_cor(aes(label = after_stat(r.label)), label.x = 0.1, label.y = 0.9, 
  #          size = 4, color = "black", method = "spearman") + # Display Spearman correlation
  labs(x = "Absolute Weights", y = "Scaled Weights") + # Label axes
  theme_minimal() # Apply a minimal theme for the plot

# Display the scatter plot
plot_abs_scaled
```


```{r, fig.align='center', results = FALSE, message = FALSE}
# Calculate the Transition Network Analysis (TNA) on the `group_regulation` 
# data with ranked weights
model_ranked <- ftna(group_regulation, scaling = "rank")
print(model_ranked) # Print the FTNA model with ranked weights
```


```{r, fig.align='center'}
# Combine weights from absolute and ranked models into a data frame for plotting
weights_data <- data.frame(
  Absolute = as.vector(model$weights), # Extract absolute weights as a vector
  Ranked = as.vector(model_ranked$weights) # Extract ranked weights as a vector
)

# Create a scatter plot comparing Absolute vs. Ranked weights with correlation annotations
plot_abs_ranked <- ggplot(weights_data, aes(x = Absolute, y = Ranked)) +
  geom_point(color = "steelblue", alpha = 0.6, size = 2) +  # Add points with specified aesthetics
  geom_smooth(formula = y ~ x, method = "lm", color = "red", linetype = "dashed") +  # Add a linear trend line
  # stat_cor(aes(label = paste("Spearman: ", round(after_stat(r), 2))), 
  #          method = "spearman", label.x = 0.1, label.y = 0.9, size = 4, color = "black") + # Spearman correlation annotation
  # stat_cor(aes(label = paste("Pearson: ", round(after_stat(r), 2))), 
  #          method = "pearson", label.x = 0.1, label.y = 0.8, size = 4, color = "darkgreen") + # Pearson correlation annotation
  labs(x = "Absolute Weights", y = "Ranked Weights") + # Label axes
  theme_minimal() # Apply a minimal theme for the plot

# Display the scatter plot
plot_abs_ranked
```


## Pruning

```{r, fig.show='hold', fig.width=9, fig.height=9}
layout(matrix(1:4, ncol = 2))
# Pruning with different methods
pruned_threshold <- prune(model_scaled, method = "threshold", threshold = 0.1)
pruned_lowest <- prune(model_scaled, method = "lowest", lowest = 0.15)
pruned_disparity <- prune(model_scaled, method = "disparity", alpha = 0.5)

# Plotting for comparison
plot(pruned_threshold)
plot(pruned_lowest)
plot(pruned_disparity)
plot(model_scaled)
```

## Patterns

```{r, fig.align='center', fig.width=3, fig.height=2}
# Identify 2-cliques (dyads) from the FTNA model with a weight threshold, 
# excluding loops in visualization.
# A 2-clique represents a pair of nodes that are strongly connected based on 
# the specified weight threshold.
layout(matrix(1:6, ncol = 3))
cliques_of_two <- cliques(
  model_scaled,      # The FTNA model with scaled edge weights
  size = 2,          # Looking for pairs of connected nodes (dyads)
  threshold = 0.1    # Only include edges with weights greater than 0.1
)

# Print and visualize the identified 2-cliques (dyads)
print(cliques_of_two)  # Display details of 2-cliques
plot(cliques_of_two, ask = F)   # Visualize 2-cliques in the network
```
```{r, fig.align='center'}
layout(matrix(1:6, ncol = 3))
# Identify 3-cliques (triads) from the FTNA model.
# A 3-clique is a fully connected set of three nodes, indicating a strong 
# triplet structure.
cliques_of_three <- cliques(
  model_scaled,      # The FTNA model with scaled edge weights
  size = 3,          # Looking for triplets of fully connected nodes (triads)
  threshold = 0.05   # Only include edges with weights greater than 0.05
)

# Print and visualize the identified 3-cliques (triads)
# Uncomment the code below to view the results
print(cliques_of_three) # Display details of 3-cliques
plot(cliques_of_three, ask = FALSE)  # Visualize 3-cliques in the network
```
```{r, fig.align='center', fig.width=6, fig.height=4}
layout(matrix(1:6, ncol = 3))
# Identify 4-cliques (quadruples) from the FTNA model.
# A 4-clique includes four nodes where each node is connected to every other 
# node in the group.
# Uncomment the code below to view the results
cliques_of_four <- cliques(
  model_scaled,      # The FTNA model with scaled edge weights
  size = 4,          # Looking for quadruples of fully connected nodes (4-cliques)
  threshold = 0.03   # Only include edges with weights greater than 0.03
)

# Print and visualize the identified 4-cliques (quadruples) 
# Uncomment the code below to view the results
print(cliques_of_four)  # Display details of 4-cliques
plot(cliques_of_four, ask = FALSE)   # Visualize 4-cliques in the network
```
```{r, fig.align='center'}
# Identify 5-cliques (quintuples) from the FTNA model, summing edge weights.
# Here, the sum of edge weights in both directions must meet the specified 
# threshold for inclusion.
# Uncomment the code below to view the results
cliques_of_five <- cliques(
  model_scaled,      # The FTNA model with scaled edge weights
  size = 5,          # Looking for quintuples of fully connected nodes (5-cliques)
  threshold = 0.1,   # Only edges with total bidirectional weights greater than 0.1
  sum_weights = TRUE # Sum edge weight in both directions when computing  threshold
)

# Print and visualize the identified 5-cliques (quintuples)
print(cliques_of_five)  # Display details of 5-cliques
plot(cliques_of_five, ask = FALSE)   # Visualize 5-cliques in the network
```

## Graph level measures

```{r}
summary(model_scaled)
```

```{r}
summary(pruned_disparity)
```


## Node level measures

```{r, fig.width=7, fig.height=7}
# Compute centrality measures for the FTNA model
centrality_measures <- centralities(model_scaled)

# Print the calculated centrality measures in the FTNA model
print(centrality_measures)
plot(centrality_measures)
```

```{r}
# Convert the FTNA model to an igraph object and 
# calculate HITS (Hub and Authority) scores
hits_results <- igraph::hits_scores(as.igraph(model_scaled))

# Extract the hub and authority scores from the HITS results for further analysis
hub_scores <- hits_results$hub
authority_scores <- hits_results$authority
```

```{r}
# Print the hub and authority scores to view influential nodes
print(hub_scores)
print(authority_scores)
```


## Edge level measures

```{r, fig.align='center', fig.width=5, fig.height=5}
edge_between <- betweenness_network(model_scaled)
plot(edge_between)
```

## Community detection

```{r, fig.align='center', fig.width=5, fig.height=5}
detected_communities <- communities(model_scaled)
plot(detected_communities, method = "leading_eigen")
print(detected_communities, method = "leading_eigen")
```

## Bootstrapping

```{r}
# Perform bootstrapping on the FTNA model with a fixed seed for reproducibility
set.seed(265)
boot <- bootstrap(model_scaled, threshold = 0.05)

# Print the combined results data frame containing
print(summary(boot))

# View non-significant edges  which are less likely to be stable across bootstrap samples
print(boot, type = "nonsig")
```

## Comparing Models

```{r, fig.align='center', fig.height=5, fig.width=5, layout = c(1,1)}
# Create FTNA for the high-achievers subset (rows 1 to 1000)
Hi <- ftna(group_regulation[1:1000, ], scaling = "minmax")

# Create FTNA for the low-achievers subset (rows 1001 to 2000)
Lo <- ftna(group_regulation[1001:2000, ], scaling = "minmax")

# Plot a comparison of the "Hi" and "Lo" models
# The 'minimum' parameter is set to 0.001, so edges with weights >= 0.001 are shown
plot_compare(Hi, Lo, minimum = 0.0001)

# Run a permutation test to determine statistical significance of 
# differences between "Hi" and "Lo"
# The 'it' parameter is set to 1000, meaning 1000 permutations are performed
Permutation <- permutation_test(Hi, Lo, it = 1000)

# Plot the significant differences identified in the permutation test
plot(Permutation)
```

## Centrality stability

```{r, fig.width=7, fig.height=4}
Centrality_stability <- estimate_centrality_stability(model_scaled, iter = 100)
plot(Centrality_stability)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{hist.tna}
\alias{hist.tna}
\title{Plot a Histogram of Edge Weights in the Network}
\usage{
\method{hist}{tna}(x, breaks, col = "lightblue", main, xlab, border = "white", ...)
}
\arguments{
\item{x}{a vector of values for which the histogram is desired.}

\item{breaks}{one of:
    \itemize{
      \item a vector giving the breakpoints between histogram cells,
      \item a function to compute the vector of breakpoints,
      \item a single number giving the number of cells for the histogram,
      \item a character string naming an algorithm to compute the
      number of cells (see \sQuote{Details}),
      \item a function to compute the number of cells.
    }
    In the last three cases the number is a suggestion only; as the
    breakpoints will be set to \code{\link{pretty}} values, the number
    is limited to \code{1e6} (with a warning if it was larger).  If
    \code{breaks} is a function, the \code{x} vector is supplied to it
    as the only argument (and the number of breaks is only limited by
    the amount of available memory).
  }

\item{col}{a colour to be used to fill the bars.}

\item{main}{A \code{character} string defining the title of the plot.}

\item{xlab}{A \code{character} string defining the vertical axis label.}

\item{border}{the color of the border around the bars.  The default
    is to use the standard foreground color.}

\item{...}{Additional arguments passed to \code{\link[graphics:hist]{graphics::hist()}}.}
}
\value{
A \code{histogram} object of edge weights.
}
\description{
Plot a Histogram of Edge Weights in the Network
}
\examples{
model <- tna(group_regulation)
hist(model)

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.tna_stability}
\alias{plot.tna_stability}
\title{Plot Centrality Stability Results}
\usage{
\method{plot}{tna_stability}(x, level = 0.05, ...)
}
\arguments{
\item{x}{A \code{tna_stability} object produced by \code{estimate_cs}.}

\item{level}{A \code{numeric} value representing the significance level for
the confidence intervals. Defaults to \code{0.05}.}

\item{...}{Ignored.}
}
\value{
A \code{ggplot} object displaying the stability analysis plot.
}
\description{
This function visualizes the centrality stability results produced by the
\code{estimate_centrality_stability} function. It shows how different centrality
measures' correlations change as varying proportions of cases are dropped,
along with their confidence intervals (CIs).
}
\details{
The function aggregates the results for each centrality measure across
multiple proportions of dropped cases (e.g., 0.1, 0.2, ..., 0.9) and
calculates the mean and the desired quantiles for each proportion.
The confidence intervals (CIs) are computed based on the quantiles
and displayed in the plot.

If no valid data is available for a centrality measure
(e.g., missing or NA values), the function skips that measure with a warning.

The plot includes:
\itemize{
\item The mean correlation for each centrality measure as a function of the
proportion of dropped cases.
\item Shaded confidence intervals representing CIs for each centrality measure.
\item A horizontal dashed line at the threshold value used for calculating
the CS-coefficient.
\item A subtitle listing the CS-coefficients for each centrality measure.
}
}
\examples{
model <- tna(group_regulation)
cs <- estimate_cs(model, iter = 10)
plot(cs)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groups.R
\name{rename_groups}
\alias{rename_groups}
\title{Rename Groups}
\usage{
rename_groups(x, new_names)
}
\arguments{
\item{x}{A \code{group_tna} object.}

\item{new_names}{A \code{character} vector containing one name per cluster.}
}
\value{
A renamed \code{group_tna} object.
}
\description{
Rename Groups
}
\examples{
model <- group_model(engagement_mmm)
model_renamed <- rename_groups(model, c("A", "B", "C"))

}
\seealso{
Cluster-related functions
\code{\link{communities}()},
\code{\link{group_model}()},
\code{\link{mmm_stats}()}
}
\concept{clusters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.tna_permutation}
\alias{plot.tna_permutation}
\title{Plot the Significant Differences from a Permutation Test}
\usage{
\method{plot}{tna_permutation}(x, colors, posCol = "#009900", negCol = "red", edge_labels = TRUE, ...)
}
\arguments{
\item{x}{A \code{tna_permutation} object.}

\item{colors}{See \code{\link[cograph:splot]{cograph::splot()}}.}

\item{posCol}{Color for plotting edges
the difference in edge weights is positive. See \code{\link[cograph:splot]{cograph::splot()}}.}

\item{negCol}{Color for plotting edges when
the the difference in edge weights is negative. See \code{\link[cograph:splot]{cograph::splot()}}.}

\item{edge_labels}{Boolean indicating whether edge labels should be
ploted. Defaults to \code{TRUE}}

\item{...}{Arguments passed to \code{\link[=plot_model]{plot_model()}}.}
}
\value{
A \code{cograph_network} object containing only the significant edges
according to the permutation test.
}
\description{
Plot the Significant Differences from a Permutation Test
}
\examples{
model_x <- tna(group_regulation[1:200, ])
model_y <- tna(group_regulation[1001:1200, ])
# Small number of iterations for CRAN
perm <- permutation_test(model_x, model_y, iter = 20)
plot(perm)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.tna_cliques}
\alias{plot.tna_cliques}
\title{Plot Cliques of a TNA Network}
\usage{
\method{plot}{tna_cliques}(x, n = 6, first = 1, show_loops = FALSE, colors, ask = TRUE, ...)
}
\arguments{
\item{x}{A \code{tna_cliques} object.}

\item{n}{An \code{integer} defining the maximum number of cliques to show.
The defaults is \code{6}.}

\item{first}{An \code{integer} giving the index of the first clique to show.
The default index is \code{1}.}

\item{show_loops}{A \code{logical} value indicating whether to include loops
in the plots or not.}

\item{colors}{Optional \code{character} vector of colors values to use
for the nodes.}

\item{ask}{A \code{logical} value. When \code{TRUE}, show plots one by one and asks
to plot the next plot in interactive mode.}

\item{...}{Ignored.}
}
\value{
\code{NULL} (invisibly).
}
\description{
Plot Cliques of a TNA Network
}
\examples{
model <- tna(group_regulation)
cliq <- cliques(model, size = 2)
plot(cliq, n = 1, ask = FALSE)

}
\seealso{
Clique-related functions
\code{\link{cliques}()},
\code{\link{plot.group_tna_cliques}()},
\code{\link{print.group_tna_cliques}()},
\code{\link{print.tna_cliques}()}
}
\concept{cliques}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{compare.group_tna}
\alias{compare.group_tna}
\title{Compare Grouped TNA Models with Comprehensive Metrics}
\usage{
\method{compare}{group_tna}(
  x,
  i = 1L,
  j = 2L,
  scaling = "none",
  measures = character(0),
  network = TRUE,
  ...
)
}
\arguments{
\item{x}{A \code{group_tna} object.}

\item{i}{An \code{integer} index or the name of the principal cluster as a
\code{character} string.}

\item{j}{An \code{integer} index or the name of the secondary cluster as a
\code{character} string.}

\item{scaling}{A  \code{character} string naming a scaling method to
apply to the weights before comparing them. The supported options are:
\itemize{
\item \code{"none"}: No scaling is performed. The weights are used as is.
\item \code{"minmax"}: Performs min-max normalization, i.e., the minimum value is
subtracted and the differences are scaled by the range.
\item \code{"max"}: Max-normalization: the values are divided by the maximum value.
\item \code{"rank"}: Applies min-max normalization to the ranks of the weights
(computed with \code{ties.method = "average"}).
\item \code{"zscore"}: Computes the standard score, i.e. the mean weight is
subtracted and the differences are scaled by the standard deviation.
\item \code{"robust"}: Computes the robust z-score, i.e. the median weight is
subtracted and the differences are scaled by the median absolute deviation
(using \link[stats:mad]{stats::mad}).
\item \code{"log"}: Simply the natural logarithm of the weights.
\item \code{"log1p"}: As above, but adds 1 to the values before taking the logarithm.
Useful for scenarios with zero weights.
\item \code{"softmax"}: Performs softmax normalization.
\item \code{"quantile"}: Uses the empirical quantiles of the weights
via \link[stats:ecdf]{stats::ecdf}.
}}

\item{measures}{A \code{character} vector indicating which centrality
measures should be computed. See \code{\link[=centralities]{centralities()}} for the
available measures. No measures are included by default.}

\item{network}{A \code{logical} value indicating whether network metrics should
be included in the comparison. The default is \code{TRUE}.}

\item{...}{Additional arguments passed to \code{\link[=compare.tna]{compare.tna()}}.}
}
\value{
A \code{tna_comparison} object. See \code{\link[=compare.tna]{compare.tna()}} for details.
}
\description{
Compare Grouped TNA Models with Comprehensive Metrics
}
\examples{
model <- group_model(engagement_mmm)
compare(model, i = 1, j = 2)

}
\seealso{
Model comparison functions
\code{\link{compare}()},
\code{\link{compare_sequences}()},
\code{\link{plot.tna_comparison}()},
\code{\link{plot.tna_sequence_comparison}()},
\code{\link{plot_compare}()},
\code{\link{plot_compare.group_tna}()},
\code{\link{print.tna_comparison}()},
\code{\link{print.tna_sequence_comparison}()}
}
\concept{comparison}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.group_tna}
\alias{plot.group_tna}
\title{Plot a Grouped Transition Network Analysis Model}
\usage{
\method{plot}{group_tna}(x, title, which, ...)
}
\arguments{
\item{x}{A \code{group_model} object.}

\item{title}{A title for each plot. It can be a single string (the same one
will be used for all plots) or a list (one per group)}

\item{which}{An optional \code{integer} vector of groups to plot. By default, all
groups are plotted.}

\item{...}{
  Arguments passed on to \code{\link[=plot.tna]{plot.tna}}
  \describe{
    \item{\code{node_list}}{An optional \code{list} of two \code{character} vectors that define
two mutually exclusive groups of node labels.}
    \item{\code{use_list_order}}{A \code{logical} value. If \code{node_list} is provided,
defines how the order of the nodes in the plot is defined. A \code{TRUE} value
uses the order in \code{node_list}. Otherwise, the nodes are ranked based on
edge weights and ordered according to the rank.}
    \item{\code{scale_nodes}}{A \code{character} string giving the name of a centrality
measure to scale the node size by. See \code{\link[=centralities]{centralities()}} for valid names.
If missing (the default), uses default \code{\link[cograph:splot]{cograph::splot()}} scaling.
The value of \code{node_size} provided via \code{...} is used as baseline size.}
    \item{\code{scaling_factor}}{A \code{numeric} value specifying how strongly to scale
the nodes when \code{scale_nodes} is provided. Values
between 0 and 1 will result in smaller differences and values larger
than 1 will result in greater differences. The default is \code{0.5}.}
  }}
}
\value{
\code{NULL} (invisibly).
}
\description{
Plots a transition network of each cluster using \code{cograph}.
}
\examples{
model <- group_model(engagement_mmm)
plot(model, which = 1)

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_compare}
\alias{plot_compare}
\alias{plot_compare.tna}
\title{Plot the Difference Network Between Two Models}
\usage{
plot_compare(x, ...)

\method{plot_compare}{tna}(x, y, posCol = "#009900", negCol = "red", ...)
}
\arguments{
\item{x}{A \code{tna} object. This is the the principal model.}

\item{...}{Additional arguments passed to \code{\link[cograph:splot]{cograph::splot()}}.}

\item{y}{A \code{tna} object. This is the model subtracted from the
principal model.}

\item{posCol}{Color for plotting edges and pie when
the first group has a higher value. See \code{\link[cograph:splot]{cograph::splot()}}.}

\item{negCol}{Color for plotting edges and pie when
the second group has a higher value. See \code{\link[cograph:splot]{cograph::splot()}}.}
}
\value{
A \code{cograph_network} object displaying the difference network
between the two models.
}
\description{
Plots the difference network between model \code{x} and model \code{y}. The edges are
computed from subtracting the two models. The pie chart is the difference in
initial probabilities between model \code{x} and model \code{y}. Green color indicates
that \code{x}is greater than \code{y}and red indicates otherwise.
}
\examples{
model_x <- tna(group_regulation[group_regulation[, 1] == "plan", ])
model_y <- tna(group_regulation[group_regulation[, 1] != "plan", ])
plot_compare(model_x, model_y)

}
\seealso{
Model comparison functions
\code{\link{compare}()},
\code{\link{compare.group_tna}()},
\code{\link{compare_sequences}()},
\code{\link{plot.tna_comparison}()},
\code{\link{plot.tna_sequence_comparison}()},
\code{\link{plot_compare.group_tna}()},
\code{\link{print.tna_comparison}()},
\code{\link{print.tna_sequence_comparison}()}
}
\concept{comparison}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.tna_communities}
\alias{print.tna_communities}
\title{Print Detected Communities}
\usage{
\method{print}{tna_communities}(x, ...)
}
\arguments{
\item{x}{A \code{tna_communities} object.}

\item{...}{Additional arguments passed to the generic \code{print} method.}
}
\value{
\code{x} (invisibly).
}
\description{
Print Detected Communities
}
\examples{
model <- tna(group_regulation)
comm <- communities(model)
print(comm)

}
\seealso{
Community detection functions
\code{\link{communities}()},
\code{\link{plot.group_tna_communities}()},
\code{\link{plot.tna_communities}()},
\code{\link{print.group_tna_communities}()}
}
\concept{communities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tna-package.R
\docType{data}
\name{engagement}
\alias{engagement}
\title{Example Data on Student Engagement}
\format{
A \code{stslist} object (sequence data).
}
\source{
\doi{10.1016/j.compedu.2023.104787}
}
\usage{
engagement
}
\description{
Students' engagement states (Active / Average / Disengaged)
throughout a whole study program. The data was generated synthetically
based on the article "The longitudinal association between engagement and
achievement varies by time, students' profiles, and achievement state:
A full program study"
}
\seealso{
Datasets
\code{\link{engagement_mmm}},
\code{\link{group_regulation}},
\code{\link{group_regulation_long}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.tna_comparison}
\alias{plot.tna_comparison}
\title{Plot the Comparison of Two TNA Models or Matrices}
\usage{
\method{plot}{tna_comparison}(
  x,
  type = "heatmap",
  population = "difference",
  method = "pearson",
  name_x = "x",
  name_y = "y",
  ...
)
}
\arguments{
\item{x}{A \code{tna_comparison} object.}

\item{type}{A \code{character} string naming the type of plot to produce. The
available options are \code{"heatmap"} (the default), \code{"scatterplot"},
\code{"centrality_heatmap"}, and \code{"weight_density"}.}

\item{population}{A \code{"character"} string naming the population for which
to produce the heatmaps, i.e, one of \code{"x"}, \code{"y"}, or \code{"difference"} for
the differences. Ignored for \code{type = "scatterplot"}. Defaults to \code{"diff"}.}

\item{method}{A \code{character} string naming the correlation coefficient to
use when plotting a scatterplot. The available options are \code{"pearson"}
(the default), \code{"kendall"}, \code{"spearman"}, and \code{"distance"}. The final
option is the distance correlation coefficient of
Szekely, Rizzo, and Bakirov (2007). See also the \code{energy} package for
further information on this measure.}

\item{name_x}{An optional \code{character} string to use as the name of the
first population in the plots. The default is \code{"x"}.}

\item{name_y}{An optional \code{character} string to use as the name of the
second population in the plots. The default is \code{"y"}.}

\item{...}{Ignored.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot the Comparison of Two TNA Models or Matrices
}
\examples{
model_x <- tna(group_regulation[1:200, ])
model_y <- tna(group_regulation[1001:1200, ])
comp <- compare(model_x, model_y)
plot(comp)

}
\references{
Szekely, G.J., Rizzo, M.L., and Bakirov, N.K. (2007),
Measuring and Testing Dependence by Correlation of Distances,
\emph{Annals of Statistics}, 35(6), 2769-2794.
doi:10.1214/009053607000000505
}
\seealso{
Model comparison functions
\code{\link{compare}()},
\code{\link{compare.group_tna}()},
\code{\link{compare_sequences}()},
\code{\link{plot.tna_sequence_comparison}()},
\code{\link{plot_compare}()},
\code{\link{plot_compare.group_tna}()},
\code{\link{print.tna_comparison}()},
\code{\link{print.tna_sequence_comparison}()}
}
\concept{comparison}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.tna}
\alias{print.tna}
\title{Print a \code{tna} Object}
\usage{
\method{print}{tna}(x, generic = FALSE, ...)
}
\arguments{
\item{x}{A \code{tna} object.}

\item{generic}{A \code{logical} value. If \code{TRUE}, use generic print method
instead. Defaults to \code{FALSE}.}

\item{...}{Additional arguments passed to the generic \code{print} methods.}
}
\value{
The \code{tna} object passed as argument \code{x} (invisibly).
}
\description{
Print a \code{tna} Object
}
\examples{
model <- tna(group_regulation)
print(model)

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.tna_sequence_comparison}
\alias{plot.tna_sequence_comparison}
\title{Plot a Sequence Comparison}
\usage{
\method{plot}{tna_sequence_comparison}(
  x,
  n = 10,
  legend = TRUE,
  cells = TRUE,
  text_color = "white",
  digits = 2L,
  ...
)
}
\arguments{
\item{x}{A \code{tna_sequence_comparison} object.}

\item{n}{An \code{integer} giving the number of patterns to plot.
The default is \code{10}.}

\item{legend}{A \code{logical} value indicating whether to show the color scale
legend. The default is \code{TRUE}.}

\item{cells}{A \code{logical} value indicating whether to display the
numeric values in each cell. The default is \code{TRUE}.}

\item{text_color}{A \code{character} string specifying the text color to use for
the cell values. The default is \code{"white"}.}

\item{digits}{An \code{integer} specifying the number of digits for the cell
values.}

\item{...}{Not used.}
}
\value{
A \code{ggplot} object.
}
\description{
Visualize the differences in pattern counts by plotting the standardized
residuals by pattern and group.
}
\examples{
\donttest{
idx <- c(1:500, 1001:1500)
group <- c(rep("High", 500), rep("Low", 500))
comp <- compare_sequences(group_regulation[idx, ], group)
plot(comp)
}

}
\seealso{
Model comparison functions
\code{\link{compare}()},
\code{\link{compare.group_tna}()},
\code{\link{compare_sequences}()},
\code{\link{plot.tna_comparison}()},
\code{\link{plot_compare}()},
\code{\link{plot_compare.group_tna}()},
\code{\link{print.tna_comparison}()},
\code{\link{print.tna_sequence_comparison}()}
}
\concept{comparison}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centralities.R
\name{estimate_cs}
\alias{estimate_cs}
\alias{estimate_centrality_stability}
\alias{estimate_cs.tna}
\alias{estimate_centrality_stability.tna}
\alias{estimate_cs.group_tna}
\alias{estimate_centrality_stability.group_tna}
\title{Estimate Centrality Stability}
\usage{
estimate_cs(
  x,
  loops,
  normalize,
  invert,
  measures,
  iter,
  method,
  drop_prop,
  threshold,
  certainty,
  progressbar = NULL,
  detailed = NULL
)

estimate_centrality_stability(
  x,
  loops,
  normalize,
  invert,
  measures,
  iter,
  method,
  drop_prop,
  threshold,
  certainty,
  progressbar = NULL,
  detailed = NULL
)

\method{estimate_cs}{tna}(
  x,
  loops = FALSE,
  normalize = FALSE,
  invert = TRUE,
  measures = c("InStrength", "OutStrength", "Betweenness"),
  iter = 1000,
  method = "pearson",
  drop_prop = seq(0.1, 0.9, by = 0.1),
  threshold = 0.7,
  certainty = 0.95,
  progressbar = FALSE,
  detailed = NULL
)

\method{estimate_centrality_stability}{tna}(
  x,
  loops = FALSE,
  normalize = FALSE,
  invert = TRUE,
  measures = c("InStrength", "OutStrength", "Betweenness"),
  iter = 1000,
  method = "pearson",
  drop_prop = seq(0.1, 0.9, by = 0.1),
  threshold = 0.7,
  certainty = 0.95,
  progressbar = FALSE,
  detailed = NULL
)

\method{estimate_cs}{group_tna}(
  x,
  loops = FALSE,
  normalize = FALSE,
  invert = TRUE,
  measures = c("InStrength", "OutStrength", "Betweenness"),
  iter = 1000,
  method = "pearson",
  drop_prop = seq(0.1, 0.9, by = 0.1),
  threshold = 0.7,
  certainty = 0.95,
  progressbar = FALSE,
  detailed = NULL
)

\method{estimate_centrality_stability}{group_tna}(
  x,
  loops = FALSE,
  normalize = FALSE,
  invert = TRUE,
  measures = c("InStrength", "OutStrength", "Betweenness"),
  iter = 1000,
  method = "pearson",
  drop_prop = seq(0.1, 0.9, by = 0.1),
  threshold = 0.7,
  certainty = 0.95,
  progressbar = FALSE,
  detailed = NULL
)
}
\arguments{
\item{x}{A \code{tna} or a \code{group_tna} object representing the temporal network
analysis data. The object should be created from a sequence data object.}

\item{loops}{A \code{logical} value indicating whether to include loops in the
network when computing the centrality measures. The default is \code{FALSE}.}

\item{normalize}{A \code{logical} value indicating whether to normalize
the centrality measures. The default is \code{FALSE}.}

\item{invert}{A \code{logical} value indicating whether the weights should be
inverted for distance-based measures. The default is \code{TRUE}.}

\item{measures}{A \code{character} vector of centrality measures to estimate.
The default measures are \code{"InStrength"}, \code{"OutStrength"},
and \code{"Betweenness"}. See \code{\link[=centralities]{centralities()}} for a list of available
centrality measures.}

\item{iter}{An \code{integer} specifying the number of resamples to draw.
The default is 1000.}

\item{method}{A \code{character} string indicating the correlation coefficient
type. The default is \code{"pearson"}. See \code{\link[stats:cor]{stats::cor()}} for details.}

\item{drop_prop}{A \code{numeric} vector specifying the proportions of
cases to drop in each sampling iteration. Default is a sequence from 0.1
to 0.9 in increments of 0.1.}

\item{threshold}{A \code{numeric} value specifying the correlation threshold for
calculating the CS-coefficient. The default is 0.7.}

\item{certainty}{A \code{numeric} value specifying the desired level of certainty
for the CS-coefficient. Default is 0.95.}

\item{progressbar}{A \code{logical} value. If \code{TRUE}, a progress bar is displayed
Defaults to \code{FALSE}}

\item{detailed}{Deprecated. This argument is ignored and will be removed in
a future version.}
}
\value{
A \code{tna_stability} object which is a \code{list} with an element for each
\code{measure} with the following elements:
\itemize{
\item \code{cs_coefficient}: The centrality stability (CS) coefficient
of the measure.
\item \code{correlations}: A \code{matrix} of correlations between the original
centrality and the resampled centralities for each drop proportion.
}

If \code{x} is a \code{group_tna} object, a \code{group_tna_stability} object is returned
instead, which is a \code{list} of \code{tna_stability} objects.
}
\description{
Estimates the stability of centrality measures in a network
using subset sampling without replacement. It allows for dropping varying
proportions of cases and calculates correlations between the original
centralities and those computed using sampled subsets.
}
\details{
The function works by repeatedly resampling the data, dropping varying
proportions of cases, and calculating centrality measures on the subsets.
The correlation between the original centralities and the resampled
centralities is calculated for each drop proportion. The stability of each
centrality measure is then summarized using a centrality stability (CS)
coefficient, which represents the proportion of dropped cases at which
the correlations drop below a given threshold (default 0.7).

The results can be visualized by plotting the output object showing the
stability of the centrality measures across different drop proportions,
along with confidence intervals. The CS-coefficients are displayed in the
subtitle.
}
\examples{
\donttest{
model <- tna(group_regulation)
# Small number of iterations and drop proportions for CRAN
estimate_cs(
  model,
  drop_prop = seq(0.3, 0.9, by = 0.2),
  measures = c("InStrength", "OutStrength"),
  iter = 10
)
}

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pruning.R
\name{prune}
\alias{prune}
\alias{prune.tna}
\alias{prune.group_tna}
\title{Prune a Transition Network based on Transition Probabilities}
\usage{
prune(x, ...)

\method{prune}{tna}(
  x,
  method = "threshold",
  threshold = 0.1,
  lowest = 0.05,
  level = 0.5,
  boot = NULL,
  ...
)

\method{prune}{group_tna}(x, ...)
}
\arguments{
\item{x}{An object of class \code{tna} or \code{group_tna}}

\item{...}{Arguments passed to \code{\link[=bootstrap]{bootstrap()}} when
using \code{method = "bootstrap"} and when a \code{tna_bootstrap} is not supplied.}

\item{method}{A \code{character} string describing the pruning method.
The available options are \code{"threshold"}, \code{"lowest"}, \code{"bootstrap"} and
\code{"disparity"}, corresponding to the methods listed in Details. The default
is \code{"threshold"}.}

\item{threshold}{A numeric value specifying the edge weight threshold.
Edges with weights below or equal to this threshold will be considered for
removal.}

\item{lowest}{A \code{numeric} value specifying the lowest percentage
of non-zero edges. This percentage of edges with the lowest weights will be
considered for removal. The default is \code{0.05}.}

\item{level}{A \code{numeric} value representing the significance level for the
disparity filter. Defaults to \code{0.5}.}

\item{boot}{A \code{tna_bootstrap} object to be used for pruning with method
\code{"boot"}. The method argument is ignored if this argument is supplied.}
}
\value{
A pruned \code{tna} or \code{group_tna} object. Details on the pruning can be
viewed with \code{\link[=pruning_details]{pruning_details()}}. The original model can be restored with
\code{\link[=deprune]{deprune()}}.
}
\description{
Prunes a network represented by a \code{tna} object by removing
edges based on a specified threshold, lowest percent of non-zero edge
weights, or the disparity filter algorithm (Serrano et al., 2009).
It ensures the network remains weakly connected.

Prunes a network represented by a \code{tna} object by removing
edges based on a specified threshold, lowest percent of non-zero edge
weights, or the disparity filter algorithm (Serrano et al., 2009).
It ensures the network remains weakly connected.
}
\examples{
model <- tna(group_regulation)
pruned_threshold <- prune(model, method = "threshold", threshold = 0.1)
pruned_percentile <- prune(model, method = "lowest", lowest = 0.05)
pruned_disparity <- prune(model, method = "disparity", level = 0.5)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}

Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{as.igraph.matrix}
\alias{as.igraph.matrix}
\title{Coerce a Weight Matrix into an \code{igraph} Object.}
\usage{
\method{as.igraph}{matrix}(x, mode = "directed", ...)
}
\arguments{
\item{x}{A \code{matrix} of edge weights.}

\item{mode}{Character scalar, specifies how igraph should interpret the
supplied matrix. See also the \code{weighted} argument, the interpretation
depends on that too. Possible values are: \code{directed},
\code{undirected}, \code{upper}, \code{lower}, \code{max}, \code{min},
\code{plus}. See details below.}

\item{...}{Ignored.}
}
\value{
An \code{igraph} object.
}
\description{
Coerce a Weight Matrix into an \code{igraph} Object.
}
\seealso{
Helper functions
\code{\link{as.igraph.group_tna}()},
\code{\link{as.igraph.tna}()}
}
\concept{helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_model}
\alias{plot_model}
\title{Plot a Transition Network Model from a Matrix of Edge Weights}
\usage{
plot_model(x, labels, colors, ...)
}
\arguments{
\item{x}{A square \code{matrix} of edge weights.}

\item{labels}{Optional \code{character} vector of node labels.}

\item{colors}{An optional \code{character} vector of node colors to use.}

\item{...}{Additional arguments passed to \code{\link[cograph:splot]{cograph::splot()}} or
\code{\link[cograph:plot_htna]{cograph::plot_htna()}}.}
}
\value{
See \code{\link[=plot.tna]{plot.tna()}}.
}
\description{
Plot a Transition Network Model from a Matrix of Edge Weights
}
\examples{
m <- matrix(rexp(25), 5, 5)
plot_model(m)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_associations}
\alias{plot_associations}
\alias{plot_associations.tna}
\title{Plot an Association Network}
\usage{
plot_associations(x, ...)

\method{plot_associations}{tna}(x, edge_color, ...)
}
\arguments{
\item{x}{A \code{tna} object.}

\item{...}{Additional arguments passed to \code{\link[=plot_model]{plot_model()}}.}

\item{edge_color}{An optional \code{character} vector of colors for the edges.
By default, the colors are specified by the magnitude of the
standardized residual.}
}
\value{
A \code{cograph_network} object.
}
\description{
Plot an Association Network
}
\examples{
model <- ftna(group_regulation)
plot_associations(model)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{simulate.tna}
\alias{simulate.tna}
\title{Simulate Data from a Transition Network Analysis Model}
\usage{
\method{simulate}{tna}(
  object,
  nsim = 1,
  seed = NULL,
  max_len = 100L,
  na_range = c(0L, 0L),
  zero_row = "self",
  format = "wide",
  ...
)
}
\arguments{
\item{object}{A \code{tna} object. The edge weights must be transition
probabilities or frequencies, i.e., the model must have
\code{type = "relative"} or \code{type = "frequency"}.}

\item{nsim}{An \code{integer} giving the number of sequences to simulate.
The default is 1.}

\item{seed}{an object specifying if and how the random number
    generator should be initialized (\sQuote{seeded}).\cr
    For the \code{"lm"} method, either \code{NULL} or an integer that will be
    used in a call to \code{set.seed} before simulating the response
    vectors.  If set, the value is saved as the \code{"seed"} attribute
    of the returned value.  The default, \code{NULL} will not change the
    random generator state, and return \code{\link{.Random.seed}} as the
    \code{"seed"} attribute, see \sQuote{Value}.
  }

\item{max_len}{An \code{integer} giving the maximum length of the simulated
sequences. When no missing values are generated, this is the length of
all simulated sequences.}

\item{na_range}{An \code{integer} vector of length 2 giving the minimum and
maximum number of missing values to generate for each sequence. The number
of missing values is drawn uniformly from this range. If both values are
zero (the default), no missing values are generated.}

\item{zero_row}{A \code{character} string describing how to process zero rows
in the weight matrix. The option \code{"self"} (the default) assigns
probability 1 to the corresponding state (self loop) and option
\code{"uniform"} assigns a uniform distribution.}

\item{format}{A \code{character} string indicating whether the data should be
returned in \code{wide} or \code{long} format.}

\item{...}{Ignored.}
}
\value{
A \code{data.frame} of the simulated sequence data.
}
\description{
Simulate Data from a Transition Network Analysis Model
}
\examples{
model <- tna(group_regulation)
sim <- simulate(model, nsim = 10, max_len = 10)

}
\seealso{
Other data: 
\code{\link{import_data}()},
\code{\link{import_onehot}()},
\code{\link{list_random_state_pools}()},
\code{\link{prepare_data}()},
\code{\link{print.tna_data}()},
\code{\link{random_group_tna}()},
\code{\link{random_tna}()},
\code{\link{random_tna_mmm}()},
\code{\link{simulate.group_tna}()}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cliques.R
\name{cliques}
\alias{cliques}
\alias{cliques.tna}
\alias{cliques.group_tna}
\title{Identify Cliques in a Transition Network}
\usage{
cliques(x, ...)

\method{cliques}{tna}(x, size = 2, threshold = 0, sum_weights = FALSE, ...)

\method{cliques}{group_tna}(x, size = 2, threshold = 0, sum_weights = FALSE, ...)
}
\arguments{
\item{x}{A \code{tna} or a \code{group_tna} object.}

\item{...}{Ignored.}

\item{size}{An \code{integer} specifying the size of the cliques to identify.
Defaults to \code{2} (dyads).}

\item{threshold}{A \code{numeric} value that sets the minimum edge weight
for an edge to be considered in the clique. Edges below this value
are ignored. Defaults to \code{0}.}

\item{sum_weights}{A \code{logical} value specifying whether the sum of the
weights should be above the \code{threshold} instead of individual weights of the
directed edges. Defaults to \code{FALSE}.}
}
\value{
A \code{tna_cliques} object which is a \code{list} of two elements:
\itemize{
\item \code{weights} is a \code{matrix} of the edge weights in the clique.
\item \code{inits} is a \code{numeric} vector of initial weights for the clique.
}

If \code{x} is a \code{group_tna} object, a \code{group_tna_cliques} object is returned
instead, which is a \code{list} or \code{tna_cliques} objects.
}
\description{
This function identifies cliques of a specified size in a transition network.
It searches for cliques, i.e., complete subgraphs where every pair of nodes
is connected, of size \code{n} in the transition matrix for the specified cluster
in the \code{tna} object.
}
\examples{
model <- tna(group_regulation)

# Find  2-cliques (dyads)
cliq <- cliques(model, size = 2)

model <- group_tna(engagement_mmm)
cliques(model)

}
\seealso{
Clique-related functions
\code{\link{plot.group_tna_cliques}()},
\code{\link{plot.tna_cliques}()},
\code{\link{print.group_tna_cliques}()},
\code{\link{print.tna_cliques}()}
}
\concept{cliques}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permutation.R
\name{permutation_test}
\alias{permutation_test}
\alias{permutation_test.tna}
\title{Compare Two Networks from Sequence Data using a Permutation Test}
\usage{
permutation_test(x, ...)

\method{permutation_test}{tna}(
  x,
  y,
  adjust = "none",
  iter = 1000,
  paired = FALSE,
  level = 0.05,
  measures = character(0),
  ...
)
}
\arguments{
\item{x}{A \code{tna} object containing sequence data for the first \code{tna} model.}

\item{...}{Additional arguments passed to \code{\link[=centralities]{centralities()}}.}

\item{y}{A \code{tna} object containing sequence data for the second \code{tna} model.}

\item{adjust}{A \code{character} string for the method to adjust p-values with
for multiple comparisons. The default is \code{"none"} for no adjustment.
See the \code{method} argument of \code{\link[stats:p.adjust]{stats::p.adjust()}} for details and available
adjustment methods.}

\item{iter}{An \code{integer} giving the number of permutations to perform.
The default is 1000.}

\item{paired}{A \code{logical} value. If \code{TRUE}, perform paired permutation tests;
if \code{FALSE}, perform unpaired tests. The default is \code{FALSE}.}

\item{level}{A \code{numeric} value giving the significance level for the
permutation tests. The default is 0.05.}

\item{measures}{A \code{character} vector of centrality measures to test.
See \code{\link[=centralities]{centralities()}} for a list of available centrality measures.}
}
\value{
A \code{tna_permutation} object which is a \code{list} with two elements:
\code{edges} and \code{centralities}, both containing the following elements:
\itemize{
\item \code{stats}: A \code{data.frame} of original differences, effect sizes, and
estimated p-values for each edge or centrality measure. The effect size
is computed as the observed difference divided by the standard deviation
of the differences of the permuted samples.
\item \code{diffs_true}: A \code{matrix} of differences in the data.
\item \code{diffs_sig}: A \code{matrix} showing the significant differences.
}
}
\description{
This function compares two networks built from sequence data using
permutation tests. The function builds Markov models for two sequence
objects, computes the transition probabilities, and compares them by
performing permutation tests. It returns the differences in transition
probabilities, effect sizes, estimated p-values, and confidence intervals.
}
\examples{
model_x <- tna(group_regulation[1:200, ])
model_y <- tna(group_regulation[1001:1200, ])
# Small number of iterations for CRAN
permutation_test(model_x, model_y, iter = 20)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{import_onehot}
\alias{import_onehot}
\title{Import One-Hot Data}
\usage{
import_onehot(
  data,
  cols,
  actor,
  session,
  interval,
  window_size = 1L,
  window_type = "tumbling",
  aggregate = FALSE
)
}
\arguments{
\item{data}{A \code{data.frame} in wide format.}

\item{cols}{An \code{expression} giving a tidy selection of columns to be
considered as one-hot data.}

\item{actor}{An optional \code{character} string giving the column name of
\code{data} containing the actor identifiers.}

\item{session}{An optional \code{character} string giving the column name of
\code{data} containing the session identifiers.}

\item{interval}{An \code{integer} that defines how many windows can appear
at most per one row of the output data. If not provided (the default),
all windows are concatenated per actor/session combination.}

\item{window_size}{An \code{integer} specifying the window size for grouping.}

\item{window_type}{A \code{character} string. Either \code{"tumbling"} (the default)
for non-overlapping windows or \code{"sliding"} for one-step sliding window.}

\item{aggregate}{A \code{logical} value that determines how multiple occurrences
of the same event within a window are processed. Option \code{TRUE} aggregates
multiple occurrences into a single occurrence. Option \code{FALSE} keeps
all occurrences (the default).}
}
\value{
The processed data as a \code{data.frame}.
}
\description{
Import One-Hot Data
}
\examples{
d <- data.frame(
  actor = gl(100, 5),
  session = gl(10, 50),
  feature1 = rbinom(500, 1, prob = 0.33),
  feature2 = rbinom(500, 1, prob = 0.25),
  feature3 = rbinom(500, 1, prob = 0.50)
)
onehot1 <- import_onehot(d, feature1:feature3)
onehot2 <- import_onehot(d, feature1:feature3, "actor", "session")

}
\seealso{
Other data: 
\code{\link{import_data}()},
\code{\link{list_random_state_pools}()},
\code{\link{prepare_data}()},
\code{\link{print.tna_data}()},
\code{\link{random_group_tna}()},
\code{\link{random_tna}()},
\code{\link{random_tna_mmm}()},
\code{\link{simulate.group_tna}()},
\code{\link{simulate.tna}()}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random.R
\name{random_group_tna}
\alias{random_group_tna}
\title{Build a Random Group Transition Network Analysis Model}
\usage{
random_group_tna(
  n_groups = NULL,
  group_names = NULL,
  n_states = NULL,
  states = NULL,
  category = NULL,
  alpha = NULL,
  diag_boost = NULL,
  n_sequences = NULL,
  seq_length = NULL,
  type = "relative",
  per_group = NULL,
  seed = NULL
)
}
\arguments{
\item{n_groups}{An \code{integer} giving the number of groups. If \code{NULL}
(the default), a value is drawn from \code{2:4} on each call.}

\item{group_names}{An optional \code{character} vector of group names of
length \code{n_groups}. Defaults to \code{"Group 1"}, \code{"Group 2"}, ...}

\item{n_states}{An \code{integer} >= 2 giving the number of states. If \code{NULL}
(the default), a value is drawn from \code{7:11} on each call.}

\item{states}{An optional \code{character} vector of state labels of length
at least \code{n_states}. The first \code{n_states} are used. If \code{NULL} (the
default), labels are taken from \code{category} or auto-picked from a
built-in pool that fits \code{n_states}.}

\item{category}{An optional \code{character} string naming a built-in label
pool. Available pools are returned by \code{list_random_state_pools()}.
When \code{NULL} (the default), a pool whose size is at least \code{n_states}
is sampled at random. Ignored when \code{states} is supplied.}

\item{alpha}{A positive \code{numeric} Dirichlet concentration parameter.
Small values (e.g. \code{0.3}) produce sparse, peaked transitions; large
values (e.g. \code{5}) produce near-uniform transitions. If \code{NULL} (the
default), a value is drawn from \code{Uniform(0.5, 1.0)} on each call.}

\item{diag_boost}{A non-negative \code{numeric} added to the diagonal of the
transition matrix before re-normalising rows. Larger values make
states "stickier" (more self-transitions). If \code{NULL} (the default),
a value is drawn from \code{Uniform(1.5, 3.0)} on each call.}

\item{n_sequences}{An \code{integer} giving the number of sequences to
simulate from the true parameters. If \code{NULL} (the default), a value
is drawn from \code{500:800} on each call.}

\item{seq_length}{An \code{integer} giving the length of each simulated
sequence. If \code{NULL} (the default), a value is drawn from \code{6:20} on
each call.}

\item{type}{A \code{character} string giving the model type. One of
\code{"relative"} (the default), \code{"frequency"}, \code{"co-occurrence"},
\code{"attention"}.}

\item{per_group}{An optional \code{list} of length \code{n_groups}. Each element is
itself a \code{list} of overrides applied to that group only. Recognised
override names: \code{alpha}, \code{diag_boost}, \code{trans_matrix}, \code{init_probs},
\code{n_sequences}, \code{seq_length}. Unset entries fall back to the top-level
defaults (which may themselves be drawn at random when \code{NULL}).}

\item{seed}{An \code{integer} random seed for reproducibility, or \code{NULL}
(the default) for fresh randomness on every call.}
}
\value{
A \code{group_tna} object.
}
\description{
Construct a fully-functional \code{group_tna} object from synthetic parameters.
Each group receives its own randomly drawn transition matrix and initial
probabilities over a shared alphabet, so groups have heterogeneous
dynamics by default. Per-group overrides allow custom group sizes,
sparsity, stickiness, or hand-supplied transition matrices.
}
\examples{
# Fresh random group model on every call
model <- random_group_tna()

# Explicit two-group engagement demo with per-group differences
model <- random_group_tna(
  n_groups  = 2,
  n_states  = 3,
  category  = "engagement",
  per_group = list(
    list(n_sequences = 400, diag_boost = 3),
    list(n_sequences = 100, alpha = 0.3)
  ),
  seed = 42
)

}
\seealso{
Other data: 
\code{\link{import_data}()},
\code{\link{import_onehot}()},
\code{\link{list_random_state_pools}()},
\code{\link{prepare_data}()},
\code{\link{print.tna_data}()},
\code{\link{random_tna}()},
\code{\link{random_tna_mmm}()},
\code{\link{simulate.group_tna}()},
\code{\link{simulate.tna}()}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{hist.group_tna}
\alias{hist.group_tna}
\title{Plot a Histogram of Edge Weights for a \code{group_tna} Object.}
\usage{
\method{hist}{group_tna}(x, ...)
}
\arguments{
\item{x}{A \code{group_tna} object.}

\item{...}{Additional arguments passed to \code{\link[graphics:hist]{graphics::hist()}}.}
}
\value{
A \code{list} (invisibly) of \code{histogram} objects of the edge weights of
each cluster.
}
\description{
Plot a Histogram of Edge Weights for a \code{group_tna} Object.
}
\examples{
model <- group_model(engagement_mmm)
hist(model)

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{bootstrap}
\alias{bootstrap}
\alias{bootstrap.tna}
\alias{bootstrap.group_tna}
\title{Bootstrap Transition Networks from Sequence Data}
\usage{
bootstrap(x, iter, level, method, threshold, consistency_range)

\method{bootstrap}{tna}(
  x,
  iter = 1000,
  level = 0.05,
  method = "stability",
  threshold,
  consistency_range = c(0.75, 1.25)
)

\method{bootstrap}{group_tna}(
  x,
  iter = 1000,
  level = 0.05,
  method = "stability",
  threshold,
  consistency_range = c(0.75, 1.25)
)
}
\arguments{
\item{x}{A \code{tna} or a \code{group_tna} object created from sequence data.}

\item{iter}{An \code{integer} specifying the number of bootstrap samples to
draw. Defaults to \code{1000}.}

\item{level}{A \code{numeric} value representing the significance level for
hypothesis testing and confidence intervals. Defaults to \code{0.05}.}

\item{method}{A \code{character} string. This argument defines the bootstrap
test statistic. The \code{"stability"} option (the default) compares edge weights
against a range of "consistent" values defined by \code{consistency_range}.
Weights that fall outside this range are considered insignificant. In other
words, an edge is considered significant if its value is within the range
in \code{(1 - level)} * 100\% of the bootstrap samples. The \code{"threshold"} option
instead compares the edge weights against a user-specified \code{threshold} value.}

\item{threshold}{A \code{numeric} value to compare edge weights against.
The default is the 10th percentile of the edge weights. Used only when
\code{method = "threshold"}.}

\item{consistency_range}{A \code{numeric} vector of length 2. Determines how much
the edge weights may deviate (multiplicatively) from their observed values
(below and above) before they are considered insignificant. The default is
\code{c(0.75, 1.25)} which corresponds to a symmetric 25\% deviation range. Used
only when \code{method = "stability"}.}
}
\value{
A \code{tna_bootstrap} object which is a \code{list} containing the
following elements:
\itemize{
\item \code{weights_orig}: The original edge weight \code{matrix}.
\item \code{weights_sig}: The \code{matrix} of significant transitions
(those with estimated p-values below the significance level).
\item \code{weights_mean}: The mean weight \code{matrix} from the bootstrap samples.
\item \code{weights_sd}: The standard deviation \code{matrix} from the bootstrap samples.
\item \code{cr_lower}: The lower bound \code{matrix} of the consistency range for the
edge weights.
\item \code{cr_upper}: The upper bound \code{matrix} of the consistency range for the
edge weights.
\item \code{ci_lower}: The lower bound \code{matrix} of the bootstrap confidence
intervals for the edge weights.
\item \code{ci_upper}: The upper bound \code{matrix} of the bootstrap confidence
intervals for the edge weights.
\item \code{p_values}: The \code{matrix} of estimated p-values for the edge weights.
\item \code{summary}: A \code{data.frame} summarizing the edges, their weights,
p-values, statistical significance, consistency ranges, and
confidence intervals.
}

If \code{x} is a \code{group_tna} object, the output is a \code{group_tna_bootstrap}
object, which is a \code{list} of \code{tna_bootstrap} objects.
}
\description{
Perform bootstrapping on transition networks created from
sequence data stored in a \code{tna} object. Bootstrapped estimates
of edge weights are returned with confidence intervals and significance
testing.
}
\details{
The function first computes the original edge weights for the specified
cluster from the \code{tna} object. It then performs bootstrapping by resampling
the sequence data and recalculating the edge weights for each
bootstrap sample. The mean and standard deviation of the transitions are
computed, and confidence intervals are derived. The function also estimates
p-values for each edge and identifies significant edges based on
the specified significance level. A matrix of significant edges
(those with estimated p-values below the significance level) is generated.
Additional statistics on removed edges (those not considered
significant) are provided.

All results, including the original transition matrix, bootstrapped
estimates, and summary statistics for removed edges, are returned in a
structured list.
}
\examples{
model <- tna(group_regulation)
# Small number of iterations for CRAN
bootstrap(model, iter = 10)

}
\seealso{
Validation functions
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.tna_sequence_comparison}
\alias{print.tna_sequence_comparison}
\title{Print a Comparison of Sequences}
\usage{
\method{print}{tna_sequence_comparison}(x, ...)
}
\arguments{
\item{x}{A \code{tna_sequence_comparison} object.}

\item{...}{Arguments passed to the generic \code{print} method.}
}
\value{
\code{x} (invisibly).
}
\description{
Print a Comparison of Sequences
}
\examples{
\donttest{
idx <- c(1:500, 1001:1500)
group <- c(rep("High", 500), rep("Low", 500))
comp <- compare_sequences(group_regulation[idx, ], group)
print(comp)
}

}
\seealso{
Model comparison functions
\code{\link{compare}()},
\code{\link{compare.group_tna}()},
\code{\link{compare_sequences}()},
\code{\link{plot.tna_comparison}()},
\code{\link{plot.tna_sequence_comparison}()},
\code{\link{plot_compare}()},
\code{\link{plot_compare.group_tna}()},
\code{\link{print.tna_comparison}()}
}
\concept{comparison}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pruning.R
\name{deprune}
\alias{deprune}
\alias{deprune.tna}
\alias{reprune.tna}
\alias{deprune.group_tna}
\title{Restore a Pruned Transition Network Analysis Model}
\usage{
deprune(x, ...)

\method{deprune}{tna}(x, ...)

\method{reprune}{tna}(x, ...)

\method{deprune}{group_tna}(x, ...)
}
\arguments{
\item{x}{A \code{tna} or \code{group_tna} object.}

\item{...}{Ignored.}
}
\value{
A \code{tna} or \code{group_tna} object that has not been pruned.
}
\description{
Restore a Pruned Transition Network Analysis Model
}
\examples{
model <- tna(group_regulation)
pruned_model <- prune(model, method = "threshold", threshold = 0.1)
depruned_model <- deprune(pruned_model) # restore original model

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap.R
\name{bootstrap_cliques}
\alias{bootstrap_cliques}
\alias{bootstrap_cliques.tna}
\title{Bootstrap Cliques of Transition Networks from Sequence Data}
\usage{
bootstrap_cliques(x, size, threshold, iter, level, consistency_range)

\method{bootstrap_cliques}{tna}(
  x,
  size = 2L,
  threshold = 0,
  iter = 1000,
  level = 0.05,
  consistency_range = c(0.75, 1.25)
)
}
\arguments{
\item{x}{A \code{tna} or a \code{group_tna} object.}

\item{size}{An \code{integer} specifying the size of the cliques to identify.
Defaults to \code{2} (dyads).}

\item{threshold}{A \code{numeric} value that sets the minimum edge weight
for an edge to be considered in the clique. Edges below this value
are ignored. Defaults to \code{0}.}

\item{iter}{An \code{integer} specifying the number of bootstrap samples to
draw. Defaults to \code{1000}.}

\item{level}{A \code{numeric} value representing the significance level for
hypothesis testing and confidence intervals. Defaults to \code{0.05}.}

\item{consistency_range}{A \code{numeric} vector of length 2. Determines how much
the edge weights may deviate (multiplicatively) from their observed values
(below and above) before they are considered insignificant. The default is
\code{c(0.75, 1.25)} which corresponds to a symmetric 25\% deviation range. Used
only when \code{method = "stability"}.}
}
\value{
A \code{data.frame} (also of class \code{tna_bootstrap_cliques}) with
one row per clique and the columns \code{clique}, \code{mean_weight}, \code{p_values},
\code{sig}, \code{cr_lower}, \code{cr_upper}, \code{ci_lower}, \code{ci_upper}.
}
\description{
Bootstrap the edge weights of all cliques of a given size in a \code{tna}
model, producing per-clique mean weights, p-values, confidence intervals,
and consistency-range bounds.
}
\examples{
model <- tna(group_regulation)
# Small number of iterations for CRAN
boot_cliq <- bootstrap_cliques(model, size = 2, iter = 10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_sequences}
\alias{plot_sequences}
\alias{plot_sequences.tna}
\alias{plot_sequences.tna_data}
\alias{plot_sequences.default}
\alias{plot_sequences.group_tna}
\title{Create a Sequence Index Plot or a Distribution Plot}
\usage{
plot_sequences(x, ...)

\method{plot_sequences}{tna}(
  x,
  group,
  type = "index",
  scale = "proportion",
  geom = "bar",
  include_na = FALSE,
  na_color = "white",
  sort_by,
  show_n = TRUE,
  border,
  title,
  legend_title,
  xlab,
  ylab,
  tick = 5,
  ncol = 2L,
  ...
)

\method{plot_sequences}{tna_data}(
  x,
  group,
  type = "index",
  scale = "proportion",
  geom = "bar",
  include_na = FALSE,
  colors,
  na_color = "white",
  sort_by,
  show_n = TRUE,
  border,
  title,
  legend_title,
  xlab,
  ylab,
  tick = 5,
  ncol = 2L,
  ...
)

\method{plot_sequences}{default}(
  x,
  cols = tidyselect::everything(),
  group,
  type = "index",
  scale = "proportion",
  geom = "bar",
  include_na = FALSE,
  colors,
  na_color = "white",
  sort_by,
  show_n = TRUE,
  border,
  title,
  legend_title,
  xlab,
  ylab,
  tick = 5,
  ncol = 2L,
  ...
)

\method{plot_sequences}{group_tna}(
  x,
  type = "index",
  scale = "proportion",
  geom = "bar",
  include_na = FALSE,
  na_color = "white",
  sort_by,
  show_n = TRUE,
  border,
  title,
  legend_title,
  xlab,
  ylab,
  tick = 1,
  ncol = 2L,
  ...
)
}
\arguments{
\item{x}{A \code{tna}, \code{group_tna}, \code{tna_data} or a \code{data.frame} object with
sequence data in wide format.}

\item{...}{Ignored.}

\item{group}{A vector indicating the group assignment of each
row of the data. Must have the same length as the
number of rows of \code{x}. Alternatively, a single \code{character} string giving
the column name of the data that defines the group when \code{x} is a wide
format \code{data.frame} or a \code{tna_data} object. Used for faceting the plot.}

\item{type}{A \code{character} string for the type of plot to generate. The
available options are  \code{"index"} (the default) for a sequence index plot,
and \code{"distribution"} showing the distribution of the states over time.}

\item{scale}{A \code{character} string that determines the scaling of the
vertical axis for distribution plots. The options are \code{"proportion"}
(the default) and \code{"count"} for proportions and raw counts of states,
respectively.}

\item{geom}{A \code{character} string for the type of geom to use for
distribution plots. The options are \code{"bar"} (the default) and \code{"area"}.}

\item{include_na}{A \code{logical} value for whether to include missing values
for distribution plots. The default is \code{FALSE}. If \code{TRUE}, the missing
values are converted to a new state and included in the plot.}

\item{na_color}{A \code{character} string giving the color to use for missing
values. The default is \code{"white"}.}

\item{sort_by}{An optional \code{expression} giving a tidy selection of column
names of \code{x} to sort by or \code{"everything"}.}

\item{show_n}{A \code{logical} value for whether to add the number of
observations (total or by group) to the plot title.}

\item{border}{A \code{character} string giving the color for borders. For index
plots, this is the color of borders between cells (tiles). For
distribution plot with \code{geom = "bar"}, this is the color of bar outlines.
Not applicable to \code{geom = "area"}.}

\item{title}{An optional \code{character} string providing a title for the plot.}

\item{legend_title}{An optional \code{character} string providing a title for the
legend.}

\item{xlab}{A \code{character} string giving the label for the horizontal axis.
The default is \code{"Time"}.}

\item{ylab}{A \code{character} string giving the label for the vertical axis.
The default is \code{"Sequence"} for index plots, and \code{"Proportion"} or
\code{"Count"} based on \code{scale} for distribution plots.}

\item{tick}{An \code{integer} specifying the horizontal axis label interval. The
default value \code{tick = 5} shows every 5th label. Setting this to 1 will show
every label.}

\item{ncol}{Number of columns to use for the facets. The default is 2.}

\item{colors}{A named \code{character} vector mapping states to colors, or an
unnamed \code{character} vector. If missing, a default palette is used.}

\item{cols}{An \code{expression} giving a tidy selection of column names to be
treated as time points. By default, all columns will be used.}
}
\value{
A \code{ggplot} object containing either a sequence index plot or
a state distribution plot, faceted by \code{group} when supplied.
}
\description{
Create a Sequence Index Plot or a Distribution Plot
}
\examples{
# Sequence index plot (default)
plot_sequences(
  group_regulation,
  group = rep(1:2, each = 1000),
)
# State distribution plot
plot_sequences(
  group_regulation,
  group = rep(1:2, each = 1000),
  type = "distribution",
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.R
\name{summary.tna}
\alias{summary.tna}
\title{Calculate Summary of Network Metrics for a Transition Network}
\usage{
\method{summary}{tna}(object, ...)
}
\arguments{
\item{object}{A \code{tna} object.}

\item{...}{Ignored.}
}
\value{
A named \code{list} containing the following network metrics (invisibly):
\itemize{
\item \code{node_count}: The total number of nodes.
\item \code{edge_count}: The total number of edges.
\item \code{network_Density}: The density of the network.
\item \code{mean_distance}: The mean shortest path length.
\item \code{mean_out_strength}: The mean out-strength of nodes.
\item \code{sd_out_strength}: The standard deviation of out-strength.
\item \code{mean_in_strength}: The mean in-strength of nodes.
\item \code{sd_in_strength}: The standard deviation of in-strength.
\item \code{mean_out_degree}: The mean out-degree of nodes.
\item \code{sd_out_degree}: The standard deviation of out-degree.
\item \code{centralization_out_degree}: The centralization of out-degree.
\item \code{centralization_in_degree}: The centralization of in-degree.
\item \code{reciprocity}: The reciprocity of the network.
}
}
\description{
This function calculates a variety of network metrics for a \code{tna} object.
It computes key metrics such as node and edge counts, network density,
mean distance, strength measures, degree centrality, and reciprocity.
}
\details{
The function extracts the \code{igraph} network  and
computes the following network metrics:
\itemize{
\item Node count: Total number of nodes in the network.
\item Edge count: Total number of edges in the network.
\item Network density: Proportion of possible edges that
are present in the network.
\item Mean distance: The average shortest path length between nodes.
\item Mean and standard deviation of out-strength and in-strength: Measures
of the total weight of outgoing and incoming edges for each node.
\item Mean and standard deviation of out-degree: The number of outgoing
edges from each node.
\item Centralization of out-degree and in-degree: Measures of how
centralized the network is based on the degrees of nodes.
\item Reciprocity: The proportion of edges that are reciprocated
(i.e., mutual edges between nodes).
}

A summary of the metrics is printed to the console.
}
\examples{
model <- tna(group_regulation)
summary(model)

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.tna_comparison}
\alias{print.tna_comparison}
\title{Print Comparison Results}
\usage{
\method{print}{tna_comparison}(x, ...)
}
\arguments{
\item{x}{A \code{tna_comparison} object.}

\item{...}{Additional arguments passed to the tibble \code{print} method.}
}
\value{
\code{x} (invisibly).
}
\description{
Print Comparison Results
}
\examples{
model_x <- tna(group_regulation[1:200, ])
model_y <- tna(group_regulation[1001:1200, ])
comp <- compare(model_x, model_y)
print(comp)

}
\seealso{
Model comparison functions
\code{\link{compare}()},
\code{\link{compare.group_tna}()},
\code{\link{compare_sequences}()},
\code{\link{plot.tna_comparison}()},
\code{\link{plot.tna_sequence_comparison}()},
\code{\link{plot_compare}()},
\code{\link{plot_compare.group_tna}()},
\code{\link{print.tna_sequence_comparison}()}
}
\concept{comparison}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_mosaic.tna_data}
\alias{plot_mosaic.tna_data}
\title{Plot State Frequencies as a Mosaic Between Two Groups}
\usage{
\method{plot_mosaic}{tna_data}(x, group, label = "Group", ...)
}
\arguments{
\item{x}{A \code{tna_data} object.}

\item{group}{A \code{character} string giving the column name of the (meta) data
to contrast the frequencies with or a vector of group indicators with the
the same length as the number of rows in the sequence data.}

\item{label}{An optional \code{character} string that specifies a label for the
grouping variable when \code{group} is not a column name of the data.}

\item{...}{Ignored.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot State Frequencies as a Mosaic Between Two Groups
}
\examples{
d <- data.frame(
  time = rep(1:5, rep = 4),
  group = rep(1:4, each = 5),
  event = sample(LETTERS[1:3], 20, replace = TRUE)
)
sequence_data <- prepare_data(
  d,
  time = "time",
  actor = "group",
  action = "event"
)
plot_mosaic(sequence_data, group = "group")

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pruning.R
\name{pruning_details}
\alias{pruning_details}
\alias{pruning_details.tna}
\alias{pruning_details.group_tna}
\title{Print Detailed Information on the Pruning Results}
\usage{
pruning_details(x, ...)

\method{pruning_details}{tna}(x, ...)

\method{pruning_details}{group_tna}(x, ...)
}
\arguments{
\item{x}{A \code{tna} or \code{group_tna} object.}

\item{...}{Ignored.}
}
\value{
A \code{data.frame} containing the removed edges if \code{x} is a \code{tna} object,
or a \code{list} of \code{data.frame} objects in the case of \code{group_tna} object.
}
\description{
Print Detailed Information on the Pruning Results
}
\examples{
model <- tna(group_regulation)
pruned_threshold <- prune(model, method = "threshold", threshold = 0.1)
pruning_details(pruned_threshold)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.tna_clustering}
\alias{print.tna_clustering}
\title{Print the Results of Clustering}
\usage{
\method{print}{tna_clustering}(x, ...)
}
\arguments{
\item{x}{A \code{tna_clustering} object.}

\item{...}{Additional arguments passed to the generic print method.}
}
\value{
\code{x} (invisibly).
}
\description{
Print the Results of Clustering
}
\examples{
data <- data.frame(
  T1 = c("A", "B", "A", "C", "A", "B"),
  T2 = c("B", "A", "B", "A", "C", "A"),
  T3 = c("C", "C", "A", "B", "B", "C")
)

# PAM clustering with optimal string alignment (default)
result <- cluster_sequences(data, k = 2)
print(result)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_compare.group_tna}
\alias{plot_compare.group_tna}
\title{Plot the Difference Network Between Two Groups}
\usage{
\method{plot_compare}{group_tna}(x, i = NULL, j = NULL, ...)
}
\arguments{
\item{x}{A \code{group_tna} object.}

\item{i}{An \code{integer} index or the name of the principal cluster as a
\code{character} string. When \code{NULL}, defaults are chosen automatically (see
\code{\link[cograph:plot_compare]{cograph::plot_compare()}}).}

\item{j}{An \code{integer} index or the name of the secondary cluster as a
\code{character} string. When \code{NULL}, defaults are chosen automatically (see
\code{\link[cograph:plot_compare]{cograph::plot_compare()}}).}

\item{...}{Additional arguments passed to \code{\link[cograph:plot_compare]{cograph::plot_compare()}}.}
}
\value{
A \code{cograph_network} object displaying the difference network between
the two clusters
}
\description{
Plot the Difference Network Between Two Groups
}
\examples{
model <- group_model(engagement_mmm)
plot_compare(model)

}
\seealso{
Model comparison functions
\code{\link{compare}()},
\code{\link{compare.group_tna}()},
\code{\link{compare_sequences}()},
\code{\link{plot.tna_comparison}()},
\code{\link{plot.tna_sequence_comparison}()},
\code{\link{plot_compare}()},
\code{\link{print.tna_comparison}()},
\code{\link{print.tna_sequence_comparison}()}
}
\concept{comparison}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{prepare_data}
\alias{prepare_data}
\title{Compute User Sessions from Event Data}
\usage{
prepare_data(
  data,
  actor,
  time,
  action,
  order,
  time_threshold = 900,
  custom_format = NULL,
  is_unix_time = FALSE,
  unix_time_unit = "seconds",
  unused_fn = dplyr::first
)
}
\arguments{
\item{data}{A \code{data.frame} or containing the action/event data.}

\item{actor}{A \code{character} vector or an \code{expression} that represents
a tidy selection of the names of the columns that
represent a user/actor identifiers. If not provided and neither \code{time} nor
\code{order} is specified, the entire dataset is treated as a single session.
In the case of multiple actors, a new \code{.actor} column is added that
represents the interaction of the given columns.}

\item{time}{A \code{character} string or an \code{expression} giving the name of
the column representing timestamps of the action events.}

\item{action}{A \code{character} string or an \code{expression} giving the name of
the column holding the information about the action taken.}

\item{order}{A \code{character} string or an \code{expression} giving the name of a
column with sequence numbers or non-unique orderable values that indicate
order within an \code{actor} group, if not present it will be ordered with all
the data if no \code{actor} is available, used when widening the data.
If both \code{actor} and \code{time} are specified, then the sequence order should
be specified such that it determines the order of events within \code{actor}
and each session.}

\item{time_threshold}{An \code{integer} specifying the time threshold in seconds
for creating new time-based sessions. Defaults to 900 seconds.}

\item{custom_format}{A \code{character} string giving the format used to
parse the \code{time} column.}

\item{is_unix_time}{A \code{logical} value indicating whether the \code{time} column
is in Unix time. The default is \code{FALSE}.}

\item{unix_time_unit}{A \code{character} string giving the Unix time unit when
\code{is_unix_time} is \code{TRUE}. The default is \code{"seconds"}. Valid options are
\code{"seconds"}, \code{"milliseconds"}, or \code{"microseconds"}.}

\item{unused_fn}{How to handle extra columns when pivoting to wide format.
See \code{\link[tidyr:pivot_wider]{tidyr::pivot_wider()}}. The default is to keep all columns and to
use the first value.}
}
\value{
A \code{tna_data} object, which is a \code{list} with the following elements:
\itemize{
\item \code{long_data}: The processed data in long format.
\item \code{sequence_data}: The processed data on the sequences in wide format,
with actions/events as different variables structured with sequences.
\item \code{meta_data}: Other variables from the original data in wide format.
\item \code{statistics}: A \code{list} containing summary statistics: total
sessions, total actions, unique users, time range (if applicable), and
top sessions and user by activities.
}
}
\description{
Processes a dataset to create user sessions based on time gaps,
ordering columns, or actor groupings. It supports different ways to
understand order in user behavior and provides flexibility when
widening the data.
}
\examples{
results <- prepare_data(
  group_regulation_long, actor = "Actor", time = "Time", action = "Action"
)
print(results$sequence_data)
print(results$meta_data)
print(results$statistics)

# Custom order column
data_ordered <- tibble::tibble(
   user = c("A", "A", "A", "B", "B", "C", "C", "C"),
   order = c(1, 2, 3, 1, 2, 1, 2, 3),
   action = c(
     "view", "click", "add_cart", "view",
     "checkout", "view", "click", "share"
   )
)
results_ordered <- prepare_data(
  data_ordered, actor = "user", order = "order", action = "action"
)
print(results_ordered$sequence_data)
print(results_ordered$meta_data)
print(results_ordered$statistics)

# No actor scenario leading to a single session
data_single_session <- tibble::tibble(
  action = c(
    "view", "click", "add_cart", "view",
    "checkout", "view", "click", "share"
   )
)
results_single <- prepare_data(data_single_session, action = "action")
print(results_single$sequence_data)
print(results_single$meta_data)
print(results_single$statistics)

# Multiple actors
data_multi_actor <- tibble::tibble(
  user = c("A", "A", "A", "A", "B", "B", "B", "B"),
  session = c(1, 1, 2, 2, 1, 1, 2, 2),
  action = c(
    "view", "click", "add_cart", "view",
    "checkout", "view", "click", "share"
  )
)
results_multi_actor <- prepare_data(
  data_multi_actor, actor = c("user", "session"), action = "action"
)
print(results_multi_actor$sequence_data)
print(results_multi_actor$meta_data)
print(results_multi_actor$statistics)

}
\seealso{
Other data: 
\code{\link{import_data}()},
\code{\link{import_onehot}()},
\code{\link{list_random_state_pools}()},
\code{\link{print.tna_data}()},
\code{\link{random_group_tna}()},
\code{\link{random_tna}()},
\code{\link{random_tna_mmm}()},
\code{\link{simulate.group_tna}()},
\code{\link{simulate.tna}()}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/communities.R
\name{communities}
\alias{communities}
\alias{communities.tna}
\alias{communities.group_tna}
\title{Community Detection for Transition Networks}
\usage{
communities(x, methods, gamma)

\method{communities}{tna}(x, methods, gamma = 1)

\method{communities}{group_tna}(x, methods, gamma = 1)
}
\arguments{
\item{x}{A \code{tna} or a \code{group_tna} object.}

\item{methods}{A \code{character} vector of community detection algorithms to
apply to the network. The supported options are:
\itemize{
\item \code{"walktrap"}: A community detection method using short random walks.
\item \code{"fast_greedy"}: A method based on modularity optimization.
\item \code{"label_prop"}: A method that uses label propagation.
\item \code{"infomap"}: A method that uses information flow to detect communities.
\item \code{"edge_betweenness"}: A method that uses edge betweenness to find
communities.
\item \code{"leading_eigen"}: A method using the leading eigenvector of the
modularity matrix.
\item \code{"spinglass"}: A method based on the spinglass model.
}

If not provided, all methods are applied.}

\item{gamma}{A \code{numeric} value depicting a parameter that affects the
behavior of certain algorithms like the Spin Glass method. Defaults to \code{1}.}
}
\value{
An object of class \code{tna_communities} which is a \code{list} with an
element for each cluster containing:
\itemize{
\item \code{counts}: A \code{list} with the number of communities found
by each algorithm.
\item \code{assignments}: A \code{data.frame} where each row corresponds to a
node and each column to a community detection algorithm,
with color-coded community assignments.
}

If \code{x} is a \code{group_tna} object, a \code{group_tna_communities} object is returned
instead, which is a \code{list} of \code{tna_communities} objects.
}
\description{
This function detects communities within the transition networks
(represented by the \code{tna} object).
It uses various algorithms to find communities in the graph representation
of transitions and returns a \code{list} of communities for each cluster or a
specified cluster. If multiple transition matrices exist, the function
iterates over each cluster in the \code{tna} object to find communities using
different algorithms. The function uses the \code{igraph} package to convert
the transition matrices into graphs and then applies community detection
algorithms (e.g., Walktrap, Fast Greedy, Label Propagation, Infomap,
Edge Betweenness, Leading Eigenvector, and Spin Glass).
}
\examples{
model <- tna(group_regulation)
comm <- communities(model)

}
\seealso{
Community detection functions
\code{\link{plot.group_tna_communities}()},
\code{\link{plot.tna_communities}()},
\code{\link{print.group_tna_communities}()},
\code{\link{print.tna_communities}()}

Cluster-related functions
\code{\link{group_model}()},
\code{\link{mmm_stats}()},
\code{\link{rename_groups}()}
}
\concept{clusters}
\concept{communities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.tna_communities}
\alias{plot.tna_communities}
\title{Plot Communities}
\usage{
\method{plot}{tna_communities}(x, colors, method, ...)
}
\arguments{
\item{x}{A \code{communities} object generated by the \code{find_communities} method.
Each community detection method maps nodes or points in to a specific
communities.}

\item{colors}{A \code{character} vector of color values used for visualizing
community assignments.}

\item{method}{A \code{character} string naming a community detection method to
use for coloring the plot. The default is to use the first available
method in \code{x}. See \code{\link[=communities]{communities()}} for details.}

\item{...}{Additional arguments passed to \code{\link[cograph:splot]{cograph::splot()}}.}
}
\value{
A \code{cograph_network} object in which the nodes are colored by
community.
}
\description{
This function visualizes the communities detected within a \code{tna} object
based on different community detection algorithms and their corresponding
color mappings.
}
\examples{
model <- tna(group_regulation)
comm <- communities(model)
plot(comm, method = "leading_eigen")

}
\seealso{
Community detection functions
\code{\link{communities}()},
\code{\link{plot.group_tna_communities}()},
\code{\link{print.group_tna_communities}()},
\code{\link{print.tna_communities}()}
}
\concept{communities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tna-package.R
\docType{package}
\name{tna-package}
\alias{tna-package}
\title{The \code{tna} Package.}
\description{
Provides tools for performing transition network analysis (TNA),
including functions for building TNA models, plotting transition networks,
and calculating centrality measures. The package relies on the  \code{igraph} for
centrality measure calculations.
}
\references{
Saqr M., López-Pernas S., Törmänen T., Kaliisa R., Misiejuk K., Tikka S. (2025).
Transition Network Analysis: A Novel Framework for Modeling, Visualizing,
and Identifying the Temporal Patterns of Learners and Learning Processes.
In \emph{Proceedings of the 15th International Learning Analytics and Knowledge Conference (LAK '25)}, 351-361.

Banerjee A., Chandrasekhar A., Duflo E., Jackson M. (2014).
Gossip: Identifying Central Individuals in a Social Network.
Working Paper.

Kivimaki, I., Lebichot, B., Saramaki, J., Saerens, M. (2016).
Two betweenness centrality measures based on Randomized Shortest Paths.
\emph{Scientific Reports}, 6, 19668.

Serrano, M. A., Boguna, M., Vespignani, A. (2009).
Extracting the multiscale backbone of complex weighted networks.
\emph{Proceedings of the National Academy of Sciences}, 106, 6483-6488.

Zhang, B., Horvath, S. (2005).
A general framework for weighted gene co-expression network analysis.
\emph{Statistical Applications in Genetics and Molecular Biology}, 4(1).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/sonsoleslp/tna/}
  \item \url{https://sonsoles.me/tna/}
  \item Report bugs at \url{https://github.com/sonsoleslp/tna/issues/}
}


Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()}
}
\author{
Sonsoles López-Pernas, Santtu Tikka, Mohammed Saqr
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_frequencies.group_tna}
\alias{plot_frequencies.group_tna}
\title{Plot the Frequency Distribution of States}
\usage{
\method{plot_frequencies}{group_tna}(
  x,
  label,
  colors,
  width = 0.7,
  palette = "Set2",
  show_label = TRUE,
  position = "dodge",
  hjust = 1.2,
  ...
)
}
\arguments{
\item{x}{A \code{group_tna} object.}

\item{label}{An optional \code{character} string that can be provided to specify
the grouping factor name if \code{x} was not constructed using a column name of
the original data.}

\item{colors}{A \code{character} vector of colors to be used in the plot
(one per group).}

\item{width}{A \code{numeric} value for the width of the bars.
The default is \code{0.7}.}

\item{palette}{A \code{character} string that specifies the palette to be used if
\code{colors} are not passed.}

\item{show_label}{A \code{logical} value indicating whether to show a label with
the frequency counts. Default is \code{TRUE}.}

\item{position}{Position of the bars:\code{"dodge"}, \code{"dodge2"}, \code{"fill"} or
\code{"stack"}.}

\item{hjust}{A \code{numeric} value for the horizontal adjustment of the labels.
The default is \code{1.2}.}

\item{...}{Ignored.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot the Frequency Distribution of States
}
\examples{
\donttest{
model <- group_model(engagement_mmm)
# Default
plot_frequencies(model)
# Default labels outside and custom colors
plot_frequencies(
  model,
  width = 0.9,
  hjust = -0.3,
  colors = c("#218516", "#f9c22e", "#53b3cb")
)
# Stacked with no labels
plot_frequencies(model, position = "stack", show_label = FALSE)
# Fill
plot_frequencies(model, position = "fill", hjust = 1.1)
}

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_frequencies}
\alias{plot_frequencies}
\alias{plot_frequencies.tna}
\title{Plot the Frequency Distribution of States}
\usage{
plot_frequencies(x, ...)

\method{plot_frequencies}{tna}(x, width = 0.7, hjust = 1.2, show_label = TRUE, colors, ...)
}
\arguments{
\item{x}{A \code{tna} object created from sequence data.}

\item{...}{Ignored.}

\item{width}{A \code{numeric} value for the Width of the bars. Default is 0.7,}

\item{hjust}{A \code{numeric} value for the horizontal adjustment of the labels.
Default is 1.2.}

\item{show_label}{A \code{logical} value indicating whether to show a label with
the frequency counts. Default is \code{TRUE}.}

\item{colors}{A \code{character} vector of colors to be used in the plot
(one per label) or a single color.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot the Frequency Distribution of States
}
\examples{
model <- tna(group_regulation)
plot_frequencies(model)
plot_frequencies(model, width =  0.5, colors = "pink")

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.tna_centralities}
\alias{plot.tna_centralities}
\title{Plot Centrality Measures}
\usage{
\method{plot}{tna_centralities}(
  x,
  reorder = TRUE,
  ncol = 3,
  scales = c("free_x", "fixed"),
  colors,
  labels = TRUE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{tna_centralities}.}

\item{reorder}{A \code{logical} value indicating whether to reorder the values
for each centrality in a descending order. The default is \code{TRUE}.}

\item{ncol}{Number of columns to use for the facets. The default is 3.}

\item{scales}{Either \code{"fixed"} or \code{"free_x"} (the default). If \code{"free_x"},
the horizontal axis is scaled individually in each facet. If \code{"fixed"},
the same values are used for all axes.}

\item{colors}{The colors for each node (default is the model colors
if the \code{tna} model object is passed, otherwise \code{"black"}).}

\item{labels}{A \code{logical} value indicating whether to show the centrality
numeric values. The default is \code{TRUE}.}

\item{...}{Ignored.}
}
\value{
A \code{ggplot} object displaying the lollipop charts for each centrality
measure.
}
\description{
Plots the centrality measures of a \code{tna_centralities} object as a
lollipop chart. The resulting plot includes facets for each centrality
measure, showing the values for each state. The returned plot is a
\code{ggplot2} object, so it can be easily modified and styled. See
\code{\link[=centralities]{centralities()}} for details on the centrality measures.
}
\examples{
tna_model <- tna(group_regulation)
cm <- centralities(tna_model)
plot(cm, ncol = 3, reorder = TRUE)

}
\seealso{
Centrality measure functions
\code{\link{betweenness_network}()},
\code{\link{centralities}()},
\code{\link{plot.group_tna_centralities}()},
\code{\link{print.group_tna_centralities}()},
\code{\link{print.tna_centralities}()}
}
\concept{centralities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tna-package.R
\docType{data}
\name{group_regulation}
\alias{group_regulation}
\title{Example Wide Data on Group Regulation}
\format{
A \code{data.frame} object.
}
\source{
The data was generated synthetically.
}
\usage{
group_regulation
}
\description{
Students' regulation during collaborative learning. Students' interactions
were coded as:  "adapt", "cohesion", "consensus", "coregulate", "discuss",
"emotion", "monitor", "plan", "synthesis"
}
\seealso{
Datasets
\code{\link{engagement}},
\code{\link{engagement_mmm}},
\code{\link{group_regulation_long}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusters.R
\name{cluster_data}
\alias{cluster_data}
\alias{cluster_sequences}
\title{Clustering via Dissimilarity Matrix based on String Distances}
\usage{
cluster_data(
  data,
  k,
  dissimilarity = "hamming",
  method = "pam",
  na_syms = c("*", "\%"),
  weighted = FALSE,
  lambda = 1,
  ...
)

cluster_sequences(
  data,
  k,
  dissimilarity = "hamming",
  method = "pam",
  na_syms = c("*", "\%"),
  weighted = FALSE,
  lambda = 1,
  ...
)
}
\arguments{
\item{data}{A \code{data.frame}, a \code{matrix} in wide format, or a \code{tna_data}
object (in which case, the sequence data is extracted automatically).}

\item{k}{An \code{integer} giving the number of clusters.}

\item{dissimilarity}{A \code{character} string specifying the
dissimilarity measure. The available options are: \code{"osa"}, \code{"lv"}, \code{"dl"},
\code{"hamming"}, \code{"lcs"}, \code{"qgram"}, \code{"cosine"}, \code{"jaccard"}, and \code{"jw"}.
See \link[stringdist:stringdist-metrics]{stringdist::stringdist-metrics} for more information on
these measures.}

\item{method}{A \code{character} string specifying clustering method.
The available methods are \code{"pam"}, \code{"ward.D"}, \code{"ward.D2"},
\code{"complete"},\code{"average"}, \code{"single"}, \code{"mcquitty"}, \code{"median"}, and
\code{"centroid"}. See \code{\link[cluster:pam]{cluster::pam()}} and \code{\link[stats:hclust]{stats::hclust()}} for more
information on these methods.}

\item{na_syms}{A \code{character} vector of symbols or factor levels to convert
to explicit missing values.}

\item{weighted}{A \code{logical} value indicating whether the dissimilarity
measure should be weighted (the default is \code{FALSE} for no weighting).
If \code{TRUE}, earlier observations of the sequences receive a greater weight
in the distance calculation with an exponential decay. Currently only
supported for the Hamming distance.}

\item{lambda}{A \code{numeric} value defining the strength of the decay when
\code{weighted = TRUE}. The default is \code{1.0}.}

\item{...}{Additional arguments passed to \code{\link[stringdist:stringdist]{stringdist::stringdist()}}.}
}
\value{
A \code{tna_clustering} object which is a  \code{list} containing:
\itemize{
\item \code{data}: The original data.
\item \code{k}: The number of clusters.
\item \code{assignments}: An \code{integer} vector of cluster assignments.
\item \code{silhouette}: Silhouette score measuring clustering quality.
\item \code{sizes}: An \code{integer} vector of cluster sizes.
\item \code{method}: The clustering method used.
\item \code{distance}: The distance matrix.
}
}
\description{
Performs clustering using specified dissimilarity measures
and clustering methods. The rows of the data are first converted to strings
and compared using the dissimilarity measures available in the
\code{stringdist} package.
}
\examples{
data <- data.frame(
  T1 = c("A", "B", "A", "C", "A", "B"),
  T2 = c("B", "A", "B", "A", "C", "A"),
  T3 = c("C", "C", "A", "B", "B", "C")
)

# PAM clustering with optimal string alignment (default)
result <- cluster_sequences(data, k = 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{simulate}
\alias{simulate}
\title{Simulate Method (Re-export of \link[stats:simulate]{stats::simulate})}
\usage{
simulate(object, nsim = 1, seed = NULL, ...)
}
\arguments{
\item{object}{A statistical model object. The \code{tna} package provides
methods for objects of class \code{tna} and \code{group_tna}.}

\item{nsim}{An \code{integer} giving the number of sequences to simulate.}

\item{seed}{An \code{integer} random seed for reproducibility, or \code{NULL}.}

\item{...}{Further arguments passed to the dispatched method.}
}
\value{
The value returned by the dispatched method (a \code{data.frame} of
simulated sequences for \code{tna} and \code{group_tna} objects). See the
method-specific help pages for details.
}
\description{
The \code{\link[stats:simulate]{stats::simulate()}} generic is re-exported so that the
\code{simulate.tna()} and \code{simulate.group_tna()} methods provided by this
package are dispatched correctly when the user only loads \code{tna}.
See the method-specific help pages for argument and return details.
}
\examples{
model <- tna(group_regulation)
sim <- simulate(model, nsim = 5, max_len = 10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tna-package.R
\docType{data}
\name{group_regulation_long}
\alias{group_regulation_long}
\title{Example Long Data on Group Regulation}
\format{
A \code{data.frame} object.
}
\source{
The data was generated synthetically from \code{group_regulation}
}
\usage{
group_regulation_long
}
\description{
Students' regulation during collaborative learning. This is the same dataset
as \code{group_regulation} but in long format. In addition to students'
actions (\code{Action}), it contains the student identifier (\code{Actor}),
timestamp (\code{Time}), \code{Course} name, and collaboration \code{Group}. It
also includes a column (\code{Achiever}) indicating whether the student
is a high or low achiever.
}
\seealso{
Datasets
\code{\link{engagement}},
\code{\link{engagement_mmm}},
\code{\link{group_regulation}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random.R
\name{list_random_state_pools}
\alias{list_random_state_pools}
\title{List Built-in Label Pools for \code{\link[=random_tna]{random_tna()}}}
\usage{
list_random_state_pools()
}
\value{
A named \code{integer} vector mapping pool names to pool sizes.
}
\description{
Return the names of the curated state-label pools available to
\code{random_tna()} and \code{random_group_tna()}, with their sizes.
}
\examples{
list_random_state_pools()

}
\seealso{
Other data: 
\code{\link{import_data}()},
\code{\link{import_onehot}()},
\code{\link{prepare_data}()},
\code{\link{print.tna_data}()},
\code{\link{random_group_tna}()},
\code{\link{random_tna}()},
\code{\link{random_tna_mmm}()},
\code{\link{simulate.group_tna}()},
\code{\link{simulate.tna}()}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.group_tna_centralities}
\alias{print.group_tna_centralities}
\title{Print Centrality Measures}
\usage{
\method{print}{group_tna_centralities}(x, ...)
}
\arguments{
\item{x}{A \code{group_tna_centralities} object.}

\item{...}{Ignored.}
}
\value{
\code{x} (invisibly).
}
\description{
Print Centrality Measures
}
\examples{
model <- group_model(engagement_mmm)
cm <- centralities(model)
print(cm)

}
\seealso{
Centrality measure functions
\code{\link{betweenness_network}()},
\code{\link{centralities}()},
\code{\link{plot.group_tna_centralities}()},
\code{\link{plot.tna_centralities}()},
\code{\link{print.tna_centralities}()}
}
\concept{centralities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.group_tna}
\alias{print.group_tna}
\title{Print a \code{group_tna} Object}
\usage{
\method{print}{group_tna}(x, ...)
}
\arguments{
\item{x}{A \code{group_tna} object.}

\item{...}{Arguments passed to \code{\link[=print.tna]{print.tna()}}.}
}
\value{
\code{x} (invisibly).
}
\description{
Print a \code{group_tna} Object
}
\examples{
model <- group_model(engagement_mmm)
print(model)

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.group_tna_bootstrap}
\alias{print.group_tna_bootstrap}
\title{Print \code{group_tna} Bootstrap Results}
\usage{
\method{print}{group_tna_bootstrap}(x, ...)
}
\arguments{
\item{x}{A \code{group_tna_bootstrap} object.}

\item{...}{Arguments passed to \code{\link[=print.tna_bootstrap]{print.tna_bootstrap()}}.}
}
\value{
\code{x} (invisibly).
}
\description{
Print \code{group_tna} Bootstrap Results
}
\examples{
model <- group_model(engagement_mmm)
# Low number of iteration for CRAN
boot <- bootstrap(model, iter = 10)
print(boot)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.group_tna_cliques}
\alias{plot.group_tna_cliques}
\title{Plot Found Cliques}
\usage{
\method{plot}{group_tna_cliques}(x, title, ...)
}
\arguments{
\item{x}{A \code{group_tna_cliques} object.}

\item{title}{A \code{character} vector of titles to use for each plot.}

\item{...}{Arguments passed to \code{\link[=plot.tna_cliques]{plot.tna_cliques()}}.}
}
\value{
A \code{list} (invisibly) with one element per cluster. Each element
contains a \code{cograph_network} plot when only one clique is present per
cluster, otherwise the element is \code{NULL}.
}
\description{
Plot Found Cliques
}
\examples{
model <- group_model(engagement_mmm)
cliq <- cliques(model, size = 2)
plot(cliq, ask = FALSE)

}
\seealso{
Clique-related functions
\code{\link{cliques}()},
\code{\link{plot.tna_cliques}()},
\code{\link{print.group_tna_cliques}()},
\code{\link{print.tna_cliques}()}
}
\concept{cliques}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random.R
\name{random_tna_mmm}
\alias{random_tna_mmm}
\title{Build a Random Mixture Markov Model Object}
\usage{
random_tna_mmm(
  n_clusters = NULL,
  n_states = NULL,
  states = NULL,
  category = NULL,
  alpha = NULL,
  diag_boost = NULL,
  n_sequences = NULL,
  seq_length = NULL,
  n_covariates = 1L,
  seed = NULL
)
}
\arguments{
\item{n_clusters}{An \code{integer} giving the number of mixture clusters.
If \code{NULL} (the default), drawn from \code{2:4} on each call.}

\item{n_states}{An \code{integer} >= 2 giving the number of states. If \code{NULL}
(the default), a value is drawn from \code{7:11} on each call.}

\item{states}{An optional \code{character} vector of state labels of length
at least \code{n_states}. The first \code{n_states} are used. If \code{NULL} (the
default), labels are taken from \code{category} or auto-picked from a
built-in pool that fits \code{n_states}.}

\item{category}{An optional \code{character} string naming a built-in label
pool. Available pools are returned by \code{list_random_state_pools()}.
When \code{NULL} (the default), a pool whose size is at least \code{n_states}
is sampled at random. Ignored when \code{states} is supplied.}

\item{alpha}{A positive \code{numeric} Dirichlet concentration parameter.
Small values (e.g. \code{0.3}) produce sparse, peaked transitions; large
values (e.g. \code{5}) produce near-uniform transitions. If \code{NULL} (the
default), a value is drawn from \code{Uniform(0.5, 1.0)} on each call.}

\item{diag_boost}{A non-negative \code{numeric} added to the diagonal of the
transition matrix before re-normalising rows. Larger values make
states "stickier" (more self-transitions). If \code{NULL} (the default),
a value is drawn from \code{Uniform(1.5, 3.0)} on each call.}

\item{n_sequences}{An \code{integer} giving the number of sequences to
simulate from the true parameters. If \code{NULL} (the default), a value
is drawn from \code{500:800} on each call.}

\item{seq_length}{An \code{integer} giving the length of each simulated
sequence. If \code{NULL} (the default), a value is drawn from \code{6:20} on
each call.}

\item{n_covariates}{An \code{integer} >= 1 giving the number of regression
variables (including the intercept) used to predict cluster membership.
Default is 1 (intercept only). When > 1, additional rows are added to
the coefficient matrix.}

\item{seed}{An \code{integer} random seed for reproducibility, or \code{NULL}
(the default) for fresh randomness on every call.}
}
\value{
An object of class \code{tna_mmm} containing fields
\code{observations}, \code{transition_probs}, \code{initial_probs}, \code{coefficients},
\code{vcov}, \code{most_probable_cluster}, \code{cluster_names}, \code{state_names},
\code{n_clusters}, \code{n_states}, \code{n_sequences}, \code{n_covariates}.
}
\description{
Construct a synthetic \code{tna_mmm} object that mirrors the \emph{structure} a
fitted seqHMM mixed Markov model exposes to \link{tna} without depending on
seqHMM at runtime. The returned object can be passed to
\code{\link[=group_model]{group_model()}} (dispatching via \code{group_model.tna_mmm}) and to
\code{\link[=mmm_stats]{mmm_stats()}} (dispatching via \code{mmm_stats.tna_mmm}).

Real seqHMM \code{mhmm} objects continue to dispatch via the original
\verb{*.mhmm} methods.
}
\examples{
model <- random_tna_mmm(seed = 1)
mmm_stats(model)
grp <- group_model(model)

}
\seealso{
Other data: 
\code{\link{import_data}()},
\code{\link{import_onehot}()},
\code{\link{list_random_state_pools}()},
\code{\link{prepare_data}()},
\code{\link{print.tna_data}()},
\code{\link{random_group_tna}()},
\code{\link{random_tna}()},
\code{\link{simulate.group_tna}()},
\code{\link{simulate.tna}()}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{compare}
\alias{compare}
\alias{compare.tna}
\alias{compare.matrix}
\title{Compare Two Matrices or TNA Models with Comprehensive Metrics}
\usage{
compare(x, ...)

\method{compare}{tna}(x, y, scaling = "none", measures = character(0), network = TRUE, ...)

\method{compare}{matrix}(x, y, scaling = "none", measures = character(0), network = TRUE, ...)
}
\arguments{
\item{x}{A \code{tna} object or a \code{matrix} of weights.}

\item{...}{Ignored.}

\item{y}{A \code{tna} object or a \code{matrix} of weights.}

\item{scaling}{A  \code{character} string naming a scaling method to
apply to the weights before comparing them. The supported options are:
\itemize{
\item \code{"none"}: No scaling is performed. The weights are used as is.
\item \code{"minmax"}: Performs min-max normalization, i.e., the minimum value is
subtracted and the differences are scaled by the range.
\item \code{"max"}: Max-normalization: the values are divided by the maximum value.
\item \code{"rank"}: Applies min-max normalization to the ranks of the weights
(computed with \code{ties.method = "average"}).
\item \code{"zscore"}: Computes the standard score, i.e. the mean weight is
subtracted and the differences are scaled by the standard deviation.
\item \code{"robust"}: Computes the robust z-score, i.e. the median weight is
subtracted and the differences are scaled by the median absolute deviation
(using \link[stats:mad]{stats::mad}).
\item \code{"log"}: Simply the natural logarithm of the weights.
\item \code{"log1p"}: As above, but adds 1 to the values before taking the logarithm.
Useful for scenarios with zero weights.
\item \code{"softmax"}: Performs softmax normalization.
\item \code{"quantile"}: Uses the empirical quantiles of the weights
via \link[stats:ecdf]{stats::ecdf}.
}}

\item{measures}{A \code{character} vector indicating which centrality
measures should be computed. See \code{\link[=centralities]{centralities()}} for the
available measures. No measures are included by default.}

\item{network}{A \code{logical} value indicating whether network metrics should
be included in the comparison. The default is \code{TRUE}.}
}
\value{
A \code{tna_comparison} object, which is a \code{list} containing the
following elements:
\itemize{
\item \code{matrices}: A \code{list} containing the scaled matrices of the input \code{tna}
objects or the scaled inputs themselves in the case of matrices.
\item \code{difference_matrix}: A \code{matrix} of differences \code{x - y}.
\item \code{edge_metrics}: A \code{data.frame} of edge-level metrics about the differences.
\item \code{summary_metrics}: A \code{data.frame} of summary metrics of the differences
across all edges.
\item \code{network_metrics}: A \code{data.frame} of network metrics for both \code{x} and \code{y}.
\item \code{centrality_differences}: A \code{data.frame} of differences in centrality
measures computes from \code{x} and \code{y}.
\item \code{centrality_correlations}: A \code{numeric} vector of correlations of the
centrality measures between \code{x} and \code{y}.
}
}
\description{
Various distances, measures of dissimilarity and similarity, correlations
and other metrics are computed to compare the models. Optionally, the weight
matrices of the models can be scaled before comparison. The resulting object
can be used to produce heatmap plots and scatterplots to further illustrate
the differences.
}
\examples{
# Comparing TNA models
model_x <- tna(group_regulation[1:200, ])
model_y <- tna(group_regulation[1001:1200, ])
comp1 <- compare(model_x, model_y)

# Comparing matrices
mat_x <- model_x$weights
mat_y <- model_y$weights
comp2 <- compare(mat_x, mat_y)

# Comparing a matrix to a TNA model
comp3 <- compare(mat_x, model_y)

}
\seealso{
Model comparison functions
\code{\link{compare.group_tna}()},
\code{\link{compare_sequences}()},
\code{\link{plot.tna_comparison}()},
\code{\link{plot.tna_sequence_comparison}()},
\code{\link{plot_compare}()},
\code{\link{plot_compare.group_tna}()},
\code{\link{print.tna_comparison}()},
\code{\link{print.tna_sequence_comparison}()}
}
\concept{comparison}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mmm.R
\name{mmm_stats}
\alias{mmm_stats}
\alias{mmm_stats.tna_mmm}
\alias{mmm_stats.mhmm}
\title{Retrieve Statistics from a Mixture Markov Model (MMM)}
\usage{
mmm_stats(x, level = 0.05)

\method{mmm_stats}{tna_mmm}(x, level = 0.05)

\method{mmm_stats}{mhmm}(x, level = 0.05)
}
\arguments{
\item{x}{A \code{mhmm} object (from the \code{seqHMM} package) or a \code{tna_mmm}
object built by \code{\link[=random_tna_mmm]{random_tna_mmm()}}.}

\item{level}{A \code{numeric} value representing the significance level for
hypothesis testing and confidence intervals. Defaults to \code{0.05}.}
}
\value{
A \code{data.frame} object.
}
\description{
Retrieve Statistics from a Mixture Markov Model (MMM)
}
\examples{
mmm_stats(engagement_mmm)

}
\seealso{
Cluster-related functions
\code{\link{communities}()},
\code{\link{group_model}()},
\code{\link{rename_groups}()}
}
\concept{clusters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.tna_stability}
\alias{print.tna_stability}
\title{Print Centrality Stability Results}
\usage{
\method{print}{tna_stability}(x, ...)
}
\arguments{
\item{x}{A \code{tna_stability} object.}

\item{...}{Additional arguments passed to the generic \code{print} method.}
}
\value{
\code{x} (invisibly).
}
\description{
Print Centrality Stability Results
}
\examples{
model <- tna(group_regulation)
# Small number of iterations and drop proportions for CRAN
cs <- estimate_cs(
  model,
  measures = c("InStrength", "OutStrength"),
  drop_prop = seq(0.3, 0.9, by = 0.2),
  iter = 10
)
print(cs)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.group_tna_centralities}
\alias{plot.group_tna_centralities}
\title{Plot Centrality Measures}
\usage{
\method{plot}{group_tna_centralities}(
  x,
  reorder = TRUE,
  ncol = 3,
  scales = c("free_x", "fixed"),
  colors,
  palette = "Set2",
  labels = TRUE,
  ...
)
}
\arguments{
\item{x}{A \code{group_tna_centralities} object.}

\item{reorder}{A \code{logical} value indicating whether to reorder the values
for each centrality in a descending order. The default is \code{TRUE}.}

\item{ncol}{Number of columns to use for the facets. The default is 3.}

\item{scales}{Either \code{"fixed"} or \code{"free_x"} (the default). If \code{"free_x"},
the horizontal axis is scaled individually in each facet. If \code{"fixed"},
the same values are used for all axes.}

\item{colors}{The colors for each node (default is the model colors
if the \code{tna} model object is passed, otherwise \code{"black"}).}

\item{palette}{A color palette to be applied if \code{colors} is not specified.}

\item{labels}{A \code{logical} value indicating whether to show the centrality
numeric values. The default is \code{TRUE}.}

\item{...}{Ignored.}
}
\value{
A \code{ggplot} object displaying a line chart for each centrality
with one line per cluster.
}
\description{
Plot Centrality Measures
}
\examples{
model <- group_model(engagement_mmm)
cm <- centralities(model)
plot(cm)

}
\seealso{
Centrality measure functions
\code{\link{betweenness_network}()},
\code{\link{centralities}()},
\code{\link{plot.tna_centralities}()},
\code{\link{print.group_tna_centralities}()},
\code{\link{print.tna_centralities}()}
}
\concept{centralities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{compare_sequences}
\alias{compare_sequences}
\alias{compare_sequences.default}
\alias{compare_sequences.group_tna}
\title{Compare Sequences Between Groups}
\usage{
compare_sequences(x, ...)

\method{compare_sequences}{default}(
  x,
  group,
  sub,
  min_freq = 5L,
  test = TRUE,
  iter = 1000L,
  adjust = "bonferroni",
  ...
)

\method{compare_sequences}{group_tna}(
  x,
  sub,
  min_freq = 5L,
  test = TRUE,
  iter = 1000L,
  adjust = "bonferroni",
  ...
)
}
\arguments{
\item{x}{A \code{group_tna} object or a \code{data.frame} containing sequence data in
wide format.}

\item{...}{Not used.}

\item{group}{A \code{vector} indicating the group assignment of each
row of the data/sequence. Must have the same length as the number of
rows/sequences of \code{x}. Alternatively, a single \code{character} string giving
the column name of the data that defines the group when \code{x} is a wide
format \code{data.frame} or a \code{tna_data} object.}

\item{sub}{An \code{integer} vector of pattern lengths to analyze.
The default is \code{2:5}.}

\item{min_freq}{An \code{integer} giving the minimum number of times that a
specific pattern has to be observed in each group to be included in the
analysis. The default is \code{5}.}

\item{test}{A \code{logical} value indicating whether to test the differences
of pattern counts between the groups using a permutation test.
The default is \code{TRUE}.}

\item{iter}{An \code{integer} giving the number of iterations for the permutation
test. The default is \code{1000}.}

\item{adjust}{A \code{character} string naming the multiple comparison
correction method (default: \code{"bonferroni"}). Supports all
\link[stats:p.adjust]{stats::p.adjust} methods: \code{"holm"}, \code{"hochberg"}, \code{"hommel"},
\code{"bonferroni"}, \code{"BH"}, \code{"BY"}, \code{"fdr"}, \code{"none"}. The adjustment
is carried out within sequences of the same length.}
}
\value{
A \code{tna_sequence_comparison} object, which is a \code{data.frame} with
columns giving the names of the patterns, pattern frequencies, pattern
proportions (within patterns of the same length), effect sizes,
and p-values of the tests.
}
\description{
Performs comprehensive sequence comparison analysis between groups. All
patterns of the sequences (subsequences of specific length) are extracted
from all sequences in each group. The pattern frequencies are compared
between the groups using a permutation test. The reported effect size is the
difference between the observed test statistic (sum of squared differences
between the observed and expected counts) and the mean value over the
permutation samples divided by their standard deviation times square root of
the number of observations.
}
\examples{
\donttest{
# Use a subset to keep the example fast.
idx <- c(1:500, 1001:1500)
group <- c(rep("High", 500), rep("Low", 500))
comp <- compare_sequences(group_regulation[idx, ], group)

# With permutation test (small number of iterations for CRAN)
comp_test <- compare_sequences(
  group_regulation[idx, ],
  group,
  test = TRUE,
  iter = 5
)
}

}
\seealso{
Model comparison functions
\code{\link{compare}()},
\code{\link{compare.group_tna}()},
\code{\link{plot.tna_comparison}()},
\code{\link{plot.tna_sequence_comparison}()},
\code{\link{plot_compare}()},
\code{\link{plot_compare.group_tna}()},
\code{\link{print.tna_comparison}()},
\code{\link{print.tna_sequence_comparison}()}
}
\concept{comparison}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/betweenness.R
\name{betweenness_network}
\alias{betweenness_network}
\alias{betweenness_network.tna}
\title{Build and Visualize a Network with Edge Betweenness}
\usage{
betweenness_network(x, directed = TRUE, invert = TRUE)

\method{betweenness_network}{tna}(x, directed = TRUE, invert = TRUE)
}
\arguments{
\item{x}{A \code{tna} object.}

\item{directed}{A \code{logical} value. If \code{TRUE}, the network is considered
directed.}

\item{invert}{A \code{logical} value indicating whether the weights should be
inverted for distance-based measures. The default is \code{TRUE}.}
}
\value{
A \code{tna} object where the edge weights are edge betweenness values.
}
\description{
This function builds a network from a transition matrix in a \code{tna} object
and computes edge betweenness for the network.
}
\examples{
model <- tna(group_regulation)
betweenness_network(model)

}
\seealso{
Centrality measure functions
\code{\link{centralities}()},
\code{\link{plot.group_tna_centralities}()},
\code{\link{plot.tna_centralities}()},
\code{\link{print.group_tna_centralities}()},
\code{\link{print.tna_centralities}()}
}
\concept{centralities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/random.R
\name{random_tna}
\alias{random_tna}
\title{Build a Random Transition Network Analysis Model}
\usage{
random_tna(
  n_states = NULL,
  states = NULL,
  category = NULL,
  alpha = NULL,
  diag_boost = NULL,
  trans_matrix = NULL,
  init_probs = NULL,
  n_sequences = NULL,
  seq_length = NULL,
  type = "relative",
  return_params = FALSE,
  seed = NULL
)
}
\arguments{
\item{n_states}{An \code{integer} >= 2 giving the number of states. If \code{NULL}
(the default), a value is drawn from \code{7:11} on each call.}

\item{states}{An optional \code{character} vector of state labels of length
at least \code{n_states}. The first \code{n_states} are used. If \code{NULL} (the
default), labels are taken from \code{category} or auto-picked from a
built-in pool that fits \code{n_states}.}

\item{category}{An optional \code{character} string naming a built-in label
pool. Available pools are returned by \code{list_random_state_pools()}.
When \code{NULL} (the default), a pool whose size is at least \code{n_states}
is sampled at random. Ignored when \code{states} is supplied.}

\item{alpha}{A positive \code{numeric} Dirichlet concentration parameter.
Small values (e.g. \code{0.3}) produce sparse, peaked transitions; large
values (e.g. \code{5}) produce near-uniform transitions. If \code{NULL} (the
default), a value is drawn from \code{Uniform(0.5, 1.0)} on each call.}

\item{diag_boost}{A non-negative \code{numeric} added to the diagonal of the
transition matrix before re-normalising rows. Larger values make
states "stickier" (more self-transitions). If \code{NULL} (the default),
a value is drawn from \code{Uniform(1.5, 3.0)} on each call.}

\item{trans_matrix}{An optional square \code{numeric} matrix of transition
probabilities. When supplied, \code{alpha} and \code{diag_boost} are ignored
for the transition matrix. Rows are renormalised to sum to one.}

\item{init_probs}{An optional \code{numeric} vector of initial state
probabilities. Renormalised to sum to one. If \code{NULL}, drawn from a
Dirichlet on the same alphabet.}

\item{n_sequences}{An \code{integer} giving the number of sequences to
simulate from the true parameters. If \code{NULL} (the default), a value
is drawn from \code{500:800} on each call.}

\item{seq_length}{An \code{integer} giving the length of each simulated
sequence. If \code{NULL} (the default), a value is drawn from \code{6:20} on
each call.}

\item{type}{A \code{character} string giving the model type. One of
\code{"relative"} (the default), \code{"frequency"}, \code{"co-occurrence"},
\code{"attention"}.}

\item{return_params}{A \code{logical}. If \code{TRUE}, returns a \code{list} with the
fitted model and the ground-truth parameters used to generate it.
Default is \code{FALSE}.}

\item{seed}{An \code{integer} random seed for reproducibility, or \code{NULL}
(the default) for fresh randomness on every call.}
}
\value{
A \code{tna} object, or a \code{list} with elements \code{model},
\code{trans_matrix}, \code{init_probs}, \code{sequences}, \code{labels}, and \code{category}
when \code{return_params = TRUE}.
}
\description{
Construct a fully-functional \code{tna} object from synthetic parameters
without needing pre-existing sequence data. Random transition probabilities
are drawn from a Dirichlet distribution, sequences are simulated from the
resulting model, and a canonical \code{tna} object is fitted on those sequences.

Calling \code{random_tna()} with no arguments returns a fresh, sticky network
with a coherent alphabet, drawn fresh on every call.
}
\examples{
# Fresh random model on every call
model <- random_tna()

# Explicit small-state demo using the engagement pool
model <- random_tna(n_states = 3, category = "engagement")

# Reproducible model
model <- random_tna(seed = 42)

# Recover the ground-truth parameters
out <- random_tna(seed = 7, return_params = TRUE)
out$trans_matrix

}
\seealso{
Other data: 
\code{\link{import_data}()},
\code{\link{import_onehot}()},
\code{\link{list_random_state_pools}()},
\code{\link{prepare_data}()},
\code{\link{print.tna_data}()},
\code{\link{random_group_tna}()},
\code{\link{random_tna_mmm}()},
\code{\link{simulate.group_tna}()},
\code{\link{simulate.tna}()}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/permutation.R
\name{permutation_test.group_tna}
\alias{permutation_test.group_tna}
\title{Compare Networks using a Permutation Test}
\usage{
\method{permutation_test}{group_tna}(
  x,
  groups,
  adjust = "none",
  iter = 1000,
  paired = FALSE,
  level = 0.05,
  measures = character(0),
  consecutive = FALSE,
  ...
)
}
\arguments{
\item{x}{A \code{group_tna} object}

\item{groups}{An \code{integer} vector or a \code{character} vector of group indices
or names, respectively, defining which groups to compare. When not provided,
all pairs are compared (the default).}

\item{adjust}{A \code{character} string for the method to adjust p-values with
for multiple comparisons. The default is \code{"none"} for no adjustment.
See the \code{method} argument of \code{\link[stats:p.adjust]{stats::p.adjust()}} for details and available
adjustment methods.}

\item{iter}{An \code{integer} giving the number of permutations to perform.
The default is 1000.}

\item{paired}{A \code{logical} value. If \code{TRUE}, perform paired permutation tests;
if \code{FALSE}, perform unpaired tests. The default is \code{FALSE}.}

\item{level}{A \code{numeric} value giving the significance level for the
permutation tests. The default is 0.05.}

\item{measures}{A \code{character} vector of centrality measures to test.
See \code{\link[=centralities]{centralities()}} for a list of available centrality measures.}

\item{consecutive}{A \code{logical} value. If \code{FALSE} (the default), all pairwise
comparisons are performed in lexicographic order with respect to the order
of the groups. If \code{TRUE}, only comparisons between consecutive pairs
of groups are performed.}

\item{...}{Additional arguments passed to \code{\link[=centralities]{centralities()}}.}
}
\description{
Test edge weight differences between all pairs or a subset of pairs of
a \code{group_tna} object. See \code{\link[=permutation_test.tna]{permutation_test.tna()}} for more details.
}
\examples{
model <- group_model(engagement_mmm)
# Small number of iterations for CRAN
permutation_test(model, iter = 20)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_mosaic.group_tna}
\alias{plot_mosaic.group_tna}
\title{Plot State Frequencies as a Mosaic Between Two Groups}
\usage{
\method{plot_mosaic}{group_tna}(x, label, ...)
}
\arguments{
\item{x}{A \code{group_tna} object.}

\item{label}{An optional \code{character} string that can be provided to specify
the grouping factor name if \code{x} was not constructed using a column name of
the original data.}

\item{...}{Ignored.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot State Frequencies as a Mosaic Between Two Groups
}
\examples{
model <- group_model(engagement, group = rep(1:3, length.out = 1000))
plot_mosaic(model)

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliability.R
\name{reliability}
\alias{reliability}
\alias{reliability.tna}
\title{Assess Model Reliability}
\usage{
reliability(x, ...)

\method{reliability}{tna}(
  x,
  types = "relative",
  split = 0.5,
  iter = 1000,
  scaling = "none",
  ...
)
}
\arguments{
\item{x}{A \code{tna} object.}

\item{...}{Ignored.}

\item{types}{A \code{character} vector giving the model types to fit.
See \code{\link[=build_model]{build_model()}} for available options.}

\item{split}{A \code{numeric} value between \code{0} and \code{1} specifying the proportion
of data for the split. The default is \code{0.5} for an even split.}

\item{iter}{An \code{integer} specifying number of iterations (splits). The
default is \code{1000}.}

\item{scaling}{See \code{\link[=compare]{compare()}}.}
}
\value{
A \code{tna_reliability} object.
}
\description{
Performs reliability analysis and outputs a concise summary of key
metrics. The results can also be visualized.
}
\examples{
# Small number of iterations for CRAN
model <- tna(engagement)
rel <- reliability(model, iter = 20)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_reliability}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot_mosaic}
\alias{plot_mosaic}
\alias{plot_mosaic.tna}
\title{Create a Mosaic Plot of Transitions or Events}
\usage{
plot_mosaic(x, ...)

\method{plot_mosaic}{tna}(x, ...)
}
\arguments{
\item{x}{A \code{tna} or a \code{group_tna} object.}

\item{...}{Ignored.}
}
\value{
A \code{ggplot} object.
}
\description{
Create a Mosaic Plot of Transitions or Events
}
\examples{
ftna_model <- ftna(group_regulation)
plot_mosaic(ftna_model)

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.group_tna_communities}
\alias{plot.group_tna_communities}
\title{Plot Detected Communities}
\usage{
\method{plot}{group_tna_communities}(x, title, colors, ...)
}
\arguments{
\item{x}{A \code{group_tna_communities} object.}

\item{title}{A \code{character} vector of titles to use for each plot.}

\item{colors}{A \code{character} vector of colors to use.}

\item{...}{Arguments passed to \code{\link[=plot.tna_communities]{plot.tna_communities()}}.}
}
\value{
A \code{list} (invisibly) of \code{cograph_network} objects in which the
nodes are colored by community for each cluster.
}
\description{
Plot Detected Communities
}
\examples{
model <- group_model(engagement_mmm)
comm <- communities(model)
plot(comm)

}
\seealso{
Community detection functions
\code{\link{communities}()},
\code{\link{plot.tna_communities}()},
\code{\link{print.group_tna_communities}()},
\code{\link{print.tna_communities}()}
}
\concept{communities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tna-package.R
\docType{data}
\name{engagement_mmm}
\alias{engagement_mmm}
\title{Example Mixture Markov Model Object for the \code{engagement} Data}
\format{
A \code{tna_mmm} object containing three clusters over the
states \code{"Engaged"}, \code{"Moderate"}, \code{"Disengaged"}.
}
\source{
Generated by \code{data-raw/engagement_mmm.R} using
\code{\link[=random_tna_mmm]{random_tna_mmm()}}.
}
\usage{
engagement_mmm
}
\description{
A \code{tna_mmm} object structurally equivalent to a fitted seqHMM
mixture Markov model, used by the package's examples for
\code{group_model()} and \code{mmm_stats()}. Generated deterministically by
\code{\link[=random_tna_mmm]{random_tna_mmm()}} with engagement-flavored state labels and
three clusters; carries no foreign class references so it is safe
to bundle.
}
\details{
Re-generate via \code{data-raw/engagement_mmm.R} whenever the simulator
changes.
}
\seealso{
Datasets
\code{\link{engagement}},
\code{\link{group_regulation}},
\code{\link{group_regulation_long}}
}
\concept{datasets}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.group_tna_cliques}
\alias{print.group_tna_cliques}
\title{Print Found Cliques}
\usage{
\method{print}{group_tna_cliques}(x, ...)
}
\arguments{
\item{x}{A \code{group_tna_cliques} object.}

\item{...}{Arguments passed to \code{\link[=print.tna_cliques]{print.tna_cliques()}}.}
}
\value{
\code{x} (invisibly).
}
\description{
Print Found Cliques
}
\examples{
model <- group_model(engagement_mmm)
cliq <- cliques(model, size = 2)
print(cliq)

}
\seealso{
Clique-related functions
\code{\link{cliques}()},
\code{\link{plot.group_tna_cliques}()},
\code{\link{plot.tna_cliques}()},
\code{\link{print.tna_cliques}()}
}
\concept{cliques}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.tna_data}
\alias{print.tna_data}
\title{Print a TNA Data Object}
\usage{
\method{print}{tna_data}(x, data = "sequence", ...)
}
\arguments{
\item{x}{A \code{tna_data} object.}

\item{data}{A \code{character} string that defines the data to be printed
tibble. Accepts either \code{"sequence"} (default) for wide format sequence data,
\code{"meta"}, for the wide format metadata, or \code{"long"} for the long format
data.}

\item{...}{Arguments passed to the tibble \code{print} method.}
}
\value{
\code{x} (invisibly).
}
\description{
Print a TNA Data Object
}
\examples{
res <- prepare_data(group_regulation_long, action = "Action", actor = "Actor",
time = "Time")
print(res, which = "sequence")
print(res, which = "meta")
print(res, which = "long")

}
\seealso{
Other data: 
\code{\link{import_data}()},
\code{\link{import_onehot}()},
\code{\link{list_random_state_pools}()},
\code{\link{prepare_data}()},
\code{\link{random_group_tna}()},
\code{\link{random_tna}()},
\code{\link{random_tna_mmm}()},
\code{\link{simulate.group_tna}()},
\code{\link{simulate.tna}()}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{import_data}
\alias{import_data}
\title{Import Wide Format Sequence Data as Long Format Sequence Data}
\usage{
import_data(data, cols, id_cols, window_size = 1, replace_zeros = TRUE)
}
\arguments{
\item{data}{A \code{data.frame} in wide format.}

\item{cols}{An \code{expression} giving a tidy selection of column names to be
transformed into long format (actions). This can be a vector of column
names (e.g., \code{c(feature1, feature2)}) or a range  specified as
\code{feature1:feature6} (without quotes) to include all columns from
'feature1' to 'feature6' in the order they appear in the data frame.
For more information on tidy selections, see \code{\link[dplyr:select]{dplyr::select()}}.}

\item{id_cols}{An \code{expression} giving a tidy selection of column names that
uniquely identify each observation (IDs).}

\item{window_size}{An \code{integer} specifying the size of the window for
sequence grouping. Default is 1 (each row is a separate window).}

\item{replace_zeros}{A \code{logical} value indicating whether to replace 0s
in \code{cols} with \code{NA}. The default is \code{TRUE}.}
}
\value{
A \code{data.frame} in long format with added columns for window and
sequence order.
}
\description{
This function transforms wide format data where features are in separate
columns into a long format suitable for sequence analysis. It creates
windows of data based on row order and generates sequence order within
these windows.
}
\examples{
data <- data.frame(
  ID = c("A", "A", "B", "B"),
  Time = c(1, 2, 1, 2),
  feature1 = c(10, 0, 15, 20),
  feature2 = c(5, 8, 0, 12),
  feature3 = c(2, 4, 6, 8),
  other_col = c("X", "Y", "Z", "W")
)

# Using a vector
long_data1 <- import_data(
  data = data,
  cols = c(feature1, feature2),
  id_cols = c("ID", "Time"),
  window_size = 2,
  replace_zeros = TRUE
)

# Using a column range
long_data2 <- import_data(
  data = data,
  cols = feature1:feature3,
  id_cols = c("ID", "Time"),
  window_size = 2,
  replace_zeros = TRUE
)

}
\seealso{
Other data: 
\code{\link{import_onehot}()},
\code{\link{list_random_state_pools}()},
\code{\link{prepare_data}()},
\code{\link{print.tna_data}()},
\code{\link{random_group_tna}()},
\code{\link{random_tna}()},
\code{\link{random_tna_mmm}()},
\code{\link{simulate.group_tna}()},
\code{\link{simulate.tna}()}
}
\concept{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centralities.R
\name{centralities}
\alias{centralities}
\alias{centralities.tna}
\alias{centralities.matrix}
\alias{centralities.group_tna}
\title{Calculate Centrality Measures for a Transition Matrix}
\usage{
centralities(x, loops = FALSE, normalize = FALSE, invert = TRUE, measures)

\method{centralities}{tna}(x, loops = FALSE, normalize = FALSE, invert = TRUE, measures)

\method{centralities}{matrix}(x, loops = FALSE, normalize = FALSE, invert = TRUE, measures)

\method{centralities}{group_tna}(x, loops = FALSE, normalize = FALSE, invert = TRUE, measures)
}
\arguments{
\item{x}{A \code{tna} object, a \code{group_tna} object, or
a square \code{matrix} representing edge weights.}

\item{loops}{A \code{logical} value indicating whether to include loops in the
network when computing the centrality measures. The default is \code{FALSE}.}

\item{normalize}{A \code{logical} value indicating whether the centralities
should be normalized. The default is \code{FALSE}.}

\item{invert}{A \code{logical} value indicating whether the weights should be
inverted for distance-based measures. The default is \code{TRUE}.}

\item{measures}{A \code{character} vector indicating which centrality
measures should be computed. If missing, all available measures are
returned. See 'Details' for the available measures.}
}
\value{
A \code{tna_centralities} object which is a tibble (\code{tbl_df}).
containing centrality measures for each state.
}
\description{
Calculates several centrality measures. See 'Details' for
information about the measures.
}
\details{
The following measures are provided:
\itemize{
\item \code{OutStrength}: Outgoing strength centrality, calculated using
\code{\link[igraph:strength]{igraph::strength()}} with \code{mode = "out"}. It measures the total weight
of the outgoing edges from each node.
\item \code{InStrength}: Incoming strength centrality, calculated using
\code{\link[igraph:strength]{igraph::strength()}} with \code{mode = "in"}. It measures the total weight
of the incoming edges to each node.
\item \code{ClosenessIn}: Closeness centrality (incoming), calculated using
\code{\link[igraph:closeness]{igraph::closeness()}} with \code{mode = "in"}. It measures how close a node
is to all other nodes based on the incoming paths.
\item \code{ClosenessOut}: Closeness centrality (outgoing), calculated using
\code{\link[igraph:closeness]{igraph::closeness()}} with \code{mode = "out"}. It measures how close a node
is to all other nodes based on the outgoing paths.
\item \code{Closeness}: Closeness centrality (overall), calculated using
\code{\link[igraph:closeness]{igraph::closeness()}} with \code{mode = "all"}. It measures how close a node
is to all other nodes based on both incoming and outgoing paths.
\item \code{Betweenness}: Betweenness centrality defined by the number of
geodesics calculated using \code{\link[igraph:betweenness]{igraph::betweenness()}}.
\item \code{BetweennessRSP}: Betweenness centrality based on randomized shortest
paths (Kivimäki et al. 2016). It measures the extent to which a
node lies on the shortest paths between other nodes.
\item \code{Diffusion}: Diffusion centrality of Banerjee et.al. (2014).
It measures the influence of a node in spreading information through
the network.
\item \code{Clustering}: Signed clustering coefficient of Zhang and Horvath (2005)
based on the symmetric adjacency matrix (sum of the adjacency matrix
and its transpose). It measures the degree to which nodes tend to
cluster together.
}
}
\examples{
model <- tna(group_regulation)

# Centrality measures including loops in the network
centralities(model)

# Centrality measures excluding loops in the network
centralities(model, loops = FALSE)

# Centrality measures normalized
centralities(model, normalize = TRUE)

}
\seealso{
Centrality measure functions
\code{\link{betweenness_network}()},
\code{\link{plot.group_tna_centralities}()},
\code{\link{plot.tna_centralities}()},
\code{\link{print.group_tna_centralities}()},
\code{\link{print.tna_centralities}()}
}
\concept{centralities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{as.igraph.group_tna}
\alias{as.igraph.group_tna}
\title{Coerce a Specific Group from a \code{group_tna} Object into an \code{igraph} Object.}
\usage{
\method{as.igraph}{group_tna}(x, which, ...)
}
\arguments{
\item{x}{The object to convert.}

\item{which}{The number or name of the group.}

\item{...}{Additional arguments. None currently.}
}
\value{
An \code{igraph} object.
}
\description{
Coerce a Specific Group from a \code{group_tna} Object into an \code{igraph} Object.
}
\seealso{
Helper functions
\code{\link{as.igraph.matrix}()},
\code{\link{as.igraph.tna}()}
}
\concept{helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.summary.group_tna}
\alias{print.summary.group_tna}
\title{Print a Summary of a Grouped Transition Network Analysis Model}
\usage{
\method{print}{summary.group_tna}(x, ...)
}
\arguments{
\item{x}{A \code{summary.group_tna} object.}

\item{...}{Arguments passed to the \code{tibble} print method}
}
\value{
\code{x} (invisibly).
}
\description{
Print a Summary of a Grouped Transition Network Analysis Model
}
\examples{
model <- group_model(engagement_mmm)
print(summary(model))

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{build_model}
\alias{build_model}
\alias{build_model.default}
\alias{build_model.matrix}
\alias{build_model.stslist}
\alias{build_model.data.frame}
\alias{build_model.tna_data}
\alias{build_model.tsn}
\alias{tna}
\alias{ftna}
\alias{ctna}
\alias{atna}
\alias{tsn}
\title{Build a Transition Network Analysis Model}
\usage{
build_model(x, ...)

\method{build_model}{default}(
  x,
  type = "relative",
  scaling = character(0L),
  params = list(),
  inits,
  ...
)

\method{build_model}{matrix}(
  x,
  type = "relative",
  scaling = character(0L),
  params = list(),
  inits,
  ...
)

\method{build_model}{stslist}(
  x,
  type = "relative",
  scaling = character(0L),
  cols = tidyselect::everything(),
  params = list(),
  concat = 1L,
  begin_state,
  end_state,
  ...
)

\method{build_model}{data.frame}(
  x,
  type = "relative",
  scaling = character(0L),
  cols = tidyselect::everything(),
  concat = 1L,
  params = list(),
  begin_state,
  end_state,
  ...
)

\method{build_model}{tna_data}(
  x,
  type = "relative",
  scaling = character(0L),
  params = list(),
  concat = 1L,
  begin_state,
  end_state,
  ...
)

\method{build_model}{tsn}(
  x,
  type = "relative",
  scaling = character(0L),
  params = list(),
  concat = 1L,
  begin_state,
  end_state,
  ...
)

tna(x, ...)

ftna(x, ...)

ctna(x, ...)

atna(x, ...)

tsn(x, ...)
}
\arguments{
\item{x}{A \code{stslist} (from \code{TraMineR}), \code{data.frame}, a \code{matrix}, or
a \code{tna_data} object (see \code{\link[=prepare_data]{prepare_data()}}).
For \code{stslist} and \code{data.frame} objects \code{x}
should describe a sequence of events or states to be used for building the
Markov model.  If \code{x} is a matrix, it is assumed that the element on row
\code{i} and column \code{j} is the weight of the edge representing the transition
from state \code{i} to state \code{j}. If \code{x} is a \code{data.frame}, then
it must be in wide format (see \code{cols} on how to define columns for the
time points).}

\item{...}{Ignored. For the \code{build_model} aliases (e.g., \code{tna}), this
argument matches the actual arguments to \code{build_model} beside \code{x}.}

\item{type}{A \code{character} string describing the weight matrix type.
Currently supports the following types:
\itemize{
\item \code{"relative"} for relative frequencies (probabilities, the default)
\item \code{"frequency"} for frequencies.
\item \code{"co-occurrence"} for co-occurrences.
\item \code{"n-gram"} for n-gram transitions. Captures higher-order transitions by
considering sequences of n states, useful for identifying longer
patterns.
\item \code{"gap"} allows transitions between non-adjacent states, with
transitions weighted by the gap size.
\item \code{"reverse"} considers the sequences in reverse order
(resulting in what is called a reply network in some contexts).
The resulting weight matrix is the transpose of the \code{"frequency"}
option.
\item \code{"attention"} aggregates all downstream pairs of states with an
exponential decay for the gap between states. The parameter \code{lambda}
can be used to control the decay rate (the default is 1)-
}}

\item{scaling}{A \code{character} vector describing how to scale the weights
defined by \code{type}. When a vector is provided, the scaling options are
applied in the respective order. For example, \code{c("rank", "minmax")} would
first compute the ranks, then scale them to the unit interval using
min-max normalization. An empty vector corresponds to no scaling.
Currently supports the following options:
\itemize{
\item \code{"minmax"} performs min-max normalization to scale the weights to the
unit interval. Note that if the smallest weight is positive, it
will be zero after scaling.
\item \code{"max"} Multiplies the weights by the reciprocal of the largest weight
to scale the weights to the unit interval. This options preserves
positive ranks, unlike \code{"minmax"} when all weights are positive.
\item \code{"rank"} Computes the ranks of the weights using \code{\link[base:rank]{base::rank()}} with
\code{ties.method = "average"}.
}}

\item{params}{A \code{list} of additional arguments for models of specific
\code{type}. The potential elements of this list are:
\itemize{
\item \code{n_gram}: An \code{integer} for n-gram transitions specifying the number of
adjacent events. The default value is 2.
\item \code{max_gap}: An \code{integer} for the gap-allowed transitions specifying the
largest allowed gap size. The default is 1.
\item \code{windowed}: Perform the model estimation by window. Supported for
\code{relative}, \code{frequency} and \code{co-occurrence} \code{type}s.
\item \code{window_size}: An \code{integer} for the sliding window transitions
specifying the window size. The default is 2.
\item \code{window_type}: A \code{character} string that defines the window type.
Either \code{"rolling"} or \code{"tumbling"}.
\item \code{weighted}: A \code{logical} value. If \code{TRUE}, the transitions
are weighted by the inverse of the sequence length. Can be used for
frequency, co-occurrence and reverse model types. The default is
\code{FALSE}.
\item \code{direction}: A \code{character} string specifying the direction of attention
for models of \code{type = "attention"}. The available options are
\code{"backward"}, \code{"forward"}, and \code{"both"}, for backward attention,
forward attention, and bidirectional attention, respectively.
The default is \code{"forward"}.
\item \code{decay}: A \code{function} that specifies the decay of the weights between
two time points at a specific distance. The function should take three
arguments: \code{i}, \code{j} and \code{lambda}, where \code{i} and \code{j} are \code{numeric}
vectors of time values, and \code{lambda} is  a \code{numeric} value for the
decay rate. The function should return a \code{numeric} vector of weights.
The default is \code{function(i, j, lambda) exp(-abs(i - j) / lambda)}.
\item \code{lambda}: A \code{numeric} value for the decay rate. The default is 1.
\item \code{time}: A \code{matrix} or a \code{data.frame} providing the time values
for each sequence and at time index. For \code{tna_data} objects, this can
also be a logical value, where \code{TRUE} will use the \code{time_data} element
of \code{x} for the time values. \code{Date} values are converted to \code{numeric}.
\item \code{duration}: A \code{matrix} or a \code{data.frame} providing the
time spent in each state for each sequence and time index.
This is an alternative to \code{time}.
}}

\item{inits}{An optional \code{numeric} vector of initial state probabilities
for each state. The vector will be scaled to unity.}

\item{cols}{An \code{expression} giving a tidy selection of columns that should
be considered as sequence data. By default, all columns are used.}

\item{concat}{An \code{integer} for the number of consecutive sequences
to concatenate. The default is 1 (no concatenation).}

\item{begin_state}{A \code{character} string for an additional begin state.
This state is added as the first observation for every sequence to
signify the beginning of the sequence}

\item{end_state}{A \code{character} string for an additional end state.
This state is added as the last observation for every sequence to
signify the end of the sequence.}
}
\value{
An object of class \code{tna} which is a \code{list} containing the
following elements:
\itemize{
\item \code{weights}: An adjacency \code{matrix} of the model (weight matrix).
\item \code{inits}: A \code{numeric} vector of initial values for each state.
For \code{matrix} type \code{x}, this element will be \code{NULL} if \code{inits} is not
directly provided
\item \code{labels}: A \code{character} vector of the state labels, or \code{NULL} if
there are no labels.
\item \code{data}: The original sequence data that has been converted to an
internal format used by the package when \code{x} is a \code{stslist} or a
\code{data.frame} object. Otherwise \code{NULL}.
}
}
\description{
Construct a transition network analysis (TNA) model from sequence data.
The function takes a data set of sequence of events or states as input and
builds a TNA model. It extracts the edge weights and initial probabilities
from the data along with the state labels. The function also accepts weight
matrices and initial state probabilities directly.
}
\examples{
model <- build_model(group_regulation)
print(model)

model <- tna(group_regulation)

model <- ftna(group_regulation)

model <- ctna(group_regulation)

model <- atna(group_regulation)

}
\seealso{
Basic functions
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot.tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.tna}
\alias{plot.tna}
\title{Plot a Transition Network Analysis Model}
\usage{
\method{plot}{tna}(
  x,
  node_list,
  use_list_order = TRUE,
  scale_nodes,
  scaling_factor = 0.5,
  ...
)
}
\arguments{
\item{x}{A \code{tna} object from \code{\link[=build_model]{build_model()}}.}

\item{node_list}{An optional \code{list} of two \code{character} vectors that define
two mutually exclusive groups of node labels.}

\item{use_list_order}{A \code{logical} value. If \code{node_list} is provided,
defines how the order of the nodes in the plot is defined. A \code{TRUE} value
uses the order in \code{node_list}. Otherwise, the nodes are ranked based on
edge weights and ordered according to the rank.}

\item{scale_nodes}{A \code{character} string giving the name of a centrality
measure to scale the node size by. See \code{\link[=centralities]{centralities()}} for valid names.
If missing (the default), uses default \code{\link[cograph:splot]{cograph::splot()}} scaling.
The value of \code{node_size} provided via \code{...} is used as baseline size.}

\item{scaling_factor}{A \code{numeric} value specifying how strongly to scale
the nodes when \code{scale_nodes} is provided. Values
between 0 and 1 will result in smaller differences and values larger
than 1 will result in greater differences. The default is \code{0.5}.}

\item{...}{Additional arguments passed to \code{\link[cograph:splot]{cograph::splot()}} or
\code{\link[cograph:plot_htna]{cograph::plot_htna()}}.}
}
\value{
A \code{cograph_network} plot of the transition network.
}
\description{
This function plots a transition network analysis (TNA) model using
the \code{cograph} package. The nodes in the graph represent states, with node
sizes corresponding to initial state probabilities. Edge labels represent
the edge weights of the network. See \code{\link[cograph:splot]{cograph::splot()}} for details on
how to further configure the plot.
}
\examples{
model <- tna(group_regulation)
plot(model)

}
\seealso{
Basic functions
\code{\link{build_model}()},
\code{\link{hist.group_tna}()},
\code{\link{hist.tna}()},
\code{\link{plot.group_tna}()},
\code{\link{plot_frequencies}()},
\code{\link{plot_frequencies.group_tna}()},
\code{\link{plot_mosaic}()},
\code{\link{plot_mosaic.group_tna}()},
\code{\link{plot_mosaic.tna_data}()},
\code{\link{print.group_tna}()},
\code{\link{print.summary.group_tna}()},
\code{\link{print.summary.tna}()},
\code{\link{print.tna}()},
\code{\link{summary.group_tna}()},
\code{\link{summary.tna}()},
\code{\link{tna-package}}
}
\concept{basic}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groups.R
\name{group_model}
\alias{group_model}
\alias{group_model.default}
\alias{group_model.tna_mmm}
\alias{group_model.mhmm}
\alias{group_model.tna_clustering}
\alias{group_tna}
\alias{group_ftna}
\alias{group_ctna}
\alias{group_atna}
\title{Build a Grouped Transition Network Analysis Model}
\usage{
group_model(x, ...)

\method{group_model}{default}(
  x,
  group,
  type = "relative",
  scaling = character(0L),
  groupwise = FALSE,
  cols = tidyselect::everything(),
  params = list(),
  concat = 1L,
  na.rm = TRUE,
  ...
)

\method{group_model}{tna_mmm}(
  x,
  type = "relative",
  scaling = character(0L),
  groupwise = FALSE,
  params = list(),
  na.rm = TRUE,
  ...
)

\method{group_model}{mhmm}(
  x,
  type = "relative",
  scaling = character(0L),
  groupwise = FALSE,
  params = list(),
  na.rm = TRUE,
  ...
)

\method{group_model}{tna_clustering}(
  x,
  type = "relative",
  scaling = character(0L),
  groupwise = FALSE,
  params = list(),
  na.rm = TRUE,
  ...
)

group_tna(x, ...)

group_ftna(x, ...)

group_ctna(x, ...)

group_atna(x, ...)
}
\arguments{
\item{x}{An \code{stslist} object describing a sequence of events or states to
be used for building the Markov model. The argument \code{x} also accepts
\code{data.frame} objects in wide format, and \code{tna_data} objects.
This can also be the output of clustering from
\code{\link[=cluster_sequences]{cluster_sequences()}}.}

\item{...}{Ignored.}

\item{group}{A \code{vector} indicating the group assignment of each
row of the data/sequence. Must have the same length as the number of
rows/sequences of \code{x}. Alternatively, a single \code{character} string giving
the column name of the data that defines the group when \code{x} is a wide
format \code{data.frame} or a \code{tna_data} object. If not provided, each row of
the data forms a cluster. Not used when \code{x} is a mixture Markov model
or a clustering result.}

\item{type}{A \code{character} string describing the weight matrix type.
Currently supports the following types:
\itemize{
\item \code{"relative"} for relative frequencies (probabilities, the default)
\item \code{"frequency"} for frequencies.
\item \code{"co-occurrence"} for co-occurrences.
\item \code{"n-gram"} for n-gram transitions. Captures higher-order transitions by
considering sequences of n states, useful for identifying longer
patterns.
\item \code{"gap"} allows transitions between non-adjacent states, with
transitions weighted by the gap size.
\item \code{"reverse"} considers the sequences in reverse order
(resulting in what is called a reply network in some contexts).
The resulting weight matrix is the transpose of the \code{"frequency"}
option.
\item \code{"attention"} aggregates all downstream pairs of states with an
exponential decay for the gap between states. The parameter \code{lambda}
can be used to control the decay rate (the default is 1)-
}}

\item{scaling}{A \code{character} vector describing how to scale the weights
defined by \code{type}. When a vector is provided, the scaling options are
applied in the respective order. For example, \code{c("rank", "minmax")} would
first compute the ranks, then scale them to the unit interval using
min-max normalization. An empty vector corresponds to no scaling.
Currently supports the following options:
\itemize{
\item \code{"minmax"} performs min-max normalization to scale the weights to the
unit interval. Note that if the smallest weight is positive, it
will be zero after scaling.
\item \code{"max"} Multiplies the weights by the reciprocal of the largest weight
to scale the weights to the unit interval. This options preserves
positive ranks, unlike \code{"minmax"} when all weights are positive.
\item \code{"rank"} Computes the ranks of the weights using \code{\link[base:rank]{base::rank()}} with
\code{ties.method = "average"}.
}}

\item{groupwise}{A \code{logical} value that indicates whether scaling methods
should be applied by group (\code{TRUE}) or globally (\code{FALSE}, the default).}

\item{cols}{An \code{expression} giving a tidy selection of the
columns that should be considered as sequence data.
The default is all columns. The columns are
automatically determined for \code{tna_data} objects. The \code{group} column
is automatically removed from these columns if provided.}

\item{params}{A \code{list} of additional arguments for models of specific
\code{type}. The potential elements of this list are:
\itemize{
\item \code{n_gram}: An \code{integer} for n-gram transitions specifying the number of
adjacent events. The default value is 2.
\item \code{max_gap}: An \code{integer} for the gap-allowed transitions specifying the
largest allowed gap size. The default is 1.
\item \code{windowed}: Perform the model estimation by window. Supported for
\code{relative}, \code{frequency} and \code{co-occurrence} \code{type}s.
\item \code{window_size}: An \code{integer} for the sliding window transitions
specifying the window size. The default is 2.
\item \code{window_type}: A \code{character} string that defines the window type.
Either \code{"rolling"} or \code{"tumbling"}.
\item \code{weighted}: A \code{logical} value. If \code{TRUE}, the transitions
are weighted by the inverse of the sequence length. Can be used for
frequency, co-occurrence and reverse model types. The default is
\code{FALSE}.
\item \code{direction}: A \code{character} string specifying the direction of attention
for models of \code{type = "attention"}. The available options are
\code{"backward"}, \code{"forward"}, and \code{"both"}, for backward attention,
forward attention, and bidirectional attention, respectively.
The default is \code{"forward"}.
\item \code{decay}: A \code{function} that specifies the decay of the weights between
two time points at a specific distance. The function should take three
arguments: \code{i}, \code{j} and \code{lambda}, where \code{i} and \code{j} are \code{numeric}
vectors of time values, and \code{lambda} is  a \code{numeric} value for the
decay rate. The function should return a \code{numeric} vector of weights.
The default is \code{function(i, j, lambda) exp(-abs(i - j) / lambda)}.
\item \code{lambda}: A \code{numeric} value for the decay rate. The default is 1.
\item \code{time}: A \code{matrix} or a \code{data.frame} providing the time values
for each sequence and at time index. For \code{tna_data} objects, this can
also be a logical value, where \code{TRUE} will use the \code{time_data} element
of \code{x} for the time values. \code{Date} values are converted to \code{numeric}.
\item \code{duration}: A \code{matrix} or a \code{data.frame} providing the
time spent in each state for each sequence and time index.
This is an alternative to \code{time}.
}}

\item{concat}{An \code{integer} for the number of consecutive sequences
to concatenate. The default is 1 (no concatenation).}

\item{na.rm}{A \code{logical} value that determines if observations with \code{NA}
value in \code{group} be removed. If \code{FALSE}, an additional category for \code{NA}
values will be added. The default is \code{FALSE} and a warning is issued
if \code{NA} values are detected.}
}
\value{
An object of class \code{group_tna} which is a \code{list} containing one
element per cluster. Each element is a \code{tna} object.
}
\description{
This function constructs a transition network analysis (TNA) model for
each group from a given sequence, wide-format dataframe
or a mixture Markov model.
}
\examples{
# Manually specified groups
group <- c(rep("High", 1000), rep("Low", 1000))
model <- group_model(group_regulation, group = group)

# Groups defined by a mixed Markov model
model <- group_model(engagement_mmm)

model <- group_tna(group_regulation, group = gl(2, 1000))

model <- group_ftna(group_regulation, group = gl(2, 1000))

model <- group_ctna(group_regulation, group = gl(2, 1000))

model <- group_atna(group_regulation, group = gl(2, 1000))

}
\seealso{
Cluster-related functions
\code{\link{communities}()},
\code{\link{mmm_stats}()},
\code{\link{rename_groups}()}
}
\concept{clusters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build.R
\name{sna}
\alias{sna}
\title{Build a Social Network Analysis Model}
\usage{
sna(x, aggregate = sum, ...)
}
\arguments{
\item{x}{A \code{data.frame} or a \code{matrix} with three columns: the first two
representing the states and the third giving the weights.}

\item{aggregate}{A \code{function} to use for aggregating the weights. The
default is \code{\link[=sum]{sum()}}.}

\item{...}{Additional arguments passed to \code{aggregate}.}
}
\value{
A \code{tna} object representing the model.
}
\description{
Build a Social Network Analysis Model
}
\examples{
set.seed(123)
d <- data.frame(
  from = sample(LETTERS[1:4], 100, replace = TRUE),
  to = sample(LETTERS[1:4], 100, replace = TRUE),
  weight = rexp(100)
)
model <- sna(d)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.R
\name{plot.tna_reliability}
\alias{plot.tna_reliability}
\title{Plot Reliability Analysis Results}
\usage{
\method{plot}{tna_reliability}(x, type = "histogram", metric = "Median Abs. Diff.", ...)
}
\arguments{
\item{x}{A \code{tna_reliability} object.}

\item{type}{A \code{character} string specifying the plot type. The options are:
\code{"histogram"} (default), \code{"density"}, or \code{"boxplot"}.}

\item{metric}{A \code{character} string specifying the metric to plot.
The default is the median absolute difference (\code{"Median Abs. Diff."}).}

\item{...}{Ignored}
}
\description{
Plot Reliability Analysis Results
}
\examples{
# Small number of iterations for CRAN
model <- tna(engagement)
rel <- reliability(model, iter = 20)
plot(rel)

}
\seealso{
Validation functions
\code{\link{bootstrap}()},
\code{\link{deprune}()},
\code{\link{estimate_cs}()},
\code{\link{permutation_test}()},
\code{\link{permutation_test.group_tna}()},
\code{\link{plot.group_tna_bootstrap}()},
\code{\link{plot.group_tna_permutation}()},
\code{\link{plot.group_tna_stability}()},
\code{\link{plot.tna_bootstrap}()},
\code{\link{plot.tna_permutation}()},
\code{\link{plot.tna_stability}()},
\code{\link{print.group_tna_bootstrap}()},
\code{\link{print.group_tna_permutation}()},
\code{\link{print.group_tna_stability}()},
\code{\link{print.summary.group_tna_bootstrap}()},
\code{\link{print.summary.tna_bootstrap}()},
\code{\link{print.tna_bootstrap}()},
\code{\link{print.tna_clustering}()},
\code{\link{print.tna_permutation}()},
\code{\link{print.tna_reliability}()},
\code{\link{print.tna_stability}()},
\code{\link{prune}()},
\code{\link{pruning_details}()},
\code{\link{reliability}()},
\code{\link{reprune}()},
\code{\link{summary.group_tna_bootstrap}()},
\code{\link{summary.tna_bootstrap}()}
}
\concept{validation}
